package pwc.taxtech.atms.agent;

import com.mysql.jdbc.MysqlErrorNumbers;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.sql.SQLException;

import static pwc.taxtech.atms.constant.Constant.CREATE_DB_EXISTS;
import static pwc.taxtech.atms.constant.Constant.CREATE_DB_FAILED;
import static pwc.taxtech.atms.constant.Constant.CREATE_DB_SUCCESS;

@Service
public class DatabaseAgent {
    private static Logger LOGGER = LoggerFactory.getLogger(DatabaseAgent.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;


    public int createDatabase(String dbName) {
        try {
            jdbcTemplate.execute("CREATE DATABASE  " + dbName + " default charset utf8mb4 COLLATE utf8mb4_general_ci");
            return CREATE_DB_SUCCESS;
        } catch (Exception exception) {
            if (exception.getCause() instanceof SQLException) {
                SQLException ex = (SQLException) exception.getCause();
                if (ex.getErrorCode() == MysqlErrorNumbers.ER_DB_CREATE_EXISTS) {
                    LOGGER.warn("db is exists {}", dbName);
                    return CREATE_DB_EXISTS;
                }
            }

            LOGGER.warn("Unknown Exception {}", exception.getMessage(), exception);
            return CREATE_DB_FAILED;
        }
    }

}
