package pwc.taxtech.atms.common;

import pwc.taxtech.atms.dto.PagingDto;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class PagingList<T> {
    private List<T> sourceData;
    private PagingDto pagingDto;
    /**
     * 总页数
     */
    private int totalPage = 0;

    /**
     * 当前是第几页
     */
    private int curPageNo = 0;

    /**
     * 每页的大小
     */
    private int pageSize = 0;

    /**
     * 每页默认大小
     */
    private static final int DEFAULT_PAGE_SIZE = 500;

    private int totalCount = 0;


    public PagingList(List<T> sourceData, PagingDto pagingDto) {
        this.sourceData = sourceData;
        this.pagingDto = pagingDto;
        this.pageSize = pagingDto.getPageSize();
        this.curPageNo = pagingDto.getPageIndex();
        this.totalCount = pagingDto.getTotalCount();
        init();
    }

    private void init() {
        if (pagingDto.getPageSize() <= 0) {
            throw new IllegalArgumentException("Paging size must be greater than zero.");
        }
        if (null == sourceData) {
            throw new NullPointerException("Paging resource list must be not null.");
        }
        if (totalCount % pageSize > 0) {
            this.totalPage = (totalCount / pageSize) + 1;
        } else {
            this.totalPage = totalCount / pageSize;
        }
    }

    public List<T> getPagingList() {
        if (curPageNo <= totalPage) {
            if (curPageNo < totalCount) {
                return sourceData.stream().skip((curPageNo - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
            } else {
                int lastPageItemCount = totalCount - (curPageNo - 1) * pageSize;
                return sourceData.stream().skip((curPageNo - 1) * pageSize).limit(lastPageItemCount).collect(Collectors.toList());
            }
        } else {
            return new ArrayList<>();
        }
    }
}
