package pwc.taxtech.atms.common.datasource;

import com.alibaba.druid.pool.DruidDataSource;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.pool2.KeyedPooledObjectFactory;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.exception.ParameterException;

public class DataSourceFactory implements KeyedPooledObjectFactory<String, DruidDataSource> {
    private static final Logger logger = LoggerFactory.getLogger(DataSourceFactory.class);
    private DataSourceConfig dataSourceConfig;
    private static final String SYMBOL = "demo_db_name";

    public DataSourceFactory(DataSourceConfig dataSourceConfig) {
        this.dataSourceConfig = dataSourceConfig;
    }

    @Override
    public PooledObject<DruidDataSource> makeObject(String s) throws Exception {
        if (StringUtils.isBlank(s)) {
            throw new ParameterException("empty database name.");
        }
        DruidDataSource dynamicDataSource = new DruidDataSource();
        String url = StringUtils.replace(dataSourceConfig.getUrl(), SYMBOL, s);
        dynamicDataSource.setUrl(url);
        dynamicDataSource.setUsername(dataSourceConfig.getUsername());
        dynamicDataSource.setPassword(dataSourceConfig.getPassword());
        dynamicDataSource.init();
        return new DefaultPooledObject<>(dynamicDataSource);
    }

    @Override
    public void destroyObject(String s, PooledObject<DruidDataSource> pooledObject) throws Exception {
        pooledObject.getObject().close();
    }

    @Override
    public boolean validateObject(String s, PooledObject<DruidDataSource> pooledObject) {
        try {
            return pooledObject.getObject().isEnable();
        } catch (Exception e) {
            logger.error("Failed to validate DruidDataSource: ", e);
        }
        return false;
    }

    @Override
    public void activateObject(String s, PooledObject<DruidDataSource> pooledObject) throws Exception {

    }

    @Override
    public void passivateObject(String s, PooledObject<DruidDataSource> pooledObject) throws Exception {

    }
}
