package pwc.taxtech.atms.controller;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.ApiOperation;
import pwc.taxtech.atms.dto.BusinessUnitDto;
import pwc.taxtech.atms.dto.BusinessUnitInputDto;
import pwc.taxtech.atms.dto.IdModel;
import pwc.taxtech.atms.service.BusinessUnitService;

/** @see PwC.Tax.Tech.Atms.WebApi\Controllers\BusinessUnitController.cs */
@RestController
@RequestMapping("/api/v1/businessunit")
public class BusinessUnitController {

    private static Logger logger = LoggerFactory.getLogger(BusinessUnitController.class);

    @Autowired
    private BusinessUnitService businessUnitService;

    @ApiOperation(value = "主数据事业部查询", notes = "返回所有事业部数据")
    @RequestMapping(value = "/getlist", method = RequestMethod.GET)
    public @ResponseBody List<BusinessUnitDto> getBusinessUnits() {
        logger.debug("BusinessUnitController getBusinessUnits");
        return businessUnitService.getBusinessUnits();
    }

    @ApiOperation(value = "主数据事业部增加", notes = "根据Request Body的信息添加一个或多个新的事业部数据")
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    public void addBusinessUnits(@RequestBody List<BusinessUnitInputDto> businessUnitDtoList) {
        logger.debug("BusinessUnitController addBusinessUnits");
        businessUnitService.addBusinessUnits(businessUnitDtoList);
    }

    @ApiOperation(value = "主数据事业部删除", notes = "根据给定的id删除一条事业部数据")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public @ResponseBody Integer deleteBusinessUnit(@RequestBody IdModel idModel) {
        logger.debug("BusinessUnitController deleteBusinessUnit");
        return businessUnitService.deleteBusinessUnit(idModel);
    }

    @ApiOperation(value = "主数据事业部修改", notes = "根据Request Body的信息修改一个或多个事业部数据")
    @RequestMapping(method = RequestMethod.PUT)
    public @ResponseBody Boolean updateBusinessUnits(@RequestBody List<BusinessUnitDto> businessUnitDtoList) {
        logger.debug("BusinessUnitController updateBusinessUnit");
        return businessUnitService.updateBusinessUnits(businessUnitDtoList);
    }

}
