package pwc.taxtech.atms.controller;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.dto.PagingDto;
import pwc.taxtech.atms.dto.PagingResultDto;
import pwc.taxtech.atms.dto.vatdto.CustomsInvoiceDto;
import pwc.taxtech.atms.dto.vatdto.CustomsInvoiceFilter;
import pwc.taxtech.atms.dto.vatdto.JsonCustomsInvoiceDto;
import pwc.taxtech.atms.thirdparty.ExcelUtil;
import pwc.taxtech.atms.vat.service.impl.CustomsInvoiceServiceImpl;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import static javax.servlet.http.HttpServletResponse.SC_NO_CONTENT;
import static javax.servlet.http.HttpServletResponse.SC_OK;

@RestController
@RequestMapping(value = "api/v1/CustomsInvoice")
public class CustomsInvoiceController {
    @Autowired
    CustomsInvoiceServiceImpl customsInvoiceService;

    @RequestMapping(value = "GetCustomsInvoiceDataForDisplay", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public String getCustomsInvoiceDataForDisplay(@RequestParam int fromPeriod, @RequestParam int toPeriod, @RequestParam String criteria, @RequestParam String pagination) {
        CustomsInvoiceFilter filter = new CustomsInvoiceFilter();
        if (StringUtils.isNotBlank(criteria) && !"null".equals(criteria)) {
            filter = JSONObject.parseObject(criteria, CustomsInvoiceFilter.class);
        } else {
            filter = null;
        }

        PagingDto paging = new PagingDto();
        if (StringUtils.isNotBlank(pagination) && !"null".equals(pagination)) {
            paging = JSONObject.parseObject(pagination, PagingDto.class);
        } else {
            paging = null;
        }
        return convertCustomsInvoiceDataToJson(fromPeriod, toPeriod, filter, paging);
    }

    @RequestMapping(value = "GetCustomsInvoicesByPeriodId/{periodId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity getCustomsInvoicesByPeriodId(@PathVariable Integer periodId) {
        return ResponseEntity.ok().body(customsInvoiceService.getCustomsInvoicesByPeriodId(periodId));
    }

    @RequestMapping(value = "getCustomsInvoicesValidates/{periodId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity getCustomsInvoicesValidates(@PathVariable Integer periodId) {
        return ResponseEntity.ok().body(customsInvoiceService.getCustomsInvoicesValidates(periodId));
    }

    @RequestMapping(value = "ExportQueryData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void exportQueryData(@RequestParam String criteria, HttpServletResponse response) {
        CustomsInvoiceFilter filter = new CustomsInvoiceFilter();
        if (StringUtils.isNotBlank(criteria) && !criteria.equals("null")) {
            filter = JSONObject.parseObject(criteria, CustomsInvoiceFilter.class);
        } else {
            filter = null;
        }

        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + UUID.randomUUID() + ".xlsx");
        response.setHeader("x-file-name", UUID.randomUUID() + ".xlsx");

        OutputStream os = null;
        try {
            os = response.getOutputStream();
            int count = getDownloadFilePath(filter, os);

            if (count == 0) {
                response.setStatus(SC_NO_CONTENT);
            } else {
                response.setStatus(SC_OK);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }


    }

    private int getDownloadFilePath(CustomsInvoiceFilter filter, OutputStream outputStream) {
        List<JsonCustomsInvoiceDto> list = GetCustomsInvoiceDataList(filter);
        if (list.size() == 0) {
            return 0;
        }

        Map<String, String> header = new HashMap<>();
        header.put("RecordNumber", "RecordNumber");
        header.put("PeriodId", "期间");
        header.put("IssueDate", "开具日期");
        header.put("PayNum", "海关缴款书号码");
        header.put("InvoiceAmount", "金额");
        header.put("InvoiceTaxAmount", "税额");
        header.put("AuditResult", "稽核结果");

        ExcelUtil.exportExcel(header, list, outputStream);
        return list.size();
    }

    private List<JsonCustomsInvoiceDto> GetCustomsInvoiceDataList(CustomsInvoiceFilter filter) {
        List<CustomsInvoiceDto> balanceList = customsInvoiceService.getCustomsInvoicesByPeriodIds(filter.getPeriodFrom(), filter.getPeriodTo());
        if (filter != null && !balanceList.isEmpty()) {
            balanceList = filterCustomsInvoiceDataList(filter, balanceList);
        }

        List<JsonCustomsInvoiceDto> recordList = new ArrayList<>();
        JsonCustomsInvoiceDto record;
        int indexNumber = 1;
        for (CustomsInvoiceDto dto : balanceList) {
            record = new JsonCustomsInvoiceDto();
            CommonUtils.copyProperties(dto, record);
            record.setRecordNumber(String.valueOf(indexNumber));
            record.setIssueDate(DateFormat.getDateInstance().format(dto.getIssueDate()));
            record.setInvoiceTaxAmount(dto.getInvoiceTaxAmount() != null ? dto.getInvoiceTaxAmount().toString() : "0.00");
            record.setInvoiceAmount(dto.getInvoiceAmount() != null ? dto.getInvoiceAmount().toString() : "0.00");
            recordList.add(record);
            indexNumber++;
        }

        return recordList;
    }

    private String convertCustomsInvoiceDataToJson(int fromPeriod, int toPeriod, CustomsInvoiceFilter filter, PagingDto pagination) {
        String result = StringUtils.EMPTY;
        List<CustomsInvoiceDto> balanceList = customsInvoiceService.getCustomsInvoicesByPeriodIds(fromPeriod, toPeriod);
        if (filter != null && !balanceList.isEmpty()) {
            balanceList = filterCustomsInvoiceDataList(filter, balanceList);
        }

        if (pagination != null) {
            int pageAskingFor = 0;

            if (pagination.getPageIndex() > 0) {
                pageAskingFor = pagination.getPageIndex() - 1;
            }

            pagination.setTotalCount(balanceList.size());
            balanceList = balanceList.stream().skip(pageAskingFor * pagination.getPageSize()).limit(pagination.getPageSize()).collect(Collectors.toList());
        }
        List<JsonCustomsInvoiceDto> recordList = new ArrayList<>();
        JsonCustomsInvoiceDto record;

        int indexNumber = 1;
        for (CustomsInvoiceDto dto : balanceList) {
            record = new JsonCustomsInvoiceDto();
            CommonUtils.copyProperties(dto, record);
            record.setRecordNumber(String.valueOf(indexNumber));
            record.setIssueDate(DateFormat.getDateInstance().format(dto.getIssueDate()));
            record.setInvoiceTaxAmount(dto.getInvoiceTaxAmount() != null ? dto.getInvoiceTaxAmount().toString() : "0.00");
            record.setInvoiceAmount(dto.getInvoiceAmount() != null ? dto.getInvoiceAmount().toString() : "0.00");
            recordList.add(record);
            indexNumber++;
        }
        PagingResultDto<JsonCustomsInvoiceDto> listWithPaging = new PagingResultDto<>();
        listWithPaging.setPageInfo(pagination);
        listWithPaging.setList(recordList);
        result = JSONObject.toJSONString(listWithPaging);
        return result;
    }

    private List<CustomsInvoiceDto> filterCustomsInvoiceDataList(CustomsInvoiceFilter filter, List<CustomsInvoiceDto> balanceList) {
        if (!balanceList.isEmpty()) {
            if (StringUtils.isNotBlank(filter.getPayNum())) {
                String[] names = filter.getPayNum().split(" ");
                List<CustomsInvoiceDto> ds = new ArrayList<>();
                for (String name : names) {
                    ds.addAll(balanceList.stream().filter(a -> a.getPayNum().contains(name)).collect(Collectors.toList()));
                }
                balanceList = ds;
            }
        }

        if (filter.getTaxAmountFrom() != null || filter.getTaxAmountTo() != null) {
            if (filter.getTaxAmountFrom() != null && filter.getTaxAmountTo() != null) {
                balanceList = balanceList.stream()
                        .filter(a -> a.getInvoiceTaxAmount() != null
                                && (filter.getTaxAmountFrom().compareTo(a.getInvoiceAmount()) < 1
                                && filter.getTaxAmountTo().compareTo(a.getInvoiceAmount()) > -1))
                        .collect(Collectors.toList());
            } else if (filter.getTaxAmountFrom() != null) {
                balanceList = balanceList.stream()
                        .filter(a -> a.getInvoiceTaxAmount() != null
                                && filter.getTaxAmountFrom().compareTo(a.getInvoiceAmount()) < 1)
                        .collect(Collectors.toList());
            } else {
                balanceList = balanceList.stream()
                        .filter(a -> a.getInvoiceTaxAmount() != null
                                && filter.getTaxAmountTo().compareTo(a.getInvoiceAmount()) > -1)
                        .collect(Collectors.toList());
            }
        }

        if (filter.getInvoiceAmountFrom() != null || filter.getInvoiceAmountTo() != null) {
            if (filter.getInvoiceAmountFrom() != null && filter.getInvoiceAmountTo() != null) {
                balanceList = balanceList.stream()
                        .filter(a -> a.getInvoiceAmount() != null
                                && (filter.getInvoiceAmountFrom().compareTo(a.getInvoiceAmount()) < 1
                                && filter.getInvoiceAmountTo().compareTo(a.getInvoiceAmount()) > -1))
                        .collect(Collectors.toList());
            } else if (filter.getInvoiceAmountFrom() != null) {
                balanceList = balanceList.stream()
                        .filter(a -> a.getInvoiceAmount() != null
                                && filter.getInvoiceAmountFrom().compareTo(a.getInvoiceAmount()) < 1)
                        .collect(Collectors.toList());
            } else {
                balanceList = balanceList.stream()
                        .filter(a -> a.getInvoiceAmount() != null
                                && filter.getInvoiceAmountTo().compareTo(a.getInvoiceAmount()) > -1)
                        .collect(Collectors.toList());
            }
        }

        if (filter.getInvoiceStartDate() != null || filter.getInvoiceEndDate() != null) {
            if (filter.getInvoiceStartDate() != null && filter.getInvoiceEndDate() != null) {
                balanceList = balanceList.stream()
                        .filter(a -> a.getIssueDate() != null
                                && (filter.getInvoiceStartDate().compareTo(a.getIssueDate()) < 1
                                && filter.getInvoiceEndDate().compareTo(a.getIssueDate()) > -1))
                        .collect(Collectors.toList());
            } else if (filter.getInvoiceStartDate() != null) {
                balanceList = balanceList.stream()
                        .filter(a -> a.getIssueDate() != null
                                && filter.getInvoiceStartDate().compareTo(a.getIssueDate()) < 1)
                        .collect(Collectors.toList());
            } else {
                balanceList = balanceList.stream()
                        .filter(a -> a.getIssueDate() != null
                                && filter.getInvoiceEndDate().compareTo(a.getIssueDate()) > -1)
                        .collect(Collectors.toList());
            }
        }
        return balanceList;
    }
}
