package pwc.taxtech.atms.controller;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.ApiOperation;
import pwc.taxtech.atms.dto.navtree.DevTreeDto;
import pwc.taxtech.atms.dto.navtree.RolePermissionDisplayDto;
import pwc.taxtech.atms.service.PermissionService;

@RestController
@RequestMapping("/api/v1/permission/")
public class PermissionController {

    private static Logger logger = LoggerFactory.getLogger(PermissionController.class);

    @Autowired
    private PermissionService permissionService;

    @ApiOperation(value = "Active or Deactive the specific area.")
    @RequestMapping(value = "getIvhTreePermissionsByRoleID", method = RequestMethod.GET)
    public @ResponseBody RolePermissionDisplayDto getIvhTreePermissionsByRoleID(@RequestParam String roleID,
            @RequestParam String serviceType) {
        logger.info("getIvhTreePermissionsByRoleID: roleID={}, serviceType={}.", roleID, serviceType);
        return permissionService.getIvhTreePermissionsByRoleID(roleID, serviceType);
    }

    @ApiOperation(value = "通过传入的角色ID列表获取树形结构权限")
    @RequestMapping(value = "getIvhTreePermissionsByRoleIDList", method = RequestMethod.POST)
    public @ResponseBody List<DevTreeDto> getIvhTreePermissionsByRoleIDList(@RequestBody List<String> roleIDList,
            @RequestParam String serviceType) {
        return permissionService.getIvhTreePermissionsByRoleIDList(roleIDList, serviceType);
    }
    
    @ApiOperation(value = "获取所有权限树形结构列表")
    @RequestMapping(value = "getAllPermissions", method = RequestMethod.GET)
    public @ResponseBody List<DevTreeDto> getAllPermissions(@RequestParam String serviceType) {
        return permissionService.getAllPermissions(serviceType);
    }
}
