package pwc.taxtech.atms.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import pwc.taxtech.atms.constant.enums.VoucherSearchEnum;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.QueryVoucherParmDto;
import pwc.taxtech.atms.dto.vatdto.VatEnterpriseAccountDto;
import pwc.taxtech.atms.vat.service.TBDataImportService;
import pwc.taxtech.atms.vat.service.VoucherService;

import java.util.List;

@RequestMapping(value = "api/v1/voucher")
@RestController
public class VoucherController {
    @Autowired
    VoucherService voucherService;

    @Autowired
    TBDataImportService tbDataImportService;

    private static final Logger logger = LoggerFactory.getLogger(VoucherController.class);

    @RequestMapping(value = "voucherSelectAdvancedByEntry", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<Object> voucherSelectAdvancedByEntry(@RequestBody QueryVoucherParmDto parmDto) {
        try {
            return voucherService.voucherSelectAdvancedByEntry(parmDto.getListQueryCondition(), VoucherSearchEnum.values()[parmDto.getMainRelation()], parmDto.isAllJe(), parmDto.getPagingInfo());
        } catch (Exception ex) {
            logger.error("voucherSelectAdvancedByEntryError", ex);
        }
        OperationResultDto<Object> result = new OperationResultDto<>();
        result.setResult(false);
        return result;
    }

    @RequestMapping(value = "voucherSelectAdvancedCount", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<Object> voucherSelectAdvancedCount(@RequestBody QueryVoucherParmDto parmDto) {
        try {
            return voucherService.voucherSelectAdvancedCount(parmDto.getListQueryCondition(), VoucherSearchEnum.values()[parmDto.getMainRelation()], parmDto.isEntryShow(), parmDto.isAllJe());
        } catch (Exception ex) {
            logger.error("voucherSelectAdvancedCountError", ex);
        }
        OperationResultDto<Object> result = new OperationResultDto<>();
        result.setResult(false);
        return result;
    }

    @RequestMapping(value = "getEnterpriseAccount",method = RequestMethod.GET,produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public List<VatEnterpriseAccountDto> getEnterpriseAccount(){
        return tbDataImportService.getEnterpriseAccount();
    }

    @RequestMapping(value = "voucherSelectAdvancedByVoucher", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<Object> voucherSelectAdvancedByVoucher(QueryVoucherParmDto parmDto)
    {
        try {
            return voucherService.voucherSelectAdvancedByVoucher(parmDto.getListQueryCondition(), VoucherSearchEnum.values()[parmDto.getMainRelation()], parmDto.isAllJe(), parmDto.getPagingInfo());
        } catch (Exception ex) {
            logger.error("voucherSelectAdvancedByVoucherError", ex);
        }
        OperationResultDto<Object> result = new OperationResultDto<>();
        result.setResult(false);
        return result;
    }
}
