package pwc.taxtech.atms.dao.dao;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.CommonConstants;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.dao.AccountMappingMapper;
import pwc.taxtech.atms.entitiy.AccountMapping;
import pwc.taxtech.atms.entitiy.AccountMappingExample;
import pwc.taxtech.atms.entitiy.EnterpriseAccount;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class AccountMappingDao {
    @Autowired
    AccountMappingMapper mapper;
    @Autowired
    EnterpriseAccountDao epAccountDao;
    @Autowired
    AccountMappingManualDao mappingManualDao;

    public void delByEpAccountCodeList(List<String> codeList) {
        if (CollectionUtils.isNotEmpty(codeList)) {
            AccountMappingExample example = new AccountMappingExample();
            AccountMappingExample.Criteria criteria = example.createCriteria();
            criteria.andEnterpriseAccountCodeIn(codeList);
            mapper.deleteByExample(example);
        }
    }

    public boolean isStdExists(String accountSetID, String enterpirseAccountCode, String industryID, String orgID) {
        AccountMappingExample example = new AccountMappingExample();
        AccountMappingExample.Criteria criteria = example.createCriteria();
        criteria.andEnterpriseAccountCodeEqualTo(enterpirseAccountCode);
        criteria.andEnterpriseAccountSetIDEqualTo(accountSetID);
        criteria.andOrganizationIDEqualTo(orgID);
        criteria.andIndustryIDEqualTo(industryID);
        criteria.andStandardAccountCodeNotEqualTo(CommonConstants.EmptyStdCode);
        return mapper.countByExample(example) > 0;
    }

    private void updateToTable(String accountCode, String accountSetId, String stdAccountCode,
                               String industryID, String enterpriseAccountID, String orgID) {
        AccountMappingExample example = new AccountMappingExample();
        AccountMappingExample.Criteria criteria = example.createCriteria();
        criteria.andEnterpriseAccountCodeEqualTo(accountCode);
        criteria.andEnterpriseAccountSetIDEqualTo(accountSetId);
        criteria.andOrganizationIDEqualTo(orgID);
        criteria.andIndustryIDEqualTo(industryID);
        mapper.deleteByExample(example);

        AccountMapping mapping = new AccountMapping();
        mapping.setID(CommonUtils.getUUID());
        mapping.setEnterpriseAccountCode(accountCode);
        mapping.setEnterpriseAccountSetID(accountSetId);
        mapping.setOrganizationID(orgID);
        mapping.setStandardAccountCode(stdAccountCode);
        mapping.setIndustryID(industryID);
        mapper.insertSelective(mapping);
    }

    public void mapStdAccountByCode(String accountSetId, String stdAccountCode, String accountCode,
                                    String industryID, String orgId, boolean bOverwrite, int acctLevel) {

        String enterpriseAccountID = epAccountDao.getEpAccountID(accountCode, accountSetId);

        if (StringUtils.isBlank(stdAccountCode))
            stdAccountCode = CommonConstants.NullStdCode;

        //region 新逻辑
        if (!bOverwrite) {
            if (!isStdExists(accountSetId, accountCode, industryID, orgId)) {
                updateToTable(accountCode, accountSetId, stdAccountCode, industryID, enterpriseAccountID, orgId);
            }
        } else {
            updateToTable(accountCode, accountSetId, stdAccountCode, industryID, enterpriseAccountID, orgId);
        }
        //endregion
    }

    public List<AccountMapping> getAccountMapping(String epAccountCode, String epAccountSetID, String orgID) {
        AccountMappingExample example = new AccountMappingExample();
        AccountMappingExample.Criteria criteria = example.createCriteria();
        criteria.andEnterpriseAccountCodeEqualTo(epAccountCode);
        criteria.andEnterpriseAccountSetIDEqualTo(epAccountSetID);
        criteria.andOrganizationIDEqualTo(orgID);
        return mapper.selectByExample(example);
    }

    public void delMapping(String epAccountCode, String epAccountSetID, String orgID, String industryID) {
        AccountMappingExample example = new AccountMappingExample();
        AccountMappingExample.Criteria criteria = example.createCriteria();
        criteria.andEnterpriseAccountCodeEqualTo(epAccountCode);
        criteria.andEnterpriseAccountSetIDEqualTo(epAccountSetID);
        criteria.andOrganizationIDEqualTo(orgID);
        criteria.andIndustryIDEqualTo(industryID);
        mapper.deleteByExample(example);
    }

    public void delMapping(String epAccountSetID, String orgID, String industryID) {
        AccountMappingExample example = new AccountMappingExample();
        AccountMappingExample.Criteria criteria = example.createCriteria();
        criteria.andEnterpriseAccountSetIDEqualTo(epAccountSetID);
        criteria.andOrganizationIDEqualTo(orgID);
        criteria.andIndustryIDEqualTo(industryID);
        mapper.deleteByExample(example);
    }

    public void clearMapping(List<EnterpriseAccount> eaList, String industryID,
                             String orgID, Boolean isToClearAllMapping) {
        if (CollectionUtils.isEmpty(eaList)) {
            return;
        }
        List<String> codeList = eaList.stream().map(EnterpriseAccount::getCode).collect(Collectors.toList());
        if (!isToClearAllMapping) {
            for (EnterpriseAccount ea : eaList) {
                delMapping(ea.getCode(), ea.getEnterpriseAccountSetID(), industryID, orgID);
                //同时删除手动对应表的对应关系
                mappingManualDao.delMapping(ea.getFullName(), ea.getEnterpriseAccountSetID(), industryID, orgID);

                clearChildMapping(ea.getCode(), ea.getEnterpriseAccountSetID(), industryID, orgID, codeList);
            }
        } else {
            delMapping(eaList.get(0).getEnterpriseAccountSetID(), industryID, orgID);
        }
    }

    private void clearChildMapping(String eaCode, String setId, String industryID, String orgID, List<String> codeList) {
        List<EnterpriseAccount> list = epAccountDao.getEpAccountBy(eaCode, setId);
        for (EnterpriseAccount row : list) {
            String code = row.getCode();
            //如果在本身的id里面包含了，说明第一次遍历的时候已经更新过数据库了，不需要再一次更新
            if (codeList.contains(code)) continue;
            //string updatesql = "UPDATE EnterpriseAccount SET StdCode=NULL WHERE ID=@ID";
            delMapping(code, setId, orgID, industryID);
            clearChildMapping(row.getCode(), row.getEnterpriseAccountSetID(), industryID, orgID, codeList);
        }
    }
}
