package pwc.taxtech.atms.dao.dao;

import com.google.common.collect.Lists;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.constant.AccountMappingConstant;
import pwc.taxtech.atms.constant.EnterpriseAccountConstant;
import pwc.taxtech.atms.dao.EnterpriseAccountMapper;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountDto;
import pwc.taxtech.atms.entitiy.EnterpriseAccount;
import pwc.taxtech.atms.entitiy.EnterpriseAccountExample;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@Service
public class EnterpriseAccountDao {
    @Autowired
    private EnterpriseAccountMapper enterpriseAccountMapper;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public List<EnterpriseAccount> getAll() {
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        return enterpriseAccountMapper.selectByExample(example);
    }

    public List<EnterpriseAccount> getByEpAccountSetId(String epAccountSetId) {
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        EnterpriseAccountExample.Criteria criteria = example.createCriteria();
        criteria.andEnterpriseAccountSetIDEqualTo(epAccountSetId);
        return enterpriseAccountMapper.selectByExample(example);
    }

    public List<EnterpriseAccount> getEpAccountBy(String parentCode, String epAccountSetId) {
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        EnterpriseAccountExample.Criteria criteria = example.createCriteria();
        criteria.andParentCodeEqualTo(parentCode);
        criteria.andEnterpriseAccountSetIDEqualTo(epAccountSetId);
        return enterpriseAccountMapper.selectByExample(example);
    }

    public Optional<EnterpriseAccount> getEpAccount(String epAccountCode, String epAccountSetId) {
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        EnterpriseAccountExample.Criteria criteria = example.createCriteria();
        criteria.andCodeEqualTo(epAccountCode);
        criteria.andEnterpriseAccountSetIDEqualTo(epAccountSetId);
        return enterpriseAccountMapper.selectByExample(example).stream().findFirst();
    }

    public String getEpAccountID(String epAccountCode, String epAccountSetId) {
        Optional<EnterpriseAccount> optional = getEpAccount(epAccountCode, epAccountSetId);
        return optional.isPresent() ? optional.get().getID() : StringUtils.EMPTY;
    }

    public List<EnterpriseAccountDto> getAccountMappingResult(String accountSetId, int ruleId, String industryId) {
        //amk.IndustryId倒序排列可以保证在结果集中，同一个COA，本行业的匹配结果在通用关键字匹配结果之前出现
        String sql = "SELECT a.Code, a.IsLeaf, sa.Code StdCode,a.FullName as AccountFullName, amk.FullName as MappingFullName,amk.IndustryId"
                + " FROM (select * from AccountMappingKeyword where IndustryId=?) amk "
                + " INNER JOIN (select * from StandardAccount where IndustryId=?) sa ON amk.StandardCode = sa.Code"
                + " INNER JOIN EnterpriseAccount a on a.FullName like amk.FullName"
                + " WHERE amk.RuleType = ? AND sa.IsLeaf = 1 AND a.EnterpriseAccountSetID=?"
                + " ORDER BY a.Code,amk.IndustryId DESC";

        List<Map<String, Object>> queryList = jdbcTemplate.queryForList(sql, industryId, industryId, ruleId, accountSetId);
        List<EnterpriseAccountDto> list = Lists.newArrayList();
        for (Map<String, Object> obj : queryList) {
            EnterpriseAccountDto account = new EnterpriseAccountDto();
            account.setCode(MapUtils.getString(obj, "Code"));
            account.setIsLeaf(StringUtils.equals(MapUtils.getString(obj, "isLeaf"), "1"));
            account.setStdCode(MapUtils.getString(obj, "StdCode"));
            account.setFullName(MapUtils.getString(obj, "AccountFullName"));
            account.setMappingFullName(MapUtils.getString(obj, "MappingFullName"));

            list.add(account);
        }

        return list;
    }

    public List<Map<String, Object>> selectAccounts(String industryId, String accountSetId, String organizationID, int iAcctProp,
                                                    int iIsMapped, int iLevel, Boolean bIsActive) {
        //In admin module, enterprise account has IsInVoucher to identify vouchers
        //However web admin is not requried voucher, so the field is not available in web admin,
        //so set acctVoucherHasNull=true at this moment
        //But in the web admin, balance is not required at this stage, so set balanceIsNull=true
        //string sql = "select count(1) from EnterpriseAccount where IsInVoucher is null";
        //bool balanceIsNull = true;
        //bool acctVoucherHasNull = true;
        List<String> whereList = Lists.newArrayList();

        String sql = "SELECT * FROM ( " +
                "SELECT CASE IsLeaf WHEN 0 THEN '+' ELSE '' END IsEx,Code, Name, FullName, " +
                "CASE Direction WHEN 1 THEN N'借方' WHEN -1 THEN N'贷方' END Direction, " +
                "CASE StdAccountCode WHEN '0000' THEN '-' ELSE StdAccountCode END StdCode," +
                "CASE StdAccountCode WHEN '0000' THEN '-' ELSE StdName END StdName, " +
                "CASE StdAccountCode WHEN '0000' THEN '-' ELSE StdAccountCode + '/' + StdFullName END StdCodeName, " +
                "CASE StdAccountCode WHEN '0000' THEN '-' ELSE CASE StdDirection WHEN 1 THEN N'借方' WHEN -1 THEN N'贷方' END END StdDirection, " +
                "CASE WHEN AcctProp<>StdAcctProp THEN 1 ELSE 0 END IsAcctPropDiff," +
                "CASE WHEN StdAccountCode IS NULL THEN 0 ELSE 1 END MappingStatus, " +
                "CASE WHEN StdAccountCode IS NULL THEN AcctProp ELSE StdAcctProp END AcctProp, ParentCode, AcctLevel, StdAccountCode AcctStdCode, IsLeaf " +
                "        FROM (SELECT ID,Code,Name,FullName,Direction,AcctProp,ParentCode,AcctLevel,IsLeaf FROM EnterpriseAccount WHERE EnterpriseAccountSetID=?)  ea " +
                "LEFT JOIN (SELECT EnterpriseAccountCode ,StandardAccountCode FROM AccountMapping WHERE OrganizationID=? AND IndustryID=?) am ON ea.Code=am.EnterpriseAccountCode " +
                "LEFT JOIN (SELECT ID StdID, Code StdAccountCode, Name StdName, FullName StdFullName,Direction StdDirection,AcctProp StdAcctProp FROM StandardAccount WHERE IndustryID=?) sa ON am.StandardAccountCode=sa.StdAccountCode ) T";

        List<Object> params = Lists.newArrayList(accountSetId, organizationID, industryId, industryId);
        if (iAcctProp != EnterpriseAccountConstant.InvalidId) {
            sql += "AcctProp = ?";
            params.add(iAcctProp);
        }
        if (iIsMapped == AccountMappingConstant.Status.UNMAPPED.getCode()) {
            sql += "StdCode IS NULL";
        } else if (iIsMapped == AccountMappingConstant.Status.MAPPED.getCode()) {
            sql += "StdCode IS NOT NULL";
        } else if (iIsMapped == AccountMappingConstant.Status.DIRECTION_DIFFERENT.getCode()) {
            sql += "StdCode IS NOT NULL AND Direction<>StdDirection";
        } else if (iIsMapped == AccountMappingConstant.Status.ACCOUNT_TYPE_DIFFERENT.getCode()) {
            sql += "StdCode IS NOT NULL AND IsAcctPropDiff=1";
        }
        if (iLevel != EnterpriseAccountConstant.InvalidId) {
            sql += "AcctLevel = ?";
            params.add(iLevel);
        }

        sql += " ORDER BY Code";
        return jdbcTemplate.queryForList(sql, params.toArray());
    }


}
