package pwc.taxtech.atms.security;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.security.authentication.AuthenticationCredentialsNotFoundException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.util.StringUtils;

public class JwtAuthenticationFilter extends AbstractAuthenticationProcessingFilter {

    public JwtAuthenticationFilter() {
        super("/**");
    }

    protected boolean requiresAuthentication(HttpServletRequest request, HttpServletResponse response) {
        String header = request.getHeader("Authorization");
        return (header != null && (header.startsWith("bearer ") || header.startsWith("Bearer ")));
    }

    @Override
    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) {

        String header = request.getHeader("Authorization");

        if (header == null || !(header.startsWith("bearer ") || header.startsWith("Bearer "))) {
            throw new AuthenticationCredentialsNotFoundException("No JWT token found in request headers");
        }

        String authToken = header.substring(7);
        if (!StringUtils.hasText(authToken)) {
            throw new AuthenticationCredentialsNotFoundException("Empty Bearer token");
        }

        JwtAuthenticationToken authRequest = new JwtAuthenticationToken(authToken);
        
        setDetails(request, authRequest);

        return getAuthenticationManager().authenticate(authRequest);
    }
    
    protected void setDetails(HttpServletRequest request,
            UsernamePasswordAuthenticationToken authRequest) {
        authRequest.setDetails(authenticationDetailsSource.buildDetails(request));
    }

    @Override
    protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain,
            Authentication authResult) throws IOException, ServletException {
        super.successfulAuthentication(request, response, chain, authResult);

        // As this authentication is in HTTP header, after success we need to continue
        // the request normally
        // and return the response as if the resource was not secured at all
        chain.doFilter(request, response);
    }
}