package pwc.taxtech.atms.service;

import java.util.List;
import java.util.Map;

import pwc.taxtech.atms.dto.DimensionValueOrgDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.dimension.DimensionValueUpdateDto;
import pwc.taxtech.atms.dto.navtree.NavTreeDto;
import pwc.taxtech.atms.dto.organization.OrganizationDto;
import pwc.taxtech.atms.dto.role.RoleDisplayDto;
import pwc.taxtech.atms.dto.role.RoleDto;
import pwc.taxtech.atms.dto.role.RoleDtoTree;
import pwc.taxtech.atms.dto.role.RolePermissionDto;
import pwc.taxtech.atms.dto.role.UpdateRoleInfo;
import pwc.taxtech.atms.dto.user.DimensionRole;
import pwc.taxtech.atms.dto.user.DimensionUser;
import pwc.taxtech.atms.dto.user.UserOrgDto;
import pwc.taxtech.atms.dto.user.UserOrgRoleDto;
import pwc.taxtech.atms.dto.user.UserRoleDimensionValueDto;
import pwc.taxtech.atms.dto.user.UserRoleDto;
import pwc.taxtech.atms.dto.user.UserRoleInfo;
import pwc.taxtech.atms.dto.user.UserRoleUpdateDto;
import pwc.taxtech.atms.dto.user.VMUser;
import pwc.taxtech.atms.entitiy.UserDimensionValue;
import pwc.taxtech.atms.entitiy.UserDimensionValueOrg;
import pwc.taxtech.atms.entitiy.UserOrganization;

public interface RoleService {

    List<NavTreeDto> getRoleTreeList();

    List<UserRoleInfo> getUserRoleByUser(VMUser vmUser);

    List<UserRoleInfo> getAllOwnUserRoleList();

    List<UserRoleInfo> getAllUserRoleList();

    List<UserRoleDto> getAllRoleListByUserID(String userID, String serviceTypeID);

    List<UserOrgDto> getUserByDimensionValue(List<OrganizationDto> orgList,
            List<UserDimensionValue> userDimensionValueList, List<UserDimensionValueOrg> userDimensionValueOrgList,
            List<UserOrganization> userOrganizationList, List<UserOrgRoleDto> originalRoleList,
            List<UserOrgRoleDto> userDimensionValueRoleList, List<UserOrgRoleDto> userOrganizationRoleList,
            String dimensionID, String dimensionValueID);

    List<RoleDtoTree> getRoleList();
    
    List<UserRoleDto> getUsersByRoleID(String roleID);
    
    /**
     * 查询某角色额外用户
     * @param roleID
     * @return
     */
    List<UserRoleDto> getExtraUsersByRoleID(String roleID);
    
    /**
     * 为角色添加用户
     * @param roleID
     * @param serviceTypeID
     * @param userIdList
     */
    void addUsersToRole(String roleID, String serviceTypeID, List<String> userIdList);
    
    void removeUserRole(String userID, List<String> roleIDList, String serviceTypeID);
    
    /**
     * Get role list by roleTypeId
     * @param roleTypeID
     * @return List<RoleDto>
     */
    List<RoleDto> getRoleListByRoleType(String roleTypeID);
    
    /**
     * 增加角色分类
     * @param roleCategoryName
     * @param roleCategoryID
     */
    void addRoleCategory(String roleCategoryName, String roleCategoryID);
    
    /**
     * 更新角色分类名称
     * @param roleCategoryName
     * @param roleCategoryID
     */
    void updateRoleCategory(String roleCategoryName, String roleCategoryID);

    /**
     * 删除角色分类
     * @param roleCategoryID
     * @return OperationResultDto
     */
    @SuppressWarnings("rawtypes")
    OperationResultDto deleteRoleCategory(String roleCategoryID);
    
    /**
     * Add a new role info
     * @param RoleDisplayDto roleDto
     * @return roleID
     */
    String addRole(RoleDisplayDto roleDto);
    
    /**
     * Update a role info
     * @param updateRole
     * @param roleID
     */
    void updateRole(UpdateRoleInfo updateRole, String roleID);
    
    /**
     * validate if the rolename exists
     * @param roleName
     * @param oldRoleName
     * @return true - not exist; false - exist
     */
    boolean validateRoleNameUnique(String roleName, String oldRoleName);
    
    /**
     * Delete a role info
     * @param roleDto
     */
    void deleteRole(RoleDto roleDto);
    
    /**
     * check mapping between user and role
     * @param roleID
     * @return
     */
    boolean checkUserRole(String roleID);
    
    /**
     * @param model
     * @return
     */
    List<UserRoleInfo> getUserRoleList(UserRoleDimensionValueDto userRoleDimensionValueDto);
    
    /**
     * 根据serviceType获取所有角色权限list
     * @param serviceTypeID
     * @return List<RolePermissionDto>
     */
    List<RolePermissionDto> getAllRolePermission(String serviceTypeID);
    
    /**
     * 更新用户角色列表
     * @param userRoleUpdateDto
     */
    void updateUserRole(UserRoleUpdateDto userRoleUpdateDto);
    
    /**
     * 用户详情页面，增加范围
     * @param dimensionValueUpdateDto
     */
    void updateDimensionValues(DimensionValueUpdateDto dimensionValueUpdateDto);
    
    /**
     * 用户详情页面, 增加机构
     * @param orgIdList
     * @param userID
     */
    void updateUserOrg(List<String> orgIdList, String userID);
    
    /**
     * 获取维度，角色，用户列表
     * @param dimensionId
     * @param dimensionValueId
     */
    List<DimensionRole> getDimensionRoleUserList(String dimensionId, String dimensionValueId);

    /**
     * 获取某一个维度的 用户 和 机构列表，角色列表
     * @param dimensionId
     * @param dimensionValueId
     */
    List<DimensionUser> getDimensionUserRoleList(String dimensionId, String dimensionValueId);

    Map<String, Integer> getActiveUserRoleList(List<OrganizationDto> orgDtoList,
            List<DimensionValueOrgDto> dimensionValueOrgList, String areaid);
    
    List<UserRoleInfo> getActiveUserRoleListByAreaID(String areaID);
}
