package pwc.taxtech.atms.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.constant.enums.EnumValidationType;
import pwc.taxtech.atms.constant.enums.ProjectImportDataType;
import pwc.taxtech.atms.constant.enums.ServiceType;
import pwc.taxtech.atms.dao.DataImportMapper;
import pwc.taxtech.atms.dao.EnterpriseAccountMapper;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.taxadmin.DataImportedStatusDto;
import pwc.taxtech.atms.entitiy.EnterpriseAccountExample;
import pwc.taxtech.atms.entitiy.Project;
import pwc.taxtech.atms.entitiy.ProjectExample;
import pwc.taxtech.atms.vat.dao.CompanyBalanceMapper;
import pwc.taxtech.atms.vat.entity.CompanyBalance;
import pwc.taxtech.atms.vat.entity.CompanyBalanceExample;

import java.util.Date;
import java.util.List;

@Service
public class DataImportServiceImpl {

    private static final String PROJECT_DB_NAME = "test_db";//TODO:fixed use init db
    private static final String COMPANY_BALANCE_TABLE = "CompanyBalance";
    private static final String BALANCE_TABLE = "Balance";
    private static final int FIRST_OR_DEFAULT = 0;

    @Autowired
    private DataImportMapper mapper;

    @Autowired
    private CompanyBalanceMapper companyBalanceMapper;

    @Autowired
    private ProjectMapper projectMapper;

    @Autowired
    private EnterpriseAccountMapper enterpriseAccountMapper;

    public OperationResultDto importTrialBalance(List<CompanyBalance> balanceList, Integer importType,
                                                 Integer serviceTypeId, String userID) {
        OperationResultDto result = new OperationResultDto();
        if (serviceTypeId == ServiceType.VAT.getCode().intValue()) {
            int period = balanceList.get(0).getPeriodId().intValue();
            DataImportedStatusDto dataImportedStatus = new DataImportedStatusDto();

            dataImportedStatus.setPeriod(period);
            dataImportedStatus.setDbName(PROJECT_DB_NAME);
            dataImportedStatus.setDataType(ProjectImportDataType.TrialBalance.getCode());
            dataImportedStatus.setStartTime(new Date());

            //覆盖导入之前删除数据
            if (importType == EnumTbImportType.CoverImport.getCode()) {
                clearTableData(period, COMPANY_BALANCE_TABLE);
            }

//            var sqlhelper = new SqlHelper(_projectDbConnstring);
//            var companyBalance = ServiceHelper.ToDataTable(balanceList);
//            sqlhelper.SqlBulkCopyInsert(_companyBalancetable, companyBalance, 10000);
//            sqlhelper.Close();
            //TODO: sql bulk copy insert tobe impl(neo)

            //SynAccountFromTaxAdmin(period, userID);
            Boolean isSuccess = refreshTrialBalance(period, serviceTypeId, userID);
            result.setResult(isSuccess);

        } else if (serviceTypeId == ServiceType.CIT.getCode()) {//TODO import cit (neo)

        }


        return result;
    }

    private Boolean refreshTrialBalance(int period, Integer serviceTypeId, String userID) {
        CompanyBalanceExample companyBalanceExample = new CompanyBalanceExample();
        companyBalanceExample.createCriteria().andPeriodIdEqualTo(period);
        List<CompanyBalance> list = companyBalanceMapper.selectByExample(companyBalanceExample);

        if (list != null && !list.isEmpty()) {
            //同步账套 TODO: sync admin db and project db(neo)
            synAccountFromTaxAdmin(period, userID);
            //先删除数据
            clearTableData(period, BALANCE_TABLE);
            mapper.deleteTbValidation(period, EnumValidationType.TrialBalance.getCode());
        }
        return true;
    }

    private void synAccountFromTaxAdmin(int period, String userID) {
        ProjectExample example = new ProjectExample();
        example.createCriteria().andDbNameEqualTo(PROJECT_DB_NAME);
        List<Project> plist = projectMapper.selectByExample(example);
        if (plist != null && !plist.isEmpty()) {
            Project pFirst = plist.get(FIRST_OR_DEFAULT);

            if (pFirst.getEnterpriseAccountSetID() != null && !pFirst.getEnterpriseAccountSetID().isEmpty()) {
                String easId = pFirst.getEnterpriseAccountSetID();

                EnterpriseAccountExample accountExample = new EnterpriseAccountExample();
                accountExample.createCriteria().andEnterpriseAccountSetIDEqualTo(easId);

            }
        }
    }

    private void clearTableData(int period, String companyBalanceTable) {
        if (period > 0) {
            mapper.deleteByNameAndPeriod(companyBalanceTable, period);
        } else {
            mapper.truncateByName(companyBalanceTable);
        }
    }
}
