package pwc.taxtech.atms.service.impl;

import org.apache.commons.lang3.StringUtils;
import pwc.taxtech.atms.constant.enums.RegexMatchType;
import pwc.taxtech.atms.dto.RegexMatchObject;

import java.util.AbstractMap;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public final class FormulaHelper {
    private Pattern regexPattern, equalPattern, keyValuePattern;
    private String strPattern = "(?=\\+|-|\\*|/|,|\\(|$|)(?!\")(((?<columnNumber>[A-Z]{1,3})(?<rowNumber>[0-9]{1,7}))|(?<keyCode>@[a-zA-Z0-9.]{1,}))(?=\\+|-|\\*|/| |,|\\)|>|=|<|$|^)(?!\")";
    private String strKeyValuePattern = "(?=\\+|-|\\*|/|,|\\(|$|)(?!\")(?<keyCode>@[a-zA-Z0-9.]{1,})(?=\\+|-|\\*|/| |,|\\)|>|=|<|$|^)(?!\")";
    private String strEqualPattern = "(?<![<>])(?<equal>=)";

    public FormulaHelper() {
        regexPattern = Pattern.compile(strPattern);
        equalPattern = Pattern.compile(strEqualPattern);
        keyValuePattern = Pattern.compile(strKeyValuePattern);
    }

    public String formatFormula(String formula) {
        Matcher equalMatchResult = equalPattern.matcher(formula);
        Stack<RegexMatchObject> equalMatchObjectStack = new Stack<>();
        while (equalMatchResult.find()) {
            if (StringUtils.isNotBlank(equalMatchResult.group("equal"))) {
                RegexMatchObject object = new RegexMatchObject();
                object.setIndex(equalMatchResult.start());
                object.setLength(equalMatchResult.end() - equalMatchResult.start() + 1);
                object.setMatchType(RegexMatchType.Equal);
                object.setParameters(new AbstractMap.SimpleEntry<>(equalMatchResult.group("equal"), -1));
                equalMatchObjectStack.push(object);
            }
        }

        while (equalMatchObjectStack.size() > 0) {
            RegexMatchObject obj = equalMatchObjectStack.pop();
            formula = formula.substring(0, obj.getIndex()) + obj.getExpression() + formula.substring(obj.getIndex() + obj.getLength());
        }

        Matcher regexResult = regexPattern.matcher(formula);
        Stack<RegexMatchObject> regexMatchObjectStack = new Stack<>();

        while (regexResult.find()) {
            if (StringUtils.isNotBlank(regexResult.group("columnNumber")) && StringUtils.isNotBlank(regexResult.group("rowNumber"))) {
                RegexMatchObject object = new RegexMatchObject();
                object.setIndex(regexResult.start());
                object.setLength(regexResult.end() - regexResult.start() + 1);
                object.setMatchType(RegexMatchType.CellNumber);
                object.setParameters(new AbstractMap.SimpleEntry<>(regexResult.group("columnNumber"), Integer.parseInt(regexResult.group("rowNumber"))));
                regexMatchObjectStack.push(object);
            }

            if (StringUtils.isNotBlank(regexResult.group("keyCode"))) {
                RegexMatchObject object = new RegexMatchObject();
                object.setIndex(regexResult.start());
                object.setLength(regexResult.end() - regexResult.start() + 1);
                object.setMatchType(RegexMatchType.CellNumber);
                object.setParameters(new AbstractMap.SimpleEntry<>(regexResult.group("keyCode"), -1));
                regexMatchObjectStack.push(object);
            }
        }

        while (!regexMatchObjectStack.empty()) {
            RegexMatchObject obj = regexMatchObjectStack.pop();
            formula = formula.substring(0, obj.getIndex())
                    + obj.getExpression()
                    + formula.substring(obj.getIndex(), obj.getIndex() + obj.getLength());
        }
        return formula.trim();
    }
}
