package pwc.taxtech.atms.service.impl;

import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.dao.dao.FormulaConfigDao;
import pwc.taxtech.atms.dao.dao.FormulaParamConfigDao;
import pwc.taxtech.atms.dao.dao.FormulaParamMappingDao;
import pwc.taxtech.atms.dao.dao.FormulaParamOptionDao;
import pwc.taxtech.atms.dto.formula.FormulaParamConfigDto;
import pwc.taxtech.atms.dto.formula.FormulaParamMappingDto;
import pwc.taxtech.atms.dto.formula.FormulaParamOptionDto;
import pwc.taxtech.atms.entitiy.FormulaConfig;
import pwc.taxtech.atms.entitiy.FormulaParamConfig;
import pwc.taxtech.atms.entitiy.FormulaParamMapping;
import pwc.taxtech.atms.entitiy.FormulaParamOption;
import pwc.taxtech.atms.service.FormulaService;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class FormulaServiceImpl extends BaseService implements FormulaService {

    @Autowired
    FormulaConfigDao formulaConfigDao;
    @Autowired
    FormulaParamOptionDao formulaParamOptionDao;
    @Autowired
    FormulaParamConfigDao paramConfigDao;
    @Autowired
    FormulaParamMappingDao paramMappingDao;

    @Override
    public List<FormulaConfig> getAllFormulaList() {
        return formulaConfigDao.getAll();
    }

    @Override
    public List<FormulaParamConfigDto> getAllFormulaParamList() {
        List<FormulaParamOption> paramOpts = formulaParamOptionDao.getAll();
        List<FormulaParamOptionDto> paramOptions = paramOpts.stream().map(x -> {
            FormulaParamOptionDto dto = new FormulaParamOptionDto();
            return CommonUtils.copyProperties(x, dto);
        }).collect(Collectors.toList());
        List<FormulaParamConfig> paramCfg = paramConfigDao.getAll();
        List<FormulaParamConfigDto> list = paramCfg.stream().map(x -> {
            FormulaParamConfigDto dto = new FormulaParamConfigDto();
            return CommonUtils.copyProperties(x, dto);
        }).collect(Collectors.toList());
        Set<String> optionFormulaIDs = paramOptions.stream().map(FormulaParamOptionDto::getFormulaParamID).collect(Collectors.toSet());

        list.forEach(x -> {
            x.setOptions(Lists.newArrayList());
            if (optionFormulaIDs.contains(x.getID())) {
                x.getOptions().addAll(paramOptions.stream().filter(y -> StringUtils.equals(y.getFormulaParamID(),
                        x.getID())).collect(Collectors.toList()));
            }
        });
        return list;
    }

    @Override
    public List<FormulaParamMappingDto> getAllFormulaParamMappingList() {
        List<FormulaParamMapping> paramMap = paramMappingDao.getAll();
        return paramMap.stream().map(x -> {
            FormulaParamMappingDto dto = new FormulaParamMappingDto();
            return CommonUtils.copyProperties(x, dto);
        }).collect(Collectors.toList());
    }
}
