package pwc.taxtech.atms.service.impl;

import static java.util.Comparator.comparing;
import static java.util.Comparator.naturalOrder;
import static java.util.Comparator.nullsFirst;
import static java.util.stream.Collectors.toList;
import static pwc.taxtech.atms.common.CommonConstants.Comma;
import static pwc.taxtech.atms.common.CommonConstants.CommonFail;
import static pwc.taxtech.atms.common.CommonConstants.DashSignSeparator;
import static pwc.taxtech.atms.common.CommonUtils.copyProperties;
import static pwc.taxtech.atms.common.OperateLogType.OperationLogOrganization;
import static pwc.taxtech.atms.common.OperationModule.Organization;
import static pwc.taxtech.atms.common.message.EnterpriseAccountSetOrgMessage.EffectiveDateAreaProblem;
import static pwc.taxtech.atms.common.message.EnterpriseAccountSetOrgMessage.EnterpriseAccountSetOrgDateTimeOverlap;
import static pwc.taxtech.atms.common.message.LogMessage.AddOrganization;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.format.DateTimeFormat;
import org.nutz.lang.Lang;
import org.nutz.lang.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.Assert;

import pwc.taxtech.atms.exception.ApplicationException;
import pwc.taxtech.atms.common.CommonConstants;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.OperateLogType;
import pwc.taxtech.atms.common.OperationAction;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.ServiceTypeEnum;
import pwc.taxtech.atms.common.VatTaxPayerTypeEnum;
import pwc.taxtech.atms.common.message.EnterpriseAccountSetOrgMessage;
import pwc.taxtech.atms.common.message.LogMessage;
import pwc.taxtech.atms.common.message.OrganizationMessage;
import pwc.taxtech.atms.constant.AreaConstant;
import pwc.taxtech.atms.constant.DimensionConstant;
import pwc.taxtech.atms.constant.LevelConstant;
import pwc.taxtech.atms.constant.OrganizationConstant;
import pwc.taxtech.atms.dao.AreaMapper;
import pwc.taxtech.atms.dao.AreaRegionMapper;
import pwc.taxtech.atms.dao.BusinessUnitMapper;
import pwc.taxtech.atms.dao.DimensionMapper;
import pwc.taxtech.atms.dao.DimensionValueMapper;
import pwc.taxtech.atms.dao.DimensionValueOrgMapper;
import pwc.taxtech.atms.dao.EnterpriseAccountSetMapper;
import pwc.taxtech.atms.dao.EnterpriseAccountSetOrgMapper;
import pwc.taxtech.atms.dao.IndustryMapper;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.OrganizationServiceTemplateGroupMapper;
import pwc.taxtech.atms.dao.OrganizationStructureMapper;
import pwc.taxtech.atms.dao.RegionMapper;
import pwc.taxtech.atms.dao.ServiceTypeMapper;
import pwc.taxtech.atms.dao.TaxPayerReportRuleMapper;
import pwc.taxtech.atms.dao.TemplateGroupMapper;
import pwc.taxtech.atms.dao.UserDimensionValueMapper;
import pwc.taxtech.atms.dao.UserDimensionValueOrgMapper;
import pwc.taxtech.atms.dao.UserDimensionValueRoleMapper;
import pwc.taxtech.atms.dao.UserOrganizationMapper;
import pwc.taxtech.atms.dao.UserOrganizationRoleMapper;
import pwc.taxtech.atms.dao.UserRoleMapper;
import pwc.taxtech.atms.dto.AreaOrganizationStatistics;
import pwc.taxtech.atms.dto.AreaStatistics;
import pwc.taxtech.atms.dto.DimensionValueOrgDto;
import pwc.taxtech.atms.dto.IndustryDto;
import pwc.taxtech.atms.dto.OperationLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.OrganizationServiceTemplateGroupDto;
import pwc.taxtech.atms.dto.OrganizationStatistics;
import pwc.taxtech.atms.dto.OrganizationValidateDto;
import pwc.taxtech.atms.dto.StatisticAttributeDisplayDto;
import pwc.taxtech.atms.dto.UpdateLogParams;
import pwc.taxtech.atms.dto.arearegion.AreaRegionDto;
import pwc.taxtech.atms.dto.dimension.AttributeTypeEnum;
import pwc.taxtech.atms.dto.dimension.DimensinTypeOrgDto;
import pwc.taxtech.atms.dto.dimension.DimensionOrgDto;
import pwc.taxtech.atms.dto.dimension.DimensionOrgDtoDashboard;
import pwc.taxtech.atms.dto.dimension.DimensionTypeEnum;
import pwc.taxtech.atms.dto.dimension.DimensionValueDisplayDto;
import pwc.taxtech.atms.dto.dimension.DimensionValueJoinDimensionDto;
import pwc.taxtech.atms.dto.dimension.OrgDashboardParams;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountSetOrgDto;
import pwc.taxtech.atms.dto.navtree.DevTreeDto;
import pwc.taxtech.atms.dto.navtree.IvhTreeDto;
import pwc.taxtech.atms.dto.navtree.NavTreeDto;
import pwc.taxtech.atms.dto.organization.OrgBasicDto;
import pwc.taxtech.atms.dto.organization.OrgCountDto;
import pwc.taxtech.atms.dto.organization.OrgCustomDto;
import pwc.taxtech.atms.dto.organization.OrgDisplayDto;
import pwc.taxtech.atms.dto.organization.OrgDto;
import pwc.taxtech.atms.dto.organization.OrgGeneralInfoDto;
import pwc.taxtech.atms.dto.organization.OrgGeneralInfoMiddleDto;
import pwc.taxtech.atms.dto.organization.OrganizationDto;
import pwc.taxtech.atms.dto.user.NameDto;
import pwc.taxtech.atms.dto.user.UserOrgDto;
import pwc.taxtech.atms.dto.user.UserOrgRoleDto;
import pwc.taxtech.atms.entitiy.Area;
import pwc.taxtech.atms.entitiy.AreaExample;
import pwc.taxtech.atms.entitiy.BusinessUnit;
import pwc.taxtech.atms.entitiy.BusinessUnitExample;
import pwc.taxtech.atms.entitiy.Dimension;
import pwc.taxtech.atms.entitiy.DimensionExample;
import pwc.taxtech.atms.entitiy.DimensionValue;
import pwc.taxtech.atms.entitiy.DimensionValueExample;
import pwc.taxtech.atms.entitiy.DimensionValueOrg;
import pwc.taxtech.atms.entitiy.DimensionValueOrgExample;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSet;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSetOrg;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSetOrgExample;
import pwc.taxtech.atms.entitiy.Industry;
import pwc.taxtech.atms.entitiy.IndustryExample;
import pwc.taxtech.atms.entitiy.Organization;
import pwc.taxtech.atms.entitiy.OrganizationExample;
import pwc.taxtech.atms.entitiy.OrganizationExample.Criteria;
import pwc.taxtech.atms.entitiy.OrganizationServiceTemplateGroup;
import pwc.taxtech.atms.entitiy.OrganizationServiceTemplateGroupExample;
import pwc.taxtech.atms.entitiy.OrganizationStructure;
import pwc.taxtech.atms.entitiy.OrganizationStructureExample;
import pwc.taxtech.atms.entitiy.Region;
import pwc.taxtech.atms.entitiy.RegionExample;
import pwc.taxtech.atms.entitiy.ServiceType;
import pwc.taxtech.atms.entitiy.TaxPayerReportRule;
import pwc.taxtech.atms.entitiy.TaxPayerReportRuleExample;
import pwc.taxtech.atms.entitiy.TemplateGroup;
import pwc.taxtech.atms.entitiy.UserDimensionValue;
import pwc.taxtech.atms.entitiy.UserDimensionValueOrg;
import pwc.taxtech.atms.entitiy.UserDimensionValueOrgExample;
import pwc.taxtech.atms.entitiy.UserOrganization;
import pwc.taxtech.atms.service.CommonService;
import pwc.taxtech.atms.service.DimensionService;
import pwc.taxtech.atms.service.OperationLogService;
import pwc.taxtech.atms.service.OrganizationService;
import pwc.taxtech.atms.service.RoleService;
import pwc.taxtech.atms.service.StatisticAttributeService;

/**
 * @author rzhou038
 *
 */
@Service
public class OrganizationServiceImpl implements OrganizationService {

    @Autowired
    private OrganizationMapper organizationMapper;

    @Autowired
    private IndustryMapper industryMapper;

    @Autowired
    private StatisticAttributeService statisticAttributeService;

    @Autowired
    private DimensionService dimensionService;

    @Autowired
    private CommonService commonService;

    @Autowired
    private RoleService roleService;

    @Autowired
    private BusinessUnitMapper businessUnitMapper;

    @Autowired
    private DimensionMapper dimensionMapper;

    @Autowired
    private UserDimensionValueMapper userDimensionValueMapper;

    @Autowired
    private UserDimensionValueOrgMapper userDimensionValueOrgMapper;

    @Autowired
    private UserOrganizationMapper userOrganizationMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private UserDimensionValueRoleMapper userDimensionValueRoleMapper;

    @Autowired
    private RegionMapper regionMapper;

    @Autowired
    private AreaMapper areaMapper;

    @Autowired
    private OrganizationStructureMapper organizationStructureMapper;

    @Autowired
    private UserOrganizationRoleMapper userOrganizationRoleMapper;

    @Autowired
    private DimensionValueMapper dimensionValueMapper;

    @Autowired
    private OperationLogService operationLogService;

    @Autowired
    private EnterpriseAccountSetOrgMapper enterpriseAccountSetOrgMapper;

    @Autowired
    private ServiceTypeMapper serviceTypeMapper;

    @Autowired
    private OrganizationServiceTemplateGroupMapper organizationServiceTemplateGroupMapper;
    @Autowired
    private AreaRegionMapper areaRegionMapper;

    @Autowired
    private TaxPayerReportRuleMapper taxPayerReportRuleMapper;

    @Autowired
    private TemplateGroupMapper templateGroupMapper;

    @Autowired
    private DimensionValueOrgMapper dimensionValueOrgMapper;

    @Autowired
    private EnterpriseAccountSetMapper enterpriseAccountSetMapper;
    @Autowired
    private DistributedIDService distributedIDService;

    private static final Logger logger = LoggerFactory.getLogger(ProjectServiceImpl.class);

    @Override
    public boolean isOrganizationStructureExists(String organizationStructureId) {
        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andStructureIDEqualTo(organizationStructureId);
        return organizationMapper.countByExample(example) > 0;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * pwc.taxtech.atms.service.OrganizationService#getLinkedOrganization(java.util.
     * List)
     */
    @Override
    public List<Organization> getLinkedOrganization(List<Area> areaList) {

        List<String> areaIdList = areaList.stream().map(area -> area.getID()).collect(Collectors.toList());

        OrganizationExample organizationExample = new OrganizationExample();
        pwc.taxtech.atms.entitiy.OrganizationExample.Criteria criteria = organizationExample.createCriteria();
        criteria.andIsActiveEqualTo(CommonConstants.ACTIVE_STATUS);
        criteria.andAreaIDIn(areaIdList);
        List<Organization> organizationList = organizationMapper.selectByExample(organizationExample);
        return organizationList;
    }

    @Override
    public List<NavTreeDto> getOrgListToJson(Integer useType) {
        List<OrganizationDto> orgList = getOrgList(useType);
        return orgList.stream().map(this::rotateOrganizationDtoToNavTreeDto).collect(Collectors.toList());
    }

    private NavTreeDto rotateOrganizationDtoToNavTreeDto(OrganizationDto organizationDto) {
        NavTreeDto navTreeDto = new NavTreeDto();
        navTreeDto.setLabel(organizationDto.getName());
        navTreeDto.setData(organizationDto);
        navTreeDto.setChildren(organizationDto.getSubOrgs().stream().map(this::rotateOrganizationDtoToNavTreeDto)
                .collect(Collectors.toList()));
        return navTreeDto;
    }

    @Override
    public List<OrganizationDto> getOrgList(Integer useType) {
        List<OrganizationDto> orgDtoList = new ArrayList<>();
        List<Organization> orgList = findAllOrganizations();
        if (!orgList.isEmpty()) {
            // find rootOrg
            List<Organization> rootOrgs = null;
            if (useType == 1) {
                rootOrgs = orgList.stream().filter(x -> x.getParentID() == null).collect(Collectors.toList());
            } else {
                rootOrgs = orgList.stream()
                        .filter(x -> x.getParentID() == null && CommonConstants.ACTIVE_STATUS.equals(x.getIsActive()))
                        .collect(Collectors.toList());
            }

            List<ServiceType> serviceList = new ArrayList<>();
            for (Organization rootOrg : rootOrgs) {
                OrganizationDto rootDto = rotateOrganizationToOrganizationDto(rootOrg);
                OrganizationDto subOrgs = genarateSubOrgs(rootDto, orgList, serviceList, useType);
                orgDtoList.add(subOrgs);
            }
        }
        orgDtoList.sort((OrganizationDto o1, OrganizationDto o2) -> o1.getName().compareTo(o2.getName()));
        return orgDtoList;
    }

    private OrganizationDto genarateSubOrgs(OrganizationDto parentOrg, List<Organization> orgs,
            List<ServiceType> serviceList, Integer useType) {
        if (parentOrg.getParentID() != null) {
            Organization organization = orgs.stream().filter(x -> x.getID().equals(parentOrg.getParentID()))
                    .collect(Collectors.toList()).get(0);
            parentOrg.setParentName(organization == null ? null : organization.getName());
        }
        List<Organization> subOrgs = null;
        if (useType == 1) {
            subOrgs = orgs.stream().filter(x -> parentOrg.getID().equals(x.getParentID())).collect(Collectors.toList());
        } else {
            subOrgs = orgs.stream().filter(x -> parentOrg.getID().equals(x.getParentID())
                    && CommonConstants.ACTIVE_STATUS.equals(x.getIsActive())).collect(Collectors.toList());
        }
        parentOrg.setSubOrgs(new ArrayList<>());

        // create and add subtrees to the current node
        for (Organization subOrg : subOrgs) {
            OrganizationDto subDto = rotateOrganizationToOrganizationDto(subOrg);
            subDto.setLevel(parentOrg.getLevel() == null ? 1 : parentOrg.getLevel() + 1);
            parentOrg.getSubOrgs().add(genarateSubOrgs(subDto, orgs, serviceList, useType));
        }
        return parentOrg;
    }

    private OrganizationDto rotateOrganizationToOrganizationDto(Organization organization) {
        OrganizationDto organizationDto = new OrganizationDto();
        CommonUtils.copyProperties(organization, organizationDto);
        return organizationDto;
    }

    private IndustryDto rotateIndustryToIndustryDto(Industry industry) {
        IndustryDto industryDto = new IndustryDto();
        return CommonUtils.copyProperties(industry, industryDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see pwc.taxtech.atms.service.OrganizationService#GetOrganizationFilterList()
     */
    @Override
    public List<OrganizationDto> getOrganizationFilterList() {

        OrganizationExample example = new OrganizationExample();
        example.setOrderByClause("Name ASC");
        List<Organization> organizationList = organizationMapper.selectByExampleWithAssociation(example);

        if (organizationList == null || organizationList.isEmpty()) {
            return new ArrayList<>();
        }
        List<OrganizationDto> organizationDtoList = new ArrayList<>();
        for (Organization organization : organizationList) {
            OrganizationDto organizationDto = new OrganizationDto();
            CommonUtils.copyProperties(organization, organizationDto);
            organizationDto.setAreaName(organization.getArea() != null ? organization.getArea().getName() : "");
            organizationDto.setBusinessUnitName(
                    organization.getBusinessUnitID() != null && organization.getBusinessUnit()!= null ? organization.getBusinessUnit().getName() : "");
            organizationDtoList.add(organizationDto);
        }
        return organizationDtoList;
    }

    @Override
    public List<Organization> findAllOrganizations() {
        OrganizationExample organizationExample = new OrganizationExample();
        return organizationMapper.selectByExample(organizationExample);
    }

    @Override
    public List<IndustryDto> getProjectIndustryList() {
        logger.debug("获取Project Industrys Start");
        IndustryExample example = new IndustryExample();
        example.setOrderByClause("ID asc");
        pwc.taxtech.atms.entitiy.IndustryExample.Criteria criteria = example.createCriteria();
        criteria.andIsActiveEqualTo(true);

        List<Industry> industryList = industryMapper.selectByExample(example);
        List<String> distinctName = industryList.stream().map(ind -> ind.getName()).distinct()
                .collect(Collectors.toList());
        List<IndustryDto> retList = new ArrayList<IndustryDto>();
        // 遍历disName,将返回的industry转换为IndustryDto,加入到retList
        distinctName.stream().forEach(
                disName -> retList.add(rotateIndustryToIndustryDto(findFirstListByName(disName, industryList))));
        return retList;
    }

    // 返回industry中名字为industryName的第一个Industry
    private Industry findFirstListByName(String industryName, List<Industry> industryList) {
        return (industryName == null || industryList.isEmpty()) ? null
                : industryList.stream().filter(industry -> (Strings.equals(industry.getName(), industryName)))
                        .findFirst().orElse(null);
    }

    @Override
    public List<OrgBasicDto> getOrgListLevel() {
        return organizationMapper.selectIndBusiunitAreaOrgstrctReg(false);
    }

    /** Organization表中根据行业，事业部，区域关联起来 */
    private List<OrganizationDto> getOrganizationJoinResult(Boolean orgIsActive) {
        return organizationMapper.selectIndBusiunitAreaOrgstrct(orgIsActive);
    }

    public List<OrganizationDto> getOrganizationJoinResult() {
        return getOrganizationJoinResult(true);
    }

    @Override
    public Boolean taxPayerNumberUniqueValidate(OrganizationValidateDto validateDto) {
        String innerId = Strings.sBlank(validateDto.getId()).trim();
        String innerNumber = Strings.sBlank(validateDto.getValue()).trim();
        if (Strings.isBlank(innerNumber))
            return true;
        OrganizationExample example = new OrganizationExample();
        pwc.taxtech.atms.entitiy.OrganizationExample.Criteria criteria = example.createCriteria();
        criteria.andTaxPayerNumberEqualTo(innerNumber);
        List<Organization> result = organizationMapper.selectByExample(example);
        return !result.stream().anyMatch(res -> (Strings.isBlank(innerId)) || !innerId.equals(res.getID()));
    }

    /**
     * 自定义维度显示属性,在机构管理顶部点击某个维度的时候，显示下面维度的属性,Orgchart结构树（Jack）
     */
    @Override
    public List<DimensionOrgDtoDashboard> getOrgDashboard(String parentDimensionID, String parentOrgID,
            OrgDashboardParams param) {
        List<String> expandedOrgList = (param == null ? null : param.getExpandedOrgList());
        String filterFlag = (param == null ? null : param.getFilterFlag());

        // C#中在此处新建了变量List<DimensionOrgDtoDashboard> displayList，但无任何引用，故删除。
        if (expandedOrgList == null) {
            expandedOrgList = new ArrayList<>();
        }
        List<StatisticAttributeDisplayDto> rawAttrList = new ArrayList<>();
        List<DimensionValueDisplayDto> dimensionValueList = new ArrayList<>();
        List<StatisticAttributeDisplayDto> attrDtoList = statisticAttributeService
                .getStatisticAttributeListByDimensionID(parentDimensionID, rawAttrList);
        if (!DimensionConstant.OrgSubChildrenID.equals(parentDimensionID)) {
            List<OrganizationDto> query;
            query = getOrgDimensionMappingForDimension(parentDimensionID).stream()
                    .filter(org -> Objects.equals(org.getDimensionID(), parentDimensionID))
                    .collect(Collectors.toList());
            logger.debug("Size of query in getOrgDashboard is {}", query.size());
            query.stream().forEach(q -> addToDimensionValueList(q, dimensionValueList));
            logger.debug("Size of dimensionValueList in getOrgDashboard is {}", dimensionValueList.size());
            return getStatisticCount(dimensionValueList, parentDimensionID, query, attrDtoList);
        } else {
            // 机构基础数据包括区域，层级，自定义等等
            List<OrganizationDto> orgMappingResult = getOrgDimensionMappingForDimension(parentDimensionID).stream()
                    .filter(orgDto -> parentDimensionID != null && parentDimensionID.equals(orgDto.getDimensionID()))
                    .collect(Collectors.toList());

            List<StatisticAttributeDisplayDto> orgAttrList = rawAttrList.stream()
                    .sorted(Comparator.comparing(statisticAttributeDisplayDto -> Optional
                            .ofNullable(statisticAttributeDisplayDto.getOrderIndex()).orElse(0)))
                    .limit(2).collect(Collectors.toList());

            // 也就是UI显示的节点List
            List<OrganizationDto> orgDtoList = new ArrayList<>();
            String currentOrgID = "";
            if (Strings.isNotBlank(parentOrgID) && parentOrgID.contains("@@Current@@")) {
                currentOrgID = parentOrgID.replace("@@Current@@", "").trim();
            }
            if (Strings.isBlank(parentOrgID) || Strings.equals(parentOrgID, "0")) {
                // 如果父机构ID为空，那么获取第一层全部的机构
                orgDtoList = orgMappingResult.stream().filter(orgMapRes -> Strings.isBlank(orgMapRes.getParentID()))
                        .collect(Collectors.toList());
            } else if ("1".equals(parentOrgID)) {
                orgDtoList = orgMappingResult.stream().filter(orgMapRes -> Strings.isBlank(orgMapRes.getParentID())
                        && Strings.isNotBlank(orgMapRes.getTaxPayerNumber())).collect(Collectors.toList());
            } else if ("2".equals(parentOrgID)) {
                orgDtoList = orgMappingResult.stream().filter(orgMapRes -> Strings.isBlank(orgMapRes.getParentID())
                        && Strings.isBlank(orgMapRes.getTaxPayerNumber())).collect(Collectors.toList());
            } else {
                // 只获取指定节点的子节点信息

                // 创建临时变量
                String currentOrgIDTemp = currentOrgID;
                if ("0".equals(filterFlag) || Strings.isBlank(filterFlag)) {
                    if (Strings.isNotBlank(currentOrgIDTemp)) {
                        orgDtoList = orgMappingResult.stream()
                                .filter(x -> currentOrgIDTemp != null && currentOrgIDTemp.equals(x.getID()))
                                .collect(Collectors.toList());
                    } else {
                        orgDtoList = orgMappingResult.stream()
                                .filter(x -> parentOrgID != null && parentOrgID.equals(x.getParentID()))
                                .collect(Collectors.toList());
                    }
                } else if ("1".equals(filterFlag)) {
                    if (Strings.isNotBlank(currentOrgIDTemp)) {
                        orgDtoList = orgMappingResult.stream().filter(x -> currentOrgIDTemp != null
                                && currentOrgIDTemp.equals(x.getID()) && Strings.isNotBlank(x.getTaxPayerNumber()))
                                .collect(Collectors.toList());
                    } else {
                        orgDtoList = orgMappingResult.stream().filter(x -> parentOrgID != null
                                && parentOrgID.equals(x.getParentID()) && Strings.isNotBlank(x.getTaxPayerNumber()))
                                .collect(Collectors.toList());
                    }
                } else if ("2".equals(filterFlag)) {
                    if (Strings.isNotBlank(currentOrgIDTemp)) {
                        orgDtoList = orgMappingResult.stream().filter(x -> currentOrgIDTemp != null
                                && currentOrgIDTemp.equals(x.getID()) && Strings.isNotBlank(x.getTaxPayerNumber()))
                                .collect(Collectors.toList());
                    } else {
                        orgDtoList = orgMappingResult.stream().filter(x -> parentOrgID != null
                                && parentOrgID.equals(x.getParentID()) && Strings.isBlank(x.getTaxPayerNumber()))
                                .collect(Collectors.toList());
                    }
                }
            }
            if ("1".equals(parentOrgID) || "2".equals(parentOrgID)) {
                filterFlag = parentOrgID;
            }

            return getOrgChartDisplayData(orgDtoList, orgAttrList, parentDimensionID, orgMappingResult, attrDtoList,
                    expandedOrgList, filterFlag);
        }
    }

    private List<DimensionOrgDtoDashboard> getOrgChartDisplayData(List<OrganizationDto> orgList,
            List<StatisticAttributeDisplayDto> orgAttrList, String parentDimensionID,
            List<OrganizationDto> orgMappingResult, List<StatisticAttributeDisplayDto> attrDtoList,
            List<String> expandedOrgList, String filterFlag) {
        List<OrganizationDto> allOrgList = getAllOrgList();

        // 自定义维度与之关联的机构列表
        List<DimensionValueDisplayDto> dimensionValueList = getDimensionValueDisplayDtoList(orgList, orgAttrList,
                allOrgList);

        // 各种统计
        List<DimensionOrgDtoDashboard> statisticList = getStatisticCount(dimensionValueList, parentDimensionID,
                orgMappingResult, attrDtoList, allOrgList);

        // 获取orgQuery下的子机构
        // 判断是否存在子节点，如果存在的话，需要返回标记
        for (OrganizationDto item : orgList) {
            List<OrganizationDto> childrenOrgList = new ArrayList<>();
            if (!"1".equals(filterFlag) && !"2".equals(filterFlag)) {
                childrenOrgList = orgMappingResult.stream().filter(x -> Objects.equals(x.getParentID(), item.getID()))
                        .collect(Collectors.toList());
            } else if ("1".equals(filterFlag)) {
                childrenOrgList = orgMappingResult.stream().filter(
                        x -> Objects.equals(x.getParentID(), item.getID()) && Strings.isNotBlank(x.getTaxPayerNumber()))
                        .collect(Collectors.toList());
            } else if ("2".equals(filterFlag)) {
                childrenOrgList = orgMappingResult.stream().filter(
                        x -> Objects.equals(x.getParentID(), item.getID()) && Strings.isBlank(x.getTaxPayerNumber()))
                        .collect(Collectors.toList());
            }

            // 找出与之关联的统计卡片对象，并赋值给children属性
            DimensionOrgDtoDashboard childOrg = statisticList.stream()
                    .filter(x -> item.getID() != null && item.getID().equals(x.getDimensionValueID())).findFirst()
                    .orElse(null);
            Assert.notNull(childOrg, "Null childOrg");
            childOrg.setHasChildren(!childrenOrgList.isEmpty());

            if (expandedOrgList.contains(item.getID())) {
                // new added Jacob on 07-03
                List<DimensionValueDisplayDto> childrenDimensionValueList = getDimensionValueDisplayDtoList(
                        childrenOrgList, orgAttrList, allOrgList);
                List<DimensionOrgDtoDashboard> childrenStatisticList = getStatisticCount(childrenDimensionValueList,
                        parentDimensionID, orgMappingResult, attrDtoList, allOrgList);
                childOrg.setChildren(childrenStatisticList);

                getSubStatisticRecursion(childrenOrgList, orgAttrList, allOrgList, parentDimensionID, orgMappingResult,
                        attrDtoList, childrenStatisticList, expandedOrgList, filterFlag);
            }
        }
        return statisticList;
    }

    private void getSubStatisticRecursion(List<OrganizationDto> subOrgList,
            List<StatisticAttributeDisplayDto> orgAttrList, List<OrganizationDto> allOrgList, String parentDimensionID,
            List<OrganizationDto> orgMappingResult, List<StatisticAttributeDisplayDto> attrDtoList,
            List<DimensionOrgDtoDashboard> parentStatisticList, List<String> expandedOrgList, String filterFlag) {
        if (subOrgList.isEmpty())
            return;

        // 获取orgQuery下的子机构
        for (OrganizationDto item : subOrgList) {
            List<OrganizationDto> childrenOrgList = new ArrayList<>();
            if (!"1".equals(filterFlag) && !"2".equals(filterFlag)) {
                childrenOrgList = orgMappingResult.stream().filter(x -> Objects.equals(x.getParentID(), item.getID()))
                        .collect(Collectors.toList());
            } else if ("1".equals(filterFlag)) {
                childrenOrgList = orgMappingResult.stream().filter(
                        x -> Objects.equals(x.getParentID(), item.getID()) && Strings.isNotBlank(x.getTaxPayerNumber()))
                        .collect(Collectors.toList());
            } else if ("2".equals(filterFlag)) {
                childrenOrgList = orgMappingResult.stream().filter(
                        x -> Objects.equals(x.getParentID(), item.getID()) && Strings.isBlank(x.getTaxPayerNumber()))
                        .collect(Collectors.toList());
            }
            if (childrenOrgList.isEmpty()) {
                return;
            }
            // 找出与之关联的统计卡片对象，并赋值给children属性
            DimensionOrgDtoDashboard childOrg = parentStatisticList.stream()
                    .filter(x -> item.getID() != null && item.getID().equals(x.getDimensionValueID())).findFirst()
                    .orElse(null);
            Assert.notNull(childOrg, "Null childOrg");
            childOrg.setHasChildren(!childrenOrgList.isEmpty());

            if (expandedOrgList.contains(item.getID())) {
                List<DimensionValueDisplayDto> childrenDimensionValueList = getDimensionValueDisplayDtoList(
                        childrenOrgList, orgAttrList, allOrgList);
                List<DimensionOrgDtoDashboard> childrenStatisticList = getStatisticCount(childrenDimensionValueList,
                        parentDimensionID, orgMappingResult, attrDtoList, allOrgList);
                childOrg.setChildren(childrenStatisticList);

                getSubStatisticRecursion(childrenOrgList, orgAttrList, allOrgList, parentDimensionID, orgMappingResult,
                        attrDtoList, childrenStatisticList, expandedOrgList, filterFlag);
            }
        }
    }

    /**
     * 根据机构的列表 转化为展示DTO
     */
    private List<DimensionValueDisplayDto> getDimensionValueDisplayDtoList(List<OrganizationDto> orgQuery,
            List<StatisticAttributeDisplayDto> orgAttrList, List<OrganizationDto> orgAllData) {
        List<OrganizationDto> allCustomDimensionOrg = getAllCustomDimensionOrgList();
        return orgQuery.stream()
                .map(orgQ -> rotateOrgDtoToDimValDisplayDto(orgQ, orgAttrList, allCustomDimensionOrg, orgAllData))
                .filter(s -> !Strings.isBlank(s.getDimensionValueID())).distinct().collect(Collectors.toList());
    }

    private DimensionValueDisplayDto rotateOrgDtoToDimValDisplayDto(OrganizationDto orgDto,
            List<StatisticAttributeDisplayDto> orgAttrList, List<OrganizationDto> allCustomDimensionOrg,
            List<OrganizationDto> orgAllData) {
        DimensionValueDisplayDto dVDDto = new DimensionValueDisplayDto();
        dVDDto.setDimensionValueID(orgDto.getID());
        dVDDto.setDimensionValueName(orgDto.getName());
        dVDDto.setFirstAttrValue(getAttrValue(!orgAttrList.isEmpty() ? orgAttrList.get(0) : null, orgDto,
                allCustomDimensionOrg, orgAllData));
        dVDDto.setSecondAttrValue(getAttrValue(orgAttrList.size() > 1 ? orgAttrList.get(1) : null, orgDto,
                allCustomDimensionOrg, orgAllData));
        return dVDDto;
    }

    private String getAttrValue(StatisticAttributeDisplayDto statAttrDto, OrganizationDto org,
            List<OrganizationDto> allCustomDimensionOrg, List<OrganizationDto> allOrgDto) {
        if (statAttrDto == null) {
            return null;
        }
        OrganizationDto orgData = allOrgDto.stream().filter(x -> Objects.equals(org.getID(), x.getID())).findFirst()
                .orElse(null);
        Assert.notNull(orgData, "Null OrgData");
        List<OrganizationDto> dimensionValueOrgList = allCustomDimensionOrg.stream()
                .filter(x -> Objects.equals(org.getID(), x.getID())).collect(Collectors.toList());
        String attrValueName = "";

        AttributeTypeEnum enumValue = AttributeTypeEnum.valueOf(statAttrDto.getAttributeType());
        Assert.notNull(enumValue, "Null AttributeTypeEnum");
        switch (enumValue) {
        case Area:
            attrValueName = getAreaName(org, orgData.getAreaID());
            break;
        case BusinessUnit:
            attrValueName = getBusinessUnitName(org, orgData.getBusinessUnitID());
            break;
        case Industry:
            attrValueName = org.getIndustryName();
            break;
        case OrganizationStructure:
            attrValueName = getStructureName(org, orgData.getStructureID());
            break;
        case OrgCode:
            attrValueName = org.getCode();
            break;
        case TaxPayerNumber:
            attrValueName = org.getTaxPayerNumber();
            break;
        case RegionID:
            // 原语句中orgData.getStructureID()未被使用，检查是否有问题
            // attrValueName = getRegion(org, orgData.getStructureID());
            attrValueName = getRegion(org);
            break;
        case SelfDimension:
            List<OrganizationDto> list = dimensionValueOrgList.stream()
                    .filter(x -> Objects.equals(x.getAttributeID(), statAttrDto.getAttributeID()))
                    .collect(Collectors.toList());
            if (!list.isEmpty()) {
                attrValueName = list.get(0).getDimensionValueName();
            } else {
                attrValueName = null;
            }
            break;
        default:
            logger.warn("enumValue not matched in getAttrValue");
        }
        return attrValueName;
    }

    private String getRegion(OrganizationDto org) {
        // C#中原传参businessUnitID未被使用，检查是否有问题
        Region region = null;
        if (org.getRegionID() != null) {
            RegionExample example = new RegionExample();
            pwc.taxtech.atms.entitiy.RegionExample.Criteria criteria = example.createCriteria();
            criteria.andIDEqualTo(org.getRegionID());
            region = regionMapper.selectByExample(example).stream().findFirst().orElse(null);
        }

        if (region != null) {
            return region.getName();
        }
        return "";
    }

    private String getAreaName(OrganizationDto org, String areaID) {
        if (Strings.isBlank(org.getAreaName())) {
            if (areaID != null) {
                AreaExample example = new AreaExample();
                pwc.taxtech.atms.entitiy.AreaExample.Criteria criteria = example.createCriteria();
                criteria.andIDEqualTo(areaID);
                org.setAreaName(
                        areaMapper.selectByExample(example).stream().map(am -> am.getName()).findFirst().orElse(null));
            } else {
                org.setAreaName(null);
            }

        }
        return org.getAreaName();
    }

    private String getBusinessUnitName(OrganizationDto org, String businessUnitID) {
        if (Strings.isBlank(org.getBusinessUnitName())) {
            if (businessUnitID != null) {
                BusinessUnitExample example = new BusinessUnitExample();
                pwc.taxtech.atms.entitiy.BusinessUnitExample.Criteria criteria = example.createCriteria();
                criteria.andIDEqualTo(businessUnitID);
                org.setBusinessUnitName(businessUnitMapper.selectByExample(example).stream().map(bu -> bu.getName())
                        .findFirst().orElse(null));
            } else {
                org.setBusinessUnitName(null);
            }
        }
        return org.getBusinessUnitName();
    }

    private String getStructureName(OrganizationDto org, String structureID) {
        if (Strings.isBlank(org.getStructureName())) {
            if (structureID != null) {
                OrganizationStructureExample example = new OrganizationStructureExample();
                pwc.taxtech.atms.entitiy.OrganizationStructureExample.Criteria criteria = example.createCriteria();
                criteria.andIDEqualTo(structureID);
                org.setStructureName(organizationStructureMapper.selectByExample(example).stream()
                        .map(os -> os.getName()).findFirst().orElse(null));
            } else {
                org.setStructureName(null);
            }
        }
        return org.getStructureName();
    }

    private List<OrganizationDto> getAllCustomDimensionOrgList() {
        return organizationMapper.getAllCustomDimensionOrgList();
    }

    private List<OrganizationDto> getAllOrgList() {
        return organizationMapper.selectByExample(null).stream().map(this::rotateOrgToOrgDto)
                .collect(Collectors.toList());
    }

    private OrganizationDto rotateOrgToOrgDto(Organization org) {
        OrganizationDto orgDto = new OrganizationDto();
        orgDto.setID(org.getID());
        orgDto.setName(org.getName());
        orgDto.setBusinessUnitID(org.getBusinessUnitID());
        orgDto.setStructureID(org.getStructureID());
        orgDto.setRegionID(org.getRegionID());
        orgDto.setAreaID(org.getAreaID());
        orgDto.setParentID(org.getParentID());
        return orgDto;
    }

    private List<DimensionOrgDtoDashboard> getStatisticCount(List<DimensionValueDisplayDto> dimensionValueList,
            String parentDimensionID, List<OrganizationDto> query, List<StatisticAttributeDisplayDto> attrDtoList) {
        return getStatisticCount(dimensionValueList, parentDimensionID, query, attrDtoList, null);
    }

    private List<DimensionOrgDtoDashboard> getStatisticCount(List<DimensionValueDisplayDto> dimensionValueList,
            final String parentDimensionID, List<OrganizationDto> query, List<StatisticAttributeDisplayDto> attrDtoList,
            List<OrganizationDto> allOrgList) {
        List<DimensionOrgDtoDashboard> displayList = new ArrayList<>();
        // 获取自定义维度的所有机构
        List<OrganizationDto> allCustomDimensionOrg = getCustomDimensionOrgList();

        // 维度值的列表，多少维度，如产品维度，值为产品1，产品2，产品3 等
        List<UserDimensionValue> userDimensionValueList = new ArrayList<>();
        List<UserDimensionValueOrg> userDimensionValueOrgList = new ArrayList<>();
        List<UserOrganization> userOrganizationList = new ArrayList<>();
        List<UserOrgRoleDto> originalRoleList = new ArrayList<>();
        List<UserOrgRoleDto> userDimensionValueRoleList = new ArrayList<>();
        List<UserOrgRoleDto> userOrganizationRoleList = new ArrayList<>();
        List<OrganizationDto> userAllDimensionList = new ArrayList<>();
        // 如果有用户数的属性
        if (attrDtoList.stream().anyMatch(attDto -> AttributeTypeEnum.User.value().equals(attDto.getAttributeType()))) {
            populateUserRoleBasic(userDimensionValueList, userDimensionValueOrgList, userOrganizationList,
                    originalRoleList, userDimensionValueRoleList, userOrganizationRoleList);

            if (DimensionConstant.OrgSubChildrenID.equals(parentDimensionID)) {
                userAllDimensionList = getDimensionValueOrgDtoList().stream().map(this::rotateToOrganizationDto)
                        .collect(Collectors.toList());
            }
        }
        List<OrganizationDto> userAllDimensionListTemp = userAllDimensionList;
        if (logger.isDebugEnabled())
            logger.debug("function: getStatisticCount before filter, Size of dimensionValueList:{} ",
                    dimensionValueList.size());
        for (DimensionValueDisplayDto val : dimensionValueList) {
            DimensionOrgDtoDashboard dtoDashboard = new DimensionOrgDtoDashboard();
            List<OrganizationDto> valueList;

            // 维度的属性值，如产品1，产品2，产品3
            dtoDashboard.setDimensionValueID(val.getDimensionValueID());
            dtoDashboard.setDimensionValue(val.getDimensionValueName());
            dtoDashboard.setFirstAttrName(val.getFirstAttrValue());
            dtoDashboard.setSecondAttrName(val.getSecondAttrValue());
            dtoDashboard.setHasChildren(false);

            // 每个维度值下面的属性统计
            if (!DimensionConstant.OrgSubChildrenID.equals(parentDimensionID)) {
                valueList = query.stream()
                        .filter(q -> val.getDimensionValueID() != null
                                && val.getDimensionValueID().equals(q.getDimensionValueID()))
                        .collect(Collectors.toList());
            } else {
                String orgId = val.getDimensionValueID();
                // 获取该机构下所有的子机构，速度很差
                List<String> orgList = getChildrenOrgIdToList(orgId, allOrgList).stream()
                        .filter(str -> orgId != null && !orgId.equals(str)).collect(Collectors.toList());
                // 子机构
                valueList = query.stream().filter(q -> orgList.contains(q.getID())).collect(Collectors.toList());
            }

            List<UserOrgDto> dimensionUser = new ArrayList<>();
            // 如果有用户数的属性
            if (attrDtoList.stream()
                    .anyMatch(attrDto -> AttributeTypeEnum.User.value().equals(attrDto.getAttributeType()))) {
                if (DimensionConstant.OrgSubChildrenID.equals(parentDimensionID)) {
                    // 某个机构下的维度列表
                    List<OrganizationDto> orgDimensionTempList = userAllDimensionListTemp.stream()
                            .filter(userDimension -> val.getDimensionValueID() != null
                                    && val.getDimensionValueID().equals(userDimension.getID()))
                            .collect(Collectors.toList());
                    dimensionUser = roleService.getUserByDimensionValue(orgDimensionTempList, userDimensionValueList,
                            userDimensionValueOrgList, userOrganizationList, originalRoleList,
                            userDimensionValueRoleList, userOrganizationRoleList, parentDimensionID,
                            val.getDimensionValueID());
                } else {
                    dimensionUser = roleService.getUserByDimensionValue(valueList, userDimensionValueList,
                            userDimensionValueOrgList, userOrganizationList, originalRoleList,
                            userDimensionValueRoleList, userOrganizationRoleList, parentDimensionID,
                            val.getDimensionValueID());

                }
            }

            // 自定义显示维度属性
            // 传参String parentDimensionID 和 dimensionValueID 在C#中未被引用，故删除
            dtoDashboard.setOrgDtoList(
                    getOrgDtoStatisticsList(valueList, attrDtoList, allCustomDimensionOrg, dimensionUser));

            if (!displayList.stream().anyMatch(display -> dtoDashboard.getDimensionValueID() != null
                    && dtoDashboard.getDimensionValueID().equals(display.getDimensionValueID()))) {
                displayList.add(dtoDashboard);
            }

        }
        if (logger.isDebugEnabled())
            logger.debug("function: getStatisticCount after filter, Size of displayList: {}", displayList.size());
        return displayList;
    }

    // 传参String parentDimensionID 和 dimensionValueID 在C#中未被引用，故删除
    /** 根据自定义属性，统计属性的值 */
    private List<DimensionOrgDto> getOrgDtoStatisticsList(List<OrganizationDto> orgDtoList,
            List<StatisticAttributeDisplayDto> attributeDtoList, List<OrganizationDto> allCustomDimensionOrg,
            List<UserOrgDto> dimensionUserList) {
        List<DimensionOrgDto> dtoList = new ArrayList<>();
        // 统计的时候去掉没有机构的数据
        List<OrganizationDto> orgDtoListTemp = orgDtoList.stream().filter(s -> Strings.isNotBlank(s.getID()))
                .collect(Collectors.toList());
        // 自定义显示属性列表统计
        for (StatisticAttributeDisplayDto item : attributeDtoList) {
            DimensionOrgDto temp = new DimensionOrgDto();
            // 自定义维度ID
            temp.setAttributeID(item.getAttributeID());
            // 自定义维度名字
            temp.setAttributeName(item.getAttributeName());

            // 事业部
            if (AttributeTypeEnum.BusinessUnit.value().equals(item.getAttributeType())) {
                temp.setAttributeType(AttributeTypeEnum.BusinessUnit.value());
                // 统计当前维度，多少个事业部
                temp.setAttributeValueCount((int) (orgDtoListTemp.stream()
                        .map(org -> org.getBusinessUnitID() + org.getBusinessUnitName()).distinct().count()));
            } else if (AttributeTypeEnum.Area.value().equals(item.getAttributeType())) {
                // 覆盖区域
                temp.setAttributeType(AttributeTypeEnum.Area.value());
                // 统计当前维度，多少个覆盖区域
                temp.setAttributeValueCount((int) (orgDtoListTemp.stream().map(org -> {
                    return org.getAreaID();
                }).distinct().count()));
            } else if (AttributeTypeEnum.OrganizationStructure.value().equals(item.getAttributeType())) {
                // 机构层级
                temp.setAttributeType(AttributeTypeEnum.OrganizationStructure.value());
                // 统计当前维度，多少个子公司
                temp.setAttributeValueCount((int) (orgDtoListTemp.stream().map(org -> {
                    return org.getStructureID();
                }).distinct().count()));
            } else if (AttributeTypeEnum.OrgSubChildren.value().equals(item.getAttributeType())) {
                // 机构数
                temp.setAttributeType(AttributeTypeEnum.OrgSubChildren.value());
                // 统计当前维度，多少个机构
                temp.setAttributeValueCount(
                        (int) (orgDtoListTemp.stream().map(OrganizationDto::getID).distinct().count()));
            } else if (AttributeTypeEnum.User.value().equals(item.getAttributeType())) {
                // 用户数
                temp.setAttributeType(AttributeTypeEnum.User.value());
                // 统计当前维度，多少个用户数
                temp.setAttributeValueCount((int) (dimensionUserList.stream().map(du -> {
                    return (du.getUserID());
                }).distinct().count()));
            } else if (AttributeTypeEnum.Industry.value().equals(item.getAttributeType())) {
                // 所属行业
                temp.setAttributeType(AttributeTypeEnum.Industry.value());
                // 统计当前维度，多少个所属行业
                temp.setAttributeValueCount((int) (orgDtoListTemp.stream().map(org -> {
                    return org.getIndustryID();
                }).distinct().count()));
            } else {
                // 自定义的维度
                temp.setAttributeType(AttributeTypeEnum.SelfDimension.value());
                List<OrganizationDto> query = new ArrayList<>();
                allCustomDimensionOrg.stream().forEach(customDO -> {
                    orgDtoListTemp.stream().forEach(orgDto -> {
                        if (Objects.equals(customDO.getID(), orgDto.getID())) {
                            OrganizationDto tempQuery = new OrganizationDto();
                            tempQuery.setAttributeID(customDO.getAttributeID());
                            tempQuery.setAttributeName(customDO.getAttributeName());
                            tempQuery.setDimensionValueID(customDO.getDimensionValueID());
                            tempQuery.setDimensionValueName(customDO.getDimensionValueName());
                            query.add(tempQuery);
                        }
                    });
                });
                temp.setAttributeValueCount(
                        (int) query.stream().filter(q -> Objects.equals(item.getAttributeID(), q.getAttributeID()))
                                .map(q -> q.getDimensionValueID()).distinct().count());
            }
            dtoList.add(temp);
        }

        return dtoList;
    }

    private OrganizationDto rotateToOrganizationDto(DimensionValueOrgDto sDto) {
        OrganizationDto tDto = new OrganizationDto();
        tDto.setID(sDto.getOrganizationID());
        tDto.setName(sDto.getOrganizationName());
        tDto.setDimensionID(sDto.getDimensionID());
        tDto.setDimensionName(sDto.getDimensionName());
        tDto.setDimensionValueID(sDto.getDimensionValueID());
        tDto.setDimensionValueName(sDto.getDimensionValue());
        return tDto;
    }

    private List<String> getChildrenOrgIdToList(String parentOrgID, List<OrganizationDto> orgList) {
        List<String> childrenOrgIdList = new ArrayList<>();
        childrenOrgIdList.add(parentOrgID);
        List<OrganizationDto> query = orgList.stream()
                .filter(org -> parentOrgID != null && parentOrgID.equals(org.getParentID()))
                .collect(Collectors.toList());
        query.stream().forEach(item -> {
            childrenOrgIdList.add(item.getID());
            getChildrenOrgIdRecursion(item.getID(), childrenOrgIdList, orgList);
        });
        return childrenOrgIdList;
    }

    private void getChildrenOrgIdRecursion(String orgId, List<String> childrenOrgIdList,
            List<OrganizationDto> orgList) {
        List<OrganizationDto> query = orgList.stream().filter(org -> orgId != null && orgId.equals(org.getParentID()))
                .collect(Collectors.toList());
        if (query.isEmpty()) {
            return;
        }
        query.stream().forEach(item -> {
            childrenOrgIdList.add(item.getID());
            getChildrenOrgIdRecursion(item.getID(), childrenOrgIdList, orgList);
        });
    }

    @Override
    public List<DimensionValueOrgDto> getDimensionValueOrgDtoList() {
        return organizationMapper.getDimensionValueOrgDtoList();
    }

    /**
     * 由于C#中OrganizationService.cs line 2356~2357以下代码被注释，故JAVA中未join
     * statDimension表做联合查询。 join sd in stat Dimension on d.ID equals sd.DimensionID
     * from sdTp in sdTemp.DefaultIfEmpty()
     */
    private List<OrganizationDto> getCustomDimensionOrgList() {
        return organizationMapper.getCustomDimensionOrgList();
    }

    private void populateUserRoleBasic(List<UserDimensionValue> userDimensionValueList,
            List<UserDimensionValueOrg> userDimensionValueOrgList, List<UserOrganization> userOrganizationList,
            List<UserOrgRoleDto> originalRoleList, List<UserOrgRoleDto> userDimensionValueRoleList,
            List<UserOrgRoleDto> userOrganizationRoleList) {

        userDimensionValueList.addAll(userDimensionValueMapper.getUserDimensionValueList());
        userDimensionValueOrgList
                .addAll(userDimensionValueOrgMapper.selectByExample(new UserDimensionValueOrgExample()));
        userOrganizationList.addAll(userOrganizationMapper.getUserOrganizationList());
        /*
         * .NET中以下代码单纯从数据库查询出role表并join下列表进行联合查询，JAVA中将该逻辑直接在MapperXML中表达 var roleList =
         * (from r in this._dbContext.Set<Role>() select r);
         */
        originalRoleList.addAll(userRoleMapper.getOriginalRoleList());
        userDimensionValueRoleList.addAll(userDimensionValueRoleMapper.getUserDimensionValueRoleList());
        userOrganizationRoleList.addAll(userOrganizationRoleMapper.getUserOrganizationRoleList());
    }

    private void addToDimensionValueList(OrganizationDto orgDto, List<DimensionValueDisplayDto> dimensionValueList) {
        // 判定当Dimension List中没有任何一个DimensionValueID 与传入的Organization相等时执行
        Boolean hasValueWithDimensionValueID = !dimensionValueList.stream()
                .anyMatch(dv -> Objects.equals(orgDto.getDimensionValueID(), dv.getDimensionValueID()));

        if (StringUtils.isNotEmpty(orgDto.getDimensionID()) && hasValueWithDimensionValueID) {
            DimensionValueDisplayDto dValue = new DimensionValueDisplayDto();
            dValue.setDimensionValueID(orgDto.getDimensionValueID());
            dValue.setDimensionValueName(orgDto.getDimensionValueName());
            dimensionValueList.add(dValue);
        }
    }

    private List<OrganizationDto> getOrgDimensionMappingForDimension(String dimensionID) {
        List<OrganizationDto> basicQuery = new ArrayList<>();
        DimensinTypeOrgDto dimensionDto = dimensionService.getDimensionById(dimensionID);
        Integer dimensionType = dimensionDto.getDimensionType();
        List<OrganizationDto> query = basicQuery;
        if (DimensionTypeEnum.BusinessUnit.value().equals(dimensionType)) {
            basicQuery = getBusinessUnitListJoinResult();
            query = basicQuery.stream().map(bQuery -> transBUAndOSCResult(bQuery, dimensionID, dimensionDto)).distinct()
                    .collect(Collectors.toList());
        } else if (DimensionTypeEnum.OrgSubChildren.value().equals(dimensionType)) {
            basicQuery = getOrganizationJoinResult();
            query = basicQuery.stream().map(bQuery -> transBUAndOSCResult(bQuery, dimensionID, dimensionDto)).distinct()
                    .collect(Collectors.toList());
        } else if (DimensionTypeEnum.SelfDimension.value().equals(dimensionType)) {
            // query = dimensionMapper.getBusinessUnitListJoinDimensionResult(dimensionID,
            // true).stream()
            // .map(this::filterOrgDtoNullValueForSelfDimension).collect(Collectors.toList());
            query = dimensionMapper.getBusinessUnitListJoinDimensionResult(dimensionID, true);
        } else {
            /**
             * C#中源代码将查询结果赋值给basicQuery未做任何后续传出处理，无意义 basicQuery =
             * getOrganizationJoinResult();
             */
        }
        return query;
    }

    private List<OrganizationDto> getBusinessUnitListJoinResult() {
        return businessUnitMapper.getBusinessUnitListJoinResult();
    }

    // private OrganizationDto filterOrgDtoNullValue(OrganizationDto orgDto) {
    // orgDto.setStructureName(orgDto.getStructureName() == null ? "" :
    // orgDto.getStructureName());
    // orgDto.setIndustryID(orgDto.getIndustryID() == null ? "" :
    // orgDto.getIndustryID());
    // orgDto.setRegionID(orgDto.getRegionID() == null ? "" : orgDto.getRegionID());
    // orgDto.setRegionName(orgDto.getRegionName() == null ? "" :
    // orgDto.getRegionName());
    // orgDto.setStructureID(orgDto.getStructureID() == null ? "" :
    // orgDto.getStructureID());
    // orgDto.setAreaID(orgDto.getAreaID() == null ? "" : orgDto.getAreaID());
    // orgDto.setAreaName(orgDto.getAreaName() == null ? "" : orgDto.getAreaName());
    // orgDto.setClientCode(orgDto.getClientCode() == null ? "" :
    // orgDto.getClientCode());
    // orgDto.setCode(orgDto.getCode() == null ? "" : orgDto.getCode());
    // orgDto.setParentID(orgDto.getParentID() == null ? "" : orgDto.getParentID());
    // orgDto.setpLevel(orgDto.getpLevel() == null ? 0 : orgDto.getpLevel());
    // return orgDto;
    // }

    private OrganizationDto transBUAndOSCResult(OrganizationDto rawDto, String dimensionID,
            DimensinTypeOrgDto dimensionDto) {
        OrganizationDto newDto = new OrganizationDto();
        newDto.setBusinessUnitID(rawDto.getBusinessUnitID());
        newDto.setBusinessUnitName(rawDto.getBusinessUnitName());
        newDto.setID(rawDto.getID());
        newDto.setName(rawDto.getName());
        newDto.setDimensionID(dimensionID);
        newDto.setDimensionName(dimensionDto.getName());
        newDto.setDimensionValueID(rawDto.getBusinessUnitID());
        newDto.setDimensionValueName(rawDto.getBusinessUnitName());

        newDto.setAttributeID(rawDto.getAttributeID());
        newDto.setIndustryID(rawDto.getIndustryID());
        newDto.setIndustryName(rawDto.getIndustryName());
        newDto.setRegionID(rawDto.getRegionID());
        newDto.setRegionName(rawDto.getRegionName());
        newDto.setStructureID(rawDto.getStructureID());
        newDto.setAreaID(rawDto.getAreaID());
        newDto.setAreaName(rawDto.getAreaName());
        newDto.setClientCode(rawDto.getClientCode());
        newDto.setCode(rawDto.getCode());
        newDto.setParentID(rawDto.getParentID());
        newDto.setpLevel(rawDto.getpLevel());
        newDto.setTaxPayerNumber(rawDto.getTaxPayerNumber());

        // OrganizationDto newDto = CommonUtils.copyProperties(rawDto, new
        // OrganizationDto());
        // newDto.setDimensionID(dimensionID);
        // newDto.setDimensionName(dimensionDto.getName());
        // newDto.setDimensionValueID(rawDto.getBusinessUnitID());
        // newDto.setDimensionValueName(rawDto.getBusinessUnitName());
        // newDto.setStructureName(null);
        return newDto;
    }

    // private OrganizationDto filterOrgDtoNullValueForSelfDimension(OrganizationDto
    // orgDto) {
    // orgDto.setID(orgDto.getID() == null ? "" : orgDto.getID());
    // orgDto.setName(orgDto.getName() == null ? "" : orgDto.getName());
    // orgDto.setpLevel(orgDto.getpLevel() == null ? 0 : orgDto.getpLevel());
    // orgDto.setParentID(orgDto.getParentID() == null ? "" : orgDto.getParentID());
    // orgDto.setCode(orgDto.getCode() == null ? "" : orgDto.getCode());
    // orgDto.setClientCode(orgDto.getClientCode() == null ? "" :
    // orgDto.getClientCode());
    // orgDto.setBusinessUnitName(orgDto.getBusinessUnitName() == null ? "" :
    // orgDto.getBusinessUnitName());
    // orgDto.setAreaName(orgDto.getAreaName() == null ? "" : orgDto.getAreaName());
    // orgDto.setAreaID(orgDto.getAreaID() == null ? "" : orgDto.getAreaID());
    // orgDto.setStructureID(orgDto.getStructureID() == null ? "" :
    // orgDto.getStructureID());
    // orgDto.setRegionName(orgDto.getRegionName() == null ? "" :
    // orgDto.getRegionName());
    // orgDto.setRegionID(orgDto.getRegionID() == null ? "" : orgDto.getRegionID());
    // orgDto.setIndustryName(orgDto.getIndustryName() == null ? "" :
    // orgDto.getIndustryName());
    // orgDto.setIndustryID(orgDto.getIndustryID() == null ? "" :
    // orgDto.getIndustryID());
    // orgDto.setBusinessUnitID(orgDto.getBusinessUnitID() == null ? "" :
    // orgDto.getStructureName());
    // orgDto.setAttributeID(orgDto.getAttributeID() == null ? "" :
    // orgDto.getAttributeID());
    // return orgDto;
    // }

    @Override
    public List<OrganizationDto> getOrganizationListByAreaID(String areaID, String attributeID) {

        List<AreaRegionDto> provinceRegionList = areaRegionMapper
                .selectByRegionLevelType(CommonConstants.REGION_LEVELTYPE_PROVINCE);

        List<Organization> organizationList = organizationMapper.selectByExample(null);
        List<OrganizationDto> allList = new ArrayList<>();
        organizationList.forEach(a -> {
            OrganizationDto organizationDto = new OrganizationDto();
            organizationDto.setID(a.getID());
            organizationDto.setParentID(a.getParentID());
            organizationDto.setName(a.getName());
            allList.add(organizationDto);
        });

        List<OrganizationDto> orgList = organizationMapper.selectIndBusUnitAreaOrgStrctReg(true, areaID,
                CommonConstants.REGION_LEVELTYPE_CITY);
        orgList.forEach(a -> {
            if (StringUtils.isNotEmpty(a.getParentName())) {
                List<AreaRegionDto> areaRegionList = new ArrayList<>();
                provinceRegionList.forEach(b -> {
                    if (StringUtils.equals(b.getRegionID(), a.getParentName()))
                        areaRegionList.add(b);
                });
                String provinceName = areaRegionList.isEmpty() ? null : areaRegionList.get(0).getRegionName();
                if (!StringUtils.equals(a.getRegionName(), provinceName)) {
                    a.setRegionName(provinceName + a.getRegionName());
                }
            }
            a.setParentID(getParentID(a, allList, orgList));
        });

        return orgList;
    }

    @Override
    public AreaOrganizationStatistics getAreaStatistics() {
        AreaExample example = new AreaExample();
        example.createCriteria().andIsActiveEqualTo(true);
        List<Area> areaList = areaMapper.selectByExample(example);

        List<Area> firstRegionLevelList = areaList.stream().filter(sa -> sa.getParentID() == null).collect(toList());
        List<AreaRegionDto> provinceRegionList = areaRegionMapper
                .selectByRegionLevelType(AreaConstant.LevelTypeProvince);
        // 保证AreaID不为null
        provinceRegionList.stream().forEach(x -> x.setAreaID(Strings.sBlank(x.getAreaID())));

        OrganizationExample exampleOrg = new OrganizationExample();
        exampleOrg.createCriteria().andIsActiveEqualTo(true);
        List<OrganizationStatistics> orgList = organizationMapper.selectByExample(exampleOrg).stream()
                .map(this::convertToOrganizationStatistics).collect(toList());

        List<OrganizationDto> orgDtoList = orgList.stream().map(from -> {
            OrganizationDto to = new OrganizationDto();
            to.setAreaID(from.getAreaID());
            to.setID(from.getID());
            to.setName(from.getName());
            to.setBusinessUnitID(from.getBusinessUnitID());
            return to;
        }).collect(toList());

        List<DimensionValueOrgDto> dimensionValueOrgList = getDimensionValueOrgDtoList();

        Map<String, Integer> areaUserCount = roleService.getActiveUserRoleList(orgDtoList, dimensionValueOrgList,
                DimensionConstant.AreaID);

        List<AreaStatistics> areaRetList = new ArrayList<>();
        List<StatisticAttributeDisplayDto> rawAttrList = new ArrayList<>();
        statisticAttributeService.getStatisticAttributeListByDimensionID(DimensionConstant.AreaID, rawAttrList);

        for (Area item : firstRegionLevelList) {
            AreaStatistics areaStatics = new AreaStatistics();
            areaStatics.setID(item.getID());
            areaStatics.setName(item.getName());

            Map<String, Integer> tmpAttributeStatics = getAttributeStaticsByAreaID(item.getID(), orgList, rawAttrList,
                    areaUserCount, dimensionValueOrgList);
            areaStatics.setAttributeStatics(tmpAttributeStatics);

            List<String> provinceNameList = getProvinceNameList(item.getID(), provinceRegionList, areaList);
            areaStatics.setProvinceNameList(provinceNameList.stream().distinct().collect(toList()));
            areaRetList.add(areaStatics);
        }

        AreaOrganizationStatistics result = new AreaOrganizationStatistics();
        result.setAreaList(areaRetList);
        result.setAttributeList(rawAttrList);
        return result;
    }

    private List<String> getProvinceNameList(String areaID, List<AreaRegionDto> provinceList, List<Area> areaList) {
        // 找到城市列表
        List<AreaRegionDto> citys = provinceList.stream().filter(sa -> Objects.equals(areaID, sa.getAreaID()))
                .collect(toList());
        List<String> retList = new ArrayList<>();
        if (!Lang.isEmpty(citys)) {
            List<String> tmpList = citys.stream().map(AreaRegionDto::getRegionName).distinct().collect(toList());
            retList.addAll(tmpList);
        }
        List<Area> subArea = areaList.stream().filter(sa -> Objects.equals(areaID, sa.getParentID())).collect(toList());
        for (Area sub : subArea) {
            List<String> tempCitys = getProvinceNameList(sub.getID(), provinceList, areaList);
            if (!Lang.isEmpty(tempCitys))
                retList.addAll(tempCitys);

        }
        return retList;
    }

    private Map<String, Integer> getAttributeStaticsByAreaID(String areaID, List<OrganizationStatistics> allOrgList,
            List<StatisticAttributeDisplayDto> rawAttrList, Map<String, Integer> areaUserCount,
            List<DimensionValueOrgDto> dimensionValueOrgList) {
        int userCount = Optional.ofNullable(areaUserCount.get(areaID)).orElse(0);
        List<OrganizationStatistics> orgList = allOrgList.stream().filter(sa -> Objects.equals(areaID, sa.getAreaID()))
                .collect(toList());
        // 机构数，用户数，区域，事业部，行业，地区
        List<DimensionValueOrgDto> selfDimensionList = dimensionValueOrgList.stream().filter(
                (DimensionValueOrgDto sa) -> rawAttrList.stream().anyMatch((StatisticAttributeDisplayDto sb) -> Objects
                        .equals(sb.getBelongDimensionID(), sa.getDimensionID())))
                .collect(toList());
        Map<String, Integer> dic = new HashMap<>();
        for (StatisticAttributeDisplayDto item : rawAttrList) {
            if (AttributeTypeEnum.User.value().equals(item.getAttributeType())) {
                // 用户数
                item.setCount(userCount);
            } else if (AttributeTypeEnum.OrgSubChildren.value().equals(item.getAttributeType())) {
                // 机构数
                item.setCount((int) allOrgList.stream().filter(sa -> Objects.equals(areaID, sa.getAreaID())).count());
            } else if (AttributeTypeEnum.SelfDimension.value().equals(item.getAttributeType())) {
                // 自定义区域
                item.setCount((int) selfDimensionList.stream()
                        .filter(sa -> Objects.equals(item.getBelongDimensionID(), sa.getDimensionID()))
                        .map(DimensionValueOrgDto::getDimensionValueID).distinct().count());
            } else if (AttributeTypeEnum.Industry.value().equals(item.getAttributeType())) {
                // 行业
                item.setCount((int) orgList.stream().filter(sa -> sa.getIndustryID() != null)
                        .map(OrganizationStatistics::getIndustryID).distinct().count());
            } else if (AttributeTypeEnum.RegionID.value().equals(item.getAttributeType())) {
                // 行政地区
                item.setCount((int) orgList.stream().filter(sa -> sa.getCityID() != null)
                        .map(OrganizationStatistics::getCityID).distinct().count());
            } else if (AttributeTypeEnum.Area.value().equals(item.getAttributeType())) {
                // 区域维度
                item.setCount(1);
            } else if (AttributeTypeEnum.BusinessUnit.value().equals(item.getAttributeType())) {
                // 事业部
                item.setCount((int) orgList.stream().filter(sa -> sa.getBusinessUnitID() != null)
                        .map(OrganizationStatistics::getBusinessUnitID).distinct().count());
            } else if (AttributeTypeEnum.OrganizationStructure.value().equals(item.getAttributeType())) {
                item.setCount((int) orgList.stream().filter(sa -> sa.getStructureID() != null)
                        .map(OrganizationStatistics::getStructureID).distinct().count());
            }
            dic.put(item.getAttributeID(), item.getCount());
        }

        return dic;
    }

    private OrganizationStatistics convertToOrganizationStatistics(Organization from) {
        OrganizationStatistics to = new OrganizationStatistics();
        to.setUserCount(0);
        to.setName(from.getName());
        to.setID(from.getID());
        to.setBusinessUnitID(from.getBusinessUnitID());
        to.setCityID(from.getRegionID());
        to.setIndustryID(from.getIndustryID());
        to.setParentID(from.getParentID());
        to.setStructureID(from.getStructureID());
        to.setAreaID(from.getAreaID());
        return to;
    }

    /** @see OrganizationService.cs, Line 2644 */
    @Override
    public OrgDto getOrgCustomDashbord(final String dimensionValueID, final String parentDimensionID,
            final String attributeID) {
        Assert.hasText(parentDimensionID, "Empty parentDimensionID");
        Assert.hasText(dimensionValueID, "Empty dimensionValueID");
        OrgDto orgDto = new OrgDto();
        List<OrganizationDto> orgDtoList = getOrgCustomDashbord$getOrgDtoList(dimensionValueID, parentDimensionID);
        List<OrganizationDto> orgDtoTempList = new ArrayList<>();

        List<OrganizationDto> orgTemp = getCustomDimensionOrgList();
        // join 两个变量:orgDtoList， orgTemp
        logger.debug("join 两个变量:orgDtoList, size[{}] orgTemp, size[{}]", orgDtoList.size(), orgTemp.size());
        // 代码理解：join的结果就是取orgTemp的字段，加上orgDtoTempList的name字段
        // 疑问：orgTemp的name字段与orgDtoTempList的name字段有什么区别？应该都是等价于Organization.name?
        List<OrgCustomDto> customOrgList = new ArrayList<>();
        for (OrganizationDto p : orgDtoList) {
            for (OrganizationDto d : orgTemp) {
                if (d.getID() != null && d.getID().equals(p.getID())) {
                    // on p.ID equals d.ID
                    OrgCustomDto tmp = new OrgCustomDto();
                    tmp.setAttributeID(d.getAttributeID());
                    tmp.setAttributeName(d.getAttributeName());
                    tmp.setDimensionID(d.getDimensionID());
                    tmp.setDimensionName(d.getDimensionName());
                    tmp.setDimensionValueID(d.getDimensionValueID());
                    tmp.setDimensionValueName(d.getDimensionValueName());
                    tmp.setOrgID(p.getID()); // 其实就是d.getID()
                    tmp.setOrgName(p.getName()); // 只有orgName使用了p.name
                    customOrgList.add(tmp);
                }
            }
        }
        orgDto.setOrgCustomDtoList(customOrgList);
        orgDto.setOrgCountList(new ArrayList<>());
        orgDto.setOrgDtoList(new ArrayList<>());

        // 如果是事业部
        if (attributeID.toLowerCase().equals(DimensionConstant.Attribute.BusinessUnitID)) {
            getOrgCustomDashbord$BusinessUnitID(orgDto, orgDtoList, orgDtoTempList);
        }
        // 如果是所属行业
        else if (attributeID.toLowerCase().equals(DimensionConstant.Attribute.IndustryID)) {
            getOrgCustomDashbord$IndustryID(orgDto, orgDtoList, orgDtoTempList);
        }
        // 如果是机构数
        else if (attributeID.toLowerCase().equals(DimensionConstant.Attribute.OrgSubChildrenID)) {
            getOrgCustomDashbord$OrgSubChildrenID(orgDto, orgDtoList, orgDtoTempList);
        }
        // 如果是机构层级
        else if (attributeID.toLowerCase().equals(DimensionConstant.Attribute.OrganizationStructureID)) {
            getOrgCustomDashbord$OrganizationStructureID(orgDto, orgDtoList, orgDtoTempList);
        }
        // 覆盖区域
        else if (attributeID.toLowerCase().equals(DimensionConstant.Attribute.AreaID)) {
            getOrgCustomDashbord$AreaID(orgDto, orgDtoList, orgDtoTempList);
        }
        // 如果是自定义属性,维度值的列表，多少维度，如产品维度，值为产品1，产品2，产品3 等
        else {
            getOrgCustomDashbord$DimensionValueID(attributeID, orgDto, orgDtoList, orgDtoTempList, orgTemp);
        }

        if (attributeID.toLowerCase().equals(DimensionConstant.Attribute.OrgSubChildrenID)) {
            List<OrgDisplayDto> tmplist = populateOrgLevel(orgDtoTempList);
            orgDto.setOrgDtoList(tmplist);
        } else {
            List<OrgDisplayDto> tmplist = orgDtoTempList.stream().map(this::convertToOrgDisplayDto).collect(toList());
            orgDto.setOrgDtoList(tmplist);
        }

        return orgDto;
    }

    public void getOrgCustomDashbord$DimensionValueID(final String attributeID, OrgDto orgDto,
            List<OrganizationDto> orgDtoList, List<OrganizationDto> orgDtoTempList, List<OrganizationDto> orgTemp) {
        // 把orgDtoList LEFT JION orgTemp 放到customFilter变量
        List<OrgCustomDto> customFilter = new ArrayList<>();
        for (OrganizationDto p : orgDtoList) {
            for (OrganizationDto d : orgTemp) {
                OrganizationDto dtp = null;
                if (Objects.equals(d.getAttributeID(), attributeID) && Objects.equals(d.getID(), p.getID())) {
                    dtp = d;
                }
                OrgCustomDto joinItem = new OrgCustomDto();
                joinItem.setDimensionID(dtp == null ? "" : dtp.getDimensionID());
                joinItem.setDimensionName(dtp == null ? "" : dtp.getDimensionName());
                joinItem.setDimensionValueID(dtp == null ? "" : dtp.getDimensionValueID());
                joinItem.setDimensionValueName(dtp == null ? "" : dtp.getDimensionValueName());
                joinItem.setOrgID(p.getID());
                joinItem.setOrgName(p.getName());
                customFilter.add(joinItem);
            }
        }
        customFilter = customFilter.stream().distinct().collect(toList());

        List<Map<String, String>> dimensionValueList = customFilter.stream().map((x) -> {
            Map<String, String> map = new HashMap<>();
            map.put("ID", x.getDimensionValueID());
            map.put("Name", x.getDimensionValueName());
            return map;
        }).distinct().collect(toList());

        for (Map<String, String> val : dimensionValueList) {
            OrganizationDto orgdto = new OrganizationDto();
            orgdto.setName(val.get("Name"));
            orgdto.setLevel(LevelConstant.FirstLevel);
            // 加入第0级节点
            orgDtoTempList.add(orgdto);

            // Group by ID (DimensionValueID)
            // 代码分析：目的是为了找出OrgID的集合 groupList.item.getOrgID
            List<OrgCustomDto> groupList = customFilter.stream()
                    .filter(x -> Objects.equals(val.get("ID"), x.getDimensionValueID())).collect(toList());

            // 代码分析：通过OrgID的集合，把原始的Org集合选出来
            groupList.forEach(g -> {
                orgDtoList.stream().filter(s -> Objects.equals(s.getID(), g.getOrgID())).forEach(r -> {
                    r.setLevel(LevelConstant.FirstLevel + 1);
                    orgDtoTempList.add(r);
                });
            });

            boolean noneMatch = orgDto.getOrgCountList().stream()
                    .noneMatch(x -> Objects.equals(val.get("ID"), x.getID()));
            if (noneMatch) {
                OrgCountDto org = new OrgCountDto();
                org.setID(val.get("ID"));
                org.setName(val.get("Name"));
                org.setCount((int) customFilter.stream()
                        .filter(s -> Objects.equals(val.get("ID"), s.getDimensionValueID())).count());
                orgDto.getOrgCountList().add(org);
            }
        }
    }

    private List<OrgDisplayDto> populateOrgLevel(List<OrganizationDto> orgDtoList) {
        List<OrgDisplayDto> displayList = new ArrayList<>();

        // FIXME C#中可能有Bug， SQL字段少了IsActive, 先照原样复制过来
        // SELECT ID, ParentID from Organization
        List<OrganizationDto> orgList = organizationMapper.selectOnlyIDAndParentID();
        for (OrganizationDto item : orgDtoList) {
            if (item.getParentID() == null)
                continue;

            // 找到了parentId
            boolean foundParentId = orgDtoList.stream().anyMatch(x -> item.getParentID().equals(x.getID()));
            if (foundParentId)
                continue;

            // 重新设置ParentID
            item.setParentID(getParentID(item, orgList, orgDtoList));
        }

        List<OrganizationDto> topList = orgDtoList.stream().filter(p -> {
            return orgDtoList.stream().noneMatch(sa -> {
                return Objects.equals(sa.getID(), p.getParentID());
            });
        }).collect(toList());

        for (OrganizationDto top : topList) {
            top.setLevel(LevelConstant.FirstLevel);
            OrgDisplayDto display = convertToOrgDisplayDto(top);
            displayList.add(display);
            List<OrgDisplayDto> subList = getChildren(top, orgDtoList);
            displayList.addAll(subList);
        }

        return displayList;
    }

    private List<OrgDisplayDto> getChildren(OrganizationDto current, List<OrganizationDto> orgList) {
        List<OrgDisplayDto> retList = new ArrayList<>();
        List<OrganizationDto> subList = orgList.stream().filter(sa -> Objects.equals(sa.getParentID(), current.getID()))
                .collect(toList());
        subList.forEach(item -> {
            item.setLevel(LevelConstant.FirstLevel + 1);
            retList.add(convertToOrgDisplayDto(item));
            List<OrgDisplayDto> children = getChildren(item, orgList);
            if (Lang.isEmpty(children)) {
                // 疑问: C#代码中，为什么把level设为0？
                item.setLevel(LevelConstant.FirstLevel);
            }
            retList.addAll(children);
        });

        return retList;
    }

    private String getParentID(OrganizationDto current, List<OrganizationDto> allList,
            List<OrganizationDto> orgDtoList) {
        if (current.getParentID() == null)
            return null;

        // 这个是为了处理脏数据，那种ID 和 ParentID 是一样的情况
        if (current.getParentID().equals(current.getID()))
            return null;

        // 代码理解：从全量中找到父节点
        OrganizationDto findParent = allList.stream().filter(sa -> current.getParentID().equals(sa.getID())).findFirst()
                .orElse(null);

        // 处理脏数据
        if (findParent == null)
            return null;

        if (BooleanUtils.isTrue(findParent.getIsActive())) {
            // 代码理解：从目标集合中找到这个节点
            OrganizationDto parent = orgDtoList.stream().filter(sa -> Objects.equals(sa.getID(), findParent.getID()))
                    .findFirst().orElse(null);
            if (parent != null)
                return parent.getID();
        }
        return getParentID(findParent, allList, orgDtoList);
    }

    private OrgDisplayDto convertToOrgDisplayDto(OrganizationDto from) {
        OrgDisplayDto to = copyProperties(from, new OrgDisplayDto());
        to.setPLevel(from.getpLevel());
        to.setOrgName(from.getName());
        to.setOrgID(from.getID());
        to.setPLevel(Optional.ofNullable(to.getPLevel()).orElse(0));
        to.setLevel(Optional.ofNullable(to.getLevel()).orElse(0));
        to.setSubsidiaryCount(Optional.ofNullable(to.getSubsidiaryCount()).orElse(0));
        to.setAreaCount(Optional.ofNullable(to.getAreaCount()).orElse(0));
        return to;
    }

    /** 从getOrgCustomDashbord抽取出来的方法 */
    private void getOrgCustomDashbord$OrgSubChildrenID(OrgDto orgDto, List<OrganizationDto> orgDtoList,
            List<OrganizationDto> orgDtoTempList) {
        // 删除无用的dimensionValueList变量
        orgDtoTempList.clear();
        // C#的代码是改变指针，这里改为clear, addAll
        orgDtoTempList.addAll(orgDtoList);
        OrgCountDto org = new OrgCountDto();
        org.setName(OrganizationMessage.OrganizationCount);
        // 统计businessUnitID数量
        org.setCount(orgDtoList.size());
        orgDto.getOrgCountList().add(org);
    }

    /** 从getOrgCustomDashbord抽取出来的方法 */
    private void getOrgCustomDashbord$AreaID(OrgDto orgDto, List<OrganizationDto> orgDtoList,
            List<OrganizationDto> orgDtoTempList) {
        // 找出IndustryID的列表，去重复
        List<Map<String, String>> dimensionValueList = orgDtoList.stream().map((OrganizationDto x) -> {
            Map<String, String> map = new HashMap<>();
            map.put("ID", x.getAreaID());
            map.put("Name", x.getAreaName());
            return map;
        }).distinct().collect(toList());

        // 遍历BU ID的列表
        for (Map<String, String> val : dimensionValueList) {
            OrganizationDto orgdto = new OrganizationDto();
            orgdto.setName(val.get("Name"));
            orgdto.setLevel(LevelConstant.FirstLevel);
            // 加入第0级节点
            orgDtoTempList.add(orgdto);

            // 找出这个BU对应的OrganizationDto列表
            List<OrganizationDto> groupList = orgDtoList.stream()
                    .filter(x -> Objects.equals(val.get("ID"), x.getAreaID())).collect(toList());

            groupList.forEach((OrganizationDto r) -> {
                r.setLevel(LevelConstant.FirstLevel + 1);
                // 加入第1级节点
                orgDtoTempList.add(r);
            });

            boolean noSuchItemById = orgDto.getOrgCountList().stream()
                    .noneMatch(x -> Objects.equals(val.get("ID"), x.getID()));
            if (noSuchItemById) {
                OrgCountDto org = new OrgCountDto();
                org.setID(val.get("ID"));
                org.setName(val.get("Name"));
                // 统计businessUnitID数量
                org.setCount(
                        (int) orgDtoList.stream().filter(x -> Objects.equals(val.get("ID"), x.getAreaID())).count());
                orgDto.getOrgCountList().add(org);
            }
        }
    }

    /** 从getOrgCustomDashbord抽取出来的方法 */
    private void getOrgCustomDashbord$OrganizationStructureID(OrgDto orgDto, List<OrganizationDto> orgDtoList,
            List<OrganizationDto> orgDtoTempList) {
        // 找出IndustryID的列表，去重复
        List<Map<String, String>> dimensionValueList = orgDtoList.stream().map((OrganizationDto x) -> {
            Map<String, String> map = new HashMap<>();
            map.put("ID", x.getStructureID());
            map.put("Name", x.getStructureName());
            return map;
        }).distinct().collect(toList());

        // 遍历BU ID的列表
        for (Map<String, String> val : dimensionValueList) {

            OrganizationDto orgdto = new OrganizationDto();
            orgdto.setName(val.get("Name"));
            orgdto.setLevel(LevelConstant.FirstLevel);
            // 加入第0级节点
            orgDtoTempList.add(orgdto);

            // 找出这个BU对应的OrganizationDto列表
            List<OrganizationDto> groupList = orgDtoList.stream()
                    .filter(x -> Objects.equals(val.get("ID"), x.getStructureID())).collect(toList());

            groupList.forEach((OrganizationDto r) -> {
                r.setLevel(LevelConstant.FirstLevel + 1);
                // 加入第1级节点
                orgDtoTempList.add(r);
            });

            boolean noSuchItemById = orgDto.getOrgCountList().stream()
                    .noneMatch(x -> Objects.equals(val.get("ID"), x.getID()));
            if (noSuchItemById) {
                OrgCountDto org = new OrgCountDto();
                org.setID(val.get("ID"));
                org.setName(val.get("Name"));
                // 统计businessUnitID数量
                org.setCount((int) orgDtoList.stream().filter(x -> Objects.equals(val.get("ID"), x.getStructureID()))
                        .count());
                orgDto.getOrgCountList().add(org);
            }
        }
    }

    /** 从getOrgCustomDashbord抽取出来的方法 */
    private void getOrgCustomDashbord$IndustryID(OrgDto orgDto, List<OrganizationDto> orgDtoList,
            List<OrganizationDto> orgDtoTempList) {
        // 找出IndustryID的列表，去重复
        List<Map<String, String>> dimensionValueList = orgDtoList.stream().map((OrganizationDto x) -> {
            Map<String, String> map = new HashMap<>();
            map.put("ID", x.getIndustryID());
            map.put("Name", x.getIndustryName());
            return map;
        }).distinct().collect(toList());

        // 遍历BU ID的列表
        for (Map<String, String> val : dimensionValueList) {

            OrganizationDto orgdto = new OrganizationDto();
            orgdto.setName(val.get("Name"));
            orgdto.setLevel(LevelConstant.FirstLevel);
            // 加入第0级节点
            orgDtoTempList.add(orgdto);

            // 找出这个BU对应的OrganizationDto列表
            List<OrganizationDto> groupList = orgDtoList.stream()
                    .filter(x -> Objects.equals(val.get("ID"), x.getIndustryID())).collect(toList());

            groupList.forEach((OrganizationDto r) -> {
                r.setLevel(LevelConstant.FirstLevel + 1);
                // 加入第1级节点
                orgDtoTempList.add(r);
            });

            boolean noSuchItemById = orgDto.getOrgCountList().stream()
                    .noneMatch(x -> Objects.equals(val.get("ID"), x.getID()));
            if (noSuchItemById) {
                OrgCountDto org = new OrgCountDto();
                org.setID(val.get("ID"));
                org.setName(val.get("Name"));
                // 统计businessUnitID数量
                org.setCount((int) orgDtoList.stream().filter(x -> Objects.equals(val.get("ID"), x.getIndustryID()))
                        .count());
                orgDto.getOrgCountList().add(org);
            }
        }
    }

    /** 从getOrgCustomDashbord抽取出来的方法 */
    private void getOrgCustomDashbord$BusinessUnitID(OrgDto orgDto, List<OrganizationDto> orgDtoList,
            List<OrganizationDto> orgDtoTempList) {
        // 找出BU ID的列表，去重复
        List<Map<String, String>> dimensionValueList = orgDtoList.stream().map((OrganizationDto x) -> {
            Map<String, String> map = new HashMap<>();
            map.put("ID", x.getBusinessUnitID());
            map.put("Name", x.getBusinessUnitName());
            return map;
        }).distinct().collect(toList());

        // 遍历BU ID的列表
        for (Map<String, String> val : dimensionValueList) {

            OrganizationDto orgdto = new OrganizationDto();
            orgdto.setName(val.get("Name"));
            orgdto.setLevel(LevelConstant.FirstLevel);
            // 加入第0级节点
            orgDtoTempList.add(orgdto);

            // 找出这个BU对应的OrganizationDto列表
            List<OrganizationDto> groupList = orgDtoList.stream()
                    .filter(x -> Objects.equals(val.get("ID"), x.getBusinessUnitID())).collect(toList());

            groupList.forEach((OrganizationDto r) -> {
                r.setLevel(LevelConstant.FirstLevel + 1);
                // 加入第1级节点
                orgDtoTempList.add(r);
            });

            boolean noSuchItemById = orgDto.getOrgCountList().stream()
                    .noneMatch(x -> Objects.equals(val.get("ID"), x.getID()));
            if (noSuchItemById) {
                OrgCountDto org = new OrgCountDto();
                org.setID(val.get("ID"));
                org.setName(val.get("Name"));
                // 统计businessUnitID数量
                org.setCount((int) orgDtoList.stream().filter(x -> Objects.equals(val.get("ID"), x.getBusinessUnitID()))
                        .count());
                orgDto.getOrgCountList().add(org);
            }
        }
    }

    /** 从getOrgCustomDashbord抽取出来的方法 */
    private List<OrganizationDto> getOrgCustomDashbord$getOrgDtoList(final String dimensionValueID,
            final String parentDimensionID) {
        List<OrganizationDto> orgDtoList = null;

        if (DimensionConstant.OrgSubChildrenID.equals(parentDimensionID)) {
            // 如果是机构
            // 查找所有该父机构下的所有子机构
            List<OrganizationDto> allOrgList = getAllOrgList();
            String parentOrgID = dimensionValueID;
            // orgList 所有的子机构的ID List
            List<String> orgList = getChildrenOrgIdToList(parentOrgID, allOrgList).stream()
                    .filter(x -> !parentOrgID.equals(x)).collect(toList());

            // 然后用所有的子机构的ID List在内存中条件过滤
            orgDtoList = getOrgDimensionMappingForDimension(parentDimensionID).stream()
                    .filter(x -> orgList.contains(x.getID())).collect(toList());
        } else {
            // 如果不是机构
            // getOrgDimensionMappingForDimension查出的是全量的数据
            // 然后用dimensionId和dimensionValueId两个字段在内存中条件过滤
            orgDtoList = getOrgDimensionMappingForDimension(
                    parentDimensionID)
                            .stream()
                            .filter(organizationDto -> (parentDimensionID.equals(organizationDto.getDimensionID())
                                    && dimensionValueID.equals(organizationDto.getDimensionValueID())))
                            .collect(toList());
        }
        // 去除机构为空的值
        orgDtoList.removeIf(x -> Strings.isBlank(x.getID()));
        return orgDtoList;
    }

    /**
     * 获取维度的名称，维度值的名称
     */
    @Override
    public OrgCustomDto getDimensionValueName(String dimensionID, String dimensionValueID) {
        OrgCustomDto customDto = new OrgCustomDto();
        DimensinTypeOrgDto dimensionDto = dimensionService.getDimensionById(dimensionID);
        Integer dimensionType = dimensionDto.getDimensionType();
        customDto.setDimensionName(dimensionDto.getName());
        if (DimensionTypeEnum.BusinessUnit.value().equals(dimensionType)) {
            BusinessUnit value = dimensionValueID != null ? businessUnitMapper.selectByPrimaryKey(dimensionValueID)
                    : null;
            if (value != null) {
                customDto.setDimensionValueName(value.getName());
            }
        } else if (DimensionTypeEnum.OrganizationStructure.value().equals(dimensionType)) {
            OrganizationStructure value = dimensionValueID != null
                    ? organizationStructureMapper.selectByPrimaryKey(dimensionValueID)
                    : null;
            if (value != null) {
                customDto.setDimensionValueName(value.getName());
            }
        } else if (DimensionTypeEnum.SelfDimension.value().equals(dimensionType)) {
            DimensionValue value = dimensionValueID != null ? dimensionValueMapper.selectByPrimaryKey(dimensionValueID)
                    : null;
            if (value != null) {
                customDto.setDimensionValueName(value.getName());
            }
        }
        return customDto;
    }

    @SuppressWarnings({ "rawtypes", "deprecation" })
    @Override
    public OperationResultDto disableOrgs(List<String> orgIDs) {
        try {
            List<OperationLogDto> logList = new ArrayList<>();
            for (String orgID : orgIDs) {
                Organization rootOrg = orgID != null ? organizationMapper.selectByPrimaryKey(orgID) : null;
                Assert.notNull(rootOrg, "rootOrg is null");
                if (rootOrg.getIsActive()) {
                    rootOrg.setIsActive(false);
                    organizationMapper.updateByPrimaryKey(rootOrg);
                    OperationLogDto opLog = new OperationLogDto();
                    opLog.setAction(OperationAction.Update.value());
                    opLog.setOperationContent(LogMessage.IsActive);
                    opLog.setOperationObject(rootOrg.getName());
                    opLog.setOriginalState(BooleanUtils.isTrue(rootOrg.getIsActive()) ? "False" : "True");
                    opLog.setUpdateState(BooleanUtils.isTrue(rootOrg.getIsActive()) ? "True" : "False");
                    opLog.setModule(OperationModule.Organization.value());
                    opLog.setComment("");
                    opLog.setLogType(OperateLogType.OperationLogOrganization.value());
                    logList.add(opLog);
                }
            }
            operationLogService.addOperationLogList(logList);
            return new OperationResultDto(true);
        } catch (Exception ex) {
            logger.error("DisableOrgs出错了:" + ex, ex);
            logger.error("标记回滚, ready to call setRollbackOnly");
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new OperationResultDto(false, ex.getMessage());
        }
    }

    @SuppressWarnings({ "rawtypes" })
    @Override
    public OperationResultDto enableOrgs(String orgID) {
        Organization curOrg = orgID != null ? organizationMapper.selectByPrimaryKey(orgID) : null;
        if (curOrg == null) {
            return new OperationResultDto(false, OrganizationMessage.NotFindOrgEnable);
        } else {
            if (!curOrg.getIsActive()) {
                curOrg.setIsActive(true);
                organizationMapper.updateByPrimaryKey(curOrg);
                OperationLogDto opLog = new OperationLogDto();
                opLog.setAction(OperationAction.Update.value());
                opLog.setOperationContent(LogMessage.IsActive);
                opLog.setOperationObject(curOrg.getName());
                opLog.setOriginalState(LogMessage.Disable);
                opLog.setUpdateState(LogMessage.Enable);
                opLog.setIp("");
                opLog.setModule(OperationModule.Organization.value());
                opLog.setComment("");
                opLog.setLogType(OperateLogType.OperationLogOrganization.value());
                operationLogService.addOperationLog(opLog);
            }
            return new OperationResultDto(true);
        }
    }

    @Override
    public OrganizationDto getSingleOrgByOrgID(String orgId) {
        OrganizationDto result = organizationMapper.getSingleOrgByOrgIDToOrgDto(orgId).stream().findFirst()
                .orElse(null);
        Assert.notNull(result, "result is null");
        result.setLevel(result.getLevel() == null ? 0 : result.getLevel());
        result.setIsSystemDimension(result.getIsSystemDimension() == null ? false : result.getIsSystemDimension());

        // 查询机构账套
        List<EnterpriseAccountSetOrgDto> query1 = enterpriseAccountSetOrgMapper.getSingleOrgByOrgIDToEASODto(orgId);
        for (EnterpriseAccountSetOrgDto p : query1) {
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM");
            p.setExpiredDateStr(p.getExpiredDate() == null ? "" : formatter.format(p.getExpiredDate()));
            p.setEffectiveDateStr(p.getEffectiveDate() == null ? "" : formatter.format(p.getEffectiveDate()));
        }
        result.setEnterpriseAccountSetOrgList(query1);
        // 查询机构服务
        List<ServiceType> serviceType = serviceTypeMapper.selectByExample(null);
        List<OrganizationServiceTemplateGroupDto> query2 = organizationServiceTemplateGroupMapper
                .getSingleOrgByOrgIDToOSTGDto(orgId);
        for (OrganizationServiceTemplateGroupDto mapping : query2) {
            ServiceType st = serviceType.stream().filter(n -> Objects.equals(n.getID(), mapping.getServiceTypeID()))
                    .findFirst().orElse(null);
            if (st != null) {
                mapping.setServiceTypeName(st.getName());
            }
        }
        result.setOrganizationServiceTemplateGroupList(query2);
        if (result.getParentID() == null) {
            result.setParentName("");
        } else {
            Organization tParent = organizationMapper.selectByExample(null).stream()
                    .filter(sa -> Objects.equals(sa.getID(), result.getParentID())).findFirst().orElse(null);
            Assert.notNull(tParent, "tParent is null");
            result.setParentName(tParent.getName());
        }
        result.setDimensionValueOrgList(dimensionService.getAllDimensionOrgListByOrgID(result.getID()));
        return result;
    }

    @Override
    public Boolean codeUniqueValidate(OrganizationValidateDto validateDto) {
        String innerId = Strings.isBlank(validateDto.getId()) ? "" : validateDto.getId().trim();
        String innerCode = Strings.isBlank(validateDto.getValue()) ? "" : validateDto.getValue().trim();
        return Strings.isBlank(innerCode)
                || organizationMapper.countForCodeUniqueValidate(innerId, innerCode).equals(0);
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Override
    public OperationResultDto<Object> addOrg(OrganizationDto orgDto) {
        try {
            OperationResultDto result = checkExist(orgDto);
            if (result != null && !BooleanUtils.isTrue(result.getResult())) {
                return new OperationResultDto(result.getResult(), result.getResultMsg());
            }
            if (Objects.equals(orgDto.getParentID(), OrganizationConstant.NoParentID)) {
                orgDto.setParentID(null);
                orgDto.setpLevel(0);
            } else {
                // 机构添加层级
                Organization query = null;
                query = orgDto.getParentID() != null ? organizationMapper.selectByPrimaryKey(orgDto.getParentID())
                        : null;
                if (query != null) {
                    orgDto.setpLevel(query.getpLevel() + 1);
                }
            }

            orgDto.setID(CommonUtils.getUUID());
            orgDto.setCreateTime(new Date());
            orgDto.setUpdateTime(new Date());

            OrganizationValidateDto vailOrgDto = new OrganizationValidateDto();
            vailOrgDto.setId(orgDto.getID());
            vailOrgDto.setValue(orgDto.getCode());

            if (!codeUniqueValidate(vailOrgDto)) {
                return new OperationResultDto(false, OrganizationMessage.OrgNameOrCodeIsExiste);
            }

            vailOrgDto = new OrganizationValidateDto();
            vailOrgDto.setId(orgDto.getID());
            vailOrgDto.setValue(orgDto.getTaxPayerNumber());

            if (!taxPayerNumberUniqueValidate(vailOrgDto)) {
                return new OperationResultDto(false, OrganizationMessage.OrgNameOrCodeTaxPaymentNoIsExist);
            }

            Organization org = copyProperties(orgDto, new Organization());
            org.setpLevel(orgDto.getpLevel());
            organizationMapper.insert(org);
            OperationResultDto<Object> validateSetRet = validateEnterpriseAccountSetDateTimeOverlap(
                    orgDto.getEnterpriseAccountSetOrgList());
            if (!validateSetRet.getResult()) {
                logger.error("标记回滚, ready to call setRollbackOnly");
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return validateSetRet;
            }

            // 添加账套
            for (EnterpriseAccountSetOrgDto p : orgDto.getEnterpriseAccountSetOrgList()) {
                EnterpriseAccountSetOrg enterpriseAccountSetOrg = new EnterpriseAccountSetOrg();
                enterpriseAccountSetOrg.setID(CommonUtils.getUUID());
                enterpriseAccountSetOrg.setEnterpriseAccountSetID(p.getEnterpriseAccountSetID());
                enterpriseAccountSetOrg.setOrganizationID(orgDto.getID());
                if (!Strings.isEmpty(p.getEffectiveDateStr())) {
                    enterpriseAccountSetOrg.setEffectiveDate(DateTimeFormat.forPattern("yyyy-MM-dd")
                            .parseDateTime(p.getEffectiveDateStr() + "-01").toDate());
                    p.setEffectiveDate(DateTimeFormat.forPattern("yyyy-MM-dd")
                            .parseDateTime(p.getEffectiveDateStr() + "-01").toDate());
                }

                if (!Strings.isEmpty(p.getExpiredDateStr())) {
                    enterpriseAccountSetOrg.setExpiredDate(DateTimeFormat.forPattern("yyyy-MM-dd")
                            .parseDateTime(p.getExpiredDateStr() + "-01").plusMonths(1).minusDays(1).toDate());
                    p.setExpiredDate(DateTimeFormat.forPattern("yyyy-MM-dd")
                            .parseDateTime(p.getExpiredDateStr() + "-01").plusMonths(1).minusDays(1).toDate());
                }
                enterpriseAccountSetOrgMapper.insert(enterpriseAccountSetOrg);
            }

            // 添加服务
            // For VAT, update Rule Engine tables after updating
            // OrganizationServiceTemplateGroup
            // .NET中定义了以下变量，但未被使用，故在此处注释 defaultRule, reportRules
            OrganizationServiceTemplateGroupDto vatSetting = orgDto.getOrganizationServiceTemplateGroupList().stream()
                    .filter(t -> ServiceTypeEnum.VAT.toString().equals(t.getServiceTypeID())).findFirst().orElse(null);
            if (vatSetting != null) { // If currently no rule and new rule should be added
                TemplateGroup templateGrp = vatSetting.getTemplateGroupID() != null
                        ? templateGroupMapper.selectByPrimaryKey(vatSetting.getTemplateGroupID())
                        : null;
                // In organization module tax payer type is 1&2 while 0&1 in rule engine
                Integer newTaxPayerType = (templateGrp != null && templateGrp.getPayTaxType() != null)
                        ? templateGrp.getPayTaxType() - 1
                        : VatTaxPayerTypeEnum.General.value();
                Date now = new Date();
                TaxPayerReportRule newRule = new TaxPayerReportRule();
                newRule.setId(distributedIDService.nextId());
                newRule.setIsDefault(false);
                newRule.setOrganizationId(orgDto.getID());
                newRule.setTemplateGroupId(vatSetting.getTemplateGroupID());
                newRule.setTaxPayerType(newTaxPayerType); // Use tax payer type related to template group
                newRule.setCreateTime(now);
                newRule.setUpdateTime(now);
                taxPayerReportRuleMapper.insert(newRule);
            }
            for (OrganizationServiceTemplateGroupDto p : orgDto.getOrganizationServiceTemplateGroupList()) {
                OrganizationServiceTemplateGroup orgServiceTempGroup = new OrganizationServiceTemplateGroup();
                orgServiceTempGroup.setId(distributedIDService.nextId());
                orgServiceTempGroup.setOrganizationId(orgDto.getID());
                orgServiceTempGroup.setServiceTypeId(p.getServiceTypeID());
                orgServiceTempGroup.setTemplateGroupId(p.getTemplateGroupID());
                organizationServiceTemplateGroupMapper.insert(orgServiceTempGroup);
            }

            // CopyModelTree(orgDto);//__restoreroy__

            addOrgSelfDimensionList(orgDto);
            // C#中的内部类CommonLogParams 在JAVA中已经被合并到OperationLogDto中
            UpdateLogParams tempCommonLogParms = new UpdateLogParams();
            tempCommonLogParms.setOperationModule(Organization.value());
            tempCommonLogParms.setComment(AddOrganization);
            tempCommonLogParms.setOperateLogType(OperationLogOrganization.value());
            tempCommonLogParms.setOperationContent(org.getName());
            tempCommonLogParms.setOperationAction(OperationAction.New.value());
            tempCommonLogParms.setOperationObject(orgDto.getName());
            tempCommonLogParms.setUpdateState("");
            tempCommonLogParms.setOriginalState("");
            operationLogService.addOrDeleteDataAddLog(tempCommonLogParms);
            return new OperationResultDto(true);
        } catch (Exception e) {
            logger.error("addOrg出错:" + e, e);
            logger.error("标记回滚, ready to call setRollbackOnly");
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new OperationResultDto(false, CommonFail + e.getMessage());
        }
    }

    // C#中形参userName在函数中未被使用，故删除
    private void addOrgSelfDimensionList(OrganizationDto orgDto) {
        if (orgDto.getDimensionValueOrgList() == null || orgDto.getDimensionValueOrgList().isEmpty()) {
            return;
        }
        List<DimensionValueOrg> addList = new ArrayList<>();
        // C#中的内部类CommonLogParams 在JAVA中已经被合并到OperationLogDto中
        // C#中进行联合查询获取到DimensionValue和Dimension的表，但无调用。 dimensionLogList未被使用，故将其删除
        for (DimensionValueOrgDto item : orgDto.getDimensionValueOrgList()) {
            DimensionValueOrg dvOrg = new DimensionValueOrg();
            // C#中下文UpdateTime 与 CreateTime在数据库中并无对应字段，故删除
            dvOrg.setOrganizationID(orgDto.getID());
            dvOrg.setID(CommonUtils.getUUID());
            dvOrg.setDimensionValueID(item.getDimensionValueID());
            addList.add(dvOrg);
        }
        if (!addList.isEmpty()) {
            addList.stream().forEach(x -> dimensionValueOrgMapper.insert(x));
        }

    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    public OperationResultDto<Object> validateEnterpriseAccountSetDateTimeOverlap(
            List<EnterpriseAccountSetOrgDto> list) {
        if (list != null && !list.isEmpty()) {
            for (EnterpriseAccountSetOrgDto p : list) {
                p.setEffectiveDate(null);
                p.setExpiredDate(null);
                if (Strings.isNotBlank(p.getEffectiveDateStr())) {
                    p.setEffectiveDate(DateTimeFormat.forPattern("yyyy-MM-dd")
                            .parseDateTime(p.getEffectiveDateStr() + "-01").toDate());
                }
                if (Strings.isNotBlank(p.getExpiredDateStr())) {
                    p.setExpiredDate(DateTimeFormat.forPattern("yyyy-MM-dd")
                            .parseDateTime(p.getExpiredDateStr() + "-01").plusMonths(1).minusDays(1).toDate());
                }
            }
            if (list.stream().anyMatch(sa -> sa.getEffectiveDate().after(sa.getExpiredDate()))) {
                return new OperationResultDto(false, EffectiveDateAreaProblem);
            }

            List<EnterpriseAccountSetOrgDto> overlapList = new ArrayList<>();
            if (commonService.isOrganizationDateTimeOverlap(list)) {
                list.stream().filter(sa -> sa.getOverlapList() != null && !sa.getOverlapList().isEmpty())
                        .forEach(s -> overlapList.add(s));
            }
            if (!overlapList.isEmpty()) {
                return new OperationResultDto(false, EnterpriseAccountSetOrgDateTimeOverlap, overlapList);
            }
            // 一年之内不能设置两个一样的账套，即：不能设置这种，账套A，2017年1月到3月，账套A，2017年4月到5月
            List<String> acountList = list.stream().map(EnterpriseAccountSetOrgDto::getEnterpriseAccountSetID)
                    .distinct().collect(Collectors.toList());
            for (String accountId : acountList) {
                List<EnterpriseAccountSetOrgDto> tempList = list.stream()
                        .filter(sa -> Strings.equals(sa.getEnterpriseAccountSetID(), accountId))
                        .collect(Collectors.toList());
                if (!CommonUtils.validateOnlyOncePerYear(tempList)) {
                    return new OperationResultDto(false,
                            EnterpriseAccountSetOrgMessage.EnterpriseAccountSetOrgOnlyOncePerYear);
                }
            }
        }
        return new OperationResultDto(true);
    }

    @SuppressWarnings("rawtypes")
    private OperationResultDto checkExist(OrganizationDto orgDto) {
        Assert.notNull(orgDto, "orgDto null point error");
        Organization queryRoot = null;
        if (orgDto.getParentID() != null) {
            OrganizationExample example = new OrganizationExample();
            Criteria criteria = example.createCriteria();
            criteria.andClientCodeEqualTo(orgDto.getClientCode());
            if (orgDto.getID() != null) {
                criteria.andIDNotEqualTo(orgDto.getID());
            }
            queryRoot = organizationMapper.selectByExample(example).stream().findFirst().orElse(null);
        }
        if (queryRoot != null && orgDto.getID() != null && Strings.equals(orgDto.getID(), CommonConstants.EMPTY_UUID)) {
            return new OperationResultDto(false, OrganizationMessage.ClientCodeIsExist);
        }
        OperationResultDto operationResult = checkOrgNameOrOrgCode(orgDto);
        if (operationResult.getResult()) {
            operationResult = checkCodeTaxPaymentNo(orgDto);
        }
        return operationResult;
    }

    @SuppressWarnings("rawtypes")
    private OperationResultDto checkCodeTaxPaymentNo(OrganizationDto orgDto) {

        OrganizationExample example = new OrganizationExample();
        Criteria criteria = example.createCriteria();
        criteria.andNameEqualTo(orgDto.getName()).andCodeEqualTo(orgDto.getCode());
        if (orgDto.getTaxPayerNumber() != null) {
            criteria.andTaxPayerNumberEqualTo(orgDto.getTaxPayerNumber());
        } else {
            criteria.andTaxPayerNumberIsNull();
        }
        if (orgDto.getID() != null) {
            criteria.andIDNotEqualTo(orgDto.getID());
        }
        Organization query = organizationMapper.selectByExample(example).stream().findFirst().orElse(null);
        if (query != null) {
            return new OperationResultDto(false, OrganizationMessage.OrgNameOrCodeTaxPaymentNoIsExist);
        }
        return new OperationResultDto(true);
    }

    @SuppressWarnings("rawtypes")
    @Override
    public OperationResultDto updateOrgToDimension(String dimensionValueID, String parentDimensionID,
            List<OrgBasicDto> orgList) {
        if (orgList == null || dimensionValueID == null || parentDimensionID == null) {
            throw new ApplicationException(CommonConstants.JSONNULLOBJECT);
        }
        String dimensionName = dimensionService.getDimensionById(parentDimensionID).getName();
        // 事业部,更新机构表
        if (DimensionConstant.BusinessUnitID.equalsIgnoreCase(parentDimensionID)) {
            for (OrgBasicDto model : orgList) {
                Organization old = organizationMapper.selectByPrimaryKey(model.getID());
                if (old == null) {
                    throw new ApplicationException(CommonConstants.JSONNULLOBJECT);
                }
                Organization copy = CommonUtils.copyProperties(old, new Organization());
                old.setBusinessUnitID(dimensionValueID);
                logger.debug("Start to update organization [ {} ]", old.getID());
                organizationMapper.updateByPrimaryKey(old);
                UpdateLogParams updateLogParams = new UpdateLogParams();
                updateLogParams.setOperateLogType(OperateLogType.OperationLogOrganization.value());
                updateLogParams.setComment("");
                updateLogParams.setOperationModule(OperationModule.BasicDataBusinessUnit.value());
                updateLogParams.setOperationContent(dimensionName);
                updateLogParams.setOriginalState(copy);
                updateLogParams.setUpdateState(old);
                updateLogParams.setOperationObject(model.getName());
                updateLogParams.setOperationAction(OperationAction.Update.value());
                operationLogService.updateDataAddLog(updateLogParams);
            }
        } else if (DimensionConstant.OrgSubChildrenID.equalsIgnoreCase(parentDimensionID)) {

        } else if (DimensionConstant.AreaID.equalsIgnoreCase(parentDimensionID)) {
            List<UpdateLogParams> logList = new ArrayList<>();
            List<NameDto> areaNameDtoList = areaMapper.selectByExample(new AreaExample()).stream()
                    .map(sa -> CommonUtils.copyProperties(sa, new NameDto())).collect(Collectors.toList());
            NameDto newArea = areaNameDtoList.stream().filter(sa -> Objects.equals(sa.getID(), dimensionValueID))
                    .findFirst().orElse(null);
            String newAreaName = newArea == null ? "" : newArea.getName();
            // 区域
            for (OrgBasicDto model : orgList) {
                Organization old = organizationMapper.selectByPrimaryKey(model.getID());
                if (old == null) {
                    throw new ApplicationException(CommonConstants.JSONNULLOBJECT);
                }
                Organization copy = CommonUtils.copyProperties(old, new Organization());
                if (!dimensionValueID.equalsIgnoreCase(old.getAreaID())) {
                    old.setAreaID(dimensionValueID);
                    NameDto oldArea = areaNameDtoList.stream()
                            .filter(sa -> Objects.equals(sa.getID(), copy.getAreaID())).findFirst().orElse(null);
                    String oldAreaName = oldArea == null ? "" : oldArea.getName();
                    UpdateLogParams updateLogParams = new UpdateLogParams();
                    updateLogParams.setOperateLogType(OperateLogType.OperationLogOrganization.value());
                    updateLogParams.setComment("");
                    updateLogParams.setOperationModule(OperationModule.Organization.value());
                    updateLogParams.setOperationContent(OrganizationMessage.AreaLogMessage);
                    updateLogParams.setOriginalState(oldAreaName);
                    updateLogParams.setUpdateState(newAreaName);
                    updateLogParams.setOperationObject(copy.getName());
                    updateLogParams.setOperationAction(OperationAction.Update.value());
                    logList.add(updateLogParams);
                    logger.debug("Start to update Organization [ {} ]", old.getID());
                    organizationMapper.updateByPrimaryKey(old);
                }
            }
            operationLogService.addOrDeleteDataAddLog(logList);
        } else {
            // 自定义的时候，更新DimensionValurOrg Table
            List<DimensionValueOrgDto> orgDtoList = new ArrayList<>();
            for (OrgBasicDto model : orgList) {
                DimensionValueOrgExample dimensionValueOrgExample = new DimensionValueOrgExample();
                dimensionValueOrgExample.createCriteria().andDimensionValueIDEqualTo(dimensionValueID)
                        .andOrganizationIDEqualTo(model.getID());
                List<DimensionValueOrg> query = dimensionValueOrgMapper.selectByExample(dimensionValueOrgExample);
                if ((query == null || query.isEmpty())
                        && orgDtoList.stream().noneMatch(s -> Objects.equals(s.getOrganizationID(), model.getID())
                                && Objects.equals(s.getDimensionValueID(), dimensionValueID))) {
                    DimensionValueOrgDto dto = new DimensionValueOrgDto();
                    dto.setID(CommonUtils.getUUID());
                    dto.setDimensionValueID(dimensionValueID);
                    dto.setOrganizationID(model.getID());
                    orgDtoList.add(dto);
                }
                UpdateLogParams updateLogParams = new UpdateLogParams();
                updateLogParams.setOperateLogType(OperateLogType.OperationLogOrganization.value());
                updateLogParams.setComment("");
                updateLogParams.setOperationModule(OperationModule.BasicDataDimensionValue.value());
                updateLogParams.setOperationContent(dimensionName);
                updateLogParams.setOriginalState("");
                updateLogParams.setUpdateState("");
                updateLogParams.setOperationObject(model.getName());
                updateLogParams.setOperationAction(OperationAction.New.value());
                operationLogService.addOrDeleteDataAddLog(updateLogParams);
            }
            // 添加新数据
            List<DimensionValueOrg> attr = orgDtoList.stream()
                    .map(s -> CommonUtils.copyProperties(s, new DimensionValueOrg())).collect(Collectors.toList());
            for (DimensionValueOrg oneAttr : attr) {
                logger.debug("Start to insert DimensionValueOrg [ {} ]", oneAttr.getID());
                dimensionValueOrgMapper.insert(oneAttr);
            }
        }
        OperationResultDto operationResultDto = new OperationResultDto();
        operationResultDto.setResult(true);
        return operationResultDto;
    }

    @SuppressWarnings("rawtypes")
    private OperationResultDto checkOrgNameOrOrgCode(OrganizationDto orgDto) {
        OrganizationExample example = new OrganizationExample();
        if (orgDto.getID() == null) {
            example.or().andClientCodeEqualTo(orgDto.getClientCode()).andNameEqualTo(orgDto.getName());
            example.or().andClientCodeEqualTo(orgDto.getClientCode()).andCodeEqualTo(orgDto.getCode());
        } else {
            example.or().andClientCodeEqualTo(orgDto.getClientCode()).andIDNotEqualTo(orgDto.getID())
                    .andNameEqualTo(orgDto.getName());
            example.or().andClientCodeEqualTo(orgDto.getClientCode()).andIDNotEqualTo(orgDto.getID())
                    .andCodeEqualTo(orgDto.getCode());
        }
        Organization query = organizationMapper.selectByExample(example).stream().findFirst().orElse(null);
        if (query != null) {
            return new OperationResultDto(false, OrganizationMessage.OrgNameOrCodeIsExiste);
        }
        return new OperationResultDto(true);
    }

    /**
     * update a org info
     */
    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Override
    public OperationResultDto<Object> updateOrg(OrganizationDto orgDto) {
        try {
            Assert.notNull(orgDto, "orgDto Null Exception on updateOrg");
            if (orgDto.getEnterpriseAccountSetOrgList() == null) {
                orgDto.setEnterpriseAccountSetOrgList(new ArrayList<>());
            }

            OperationResultDto result = checkOrgNameOrOrgCode(orgDto);
            if (result != null && result.getResult()) {
                result = this.checkCodeTaxPaymentNo(orgDto);
            }
            if (result != null && !BooleanUtils.isTrue(result.getResult())) {
                return new OperationResultDto(result.getResult(), result.getResultMsg());
            }
            Organization org = orgDto.getID() != null ? organizationMapper.selectByPrimaryKey(orgDto.getID()) : null;
            if (org == null) {
                return new OperationResultDto(false, OrganizationMessage.NotFindOrg);
            }

            // code唯一验证
            OrganizationValidateDto orgValidateDto1 = new OrganizationValidateDto();
            orgValidateDto1.setId(org.getID());
            orgValidateDto1.setValue(orgDto.getCode());
            if (!codeUniqueValidate(orgValidateDto1)) {
                return new OperationResultDto(false, OrganizationMessage.OrgNameOrCodeIsExiste);
            }

            // 纳税人识别号唯一性验证
            OrganizationValidateDto orgValidateDto2 = new OrganizationValidateDto();
            orgValidateDto2.setId(org.getID());
            orgValidateDto2.setValue(orgDto.getTaxPayerNumber());
            if (!taxPayerNumberUniqueValidate(orgValidateDto2)) {
                return new OperationResultDto(false, OrganizationMessage.OrgNameOrCodeIsExiste);
            }
            Organization orgOriginal = new Organization();
            orgOriginal = CommonUtils.copyProperties(org, orgOriginal);
            orgOriginal.setpLevel(orgOriginal.getpLevel());
            // 上级机构验证
            if (Strings.isBlank(orgDto.getParentID())
                    || Strings.equals(orgDto.getParentID(), OrganizationConstant.NoParentID)) {
                orgDto.setParentID(null);
                org.setpLevel(0);
            } else {
                // 机构添加层级
                Organization query = organizationMapper.selectByPrimaryKey(orgDto.getParentID());
                if (query != null) {
                    org.setpLevel(query.getpLevel() + 1);
                }
            }

            if (!Strings.equals(orgDto.getParentID(), org.getParentID())) {
                // 如果修改了上级机构
                OperationResultDto<Object> checkParentIDRet = checkParentID(orgDto);
                if (checkParentIDRet != null && !checkParentIDRet.getResult()) {
                    return checkParentIDRet;
                }
            }
            org.setName(orgDto.getName());
            org.setTaxPayerNumber(orgDto.getTaxPayerNumber());
            org.setParentID(orgDto.getParentID());
            org.setStructureID(orgDto.getStructureID());
            org.setRegionID(orgDto.getRegionID());
            org.setAreaID(orgDto.getAreaID());
            org.setUpdateTime(new Date());
            org.setIndustryID(orgDto.getIndustryID());
            org.setBusinessUnitID(orgDto.getBusinessUnitID());
            Organization orgUpdate = org;

            organizationMapper.updateByPrimaryKey(org);

            OperationResultDto<Object> validatesetRet = validateEnterpriseAccountSetDateTimeOverlap(
                    orgDto.getEnterpriseAccountSetOrgList());
            if (!validatesetRet.getResult()) {
                return validatesetRet;
            }

            // 更新机构账套
            EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
            pwc.taxtech.atms.entitiy.EnterpriseAccountSetOrgExample.Criteria criteria = example.createCriteria();
            if (orgDto.getID() != null) {
                criteria.andOrganizationIDEqualTo(orgDto.getID());
            } else {
                criteria.andOrganizationIDIsNull();
            }
            List<EnterpriseAccountSetOrg> enterpriseAccountSetOrgList = enterpriseAccountSetOrgMapper
                    .selectByExample(example);
            for (EnterpriseAccountSetOrgDto p : orgDto.getEnterpriseAccountSetOrgList()) {
                p.setEffectiveDate(Strings.isBlank(p.getEffectiveDateStr()) ? p.getEffectiveDate()
                        : DateTimeFormat.forPattern("yyyy-MM-dd").parseDateTime(p.getEffectiveDateStr() + "-01")
                                .toDate());
                p.setExpiredDate(Strings.isBlank(p.getExpiredDateStr()) ? p.getExpiredDate()
                        : DateTimeFormat.forPattern("yyyy-MM-dd").parseDateTime(p.getExpiredDateStr() + "-01")
                                .plusMonths(1).minusDays(1).toDate());
                if (enterpriseAccountSetOrgList != null
                        && enterpriseAccountSetOrgList.stream().anyMatch(t -> Strings.equals(t.getID(), p.getID()))) {
                    EnterpriseAccountSetOrg tempDto = enterpriseAccountSetOrgList.stream()
                            .filter(t -> Strings.equals(t.getID(), p.getID())).findFirst().orElse(null);
                    Assert.notNull(tempDto, "tempDto null exception on updateOrg");
                    tempDto.setEffectiveDate(p.getEffectiveDate());
                    tempDto.setEnterpriseAccountSetID(p.getEnterpriseAccountSetID());
                    tempDto.setExpiredDate(p.getExpiredDate());
                    enterpriseAccountSetOrgMapper.updateByPrimaryKey(tempDto);
                } else {
                    EnterpriseAccountSetOrg temp = new EnterpriseAccountSetOrg();
                    temp.setID(CommonUtils.getUUID());
                    temp.setEnterpriseAccountSetID(p.getEnterpriseAccountSetID());
                    temp.setOrganizationID(p.getOrganizationID());
                    temp.setEffectiveDate(p.getEffectiveDate());
                    temp.setExpiredDate(p.getExpiredDate());
                    enterpriseAccountSetOrgMapper.insert(temp);
                }
            }

            List<EnterpriseAccountSetOrg> deleteSetList = enterpriseAccountSetOrgList.stream().filter(p -> orgDto
                    .getEnterpriseAccountSetOrgList().stream().noneMatch(sa -> Strings.equals(sa.getID(), p.getID())))
                    .collect(Collectors.toList());

            if (deleteSetList != null && !deleteSetList.isEmpty()) {
                deleteSetList.stream()
                        .forEach(delSet -> enterpriseAccountSetOrgMapper.deleteByPrimaryKey(delSet.getID()));
            }

            OrganizationServiceTemplateGroupExample oSTGExample = new OrganizationServiceTemplateGroupExample();
            pwc.taxtech.atms.entitiy.OrganizationServiceTemplateGroupExample.Criteria oSTGCriteria = oSTGExample
                    .createCriteria();
            if (orgDto.getID() != null) {
                oSTGCriteria.andOrganizationIdEqualTo(orgDto.getID());
            } else {
                oSTGCriteria.andOrganizationIdIsNull();
            }
            List<OrganizationServiceTemplateGroup> currentOrganizationServiceTemplateGroupList = organizationServiceTemplateGroupMapper
                    .selectByExample(oSTGExample);

            List<OrganizationServiceTemplateGroup> updateOrganizationServiceTemplateGroupList = new ArrayList<>();
            TaxPayerReportRuleExample taxExample = new TaxPayerReportRuleExample();
            taxExample.or().andIsDefaultEqualTo(true);
            Assert.notNull(orgDto.getID(), "orgDto ID null exception on updateOrg");
            String sqlValue = "%" + orgDto.getID() + "%";
            taxExample.or().andOrganizationIdLike(sqlValue);
            List<TaxPayerReportRule> reportRules = taxPayerReportRuleMapper.selectByExample(taxExample);
            TaxPayerReportRule activeRule = reportRules.stream()
                    .filter(t -> t.getOrganizationId() == null ? false : t.getOrganizationId().contains(orgDto.getID()))
                    .findFirst().orElse(null);
            // C#中在此处定义的defaultRule在下文中未被引用，JAVA中将其删除

            // 更新机构服务
            currentOrganizationServiceTemplateGroupList.stream()
                    .forEach(tg -> organizationServiceTemplateGroupMapper.deleteByPrimaryKey(tg.getId()));

            // For VAT, update Rule Engine tables instead of
            OrganizationServiceTemplateGroupDto vatSetting = orgDto.getOrganizationServiceTemplateGroupList().stream()
                    .filter(t -> Strings.equals(t.getServiceTypeID(), ServiceTypeEnum.VAT.value().toString()))
                    .findFirst().orElse(null);
            if (vatSetting != null) {
                TemplateGroup templateGrp = vatSetting.getTemplateGroupID() != null
                        ? templateGroupMapper.selectByPrimaryKey(vatSetting.getTemplateGroupID())
                        : null;
                // In organization module tax payer type is 1&2 while 0&1 in rule engine
                int newTaxPayerType = (templateGrp != null && templateGrp.getPayTaxType() != null)
                        ? (templateGrp.getPayTaxType() - 1)
                        : (VatTaxPayerTypeEnum.General.value());
                if ((activeRule != null)
                        && (Long.compare(activeRule.getTemplateGroupId(), vatSetting.getTemplateGroupID()) !=0)) {
                    // Multiple organizations in 1 rule
                    if (!Strings.equals(activeRule.getOrganizationId(), orgDto.getID())) {
                        activeRule.setOrganizationId(
                                activeRule.getOrganizationId().replace(orgDto.getID(), "").replace(",,", ","));
                        Date now = new Date();
                        TaxPayerReportRule newRule = new TaxPayerReportRule();
                        newRule.setId(distributedIDService.nextId());
                        newRule.setIsDefault(false);
                        newRule.setOrganizationId(orgDto.getID());
                        newRule.setTemplateGroupId(vatSetting.getTemplateGroupID());
                        newRule.setTaxPayerType(newTaxPayerType); // Use tax payer type related to template group
                        newRule.setCreateTime(now);
                        newRule.setUpdateTime(now);
                        taxPayerReportRuleMapper.insertSelective(newRule);
                    } else {
                        activeRule.setTemplateGroupId(vatSetting.getTemplateGroupID());
                        activeRule.setTaxPayerType(newTaxPayerType);
                    }
                } else { // If currently no rule and new rule should be added
                    Date now = new Date();
                    TaxPayerReportRule newRule = new TaxPayerReportRule();
                    newRule.setId(distributedIDService.nextId());
                    newRule.setIsDefault(false);
                    newRule.setOrganizationId(orgDto.getID());
                    newRule.setTemplateGroupId(vatSetting.getTemplateGroupID());
                    newRule.setTaxPayerType(newTaxPayerType); // Use tax payer type related to template group
                    newRule.setCreateTime(now);
                    newRule.setUpdateTime(now);
                    taxPayerReportRuleMapper.insertSelective(newRule);
                }
            } else if (activeRule != null) { // If no setting, remove non-default rule
                taxPayerReportRuleMapper.deleteByPrimaryKey(activeRule.getId());
            }
            for (OrganizationServiceTemplateGroupDto p : orgDto.getOrganizationServiceTemplateGroupList()) {
                OrganizationServiceTemplateGroup temp = new OrganizationServiceTemplateGroup();
                temp.setId(distributedIDService.nextId());
                temp.setOrganizationId(orgDto.getID());
                temp.setServiceTypeId(p.getServiceTypeID());
                temp.setTemplateGroupId(p.getTemplateGroupID());
                updateOrganizationServiceTemplateGroupList.add(temp);
                organizationServiceTemplateGroupMapper.insertSelective(temp);
            }

            // 更新自定义属性
            // C#中的内部类CommonLogParams 在JAVA中替换成UpdateLogParams中
            List<UpdateLogParams> selfDimensionList;
            selfDimensionList = updateSelfDimensionList(orgDto);

            UpdateLogParams updateLogParams = new UpdateLogParams();
            updateLogParams.setOriginalState(orgOriginal);
            updateLogParams.setUpdateState(orgUpdate);
            updateLogParams.setOperationModule(Organization.value());
            updateLogParams.setOperateLogType(OperationLogOrganization.value());
            updateLogParams.setOperationContent(orgUpdate.getName());
            updateLogParams.setOperationObject(org.getName());
            updateLogParams.setOperationAction(OperationAction.Update.value());
            updateLogParams.setComment("");
            operationLogService.updateDataAddLog(updateLogParams);

            updateEnterpriseAccountSetOrgLog(org.getName(), enterpriseAccountSetOrgList,
                    orgDto.getEnterpriseAccountSetOrgList());
            updateOrganizationServiceTemplateGroupLog(org.getName(), currentOrganizationServiceTemplateGroupList,
                    updateOrganizationServiceTemplateGroupList);
            operationLogService.addOrDeleteDataAddLog(selfDimensionList);
            return new OperationResultDto(true);
        } catch (Exception ex) {
            logger.error("UpdateOrg出错了:" + ex, ex);
            logger.error("标记回滚, ready to call setRollbackOnly");
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new OperationResultDto(false, ex.getMessage());
        }
    }

    private void updateOrganizationServiceTemplateGroupLog(String operateObject,
            List<OrganizationServiceTemplateGroup> currentList, List<OrganizationServiceTemplateGroup> updateList) {
        currentList = currentList == null ? new ArrayList<>() : currentList;
        updateList = updateList == null ? new ArrayList<>() : updateList;
        // 新增
        List<OrganizationServiceTemplateGroup> tempCurrentList = currentList;
        List<OrganizationServiceTemplateGroup> addList = updateList.stream()
                .filter(p -> !tempCurrentList.stream()
                        .anyMatch(sa -> Strings.equals(sa.getServiceTypeId(), p.getServiceTypeId())))
                .collect(Collectors.toList());

        // 删除
        List<OrganizationServiceTemplateGroup> tempUpdateList = updateList;
        List<OrganizationServiceTemplateGroup> deleteList = currentList.stream()
                .filter(p -> !tempUpdateList.stream()
                        .anyMatch(sa -> Strings.equals(sa.getServiceTypeId(), p.getServiceTypeId())))
                .collect(Collectors.toList());

        // 修改
        List<OrganizationServiceTemplateGroup> updates = updateList.stream()
                .filter(p -> tempCurrentList.stream()
                        .anyMatch(sa -> Strings.equals(sa.getServiceTypeId(), p.getServiceTypeId())
                                && (Long.compare(sa.getTemplateGroupId(), p.getTemplateGroupId()) != 0)))
                .collect(Collectors.toList());

        List<UpdateLogParams> addLogList = new ArrayList<>();

        List<TemplateGroup> queryTemplate = templateGroupMapper.selectByExample(null);
        List<ServiceType> queryService = serviceTypeMapper.selectByExample(null);

        for (OrganizationServiceTemplateGroup item : addList) {
            ServiceType st1 = queryService.stream().filter(sa -> Strings.equals(sa.getID(), item.getServiceTypeId()))
                    .findFirst().orElse(null);
            Assert.notNull(st1, "queryService result null exception in updateOrganizationServiceTemplateGroupLog");
            TemplateGroup st2 = queryTemplate.stream()
                    .filter(sa -> Long.compare(sa.getId(), item.getTemplateGroupId())==0).findFirst().orElse(null);
            Assert.notNull(st2, "queryTemplate result null exception in updateOrganizationServiceTemplateGroupLog");
            String updateStr = st1.getName() + CommonConstants.OneSpace + st2.getName();
            UpdateLogParams updateLogParams = new UpdateLogParams();
            updateLogParams.setUpdateState(updateStr);
            updateLogParams.setOriginalState("");
            updateLogParams.setOperationContent(OrganizationMessage.OrganizationServiceTemplateGroup);
            updateLogParams.setOperationObject(operateObject);
            updateLogParams.setOperationModule(OperationModule.Organization.value());
            updateLogParams.setOperateLogType(OperateLogType.OperationLogOrganization.value());
            updateLogParams.setComment("");
            updateLogParams.setOperationAction(OperationAction.New.value());
            addLogList.add(updateLogParams);
        }

        //todo: just for test at here, remove comment later
//        for (OrganizationServiceTemplateGroup item : deleteList) {
//            ServiceType st1 = queryService.stream().filter(sa -> Strings.equals(sa.getID(), item.getServiceTypeId()))
//                    .findFirst().orElse(null);
//            Assert.notNull(st1, "queryService result null exception on updateOrganizationServiceTemplateGroupLog");
//            TemplateGroup st2 = queryTemplate.stream()
//                    .filter(sa -> Long.compare(sa.getId(), item.getTemplateGroupId())==0).findFirst().orElse(null);
//            Assert.notNull(st2, "queryTemplate result null exception on updateOrganizationServiceTemplateGroupLog");
//            String updateStr = st1.getName() + CommonConstants.OneSpace + st2.getName();
//            UpdateLogParams updateLogParams = new UpdateLogParams();
//            updateLogParams.setUpdateState("");
//            updateLogParams.setOriginalState(updateStr);
//            updateLogParams.setOperationContent(OrganizationMessage.OrganizationServiceTemplateGroup);
//            updateLogParams.setOperationObject(operateObject);
//            updateLogParams.setOperationModule(OperationModule.Organization.value());
//            updateLogParams.setOperateLogType(OperateLogType.OperationLogOrganization.value());
//            updateLogParams.setComment("");
//            updateLogParams.setOperationAction(OperationAction.Delete.value());
//            addLogList.add(updateLogParams);
//        }

//        for (OrganizationServiceTemplateGroup item : updates) {
//            OrganizationServiceTemplateGroup current = currentList.stream()
//                    .filter(sa -> Strings.equals(sa.getServiceTypeId(), item.getServiceTypeId())).findFirst()
//                    .orElse(null);
//            if (current != null) {
//                ServiceType tService = queryService.stream()
//                        .filter(sa -> Strings.equals(sa.getID(), item.getServiceTypeId())).findFirst().orElse(null);
//                Assert.notNull(tService, "tService null exception in updateOrganizationServiceTemplateGroupLog");
//                String serviceName = tService.getName();
//
//                TemplateGroup updateTemplate = queryTemplate.stream()
//                        .filter(sa -> Long.compare(sa.getId(), item.getTemplateGroupId())==0).findFirst().orElse(null);
//                Assert.notNull(updateTemplate,
//                        "updateTemplate null exception in updateOrganizationServiceTemplateGroupLog");
//                String updateStr = updateTemplate.getName();
//
//                TemplateGroup currentTemplate = queryTemplate.stream()
//                        .filter(sa -> Long.compare(sa.getId(), current.getTemplateGroupId())==0).findFirst()
//                        .orElse(null);
//                Assert.notNull(currentTemplate,
//                        "currentTemplate null exception in updateOrganizationServiceTemplateGroupLog");
//                String currentStr = currentTemplate.getName();
//
//                UpdateLogParams updateLogParams = new UpdateLogParams();
//                updateLogParams.setUpdateState(updateStr);
//                updateLogParams.setOriginalState(currentStr);
//                updateLogParams.setOperationContent(OrganizationMessage.OrganizationServiceTemplateGroup);
//                updateLogParams.setOperationObject(operateObject + DashSignSeparator + serviceName);
//                updateLogParams.setOperationModule(OperationModule.Organization.value());
//                updateLogParams.setOperateLogType(OperateLogType.OperationLogOrganization.value());
//                updateLogParams.setComment("");
//                updateLogParams.setOperationAction(OperationAction.Update.value());
//                addLogList.add(updateLogParams);
//            }
//
//        }
        operationLogService.addOrDeleteDataAddLog(addLogList);
    }

    private void updateEnterpriseAccountSetOrgLog(String operateObject, List<EnterpriseAccountSetOrg> currentList,
            List<EnterpriseAccountSetOrgDto> updateList) {
        List<String> fromList = new ArrayList<>();
        List<String> toList = new ArrayList<>();
        List<EnterpriseAccountSet> set = enterpriseAccountSetMapper.selectByExample(null);
        if (currentList != null && !currentList.isEmpty()) {
            fromList = set.stream()
                    .filter(q -> currentList.stream()
                            .anyMatch(p -> Strings.equals(p.getEnterpriseAccountSetID(), q.getID())))
                    .sorted(comparing(EnterpriseAccountSet::getID, nullsFirst(naturalOrder())))
                    .map(EnterpriseAccountSet::getName).collect(Collectors.toList());
        }
        if (updateList != null && !updateList.isEmpty()) {
            toList = set.stream()
                    .filter(q -> updateList.stream()
                            .anyMatch(p -> Strings.equals(p.getEnterpriseAccountSetID(), q.getID())))
                    .sorted(comparing(EnterpriseAccountSet::getID, nullsFirst(naturalOrder())))
                    .map(EnterpriseAccountSet::getName).collect(Collectors.toList());
        }
        String from = String.join(Comma, fromList);
        String to = String.join(Comma, toList);

        if (!Strings.equals(from, to)) {
            UpdateLogParams log = new UpdateLogParams();
            log.setOperationAction(OperationAction.Update.value());
            log.setComment("");
            log.setOperateLogType(OperateLogType.OperationLogOrganization.value());
            log.setOperationModule(OperationModule.Organization.value());
            log.setOperationObject(operateObject);
            log.setOperationContent(OrganizationMessage.RelevantEnterpriseAccountSet);
            log.setOriginalState(from);
            log.setUpdateState(to);
            operationLogService.addOrDeleteDataAddLog(log);
        }
    }

    /**
     * 修改自定义纬度的值
     */
    private List<UpdateLogParams> updateSelfDimensionList(OrganizationDto orgDto) {
        Assert.notNull(orgDto, "orgDto null exception in updateSelfDimensionList");
        List<UpdateLogParams> logList = new ArrayList<>();
        DimensionExample example1 = new DimensionExample();
        example1.createCriteria().andIsActiveEqualTo(true);
        List<Dimension> dimensionList = dimensionMapper.selectByExample(example1);

        DimensionValueExample example2 = new DimensionValueExample();
        example2.createCriteria().andIsActiveEqualTo(true);
        List<DimensionValue> dimensionValueList = dimensionValueMapper.selectByExample(example2);

        DimensionValueOrgExample example3 = new DimensionValueOrgExample();
        example3.createCriteria().andOrganizationIDEqualTo(orgDto.getID());
        List<DimensionValueOrg> oldDimensionOrgList = dimensionValueOrgMapper.selectByExample(example3);
        List<DimensionValueOrgDto> newDimensionOrgList = orgDto.getDimensionValueOrgList();

        if (newDimensionOrgList == null) {
            newDimensionOrgList = new ArrayList<>();
        }

        if (oldDimensionOrgList.isEmpty() && newDimensionOrgList.isEmpty()) {
            return logList;
        }
        List<DimensionValueJoinDimensionDto> queryInfo = dimensionValueMapper.selectDimensionValueJoinDimension();

        List<DimensionValueOrg> updateDimensionOrgList = new ArrayList<>();
        List<DimensionValueOrg> addDimensionOrgList = new ArrayList<>();
        List<DimensionValueOrg> deleteDimensionOrgList = new ArrayList<>();

        for (Dimension dimension : dimensionList) {
            List<String> valueList = dimensionValueList.stream()
                    .filter(p -> Strings.equals(p.getDimensionID(), dimension.getID()))
                    .map(p -> p.getID().toLowerCase()).collect(Collectors.toList());

            DimensionValueOrg old = oldDimensionOrgList.stream()
                    .filter(p -> valueList.contains(p.getDimensionValueID().toLowerCase())).findFirst().orElse(null);

            DimensionValueOrgDto newObj = newDimensionOrgList.stream()
                    .filter(p -> valueList.contains(p.getDimensionValueID().toLowerCase())).findFirst().orElse(null);

            // 都没有值
            if (old == null && newObj == null) {
                continue;
            }

            // 两个都有值
            if (old != null && newObj != null) {
                old.setDimensionValueID(newObj.getDimensionID());
                updateDimensionOrgList.add(old);

                DimensionValueJoinDimensionDto info = queryInfo.stream()
                        .filter(sa -> Strings.equals(sa.getDimensionValueID(), old.getDimensionValueID())).findFirst()
                        .orElse(null);
                DimensionValueJoinDimensionDto newInfo = queryInfo.stream()
                        .filter(sa -> Strings.equals(sa.getDimensionValueID(), newObj.getDimensionValueID()))
                        .findFirst().orElse(null);
                // 修改
                UpdateLogParams updateLogParams = new UpdateLogParams();
                updateLogParams.setOperationAction(OperationAction.Update.value());
                updateLogParams.setComment("");
                updateLogParams.setOperateLogType(OperationLogOrganization.value());
                updateLogParams.setOperationModule(Organization.value());
                updateLogParams.setOperationObject(orgDto.getName());
                updateLogParams.setOperationContent(info == null ? "" : info.getDimensionName());
                updateLogParams.setOriginalState(info == null ? "" : info.getDimensionValue());
                updateLogParams.setUpdateState(newInfo == null ? "" : newInfo.getDimensionValue());
                logList.add(updateLogParams);
            }

            if (old != null && newObj == null) {
                // 需要删除
                deleteDimensionOrgList.add(old);
                DimensionValueJoinDimensionDto info = queryInfo.stream()
                        .filter(sa -> Strings.equals(sa.getDimensionValueID(), old.getDimensionValueID())).findFirst()
                        .orElse(null);

                UpdateLogParams updateLogParams = new UpdateLogParams();
                updateLogParams.setOperationAction(OperationAction.Update.value());
                updateLogParams.setComment("");
                updateLogParams.setOperateLogType(OperationLogOrganization.value());
                updateLogParams.setOperationModule(Organization.value());
                updateLogParams.setOperationObject(orgDto.getName());
                updateLogParams.setOperationContent(info == null ? "" : info.getDimensionName());
                updateLogParams.setOriginalState(info == null ? "" : info.getDimensionValue());
                updateLogParams.setUpdateState("");
                logList.add(updateLogParams);
            }

            if (old == null && newObj != null) {
                // 需要添加
                DimensionValueOrg dimensionValueOrg = new DimensionValueOrg();
                dimensionValueOrg.setDimensionValueID(newObj.getDimensionValueID());
                dimensionValueOrg.setOrganizationID(orgDto.getID());
                dimensionValueOrg.setID(CommonUtils.getUUID());
                addDimensionOrgList.add(dimensionValueOrg);

                DimensionValueJoinDimensionDto info = queryInfo.stream()
                        .filter(sa -> Strings.equals(sa.getDimensionValueID(), newObj.getDimensionValueID()))
                        .findFirst().orElse(null);
                UpdateLogParams updateLogParams = new UpdateLogParams();
                updateLogParams.setOperationAction(OperationAction.Update.value());
                updateLogParams.setComment("");
                updateLogParams.setOperateLogType(OperationLogOrganization.value());
                updateLogParams.setOperationModule(Organization.value());
                updateLogParams.setOperationObject(orgDto.getName());
                updateLogParams.setOperationContent(info == null ? "" : info.getDimensionName());
                updateLogParams.setOriginalState("");
                updateLogParams.setUpdateState(info == null ? "" : info.getDimensionValue());
                logList.add(updateLogParams);
            }
        }
        if (!addDimensionOrgList.isEmpty()) {
            addDimensionOrgList.stream().forEach(d -> dimensionValueOrgMapper.insert(d));
        }

        if (!updateDimensionOrgList.isEmpty()) {
            updateDimensionOrgList.stream().forEach(d -> dimensionValueOrgMapper.updateByPrimaryKey(d));
        }

        if (!deleteDimensionOrgList.isEmpty()) {
            deleteDimensionOrgList.stream().forEach(d -> dimensionValueOrgMapper.deleteByPrimaryKey(d.getID()));
        }
        return logList;
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    private OperationResultDto<Object> checkParentID(OrganizationDto orgDto) {
        if (Strings.equals(orgDto.getParentID(), orgDto.getID())) {
            return new OperationResultDto(false, OrganizationMessage.ParentIDCannotEqualsID);
        }

        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andIsActiveEqualTo(true);
        List<Organization> orgList = organizationMapper.selectByExample(example);
        List<Organization> subList = this.getSubOrganization(orgDto.getID(), orgList);
        Assert.notNull(subList, "subList null exception in checkParentID");
        if (subList.stream().anyMatch(sa -> Strings.equals(sa.getID(), sa.getParentID()))) {
            return new OperationResultDto(false, OrganizationMessage.ParentIDCannotBeSubID);
        }
        return new OperationResultDto(true);
    }

    /**
     * 获取所有子机构
     */
    private List<Organization> getSubOrganization(String orgid, List<Organization> orglist) {
        List<Organization> retList = new ArrayList<>();
        List<Organization> subList = orglist.stream().filter(sa -> Strings.equals(sa.getParentID(), orgid))
                .collect(Collectors.toList());

        if (!subList.isEmpty()) {
            subList.stream().forEach(s -> retList.add(s));
        }

        for (Organization item : subList) {
            List<Organization> tempList = getSubOrganization(item.getID(), orglist);
            if (!tempList.isEmpty()) {
                tempList.stream().forEach(t -> retList.add(t));
            }
        }
        return retList;
    }

    @Override
    public OrgGeneralInfoDto getGeneralInfo(String orgID) {
        return getOrgGeneralInfo(orgID);
    }

    @Override
    public List<DevTreeDto> getOrgIvhTreeList(Integer useType, String orgSetId) {
        //todo
        List<DevTreeDto> result = Lists.newArrayList();
        List<IvhTreeDto> orgDtoList = Lists.newArrayList();
        List<Organization> orgList;
        OrganizationExample orgExample = new OrganizationExample();

        if (useType < 2)
        {
            orgList = organizationMapper.selectByExample(orgExample);
        }
        else
        {
            EnterpriseAccountSetOrgExample tmpExample = new EnterpriseAccountSetOrgExample();
            EnterpriseAccountSetOrgExample.Criteria criteria = tmpExample.createCriteria();
            criteria.andEnterpriseAccountSetIDEqualTo(orgSetId);
            List<EnterpriseAccountSetOrg> enAccountSetOrgList = enterpriseAccountSetOrgMapper.selectByExample(tmpExample);
            if (CollectionUtils.isNotEmpty(enAccountSetOrgList)){
                List<String> orgIdList = enAccountSetOrgList.stream().map(EnterpriseAccountSetOrg::getOrganizationID)
                        .collect(toList());
                OrganizationExample.Criteria orgCriteria = orgExample.createCriteria();
                orgCriteria.andIDIn(orgIdList);
                orgList = organizationMapper.selectByExample(orgExample);
            }else {
                orgList = Lists.newArrayList();
            }
        }
        if (CollectionUtils.isNotEmpty(orgList)) {
            //find rootOrg
            List<Organization> rootOrgList;
            if (useType == 1) {
                rootOrgList = orgList.stream().filter(x -> StringUtils.isBlank(x.getParentID())).collect(toList());
            } else {
                rootOrgList = orgList.stream().filter(x -> StringUtils.isBlank(x.getParentID()) && x.getIsActive())
                        .collect(toList());
            }
            List<ServiceType> serviceList = Lists.newArrayList();

            for (Organization org: rootOrgList){
                IvhTreeDto root = new IvhTreeDto();
                root.setChildren(Lists.newArrayList());
                root.setLabel(org.getName());
                root.setValue(org);
                orgDtoList.add(generateIvhTreeDto(org,orgList,serviceList,useType));
            }

        }
        return orgDtoList.stream().map(this::devTreeDtoMapper).collect(toList());
    }

    private DevTreeDto devTreeDtoMapper(IvhTreeDto ivhTreeDto) {
        DevTreeDto dto = new DevTreeDto();
        dto.setText(ivhTreeDto.getLabel());
        List<IvhTreeDto> childList = ivhTreeDto.getChildren();
        if (CollectionUtils.isNotEmpty(childList)) {
            dto.setItems(Lists.newArrayList());
            for (IvhTreeDto ivh : childList) {
                dto.getItems().add(devTreeDtoMapper(ivh));
            }
        }
        dto.setId(ivhTreeDto.getId());
        dto.setData(ivhTreeDto);
        return dto;
    }

    private IvhTreeDto generateIvhTreeDto(Organization parentOrg, List<Organization> orgList,
                                          List<ServiceType> serviceList, int useType) {
        IvhTreeDto result = new IvhTreeDto();
        OrganizationDto parentOrgDto = new OrganizationDto();
        CommonUtils.copyProperties(parentOrg, parentOrgDto);
        if (null != parentOrg && StringUtils.isNotBlank(parentOrg.getID())) {
            parentOrgDto.setParentName(orgList.stream().filter(x -> StringUtils.equals(x.getID(), parentOrg.getParentID()))
                    .findFirst().map(x -> x.getName()).orElse(StringUtils.EMPTY));
        }

        List<Organization> subOrgList;
        if (useType == 1) {
            subOrgList = orgList.stream().filter(x -> StringUtils.equals(x.getParentID(), parentOrg.getID())).collect(toList());
        } else {
            subOrgList = orgList.stream().filter(x -> StringUtils.equals(x.getParentID(), parentOrg.getID()) && x.getIsActive())
                    .collect(toList());
        }
        parentOrgDto.setSubOrgs(Lists.newArrayList());
        List<IvhTreeDto> children = Lists.newArrayList();

        //create and add subtrees to the current node
        for (Organization subOrg : subOrgList) {
            children.add(generateIvhTreeDto(subOrg, orgList, serviceList, useType));
        }
        result.setLabel(parentOrgDto.getName());
        result.setValue(parentOrgDto);
        result.setChildren(children);

        return result;
    }

    private OrgGeneralInfoDto getOrgGeneralInfo(String orgID) {
        List<OrgGeneralInfoMiddleDto> query = organizationMapper.selectJoinToOrgGeneralInfo();
        List<OrgGeneralInfoMiddleDto> filter = query.stream().filter(x -> Strings.equals(x.getID(), orgID))
                .collect(Collectors.toList());
        if (filter.isEmpty()) {
            return null;
        }

        List<String> serviceList = filter.stream().map(OrgGeneralInfoMiddleDto::getServiceName).distinct()
                .collect(Collectors.toList());
        List<String> accountSetList = filter.stream().map(OrgGeneralInfoMiddleDto::getAccountSetName).distinct()
                .collect(Collectors.toList());
        OrgGeneralInfoMiddleDto row = filter.get(0);

        String areaName = "";
        String businessUnitName = "";

        if (row.getAreaName() == null) {
            areaName = getAreaNameByID(row.getAreaID());
        }

        if (row.getBusinessUnitName() == null) {
            businessUnitName = getBusinessUnitNameByID(row.getBusinessUnitID());
        }
        OrgGeneralInfoDto ret = new OrgGeneralInfoDto();
        ret.setOrgName(row.getOrgName());
        ret.setTaxPayerNumber(row.getTaxPayerNumber());
        ret.setAccountSetName(
                String.join(",", (accountSetList == null || accountSetList.isEmpty()) ? null : accountSetList));
        ret.setServiceName(String.join(",", (serviceList == null || serviceList.isEmpty()) ? null : serviceList));
        ret.setBusinessUnitName(row.getBusinessUnitName() == null ? businessUnitName : row.getBusinessUnitName());
        ret.setIndustryName(row.getIndustryName());
        ret.setAreaName(row.getAreaName() == null ? areaName : row.getAreaName());
        ret.setUserList(new ArrayList<>());
        return ret;
    }

    private String getBusinessUnitNameByID(String businessUnitID) {

        BusinessUnit businessUnit = businessUnitMapper.selectByPrimaryKey(businessUnitID);
        return businessUnit != null ? businessUnit.getName() : null;
    }

    private String getAreaNameByID(String areaID) {
        Area area = areaMapper.selectByPrimaryKey(areaID);
        return area != null ? area.getName() : null;
    }

}
