package pwc.taxtech.atms.service.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.constant.enums.TbImportType;
import pwc.taxtech.atms.dao.PeriodInfoMapper;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.taxadmin.PeriodInfoDto;
import pwc.taxtech.atms.entitiy.PeriodInfo;
import pwc.taxtech.atms.entitiy.PeriodInfoExample;
import pwc.taxtech.atms.service.ProjectInfoService;

import java.util.Date;
import java.util.List;
import java.util.UUID;

import static pwc.taxtech.atms.constant.Constant.FIRST_OR_DEFAULT;

@Service
public class ProjectInfoServiceImpl implements ProjectInfoService {
    private static final Logger LOGGER = LoggerFactory.getLogger(ProjectServiceImpl.class);
    @Autowired
    private PeriodInfoMapper periodInfoMapper;

    @Override
    public OperationResultDto<Boolean> isProjectImportedData(String projectId, Integer serviceType, Integer importTypeId) {
        try {
            PeriodInfoExample example = new PeriodInfoExample();
            PeriodInfoExample.Criteria criteria = example.createCriteria().andProjectIDEqualTo(projectId).andImportTypeEqualTo(importTypeId);
            if (serviceType != null) criteria.andServiceTypeEqualTo(serviceType);

            List<PeriodInfo> piList = periodInfoMapper.selectByExample(example);

            OperationResultDto<Boolean> ord = new OperationResultDto<>();
            ord.setResult(piList != null & !piList.isEmpty());
            ord.setData(ord.getResult());
            return ord;
        } catch (Exception e) {
            LOGGER.warn("IsProjectImportedData method error", e);
            OperationResultDto<Boolean> ord = new OperationResultDto<>();
            ord.setResult(false);
            ord.setData(false);
            return ord;
        }
    }

    @Override
    public OperationResultDto<Boolean> isProjectImportedData(String projectId, Integer importTypeId) {
        return isProjectImportedData(projectId, null, importTypeId);
    }

    @Override
    public OperationResultDto<List<PeriodInfoDto>> getProjectImportType(String projectId, List<Integer> periods, Integer serviceTypeId, String id) {
        return null;
    }

    @Override
    public Integer getImportType(String projectID, Integer periodID, String userID, Integer serviceType) {
        PeriodInfoExample example = new PeriodInfoExample();
        example.createCriteria().andProjectIDEqualTo(projectID).andProjectIDEqualTo(projectID).andServiceTypeEqualTo(serviceType);

        List<PeriodInfo> periodInfoList =periodInfoMapper.selectByExample(example);
        if(periodInfoList==null || periodInfoList.size()==0){
            PeriodInfo periodInfo = new PeriodInfo();
            periodInfo.setID(UUID.randomUUID().toString());
            periodInfo.setProjectID(projectID);
            periodInfo.setPeriod(periodID);
            periodInfo.setStatus(0);
            periodInfo.setCreatorID(userID);
            periodInfo.setImportType(TbImportType.UnImported.getCode());
            periodInfo.setServiceType(serviceType);
            periodInfo.setCreateTime(new Date());
            periodInfo.setUpdateTime(new Date());

            periodInfoMapper.insert(periodInfo);
            return TbImportType.UnImported.getCode();
        }else {
          return   periodInfoList.get(FIRST_OR_DEFAULT).getImportType();
        }

    }
}
