package pwc.taxtech.atms.service.impl;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.POIUtil;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.common.message.TemplateMessage;
import pwc.taxtech.atms.constant.enums.TemplateGroupType;
import pwc.taxtech.atms.dao.CellTemplateConfigMapper;
import pwc.taxtech.atms.dao.CellTemplateMapper;
import pwc.taxtech.atms.dao.TemplateGroupMapper;
import pwc.taxtech.atms.dao.TemplateMapper;
import pwc.taxtech.atms.dao.dao.CellTemplateConfigDao;
import pwc.taxtech.atms.dao.dao.CellTemplateDao;
import pwc.taxtech.atms.dao.dao.TemplateDao;
import pwc.taxtech.atms.dao.dao.TemplateGroupDao;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.TemplateGroupDto;
import pwc.taxtech.atms.entitiy.*;
import pwc.taxtech.atms.service.FileSystemService;
import pwc.taxtech.atms.service.TemplateGroupService;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class TemplateGroupServiceImpl extends AbstractService implements TemplateGroupService {
    @Autowired
    private FileSystemService fileSystemService;
    @Autowired
    private TemplateGroupDao templateGroupDao;
    @Autowired
    private TemplateGroupMapper templateGroupMapper;
    @Autowired
    private TemplateMapper templateMapper;
    @Autowired
    private TemplateDao templateDao;
    @Autowired
    private CellTemplateMapper cellTemplateMapper;
    @Autowired
    private CellTemplateDao cellTemplateDao;
    @Autowired
    private CellTemplateConfigMapper cellTemplateConfigMapper;
    @Autowired
    private CellTemplateConfigDao cellTemplateConfigDao;

    @Override
    public List<TemplateGroupDto> get() {
        List<TemplateGroup> templateGroups = templateGroupMapper.selectByExample(new TemplateGroupExample());
        List<TemplateGroupDto> templateGroupDtos = new ArrayList<>();
        for (TemplateGroup templateGroup : templateGroups) {
            TemplateGroupDto templateGroupDto = new TemplateGroupDto();
            CommonUtils.copyProperties(templateGroup, templateGroupDto);
            templateGroupDtos.add(templateGroupDto);
        }
        return templateGroupDtos;
    }

    @Override
    public List<TemplateGroupDto> get(int serviceTypeID, Integer taxPayType, String industryID) {
        TemplateGroupExample example = new TemplateGroupExample();

        TemplateGroupExample.Criteria criteria = example.createCriteria();

        criteria.andServiceTypeIdEqualTo(String.valueOf(serviceTypeID)).andGroupTypeEqualTo(TemplateGroupType.TaxReturn.getCode());

        if (taxPayType != null) {
            criteria.andPayTaxTypeEqualTo(TemplateGroupType.TaxReturn.getCode());
        }

        List<TemplateGroup> templateGroups = templateGroupMapper.selectByExample(example);
        List<TemplateGroupDto> templateGroupDtos = new ArrayList<>();
        for (TemplateGroup templateGroup : templateGroups) {
            TemplateGroupDto dto = new TemplateGroupDto();
            CommonUtils.copyProperties(templateGroup, dto);
            templateGroupDtos.add(dto);
        }
        return templateGroupDtos;
    }

    @Override
    public OperationResultDto<Object> updateTemplateGroupName(TemplateGroupDto templateGroupDto) {
        TemplateGroup entity = templateGroupMapper.selectByPrimaryKey(templateGroupDto.getId());
        TemplateGroupExample example = new TemplateGroupExample();
        example.createCriteria().andNameEqualTo(templateGroupDto.getName()).andIdNotEqualTo(templateGroupDto.getId()).andServiceTypeIdEqualTo(entity.getServiceTypeId()).andIndustryIdsEqualTo(entity.getIndustryIds()).andPayTaxTypeEqualTo(entity.getPayTaxType());
        List<TemplateGroup> templateGroups = templateGroupMapper.selectByExample(example);
        if (!templateGroups.isEmpty()) {
            OperationResultDto<Object> result = new OperationResultDto<>();
            result.setResult(false);
            result.setResultMsg(TemplateGroupMessage.TEMPLATE_GROUP_NAME_EXIST);
        }
        entity.setName(templateGroupDto.getName());
        templateGroupMapper.updateByPrimaryKey(entity);
        OperationResultDto<Object> result = new OperationResultDto<>();
        result.setResult(true);
        return result;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,rollbackFor = Exception.class)
    public OperationResultDto<Object> deleteTemplateGroup(TemplateGroupDto templateGroupDto) {
        OperationResultDto<Object> result = new OperationResultDto<>();
        TemplateGroup templateGroupDb = templateGroupMapper.selectByPrimaryKey(templateGroupDto.getId());

        if (templateGroupDb.getIsSystemType()) {
            result.setResult(false);
            result.setResultMsg(TemplateGroupMessage.SYSTEM_TYPE_CANNOT_DELETE);
            return result;
        }

        List<String> userOranizationNameList = organizationServiceTemplateGroupMapper.getOrgnizationServiceTemplateGroupOrgNames(templateGroupDto.getId());
        if (userOranizationNameList != null && !userOranizationNameList.isEmpty()) {
            result.setResult(false);
            result.setResultMsg(TemplateGroupMessage.ORGANIZATION_USED_TEMPLATE_GROUP);
            return result;
        }

        List<Template> allTemplateDbList = templateMapper.selectByExample(new TemplateExample());
        List<Template> templateDbList = allTemplateDbList.stream().filter(a -> a.getTemplateGroupId().equals(templateGroupDb.getId())).collect(Collectors.toList());
        List<String> pathList = new ArrayList<>();

        for (Template templateDb : templateDbList) {
            boolean anySameCodeExists = allTemplateDbList.stream().anyMatch(a -> a.getCode()==templateDb.getCode() && a.getName()==templateDb.getCode());

            if (!anySameCodeExists) {
                pathList.add((templateDb.getPath()));
            }
            deleteTemplate(templateDb);
        }
        templateGroupMapper.deleteByPrimaryKey(templateGroupDb.getId());

        result.setResult(true);
        result.setData(pathList);
        return result;
    }

    private void deleteTemplate(Template templateDb) {
        cellTemplateConfigMapper.deleteCellTemplateConfigByCellTemplate(templateDb.getId());
        keyValueReferenceMapper.deleteKeyValueReferenceByCellTemplate(templateDb.getId());
        CellTemplateExample example = new CellTemplateExample();
        example.createCriteria().andReportTemplateIdEqualTo(templateDb.getId());
        cellTemplateMapper.deleteByExample(example);
    }

    public class TemplateGroupMessage {
        static final String TEMPLATE_GROUP_NAME_EXIST = "TemplateGroupNameExist";
        static final String SYSTEM_TYPE_CANNOT_DELETE = "SystemTypeCannotDelete";
        static final String ORGANIZATION_USED_TEMPLATE_GROUP = "OrganizationUsedTemplateGroup";
    }

    @Override
    public List<String> getSheetNameList(MultipartFile file) {
        try {
            Workbook workbook = WorkbookFactory.create(file.getInputStream());
            List<String> nameList = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                nameList.add(workbook.getSheetName(i));
            }
            return nameList;
        } catch (Exception e) {
            logger.error("getSheetNameList error.", e);
        }
        return Collections.emptyList();
    }

    @Transactional
    @Override
    public void importTemplateGroupExcelFile(MultipartFile file, TemplateGroupDto templateGroupDto) throws ServiceException {
        List<String> sheetNameList = templateGroupDto.getSheetNameList();
        if (CollectionUtils.isEmpty(sheetNameList)) {
            throw new ServiceException(ErrorMessage.NoSelectSheet);
        }
        try {
            List<TemplateGroup> groupList = templateGroupDao.getByGroupName(templateGroupDto.getName());
            if (CollectionUtils.isNotEmpty(groupList)) {
                throw new ServiceException(TemplateMessage.TemplateGroupNameExist);
            }
            InputStream inputStream = file.getInputStream();
            String fileName = file.getOriginalFilename();
            Workbook workbook = WorkbookFactory.create(inputStream);
            List<Template> filePathList = Lists.newArrayList();
            Long templateGroupId = distributedIDService.nextId();
            Date now = new Date();
            TemplateGroup templateGroup = new TemplateGroup();
            CommonUtils.copyProperties(templateGroupDto, templateGroup);
            templateGroup.setId(templateGroupId);
            templateGroup.setGroupType(1);//todo 整理枚举
            templateGroup.setCopyFrom(0L);
            templateGroup.setIsSystemType(false);
            templateGroup.setUpdateTime(now);
            templateGroupMapper.insertSelective(templateGroup);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                String sheetName = workbook.getSheetName(i);
                String newName = sheetName + CommonUtils.getUUID() + POIUtil.getFileSuffix(fileName).get();
                Sheet sheet = workbook.getSheetAt(i);
                Optional<Workbook> optional = POIUtil.cloneNewSheet(sheet, fileName);
                if (!optional.isPresent()) {
                    throw new ServiceException(ErrorMessage.SystemError);
                }
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                optional.get().write(bos);
                String tmpPath = fileSystemService.uploadUserTemplate(newName, new ByteArrayInputStream(bos.toByteArray()));
                String[] arr = sheetName.split("_");
                String name = arr.length >= 2 ? arr[1] : arr[0];
                Template template = new Template();
                template.setCode(arr[0]);
                template.setCreateTime(now);
                template.setId(distributedIDService.nextId());
                template.setIsActiveAssociation(true);
                template.setIsSystemType(false);
                template.setName(name);
                template.setOrderIndex(i + 1);
                template.setPath(tmpPath);
//                template.setReportType(tmpPath);
                template.setTemplateGroupId(templateGroupId);
                filePathList.add(template);
                templateMapper.insertSelective(template);
                List<CellTemplate> cellTemplateList = Lists.newArrayList();
                List<CellTemplateConfig> cellTemplateConfigList = Lists.newArrayList();
                for (int r = sheet.getFirstRowNum(); r <= sheet.getLastRowNum(); r++) {
                    Row row = sheet.getRow(r);
                    for (int c = row.getFirstCellNum(); c <= row.getLastCellNum(); c++) {
                        Cell cell = row.getCell(c);
                        if (cell == null) {
                            continue;//todo cell == null 如何处理
                        }
                        CellTemplate cellTemplate = new CellTemplate();
                        cellTemplate.setColumnIndex(c);
                        cellTemplate.setCreateTime(now);
                        cellTemplate.setUpdateTime(now);
                        cellTemplate.setRowIndex(r);
                        cellTemplate.setRowName(StringUtils.EMPTY);
                        cellTemplate.setColumnName(StringUtils.EMPTY);
                        cellTemplate.setComment(StringUtils.EMPTY);
                        cellTemplate.setCopyFromId(0L);
                        cellTemplate.setCreateBy("Admin");
                        cellTemplate.setUpdateBy("Admin");
                        cellTemplate.setDataType(0);
                        cellTemplate.setReportTemplateId(template.getId());
                        cellTemplate.setId(distributedIDService.nextId());
                        if (cell.getCellComment() != null) {
                            cellTemplate.setComment(cell.getCellComment().getString().getString());
                        }
                        cellTemplate.setIsReadOnly(cell.getCellStyle().getLocked());
                        cellTemplateList.add(cellTemplate);

                        CellTemplateConfig config = new CellTemplateConfig();
                        config.setId(distributedIDService.nextId());
                        config.setCellTemplateId(cellTemplate.getId());
                        config.setReportTemplateId(template.getId());
                        config.setDataSourceType(1);//todo 枚举
                        config.setFormula(POIUtil.getCellFormulaString(cell));
                        //config.setFormula(cell.getCellFormula());
//                        config.setFormulaDataSource(); //todo KV相关
                        config.setUpdateTime(now);
                        config.setUpdateBy(authUserHelper.getCurrentUserID());
                        cellTemplateConfigList.add(config);
                    }
                }
                List<List<CellTemplate>> tmpList = CommonUtils.subListWithLen(cellTemplateList, CommonUtils.BATCH_NUM);
                tmpList.forEach(list -> cellTemplateMapper.batchInsert2(list));
                List<List<CellTemplateConfig>> tmpConfigList = CommonUtils.subListWithLen(cellTemplateConfigList, CommonUtils.BATCH_NUM);
                tmpConfigList.forEach(list -> cellTemplateConfigMapper.batchInsert(list));
            }

        } catch (Exception e) {
            logger.error("importTemplateGroupExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    @Transactional
    @Override
    public void importTemplateExcelFile(MultipartFile file, Long templateGroupID, String reportType, String sheetList) throws ServiceException {
        if (null == file) {
            throw new ServiceException(ErrorMessage.NoFile);
        }
        try {
            if (StringUtils.isBlank(sheetList)) {
                throw new ServiceException(ErrorMessage.NoSelectSheet);
            }
            List<String> sheetNameList = JSON.parseArray(sheetList, String.class);
            if (CollectionUtils.isEmpty(sheetNameList)) {
                throw new ServiceException(ErrorMessage.NoSelectSheet);
            }
            InputStream inputStream = file.getInputStream();
            String fileName = file.getOriginalFilename();
            Workbook workbook = WorkbookFactory.create(inputStream);
            List<Template> filePathList = Lists.newArrayList();
            Date now = new Date();
            for (int i = 0; i <= workbook.getNumberOfSheets(); i++) {
                String sheetName = workbook.getSheetName(i);
                String newName = sheetName + CommonUtils.getUUID() + POIUtil.getFileSuffix(fileName).get();
                Sheet sheet = workbook.getSheetAt(i);
                Optional<Workbook> optional = POIUtil.cloneNewSheet(sheet, fileName);
                if (!optional.isPresent()) {
                    throw new ServiceException(ErrorMessage.SystemError);
                }
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                optional.get().write(bos);
                String tmpPath = fileSystemService.uploadUserTemplate(newName, new ByteArrayInputStream(bos.toByteArray()));
                String[] arr = sheetName.split("_");
                String name = arr.length >= 2 ? arr[1] : arr[0];
                Template template = new Template();
                template.setCode(arr[0]);
                template.setCreateTime(now);
                template.setId(distributedIDService.nextId());
                template.setIsActiveAssociation(true);
                template.setIsSystemType(false);
                template.setName(name);
                template.setOrderIndex(i + 1);
                template.setPath(tmpPath);
                template.setReportType(StringUtils.isBlank(reportType) ? null : Integer.valueOf(reportType));
                template.setTemplateGroupId(templateGroupID);
                filePathList.add(template);
                templateMapper.insertSelective(template);
                List<CellTemplate> cellTemplateList = Lists.newArrayList();
                List<CellTemplateConfig> cellTemplateConfigList = Lists.newArrayList();
                for (int r = sheet.getFirstRowNum(); i <= sheet.getLastRowNum(); i++) {
                    Row row = sheet.getRow(r);
                    for (int c = row.getFirstCellNum(); c <= row.getLastCellNum(); c++) {
                        Cell cell = row.getCell(c);
                        if (cell == null) {
                            continue;//todo cell == null 如何处理
                        }
                        CellTemplate cellTemplate = new CellTemplate();
                        cellTemplate.setColumnIndex(c);
                        cellTemplate.setCreateTime(now);
                        cellTemplate.setUpdateTime(now);
                        cellTemplate.setRowIndex(r);
                        cellTemplate.setReportTemplateId(template.getId());
                        cellTemplate.setId(distributedIDService.nextId());
                        if (cell.getCellComment() != null) {
                            cellTemplate.setComment(cell.getCellComment().getString().getString());
                        }
                        cellTemplate.setIsReadOnly(cell.getCellStyle().getLocked() ? true : false);
                        cellTemplateList.add(cellTemplate);

                        CellTemplateConfig config = new CellTemplateConfig();
                        config.setId(distributedIDService.nextId());
                        config.setCellTemplateId(cellTemplate.getId());
                        config.setReportTemplateId(template.getId());
                        config.setDataSourceType(1);//todo 枚举
                        config.setFormula(POIUtil.getCellFormulaString(cell));
//                        config.setFormulaDataSource(); //todo KV相关
                        config.setUpdateTime(now);
                        config.setUpdateBy(authUserHelper.getCurrentUserID());
                        cellTemplateConfigList.add(config);
                    }
                }
                List<List<CellTemplate>> tmpList = CommonUtils.subListWithLen(cellTemplateList, CommonUtils.BATCH_NUM);
                tmpList.forEach(list -> cellTemplateMapper.batchInsert2(list));
                List<List<CellTemplateConfig>> tmpConfigList = CommonUtils.subListWithLen(cellTemplateConfigList, CommonUtils.BATCH_NUM);
                tmpConfigList.forEach(list -> cellTemplateConfigMapper.batchInsert(list));
            }
        } catch (Exception e) {
            logger.error("importTemplateExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    @Transactional
    @Override
    public void addTemplateGroup(TemplateGroupDto templateGroupDto) throws ServiceException {
        List<TemplateGroup> groupList = templateGroupDao.getByGroupName(templateGroupDto.getName());
        if (CollectionUtils.isNotEmpty(groupList)) {
            throw new ServiceException(TemplateMessage.TemplateGroupNameExist);
        }
        try {
            String uid = authUserHelper.getCurrentUserID();
            TemplateGroup templateGroup = templateGroupMapper.selectByPrimaryKey(templateGroupDto.getCopyFrom());
            Long newGroupID = distributedIDService.nextId();
            Long groupID = templateGroup.getId();
            Date now = new Date();
            templateGroup.setCopyFrom(groupID);
            templateGroup.setId(newGroupID);
            templateGroup.setCreateTime(now);
            templateGroup.setUpdateTime(now);
            templateGroupMapper.insertSelective(templateGroup);
            List<Template> templateList = templateDao.getByGroupID(groupID);
            templateList.stream().forEach(item -> {
                Long itemID = item.getId();
                Long newItemID = distributedIDService.nextId();
                item.setId(newItemID);
                item.setCreateTime(now);
                item.setUpdateTime(now);
                templateMapper.insertSelective(item);
                List<CellTemplate> cellTemplateList = cellTemplateDao.getByTemplateID(itemID);
                Map<Long, Long> idMapper = Maps.newHashMap();
                List<List<CellTemplate>> tmpList = CommonUtils.subListWithLen(cellTemplateList.stream().map(cell -> {
                    Long cellID = cell.getId();
                    Long newCellID = distributedIDService.nextId();
                    cell.setCopyFromId(cellID);
                    cell.setId(newCellID);
                    cell.setReportTemplateId(newItemID);
                    cell.setCreateTime(now);
                    cell.setUpdateTime(now);
                    idMapper.put(cellID, newCellID);
                    return cell;
                }).collect(Collectors.toList()), CommonUtils.BATCH_NUM);
                tmpList.forEach(list -> cellTemplateMapper.batchInsert(list));

                List<CellTemplateConfig> configList = cellTemplateConfigDao.getByTemplateID(itemID);
                List<List<CellTemplateConfig>> tmpConfigList = CommonUtils.subListWithLen(configList.stream().map(cell -> {
                    Long newCellID = distributedIDService.nextId();
                    cell.setCreateBy(uid);
                    cell.setUpdateBy(uid);
                    cell.setId(newCellID);
                    cell.setReportTemplateId(newItemID);
                    cell.setCellTemplateId(idMapper.get(cell.getCellTemplateId()));
                    cell.setCreateTime(now);
                    cell.setUpdateTime(now);
                    return cell;
                }).collect(Collectors.toList()), CommonUtils.BATCH_NUM);
                tmpConfigList.forEach(list -> cellTemplateConfigMapper.batchInsert(list));
            });
        } catch (Exception e) {
            logger.error("addTemplateGroup error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }

    }

}
