package pwc.taxtech.atms.vat.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.constant.enums.EnumValidationType;
import pwc.taxtech.atms.dto.FieldsMapper;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.CustomsInvoiceDto;
import pwc.taxtech.atms.vat.dao.ValidationInfoMapper;
import pwc.taxtech.atms.vat.entity.CustomsInvoice;
import pwc.taxtech.atms.vat.entity.CustomsInvoiceExample;
import pwc.taxtech.atms.vat.entity.ValidationInfo;
import pwc.taxtech.atms.vat.entity.ValidationInfoExample;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

@Service
public class CustomsInvoiceServiceImpl extends VatAbstractService {
    @Autowired
    private ValidationInfoMapper validationInfoMapper;

    public List<CustomsInvoiceDto> getCustomsInvoicesByPeriodIds(int fromPeriod, int toPeriod) {
        CustomsInvoiceExample example = new CustomsInvoiceExample();
        example.setOrderByClause(" PayNum");
        example.createCriteria().andPeriodIdBetween(fromPeriod, toPeriod);
        List<CustomsInvoiceDto> result = new ArrayList<>();
        customsInvoiceMapper.selectByExample(example).forEach(a -> {
            CustomsInvoiceDto dto = new CustomsInvoiceDto();
            CommonUtils.copyProperties(a, dto);
            result.add(dto);
        });

        int seqNo = 1;
        DecimalFormat df = new DecimalFormat("#.00");
        for (CustomsInvoiceDto dto : result) {
            dto.setSeqNo(seqNo++);
            if (dto.getInvoiceAmount() != null) {
                dto.setInvoiceAmount(new BigDecimal(df.format(dto.getInvoiceAmount())));
            }
            if (dto.getInvoiceTaxAmount() != null) {
                dto.setInvoiceTaxAmount(new BigDecimal(df.format(dto.getInvoiceTaxAmount())));
            }
        }

        return result;
    }

    public List<CustomsInvoiceDto> getCustomsInvoicesByPeriodId(Integer periodId) {
        CustomsInvoiceExample example = new CustomsInvoiceExample();
        example.createCriteria().andPeriodIdEqualTo(periodId);

        List<CustomsInvoice> result = customsInvoiceMapper.selectByExample(example);
        result.sort(Comparator.comparing(CustomsInvoice::getPayNum));
        int seqNo = 1;
        List<CustomsInvoiceDto> dtos = new ArrayList<>(result.size());
        for (CustomsInvoice ci : result) {
            CustomsInvoiceDto dto = new CustomsInvoiceDto();

            try {
                FieldsMapper.map(ci, dto);
                dto.setSeqNo(seqNo++);

                dto.setInvoiceAmount(ci.getInvoiceAmount().setScale(2, BigDecimal.ROUND_HALF_UP));
                dto.setInvoiceTaxAmount(ci.getInvoiceTaxAmount().setScale(2, BigDecimal.ROUND_HALF_UP));
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }


        return dtos;
    }

    public OperationResultDto<List<ValidationInfo>> getCustomsInvoicesValidates(Integer periodId) {
        ValidationInfoExample example = new ValidationInfoExample();
        List<ValidationInfo> result = null;
        if (periodId == null) {
            example.createCriteria().andImportTypeIdEqualTo(EnumValidationType.CustomInvoice.getCode());
            result = validationInfoMapper.selectByExample(example);
        } else {
            example.createCriteria().andImportTypeIdEqualTo(EnumValidationType.CustomInvoice.getCode()).andPeriodIdEqualTo(periodId);
            result.addAll(validationInfoMapper.selectByExample(example));
            example.createCriteria().andImportTypeIdEqualTo(EnumValidationType.CustomInvoice.getCode()).andPeriodIdIsNull();
            result.addAll(validationInfoMapper.selectByExample(example));
        }

        OperationResultDto<List<ValidationInfo>> resultOpt = new OperationResultDto<>();
        resultOpt.setResult(true);
        resultOpt.setResultMsg("");
        resultOpt.setData(result);

        return resultOpt;
    }
}
