package pwc.taxtech.atms.vat.service.impl;

import org.apache.xmlbeans.impl.store.Path;
import org.reflections.util.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import pwc.taxtech.atms.constant.enums.EnumModule;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.Calendar;
import java.util.Date;
import java.util.UUID;

import static pwc.taxtech.atms.constant.Constant.CURRENT_USER_UPLOAD_FOLDER;
import static pwc.taxtech.atms.constant.Constant.UPLOAD_FOLDER_NAME;

@Service
public class FileUploadAdapter {
    private final static Logger LOGGER = LoggerFactory.getLogger(FileUploadAdapter.class);

    @Value("${max_file_length}")
    private String maxFileSize;

    static final String FILE_NAME = "filename";
    public static final String TEMP_FILE_NAME = "tempFileName";
    public static final String CHUNK_NUMBER = "_chunkNumber";
    public static final String CHUNK_SIZE = "_chunkSize";
    public static final String TOTAL_SIZE = "_totalSize";

    private boolean containsSubFolder = true;//TODO this should not use val should use parameter(neo)

    public String getLastFilePathByFileName(String tempFileName) {
        String uploadFolder = combine(CURRENT_USER_UPLOAD_FOLDER, getSubFolder());
        String fileFullPath = combine(uploadFolder, tempFileName);

        if (new File(fileFullPath).exists()) {
            return fileFullPath;
        }

        return null;
    }

    static class FileParamBean {

        String fileName;
        String tempFileName;
        long chunkNumber;
        long chunkSize;
        long totalSize;
        boolean isCanle;
        boolean result;
        boolean isLastChunk;
        boolean isFirsChunk;
    }

    public ResponseEntity upload(MultipartHttpServletRequest request, EnumModule trailBalance, UUID userIdForUpload) {
        if (request.getFileMap().size() <= 0) return ResponseEntity.badRequest().body("NoFile");
        FileParamBean paramBean = getQueryStringParameters(request);
        if (!paramBean.result) return ResponseEntity.badRequest().body("ParametersInvalid");
        if (paramBean.totalSize > Long.valueOf(maxFileSize))
            return ResponseEntity.badRequest().body("InvalidFileSize");
        String baseFolder = prepareFolder();
        if (Utils.isEmpty(baseFolder)) return ResponseEntity.badRequest().body("PrepareFolderError");

        String tempPath = String.format("%s" + File.separator + "%s", baseFolder, paramBean.tempFileName);
        String completePath = String.format("%s" + File.separator + "%s", baseFolder, paramBean.tempFileName);

        if (!paramBean.isCanle) {
            if (paramBean.isFirsChunk) {
                LOGGER.debug("First chunk arrived at webservice");
                File tempFile = new File(tempPath);
                File completeFile = new File(completePath);

                if (tempFile.exists()) tempFile.delete();
                if (completeFile.exists()) completeFile.delete();
            }

            try (FileOutputStream fos = new FileOutputStream(new File(tempPath));
                 InputStream inputStream = request.getFileMap().get("file").getInputStream();) {

                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer, 0, buffer.length)) > 0) {
                    fos.write(buffer, 0, bytesRead);
                }

            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

            if (paramBean.isLastChunk) {
                LOGGER.debug("Last chunk arrived");
                File completeFile = new File(completePath);
                File tempFile = new File(tempPath);

                tempFile.renameTo(completeFile);
                LOGGER.info("{} upload to temp folder sucess", paramBean.fileName);
            }
        } else {
            File tempFile = new File(tempPath);
            File completeFile = new File(completePath);

            if (tempFile.exists()) tempFile.delete();
            if (completeFile.exists()) completeFile.delete();
        }
        return ResponseEntity.created(URI.create("")).body(paramBean.tempFileName);

    }

    private FileParamBean getQueryStringParameters(MultipartHttpServletRequest request) {
        FileParamBean param = new FileParamBean();
        param.fileName = request.getParameter(FILE_NAME);
        param.tempFileName = request.getParameter(TEMP_FILE_NAME);
        param.chunkNumber = Utils.isEmpty(request.getParameter(CHUNK_NUMBER)) ? -1 : Long.parseLong(
                request.getParameter(CHUNK_NUMBER));
        param.chunkSize = Utils.isEmpty(request.getParameter(CHUNK_SIZE)) ? -1 : Long.parseLong(
                request.getParameter(CHUNK_SIZE));
        param.totalSize = Utils.isEmpty(request.getParameter(TOTAL_SIZE)) ? -1 : Long.parseLong(
                request.getParameter(TOTAL_SIZE));
        param.result = (param.chunkNumber != -1L) && (param.chunkSize != -1L) || (param.totalSize != -1L);
        if (!param.result) return param;

        param.isLastChunk = (param.chunkNumber + 1) * param.chunkSize >= param.totalSize;
        param.isFirsChunk = param.chunkNumber == 0;
        param.result = true;
        return param;
    }

    private String prepareFolder() {
        String uploadFolder = CURRENT_USER_UPLOAD_FOLDER;
        if (containsSubFolder) {
            uploadFolder = combine(uploadFolder, getSubFolder());
            File file = new File(uploadFolder);

            if (!file.exists()) file.mkdirs();
        }

        return uploadFolder;
    }

    private String getSubFolder() {
        Calendar now = Calendar.getInstance();
        now.setTime(new Date());
        return String.format("%s" + File.separator + "%s", now.get(Calendar.MONTH), now.get(Calendar.DATE));
    }

    private String combine(String parent, String child) {
        return String.format("%s" + File.separator + "%s", parent, child);
    }
}
