package pwc.taxtech.atms.vat.service.impl;

import com.alibaba.fastjson.JSON;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.constant.VatErpImportValidation;
import pwc.taxtech.atms.constant.enums.ErpCheckType;
import pwc.taxtech.atms.constant.enums.ImageType;
import pwc.taxtech.atms.dto.FieldsMapper;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.DataCheckDto;
import pwc.taxtech.atms.dto.vatdto.VoucherAccountMto;
import pwc.taxtech.atms.dto.vatdto.VoucherBalanceDto;
import pwc.taxtech.atms.dto.vatdto.VoucherDto;
import pwc.taxtech.atms.vat.dao.ValidationInfoMapper;
import pwc.taxtech.atms.vat.dao.VoucherMapper;
import pwc.taxtech.atms.vat.entity.ValidationInfo;
import pwc.taxtech.atms.vat.entity.ValidationInfoExample;
import pwc.taxtech.atms.vat.entity.Voucher;
import pwc.taxtech.atms.vat.entity.VoucherExample;
import pwc.taxtech.atms.vat.service.FinanceDataImportService;
import sun.misc.BASE64Encoder;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.constant.Constant.*;
import static pwc.taxtech.atms.constant.VATCommon.ReValidate;
import static pwc.taxtech.atms.constant.VATCommon.ValidationExisted;
import static pwc.taxtech.atms.constant.enums.EnumValidationType.ErpBasicCheck;
import static pwc.taxtech.atms.constant.enums.ImageType.Alert;

@Service
public class FinanceDataImportServiceImpl implements FinanceDataImportService {
    @Autowired
    private ValidationInfoMapper validationInfoMapper;
    @Autowired
    private VoucherMapper voucherMapper;

    @Override
    public OperationResultDto<List<DataCheckDto>> getBasicValidationResults(String userId) {
        Integer number = 1;
        List<DataCheckDto> results = new ArrayList<>();
        OperationResultDto<List<DataCheckDto>> operationResult = new OperationResultDto<>();

        List<ValidationInfo> dtos = getValidationData(ErpBasicCheck.getCode());
        if (dtos != null && dtos.size() > 0) {
            dtos.forEach(b -> {
                DataCheckDto dataCheckDto = new DataCheckDto();

                dataCheckDto.setDataCheckType(b.getValidationResult());
                dataCheckDto.setImageTypeId(b.getErrorLevel() != null ? b.getErrorLevel() : -1);
                dataCheckDto.setTips(b.getValidationTips());
                dataCheckDto.setErrorCount(b.getNumber() != null ? b.getNumber() : -1);
                dataCheckDto.setErpCheckTypeId(b.getErpCheckTypeId() != null ? b.getErpCheckTypeId() : -1);
                dataCheckDto.setValidationDetails(b.getValidationDetails());

                results.add(dataCheckDto);

                operationResult.setResultMsg(ValidationExisted);

            });
        } else {
            List<VoucherAccountMto> comAcctChks = validationInfoMapper.getCompanyBalanceAcctChecks();
            List<VoucherAccountMto> vAcctChks = validationInfoMapper.getVoucherAccountChecks();
            List<VoucherBalanceDto> vDebCrdSingleChk = getSingleVoucherCheck();

            List<VoucherDto> vEmptyItem = getVoucherEmptyCheck(true, false, false, false, false);
            List<VoucherDto> vEmptyGroup = getVoucherEmptyCheck(false, false, true, false, false);
            List<VoucherDto> vEmptyAcctCode = getVoucherEmptyCheck(false, false, false, true, false);
            List<VoucherDto> vEmptySummary = getVoucherEmptyCheck(false, false, false, false, true);

            if (comAcctChks.size() == 0) {
                results.add(setDataCheckItem(ImageType.Success, VatErpImportValidation.Const_ComBalanceAcctValidate,
                        0, "", ErpCheckType.Success));
            } else {
                results.add(setDataCheckItem(ImageType.Warning, VatErpImportValidation.Const_ComBalanceAcctUnValidate, comAcctChks.size(),
                        VatErpImportValidation.Const_ComBalanceAcctUnValidate_Tips
                        , ErpCheckType.ComBalanceAcctUnValidate, 0, JSON.toJSONString(comAcctChks)));
            }


            if (vAcctChks.size() == 0) {
                results.add(setDataCheckItem(ImageType.Success, VatErpImportValidation.Const_VoucherAcctValidate,
                        0, "", ErpCheckType.Success));
            } else {

                results.add(setDataCheckItem(ImageType.Warning, VatErpImportValidation.Const_VoucherAcctUnValidate, vAcctChks.size(),
                        VatErpImportValidation.Const_VoucherAcctUnValidate_Tips
                        , ErpCheckType.VoucherAcctValidate, 0, JSON.toJSONString(vAcctChks)));
            }


            if (vDebCrdSingleChk.size() == 0) {
                results.add(setDataCheckItem(ImageType.Success, VatErpImportValidation.Const_VSingleDebitCreditValidate,
                        0, "", ErpCheckType.Success));
            } else {
                results.add(setDataCheckItem(Alert, VatErpImportValidation.Const_VSingleDebitCreditUnValidate,
                        vDebCrdSingleChk.size(), VatErpImportValidation.Const_VSingleDebitCreditUnValidate_Tips
                        , ErpCheckType.VSingleDebitCreditUnValidate, 0, JSON.toJSONString(vDebCrdSingleChk)));
            }

            if (vEmptyItem.size() == 0) {
                results.add(setDataCheckItem(ImageType.Success, VatErpImportValidation.Const_VEmptyCheckValidateItemID,
                        0, "", ErpCheckType.Success));
            } else {
                results.add(setDataCheckItem(Alert, VatErpImportValidation.Const_VEmptyCheckUnValidateItemID,
                        vEmptyItem.size(), VatErpImportValidation.Const_VEmptyCheckUnValidateItemID_Tips
                        , ErpCheckType.VEmptyCheckUnValidateItemID, 0, JSON.toJSONString(vEmptyItem)));
            }

            if (vEmptyGroup.size() == 0) {
                results.add(setDataCheckItem(ImageType.Success, VatErpImportValidation.Const_VEmptyCheckValidateGroup,
                        0, "", ErpCheckType.Success));
            } else {
                results.add(setDataCheckItem(Alert, VatErpImportValidation.Const_VEmptyCheckUnValidateGroup,
                        vEmptyGroup.size(), VatErpImportValidation.Const_VEmptyCheckUnValidateGroup_Tips
                        , ErpCheckType.VEmptyCheckUnValidateGroup, 0, JSON.toJSONString(vEmptyGroup)));
            }

            if (vEmptyAcctCode.size() == 0) {
                results.add(setDataCheckItem(ImageType.Success, VatErpImportValidation.Const_VEmptyCheckValidateAcctCode,
                        0, "", ErpCheckType.Success));
            } else {
                results.add(setDataCheckItem(ImageType.Error, VatErpImportValidation.Const_VEmptyCheckUnValidateAcctCode,
                        vEmptyAcctCode.size(), VatErpImportValidation.Const_VEmptyCheckUnValidateAcctCode_Tips
                        , ErpCheckType.VEmptyCheckUnValidateAcctCode, 0, JSON.toJSONString(vEmptyAcctCode)));
            }

            if (vEmptySummary.size() == 0) {
                results.add(setDataCheckItem(ImageType.Success, VatErpImportValidation.Const_VEmptyCheckValidateSummary,
                        0, "", ErpCheckType.Success));
            } else {
                results.add(setDataCheckItem(Alert, VatErpImportValidation.Const_VEmptyCheckUnValidateSummary,
                        vEmptySummary.size(), VatErpImportValidation.Const_VEmptyCheckUnValidateSummary_Tips
                        , ErpCheckType.VEmptyCheckUnValidateSummary, 0, JSON.toJSONString(vEmptySummary)));
            }

            operationResult.setResultMsg(ReValidate);
            convertDataCheckDtoToValidationInfo(results, ErpBasicCheck.getCode(), userId).forEach(m -> {
                validationInfoMapper.insert(m);
            });
        }

        results.sort(Comparator.comparing(DataCheckDto::getDataCheckType));
        for (DataCheckDto m : results) {
            m.setNum(number);
            m.setImagePic(setCheckImage(ImageType.fromCode(m.getImageTypeId())));
            number++;
        }

        operationResult.setResult(true);
        operationResult.setData(results);

        return operationResult;
    }

    private List<ValidationInfo> convertDataCheckDtoToValidationInfo(List<DataCheckDto> basicItems, int typeId, String userId) {
        List<ValidationInfo> results = new ArrayList<>();

        basicItems.forEach(item -> {
            ValidationInfo vi = new ValidationInfo();
            vi.setID(UUID.randomUUID().toString());
            vi.setValidationResult(item.getDataCheckType());
            vi.setValidationDetails(item.getValidationDetails());
            vi.setErrorLevel(item.getImageTypeId());
            vi.setValidationTips(item.getTips());
            vi.setNumber(item.getErrorCount());
            vi.setImportTypeId(typeId);
            vi.setErpCheckTypeId(item.getErpCheckTypeId());
            vi.setPeriodId(item.getPeriod());
            vi.setCreateTime(new Date());
            vi.setCreatorId(userId);
        });

        return results;
    }

    private String setCheckImage(ImageType imageTypeId) {
        String imgbase64 = null;
        switch (imageTypeId) {
            case Alert:
                imgbase64 = imageToBase64ByImagePath(IMAGE_PATH_INFORMATION, IMAGE_FORMART_);
                break;
            case Error:
                imgbase64 = imageToBase64ByImagePath(IMAGE_PATH_ERROR, IMAGE_FORMART_);
                break;
            case Success:
                imgbase64 = imageToBase64ByImagePath(IMAGE_PATH_SUCCESS, IMAGE_FORMART_);
                break;
            case Warning:
                imgbase64 = imageToBase64ByImagePath(IMAGE_PATH_WARN, IMAGE_FORMART_);
                break;
        }

        return imgbase64;
    }

    private String imageToBase64ByImagePath(String imagPath, String format) {

        try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
            BufferedImage bufferedImage = ImageIO.read(new File(imagPath));
            ImageIO.write(bufferedImage, format, outputStream);
            BASE64Encoder encoder = new BASE64Encoder();
            return encoder.encode(outputStream.toByteArray());
        } catch (MalformedURLException e1) {
            e1.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        throw new RuntimeException("read image some error");//TODO:should be replace some exception filter(neo)
    }

    private DataCheckDto setDataCheckItem(ImageType imageType, String type, int count, String tips,
                                          ErpCheckType checkType, int periodId, String validationDetails) {
        DataCheckDto rawData = new DataCheckDto();
        rawData.setImageTypeId(imageType.getCode());
        rawData.setDataCheckType(type);
        rawData.setErrorCount(count);
        rawData.setTips(tips);
        rawData.setPeriod(periodId);
        rawData.setErpCheckTypeId(checkType.getCode());
        rawData.setValidationDetails(validationDetails);
        return rawData;
    }

    private DataCheckDto setDataCheckItem(ImageType imageType, String type, int count, String tips,
                                          ErpCheckType checkType) {
        DataCheckDto rawData = new DataCheckDto();
        rawData.setImageTypeId(imageType.getCode());
        rawData.setDataCheckType(type);
        rawData.setErrorCount(count);
        rawData.setTips(tips);
        rawData.setPeriod(0);
        rawData.setErpCheckTypeId(checkType.getCode());
        rawData.setValidationDetails(null);
        return rawData;
    }

    private List<VoucherDto> getVoucherEmptyCheck(boolean itemID, boolean period, boolean group, boolean acctCode, boolean summary) {
        VoucherExample example = new VoucherExample();

        if (itemID) {
            example.createCriteria().andItemIDIsNotNull();
            if (voucherMapper.countByExample(example) > 0) {
                example.createCriteria().andItemIDIsNull();
            }
        } else if (period) {
            example.createCriteria().andPeriodIsNull();
        } else if (group) {
            example.createCriteria().andGroupIsNull();
        } else if (acctCode) {
            example.createCriteria().andAcctCodeIsNull();
        } else if (summary) {
            example.createCriteria().andSummaryIsNull();
        }
        List<Voucher> vouchers = voucherMapper.selectByExample(example);
        if (vouchers != null && !vouchers.isEmpty()) {
            List<VoucherDto> dtos = new ArrayList<>(vouchers.size());

            vouchers.forEach(m -> {
                VoucherDto dto = new VoucherDto();
                try {
                    FieldsMapper.map(m, dto);
                } catch (ClassNotFoundException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
                dtos.add(dto);
            });

            return dtos;
        }
        return new ArrayList<>();
    }

    private List<VoucherBalanceDto> getSingleVoucherCheck() {
        List<VoucherBalanceDto> vouchers = validationInfoMapper.getSingleVoucherCheck();
        if (vouchers != null && !vouchers.isEmpty()) {
            List<VoucherBalanceDto> parentVouchers = vouchers.stream().filter(dto -> dto.getCustomerCode() == null ||
                    dto.getCustomerCode().isEmpty()).collect(Collectors.toList());

            parentVouchers.forEach(m -> {
                List<VoucherBalanceDto> children = parentVouchers.stream().filter(dto -> m.getvID().equals(dto.getvID())
                        && m.getPeriod().intValue() == dto.getPeriod().intValue()
                        && m.getGroup().equals(dto.getGroup()) && dto.getCustomerCode() != null
                        && !dto.getCustomerCode().isEmpty()).collect(Collectors.toList());
                children.sort(Comparator.comparing(VoucherBalanceDto::getItemID));
                m.setChildren(children);
            });

            parentVouchers.sort(Comparator.comparing(VoucherBalanceDto::getAcctCode));
            parentVouchers.sort(Comparator.comparing(VoucherBalanceDto::getvID));
            parentVouchers.sort(Comparator.comparing(VoucherBalanceDto::getGroup));
            parentVouchers.sort(Comparator.comparing(VoucherBalanceDto::getItemID));

            return parentVouchers;
        }
        return new ArrayList<>();
    }

    //判断ValidationInfo中是否存在typeId的数据
    private List<ValidationInfo> getValidationData(Integer typeId, Integer periodId) {
        ValidationInfoExample example = new ValidationInfoExample();

        if (periodId != null && periodId > 0) {
            example.createCriteria().andImportTypeIdEqualTo(typeId.intValue()).andPeriodIdEqualTo(periodId.intValue());
        } else example.createCriteria().andImportTypeIdEqualTo(typeId.intValue());

        return validationInfoMapper.selectByExample(example);
    }

    private List<ValidationInfo> getValidationData(Integer typeId) {
        return getValidationData(typeId, null);
    }
}
