package pwc.taxtech.atms.vat.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dao.FormulaAdminMapper;
import pwc.taxtech.atms.dto.GroupId;
import pwc.taxtech.atms.dto.vatdto.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.entitiy.ProjectServiceType;
import pwc.taxtech.atms.entitiy.ProjectServiceTypeExample;
import pwc.taxtech.atms.service.impl.DistributedIDService;
import pwc.taxtech.atms.vat.dao.CellDataMapper;
import pwc.taxtech.atms.vat.dao.FormulaProjectMapper;
import pwc.taxtech.atms.vat.dao.ReportMapper;
import pwc.taxtech.atms.vat.entity.CellData;
import pwc.taxtech.atms.vat.entity.CellDataExample;
import pwc.taxtech.atms.vat.entity.DataSource;
import pwc.taxtech.atms.vat.entity.Report;
import pwc.taxtech.atms.vat.entity.ReportExample;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import static pwc.taxtech.atms.constant.Constant.FIRST_OR_DEFAULT;

@Repository
public class FormulaAgent extends VatAbstractService {
    @Autowired
    public FormulaAdminMapper adminMp;
    @Autowired
    public FormulaProjectMapper projectMp;
    @Autowired
    public ReportMapper reportMapper;
    @Autowired
    public CellDataMapper cellDataMapper;
    @Autowired
    public DistributedIDService distributedIDService;


    public List<CellTemplatePerGroupDto> getCellTemplateGroupDto(Long templateGroupId, String projectId) {
        ProjectServiceTypeExample pst = new ProjectServiceTypeExample();
        pst.createCriteria().andTemplateGroupIDEqualTo(Long.valueOf(templateGroupId))
                .andServiceTypeIDEqualTo(EnumServiceType.VAT.getCode() + "");
        List<ProjectServiceType> pstReult = projectServiceTypeMapper.selectByExample(pst);

        List<CellTemplatePerGroupDto> cellTemplates = adminMp.getCellTemplatePerGroupDto(templateGroupId);
        if (pstReult == null || pstReult.isEmpty()) {
            List<GroupId> groupIds = adminMp.getTemplateGroupId(projectId);
            GroupId groupId;

            if (groupIds.size() > 0) {
                groupId = groupIds.stream().filter(m -> m.isDefault != 1).findFirst().get();
                if (groupId == null) groupId = groupIds.get(0);
                cellTemplates.addAll(adminMp.getCellTemplatePerGroupDto(groupId.groupId));
            }
        }
        return cellTemplates;
    }

    public Report getReportByTemplate(String templateId, Integer periodId) {
        ReportExample example = new ReportExample();
        example.createCriteria().andTemplateIdEqualTo(Long.valueOf(templateId))
                .andPeriodEqualTo(periodId);
        List<Report> reports = reportMapper.selectByExample(example);

        if (reports != null && !reports.isEmpty()) return reports.get(FIRST_OR_DEFAULT);
        return null;
    }

    public CellData getCellDataListByTemplate(String templateId, Long reportId) {
        CellDataExample dataExample = new CellDataExample();
        dataExample.createCriteria().andCellTemplateIdEqualTo(Long.valueOf(templateId));
        List<CellData> cellDataList = cellDataMapper.selectByExample(dataExample);

        if (cellDataList != null && !cellDataList.isEmpty()) return cellDataList.get(FIRST_OR_DEFAULT);
        return null;
    }

    public void save(List<ReportCellDataSourceDto> dataSourceDtos) {
        dataSourceDtos.forEach(m -> {
            DataSource ds = new DataSource();
            ds.setAmount(m.getAmount() == null ? new BigDecimal(0) : m.getAmount());
            ds.setCreateBy(m.getCreator() == null ? "Admin" : m.getCreator());
            ds.setCreateTime(m.getCreateTime() == null ? new Date() : m.getCreateTime());
            ds.setDescription(m.getDescription() == null ? "" : m.getDescription());
            ds.setKeyValueDataId(m.getKeyValueDataID());
            ds.setName(m.getName());
            ds.setType(m.type);
            ds.setUpdateBy(m.getUpdater() == null ? "Admin" : m.getUpdater());
            ds.setUpdateTime(m.getUpdateTime() == null ? new Date() : m.getUpdateTime());
            ds.setId(distributedIDService.nextId());
            ds.setKeyValueDataId(m.getDescription() == null?"":m.getDescription());//TODO: tobe fixed
            ds.setRowName(m.getRowName() == null ? "" : m.getRowName());
            ds.setColumnName(m.getColumnName() == null ? "" : m.getColumnName());
            ds.setRowIndex(m.getRowIndex());
            ds.setColumnIndex(m.getColumnIndex());
            ds.setRemapBatchId("0");//TODO: tobe fixed
            dataSourceMapper.insert(ds);
        });

    }

}
