package pwc.taxtech.atms.vat.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.reflections.util.Utils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.PagingList;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.PagingResultDto;
import pwc.taxtech.atms.dto.vatdto.OutputVATInvoiceInfoDto;
import pwc.taxtech.atms.dto.vatdto.QueryEvidenceDto;
import pwc.taxtech.atms.dto.vatdto.QueryOutputDto;
import pwc.taxtech.atms.vat.dao.ImportFileMapper;
import pwc.taxtech.atms.vat.entity.ImportFile;
import pwc.taxtech.atms.vat.entity.ImportFileExample;
import pwc.taxtech.atms.vat.entity.OutputVATInvoiceExample;
import pwc.taxtech.atms.vat.entity.OutputVATInvoiceItem;
import pwc.taxtech.atms.vat.entity.OutputVATInvoiceItemExample;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class OutputInvoiceServiceImpl extends VatAbstractService {
    @Autowired
    private ImportFileMapper importFileMapper;

    public PagingResultDto<OutputVATInvoiceInfoDto> queryOutputInvoiceList(QueryOutputDto queryDto) {
        PagingResultDto<OutputVATInvoiceInfoDto> qResult = new PagingResultDto<>();
        qResult.setPageInfo(queryDto.getPageInfo());
        qResult.setList(new ArrayList<>());
        List<OutputVATInvoiceInfoDto> finalList = getQueryList(queryDto);
        qResult.getPageInfo().setTotalCount(finalList.size());

        if (finalList.size() > 0) {
            PagingList pagingList = new PagingList(finalList, qResult.getPageInfo());
            qResult.setList(pagingList.getPagingList());
        }
        return qResult;
    }

    public OperationResultDto<List<OutputVATInvoiceInfoDto>> getExportOutputInvoiceList(QueryOutputDto queryDto) {
        OperationResultDto<List<OutputVATInvoiceInfoDto>> result = new OperationResultDto<>();
        List<OutputVATInvoiceInfoDto> finalList = getQueryList(queryDto);
        finalList = getOutputDetailInfoList(finalList);
        result.setData(finalList);
        result.setResult(true);
        return result;
    }

    public List<ImportFile> getEvidenceList(QueryEvidenceDto queryDto) {
        ImportFileExample example = new ImportFileExample();

        if (queryDto.getCurrentPeriod()) {
            if (Utils.isEmpty(queryDto.getQueryString())) {
                example.createCriteria().andFileImportTypeEqualTo(2).andPeriodIDEqualTo(queryDto.getPeriodId().intValue());
            } else {
                example.createCriteria().andFileImportTypeEqualTo(2).andPeriodIDEqualTo(queryDto.getPeriodId().intValue()).andFileNameLike(queryDto.getQueryString());
            }
        } else {
            if (Utils.isEmpty(queryDto.getQueryString())) {
                example.createCriteria().andFileImportTypeEqualTo(2);
            } else {
                example.createCriteria().andFileImportTypeEqualTo(2).andFileNameLike(queryDto.getQueryString());
            }
        }
        return importFileMapper.selectByExample(example);
    }

    public OperationResultDto<List<OutputVATInvoiceInfoDto>> getOutputInvoiceList(Integer period) {
        List<OutputVATInvoiceInfoDto> result = outputVATInvoiceMapper.queryOutputDetailWithItem(period);
        result.sort(Comparator.comparing(OutputVATInvoiceInfoDto::getInvoiceID));
        OperationResultDto<List<OutputVATInvoiceInfoDto>> dtoResult = new OperationResultDto();

        OutputVATInvoiceExample example = new OutputVATInvoiceExample();
        example.createCriteria().andPeriodIDEqualTo(period.intValue());

        dtoResult.setData(result);
        dtoResult.setReturnCode(Long.valueOf(outputVATInvoiceMapper.countByExample(example)).intValue());
        dtoResult.setResult(true);
        return dtoResult;
    }

    private List<OutputVATInvoiceInfoDto> getOutputDetailInfoList(List<OutputVATInvoiceInfoDto> query) {
        List<OutputVATInvoiceInfoDto> list = new ArrayList<>();
        List<OutputVATInvoiceItem> outItem = outputVATInvoiceItemMapper.selectByExample(new OutputVATInvoiceItemExample());
        if (query != null && query.size() > 0) {
            for (OutputVATInvoiceInfoDto dto : query) {
                List<OutputVATInvoiceItem> itemList = outItem.stream()
                        .filter(a -> a.getInvoiceID().equals(dto.getInvoiceID()))
                        .sorted(Comparator.comparing(OutputVATInvoiceItem::getSeqNo))
                        .collect(Collectors.toList());

                if (itemList != null && itemList.size() > 0) {
                    OutputVATInvoiceInfoDto newDto = new OutputVATInvoiceInfoDto();
                    CommonUtils.copyProperties(dto, newDto);
                    newDto.setCodeVersion(itemList.get(0).getCodeVersion());
                    newDto.setProductName(itemList.get(0).getProductName());
                    newDto.setDocumentNum(itemList.get(0).getDocumentNum());
                    newDto.setProductStandard(itemList.get(0).getProductStandard());
                    newDto.setUnit(itemList.get(0).getUnit());
                    newDto.setQuantity(itemList.get(0).getQuantity());
                    newDto.setUnitPrice(itemList.get(0).getUnitPrice());
                    newDto.setAmount(itemList.get(0).getAmount());
                    newDto.setTaxRate(itemList.get(0).getTaxRate());
                    newDto.setTaxAmount(itemList.get(0).getTaxAmount());
                    newDto.setTaxClassCode(itemList.get(0).getTaxClassCode());
                    list.add(newDto);
                    if (itemList.size() > 1) {
                        for (int i = 1; i < itemList.size(); i++) {
                            OutputVATInvoiceInfoDto invoiceDto = new OutputVATInvoiceInfoDto();
                            invoiceDto.setPeriodID(dto.getPeriodID());
                            invoiceDto.setInvoiceType(null);
                            invoiceDto.setClassCode(StringUtils.EMPTY);
                            invoiceDto.setInvoiceNumber(StringUtils.EMPTY);
                            invoiceDto.setBuyerName(StringUtils.EMPTY);
                            invoiceDto.setBuyerTaxNumber(StringUtils.EMPTY);
                            invoiceDto.setBankAccount(StringUtils.EMPTY);
                            invoiceDto.setPhoneNum(StringUtils.EMPTY);
                            invoiceDto.setInvoiceDate(null);
                            invoiceDto.setCodeVersion(itemList.get(i).getCodeVersion());
                            invoiceDto.setProductName(itemList.get(i).getProductName());
                            invoiceDto.setDocumentNum(itemList.get(i).getDocumentNum());
                            invoiceDto.setProductStandard(itemList.get(i).getProductStandard());
                            invoiceDto.setUnit(itemList.get(i).getUnit());
                            invoiceDto.setQuantity(itemList.get(i).getQuantity());
                            invoiceDto.setUnitPrice(itemList.get(i).getUnitPrice());
                            invoiceDto.setAmount(itemList.get(i).getAmount());
                            invoiceDto.setTaxRate(itemList.get(i).getTaxRate());
                            invoiceDto.setTaxAmount(itemList.get(i).getTaxAmount());
                            invoiceDto.setTaxClassCode(itemList.get(i).getTaxClassCode());
                            list.add(invoiceDto);
                        }
                    }
                }
            }
        }
        return list;
    }

    private List<OutputVATInvoiceInfoDto> getQueryList(QueryOutputDto queryDto) {
        List<OutputVATInvoiceInfoDto> rList = outputVATInvoiceMapper.selectOutputVATInvoiceInfoLeftJoinItem(queryDto);
        Function<OutputVATInvoiceInfoDto, List<Object>> compositeKey = outputVATInvoiceInfoDto ->
                Arrays.asList(outputVATInvoiceInfoDto.getInvoiceID()
                        , outputVATInvoiceInfoDto.getClassCode()
                        , outputVATInvoiceInfoDto.getInvoiceNumber());
        List<OutputVATInvoiceInfoDto> groupedList = new ArrayList<>();
        rList.stream().collect(Collectors.groupingBy(compositeKey, Collectors.toList())).forEach((k, v) -> {
            if (v.stream().findFirst().isPresent()) {
                OutputVATInvoiceInfoDto fItem = v.stream().findFirst().get();
                OutputVATInvoiceInfoDto outputVATInvoiceInfoDto = new OutputVATInvoiceInfoDto();
                CommonUtils.copyProperties(fItem, outputVATInvoiceInfoDto);
                outputVATInvoiceInfoDto.setAmount(new BigDecimal(v.stream().mapToDouble(a -> a.getAmount().doubleValue()).sum()));
                outputVATInvoiceInfoDto.setTaxAmount(new BigDecimal(v.stream().mapToDouble(a -> a.getTaxAmount().doubleValue()).sum()));
                groupedList.add(outputVATInvoiceInfoDto);
            }
        });

        Stream<OutputVATInvoiceInfoDto> stream = groupedList.stream();
        if (queryDto.getInvoiceType() != null && queryDto.getInvoiceType() != 999) {
            stream = stream.filter(a -> a.getInvoiceType().equals(queryDto.getInvoiceType()));
        }
        if (queryDto.getStartInvoiceDate() != null) {
            stream = stream.filter(a -> a.getInvoiceDate().after(queryDto.getStartInvoiceDate()));
        }
        if (queryDto.getEndInvoiceDate() != null) {
            stream = stream.filter(a -> a.getInvoiceDate().before(queryDto.getEndInvoiceDate()));
        }
        if (StringUtils.isNotBlank(queryDto.getClassCode())) {
            stream = stream.filter(a -> a.getClassCode().contains(queryDto.getClassCode()));
        }
        if (StringUtils.isNotBlank(queryDto.getInvoiceNumber())) {
            stream = stream.filter(a -> a.getInvoiceNumber().contains(queryDto.getInvoiceNumber()));
        }
        if (StringUtils.isNotBlank(queryDto.getBuyerName())) {
            stream = stream.filter(a -> a.getBuyerName().contains(queryDto.getBuyerName()));
        }
        if (queryDto.getAmountStart() != null) {
            stream = stream.filter(a -> a.getAmount().compareTo(queryDto.getAmountStart()) > 0);
        }
        if (queryDto.getAmountEnd() != null) {
            stream = stream.filter(a -> a.getAmount().compareTo(queryDto.getAmountEnd()) < 0);
        }
        if (queryDto.getTaxAmountStart() != null) {
            stream = stream.filter(a -> a.getTaxAmount().compareTo(queryDto.getTaxAmountStart()) > 0);
        }
        if (queryDto.getTaxAmountEnd() != null) {
            stream = stream.filter(a -> a.getTaxAmount().compareTo(queryDto.getTaxAmountEnd()) < 0);
        }
        return stream.sorted(Comparator.comparing(OutputVATInvoiceInfoDto::getInvoiceDate)).collect(Collectors.toList());
    }
}
