package pwc.taxtech.atms.vat.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.VatEnterpriseAccountDto;
import pwc.taxtech.atms.vat.entity.VatEnterpriseAccount;
import pwc.taxtech.atms.vat.entity.VatEnterpriseAccountExample;
import pwc.taxtech.atms.vat.entity.VatStandardAccount;
import pwc.taxtech.atms.vat.entity.VatStandardAccountExample;
import pwc.taxtech.atms.vat.service.TBDataImportService;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class TBDataImportServiceImpl extends VatAbstractService implements TBDataImportService {

    @Override
    public List<TrialBalanceDto> getTrialBalanceData(int fromPeriod, int toPeriod) {
        long fromMonth = (long) fromPeriod;
        long toMonth = (long) toPeriod;
//        Map<String, Object> monthMap = new HashMap<>();
//        monthMap.put("fromMonth", fromMonth);
//        monthMap.put("toMonth", toMonth);
        List<TrialBalanceDto> queryList = companyBalanceMapper.companyBalanceLeftJoinEnterpriseAccount(Long.toString(fromMonth), Long.toString(toMonth));
        List<TrialBalanceDto> result = new ArrayList<>();
        queryList.stream().collect(Collectors.groupingBy(TrialBalanceDto::getAcctCode)).forEach((a, b) -> {
            TrialBalanceDto item = b.get(0);
            TrialBalanceDto trialBalanceDto = new TrialBalanceDto();
            trialBalanceDto.setAcctCode(item.getAcctCode());
            trialBalanceDto.setCustomerCode(item.getCustomerCode());
            trialBalanceDto.setBegDebitBal(item.getPeriodId() == fromPeriod ? item.getBegDebitBal() : BigDecimal.ZERO);
            trialBalanceDto.setBegCreditBal(item.getPeriodId() == fromPeriod ? item.getBegCreditBal() : BigDecimal.ZERO);
            trialBalanceDto.setBegBal(item.getPeriodId() == fromPeriod ? item.getBegBal() : BigDecimal.ZERO);
            trialBalanceDto.setEndBal(item.getPeriodId() == toPeriod ? item.getEndBal() : BigDecimal.ZERO);
            trialBalanceDto.setEndDebitBal(item.getPeriodId() == toPeriod ? item.getEndDebitBal() : BigDecimal.ZERO);
            trialBalanceDto.setEndCreditBal(item.getPeriodId() == toPeriod ? item.getEndCreditBal() : BigDecimal.ZERO);
            double summaryA = b.stream().mapToDouble(c -> Objects.requireNonNull(c.getDebitBal()).doubleValue()).summaryStatistics().getSum();
            trialBalanceDto.setDebitBal(BigDecimal.valueOf(summaryA));
            double summaryB = b.stream().mapToDouble(c -> Objects.requireNonNull(c.getCreditBal()).doubleValue()).summaryStatistics().getSum();
            trialBalanceDto.setCreditBal(BigDecimal.valueOf(summaryB));
            trialBalanceDto.setParentCode(item.getParentCode());
            trialBalanceDto.setAccountName(item.getAccountName());
            trialBalanceDto.setAcctProp(item.getAcctProp());
            trialBalanceDto.setDirection(item.getDirection());
            result.add(trialBalanceDto);
        });
        return result;
    }

    @Override
    public List<TrialBalanceDto> getTrialBalanceStdData(int fromPeriod, int toPeriod) {
        long fromMonth = (long) fromPeriod;
        long toMonth = (long) toPeriod;
        Map<String, Object> monthMap = new HashMap<>();
        monthMap.put("fromMonth", fromMonth);
        monthMap.put("toMonth", toMonth);
        List<TrialBalanceDto> queryList = vatStandardAccountMapper.selectProjectStandardAccountLeftJoinBalanceStdManual(Long.toString(fromMonth), Long.toString(toMonth));
        List<TrialBalanceDto> result = new ArrayList<>();
        queryList.stream().collect(Collectors.groupingBy(TrialBalanceDto::getAcctCode)).forEach((a, b) -> {
            TrialBalanceDto item = b.get(0);
            TrialBalanceDto trialBalanceDto = new TrialBalanceDto();
            trialBalanceDto.setAcctCode(item.getAcctCode());
            trialBalanceDto.setCustomerCode(item.getCustomerCode());
            trialBalanceDto.setBegDebitBal(item.getPeriodId() == fromPeriod ? item.getBegDebitBal() : BigDecimal.ZERO);
            trialBalanceDto.setBegCreditBal(item.getPeriodId() == fromPeriod ? item.getBegCreditBal() : BigDecimal.ZERO);
            trialBalanceDto.setBegBal(item.getPeriodId() == fromPeriod ? item.getBegBal() : BigDecimal.ZERO);
            trialBalanceDto.setEndBal(item.getPeriodId() == toPeriod ? item.getEndBal() : BigDecimal.ZERO);
            trialBalanceDto.setEndDebitBal(item.getPeriodId() == toPeriod ? item.getEndDebitBal() : BigDecimal.ZERO);
            trialBalanceDto.setEndCreditBal(item.getPeriodId() == toPeriod ? item.getEndCreditBal() : BigDecimal.ZERO);
            double summaryA = b.stream().mapToDouble(c -> Objects.requireNonNull(c.getDebitBal()).doubleValue()).summaryStatistics().getSum();
            trialBalanceDto.setDebitBal(BigDecimal.valueOf(summaryA));
            double summaryB = b.stream().mapToDouble(c -> Objects.requireNonNull(c.getCreditBal()).doubleValue()).summaryStatistics().getSum();
            trialBalanceDto.setCreditBal(BigDecimal.valueOf(summaryB));
            trialBalanceDto.setParentCode(item.getParentCode());
            trialBalanceDto.setAccountName(item.getAccountName());
            trialBalanceDto.setAcctProp(item.getAcctProp());
            trialBalanceDto.setDirection(item.getDirection());
            result.add(trialBalanceDto);
        });
        return result;
    }

    @Override
    public List<VatEnterpriseAccountDto> getEnterpriseAccount() {
        List<VatEnterpriseAccount> list = vatEnterpriseAccountMapper.selectByExample(new VatEnterpriseAccountExample());
        List<VatEnterpriseAccountDto> resultList = new ArrayList<>();
        list.forEach(a -> {
            VatEnterpriseAccountDto vatEnterpriseAccountDto = new VatEnterpriseAccountDto();
            CommonUtils.copyProperties(a, vatEnterpriseAccountDto);
            resultList.add(vatEnterpriseAccountDto);
        });
        return resultList;
    }

    @Override
    public List<VatEnterpriseAccountDto> getStandardAccount() {
        VatStandardAccountExample example = new VatStandardAccountExample();
        example.setDistinct(true);
        List<VatStandardAccount> vatStandardAccountList = vatStandardAccountMapper.selectByExample(example);
        List<VatEnterpriseAccountDto> vatEnterpriseAccountDtoList = new ArrayList<>();
        vatStandardAccountList.forEach(a -> {
            VatEnterpriseAccountDto vatEnterpriseAccountDto = new VatEnterpriseAccountDto();
            vatEnterpriseAccountDto.setAcctCode(a.getCode());
            vatEnterpriseAccountDto.setParentCode(StringUtils.isBlank(a.getParentCode()) ? "0" : a.getParentCode());
            vatEnterpriseAccountDto.setName(a.getName());
            vatEnterpriseAccountDto.setAcctProp(a.getAcctProp());
            vatEnterpriseAccountDto.setDirection(a.getDirection());
        });
        return vatEnterpriseAccountDtoList;
    }

    @Deprecated
    private List<TrialBalanceDto> dealNullValue(List<TrialBalanceDto> dataList) {
        int begDebitBal = 0, begCreditBal = 0, begBal = 0, endBal = 0;
        int endDebitBal = 0, endCreditBal = 0, debitBal = 0, creditBal = 0, monthId = 0, customerCode = 0;

        if (dataList != null && !dataList.isEmpty()) {
            if (dataList.stream().anyMatch(a -> a.getMonthId() != null)) {
                monthId = 1;
            }

            if (dataList.stream().anyMatch(a -> StringUtils.isNotBlank(a.getCustomerCode()))) {
                customerCode = 1;
            }

            if (dataList.stream().anyMatch(p -> p.getBegDebitBal() != null)) {
                begDebitBal = 1;
            }

            if (dataList.stream().anyMatch(p -> p.getBegCreditBal() != null)) {
                begCreditBal = 1;
            }

            if (dataList.stream().anyMatch(p -> p.getBegBal() != null)) {
                begBal = 1;
            }

            if (dataList.stream().anyMatch(p -> p.getEndBal() != null)) {
                endBal = 1;
            }

            if (dataList.stream().anyMatch(p -> p.getEndDebitBal() != null)) {
                endDebitBal = 1;
            }

            if (dataList.stream().anyMatch(p -> p.getEndCreditBal() != null)) {
                endCreditBal = 1;
            }

            if (dataList.stream().anyMatch(p -> p.getDebitBal() != null)) {
                debitBal = 1;
            }

            if (dataList.stream().anyMatch(p -> p.getCreditBal() != null)) {
                creditBal = 1;
            }

            for (TrialBalanceDto compbalance : dataList) {
                if (compbalance.getMonthId() == null && monthId == 1) {
                    compbalance.setMonthId(0);
                }
                if (customerCode == 0) {
                    compbalance.setCustomerCode(null);
                }
                if (compbalance.getBegDebitBal() == null && begDebitBal == 1) {
                    compbalance.setBegDebitBal(BigDecimal.ZERO);
                }
                if (compbalance.getBegCreditBal() == null && begCreditBal == 1) {
                    compbalance.setBegCreditBal(BigDecimal.ZERO);
                }
                if (compbalance.getBegBal() == null && begBal == 1) {
                    compbalance.setBegBal(BigDecimal.ZERO);
                }
                if (compbalance.getEndBal() == null && endBal == 1) {
                    compbalance.setEndBal(BigDecimal.ZERO);
                }
                if (compbalance.getEndDebitBal() == null && endDebitBal == 1) {
                    compbalance.setEndDebitBal(BigDecimal.ZERO);
                }
                if (compbalance.getEndCreditBal() == null && endCreditBal == 1) {
                    compbalance.setEndCreditBal(BigDecimal.ZERO);
                }
                if (compbalance.getDebitBal() == null && debitBal == 1) {
                    compbalance.setDebitBal(BigDecimal.ZERO);
                }
                if (compbalance.getCreditBal() == null && creditBal == 1) {
                    compbalance.setCreditBal(BigDecimal.ZERO);
                }
            }
        }
        return dataList;
    }
}
