package pwc.taxtech.atms.vat.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.constant.VoucherSelect;
import pwc.taxtech.atms.constant.enums.VoucherLevelEnum;
import pwc.taxtech.atms.constant.enums.VoucherSearchEnum;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.PagingDto;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.vat.entity.VatEnterpriseAccountExample;
import pwc.taxtech.atms.vat.service.VoucherService;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class VoucherServiceImpl extends VatAbstractService implements VoucherService {

    @Resource(name = "dynamicJdbcTemplate")
    JdbcTemplate jdbcTemplate;

    @Override
    public OperationResultDto<Object> voucherSelectAdvancedByEntry(List<QueryConditionDto> listQueryCondition, VoucherSearchEnum mainRelation, boolean allJe, PagingDto pagingInfo) throws Exception {
        OperationResultDto<Object> result = new OperationResultDto<>();
        StringBuffer exceptionString = new StringBuffer();
        String sqlWhere = getSqlWhereCondition(listQueryCondition, mainRelation, allJe, exceptionString);
        if (StringUtils.isNotBlank(exceptionString.toString())) {
            result.setResult(false);
            result.setResultMsg(exceptionString.toString());
            return result;
        }
        String sql = StringUtils.EMPTY;
        if (allJe) {
            if (pagingInfo != null) {
                sql = "SELECT" +
                        "`VoucherID`," +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`GROUP`," +
                        "`ItemID`," +
                        "`VID`," +
                        "`Summary`," +
                        "e.`FullName`," +
                        "`Debit`," +
                        "`Credit`," +
                        "TB.`AcctCode`," +
                        "`CustomerCode`," +
                        "`CustomerName`," +
                        "e.StdCode," +
                        "TB.VoucherStd," +
                        "s.FullName AS StdName" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`VoucherID`," +
                        "c.`CurrentPeriodBo`," +
                        "`Date`," +
                        "c.`GROUP`," +
                        "`ItemID`," +
                        "c.`VID`," +
                        "`Summary`," +
                        "`Debit`," +
                        "`Credit`," +
                        "`AcctCode`," +
                        "`CustomerCode`," +
                        "`CustomerName`," +
                        "`StdCode` AS VoucherStd" +
                        "FROM" +
                        "(" +
                        "SELECT DISTINCT" +
                        "`CurrentPeriodBo`," +
                        "`GROUP`," +
                        "`VID`" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`GROUP`," +
                        "`VID`" +
                        "FROM" +
                        "Voucher " + sqlWhere +
                        ") tbl" +
                        ") abc" +
                        "LEFT JOIN Voucher c ON abc.`CurrentPeriodBo` = c.`CurrentPeriodBo`" +
                        "AND abc.`GROUP` = c.`GROUP`" +
                        "AND abc.`VID` = c.`VID`" +
                        ") TB" +
                        "LEFT JOIN EnterpriseAccount e ON TB.AcctCode = e.AcctCode" +
                        "LEFT JOIN StandardAccount s ON e.StdCode = s. CODE" +
                        "ORDER BY" +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`GROUP`," +
                        "`VID`" +
                        "LIMIT " + pagingInfo.getPageSize() * (pagingInfo.getPageIndex() - 1) + "," + pagingInfo.getPageSize();
            } else {
                sql = "SELECT" +
                        "`VoucherID`," +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`GROUP`," +
                        "`ItemID`," +
                        "`VID`," +
                        "`Summary`," +
                        "e.`FullName`," +
                        "`Debit`," +
                        "`Credit`," +
                        "TB.`AcctCode`," +
                        "`CustomerCode`," +
                        "`CustomerName`," +
                        "e.StdCode," +
                        "TB.VoucherStd," +
                        "s.FullName AS StdName" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`VoucherID`," +
                        "c.`CurrentPeriodBo`," +
                        "`Date`," +
                        "c.`GROUP`," +
                        "`ItemID`," +
                        "c.`VID`," +
                        "`Summary`," +
                        "`Debit`," +
                        "`Credit`," +
                        "`AcctCode`," +
                        "`CustomerCode`," +
                        "`CustomerName`," +
                        "`StdCode` AS VoucherStd" +
                        "FROM" +
                        "(" +
                        "SELECT DISTINCT" +
                        "`CurrentPeriodBo`," +
                        "`GROUP`," +
                        "`VID`" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`GROUP`," +
                        "`VID`" +
                        "FROM" +
                        "Voucher " + sqlWhere +
                        ") tbl" +
                        ") abc" +
                        "LEFT JOIN Voucher c ON abc.`CurrentPeriodBo` = c.`CurrentPeriodBo`" +
                        "AND abc.`GROUP` = c.`GROUP`" +
                        "AND abc.`VID` = c.`VID`" +
                        ") TB" +
                        "LEFT JOIN EnterpriseAccount e ON TB.AcctCode = e.AcctCode" +
                        "LEFT JOIN StandardAccount s ON e.StdCode = s. CODE" +
                        "ORDER BY" +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`GROUP`," +
                        "`VID`";
            }
        } else {
            if (pagingInfo != null) {
                sql = "SELECT" +
                        "*" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`VoucherID`," +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`ItemID`," +
                        "`VID`," +
                        "`Summary`," +
                        "e.`FullName`," +
                        "`Debit`," +
                        "`Credit`," +
                        "TB.`AcctCode`," +
                        "`CustomerCode`," +
                        "`CustomerName`," +
                        "e.StdCode," +
                        "Tb.VoucherStd AS VoucherStd," +
                        "s.FullName AS StdName" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`VoucherID`," +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`ItemID`," +
                        "`VID`," +
                        "`Summary`," +
                        "`Debit`," +
                        "`Credit`," +
                        "`AcctCode`," +
                        "`CustomerCode`," +
                        "`CustomerName`," +
                        "`StdCode` AS VoucherStd" +
                        "FROM" +
                        "Voucher" + sqlWhere +
                        ") TB" +
                        "LEFT JOIN EnterpriseAccount e ON TB.AcctCode = e.AcctCode" +
                        "LEFT JOIN StandardAccount s ON e.StdCode = s. CODE" +
                        ") RowNumVoucher" +
                        "ORDER BY" +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`VID`" +
                        "LIMIT " + pagingInfo.getPageSize() * (pagingInfo.getPageIndex() - 1) + "," + pagingInfo.getPageSize();
            } else {
                sql = "SELECT" +
                        "*" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`VoucherID`," +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`ItemID`," +
                        "`VID`," +
                        "`Summary`," +
                        "e.`FullName`," +
                        "`Debit`," +
                        "`Credit`," +
                        "TB.`AcctCode`," +
                        "`CustomerCode`," +
                        "`CustomerName`," +
                        "e.StdCode," +
                        "Tb.VoucherStd AS VoucherStd," +
                        "s.FullName AS StdName" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`VoucherID`," +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`ItemID`," +
                        "`VID`," +
                        "`Summary`," +
                        "`Debit`," +
                        "`Credit`," +
                        "`AcctCode`," +
                        "`CustomerCode`," +
                        "`CustomerName`," +
                        "`StdCode` AS VoucherStd" +
                        "FROM" +
                        "Voucher" + sqlWhere +
                        ") TB" +
                        "LEFT JOIN EnterpriseAccount e ON TB.AcctCode = e.AcctCode" +
                        "LEFT JOIN StandardAccount s ON e.StdCode = s. CODE" +
                        ") RowNumVoucher" +
                        "ORDER BY" +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`VID`";
            }
            List<Map<String, Object>> queryForList = jdbcTemplate.queryForList(sql);
            List<VoucherDto> voucherDtos = new ArrayList();
            queryForList.forEach(a -> {
                VoucherDto dto = new VoucherDto();
                dto.setVoucherID(a.get("VoucherID").toString());
                dto.setPeriod(Integer.parseInt(a.get("CurrentPeriodBo").toString()));
                dto.setDate(DateTime.parse(a.get("Date").toString()).toDate());
                dto.setGroup(a.get("Group").toString());
                dto.setvID(a.get("VID").toString());
                dto.setSummary(a.get("Summary").toString());
                dto.setAcctCode(a.get("AcctCode").toString());
                dto.setDebit(BigDecimal.valueOf(Double.valueOf(a.get("Debit").toString())));
                dto.setCredit(BigDecimal.valueOf(Double.valueOf(a.get("Credit").toString())));
                dto.setGroupShow(a.get("Group").toString() + "-" + a.get("VID").toString());
                dto.setAcctCodeAndNameShow(a.get("AcctCode").toString() + (StringUtils.isNotBlank(a.get("FullName").toString()) ? "/" + a.get("FullName") : null));
                dto.setCustomerCode(a.get("CustomerCode").toString());
                dto.setCustomerName(a.get("CustomerName").toString());
                dto.setCustomerCodeAndNameShow(a.get("CustomerCode").toString() + (StringUtils.isNotBlank(a.get("CustomerName").toString()) ? "/" + a.get("CustomerName").toString() : null));
                dto.setStdCodeAndNameShow(a.get("StdCode").toString() + (StringUtils.isNotBlank(a.get("StdName").toString()) ? "/" + a.get("StdName").toString() : null));
                dto.setStdCode(a.get("VoucherStd").toString());
                voucherDtos.add(dto);
            });

            QueryVoucherDto dto = new QueryVoucherDto();
            dto.setVoucherList(voucherDtos);
            dto.setItemIDCount(voucherDtos.size());
            dto.setCreditSum(new BigDecimal(voucherDtos.stream().mapToDouble(a -> a.getCredit().doubleValue()).summaryStatistics().getSum()));
            dto.setDebitSum(new BigDecimal(voucherDtos.stream().mapToDouble(a -> a.getDebit().doubleValue()).summaryStatistics().getSum()));
            dto.setVidCount((int) voucherDtos.stream().map(a -> new VIDCountDto(a.getvID(), a.getGroup(), a.getPeriod())).distinct().count());
            result.setResult(true);
            result.setData(dto);
        }
        return result;
    }

    @Override
    public OperationResultDto<Object> voucherSelectAdvancedCount(List<QueryConditionDto> listQueryCondition, VoucherSearchEnum mainRelation, boolean isEntryShow, boolean allJe) throws Exception {
        OperationResultDto<Object> result = new OperationResultDto<>();
        StringBuffer exceptionString = new StringBuffer(StringUtils.EMPTY);
        String sqlWhere = getSqlWhereCondition(listQueryCondition, mainRelation, allJe, exceptionString);
        if (StringUtils.isNotBlank(exceptionString)) {
            result.setResult(false);
            result.setResultMsg(exceptionString.toString());
            return result;
        }

        String sql = StringUtils.EMPTY;
        if (isEntryShow) {
            if (allJe) {
                sql = "select count(1) as TotalCount from" +
                        " (select distinct CurrentPeriodBo,`Group`,VID from" +
                        " (select CurrentPeriodBo,`Group`,VID from Voucher " + sqlWhere + ")TB)abc" +
                        " left join Voucher c on abc.CurrentPeriodBo=c.CurrentPeriodBo and abc.`Group`=c.`Group` and abc.VID=c.VID";
            } else {
                sql = "select count(1) as TotalCount from Voucher " + sqlWhere;
            }
        } else {
            sql = "Select count(1) as TotalCount from (select CurrentPeriodBo,`Group`,VID from Voucher " + sqlWhere + " group by CurrentPeriodBo,`Group`,VID)TB";
        }

        int total = jdbcTemplate.queryForObject(sql, int.class);
        result.setResult(true);
        result.setData(total);
        return null;
    }

    @Override
    public OperationResultDto<Object> voucherSelectAdvancedByVoucher(List<QueryConditionDto> listQueryCondition, VoucherSearchEnum mainRelation, boolean allJe, PagingDto pagingInfo) throws Exception {
        OperationResultDto<Object> resultDto = new OperationResultDto<>();
        OperationResultDto<Object> result = new OperationResultDto<>();
        StringBuffer exceptionString = new StringBuffer();
        String sqlWhere = getSqlWhereCondition(listQueryCondition, mainRelation, allJe, exceptionString);
        if (StringUtils.isNotBlank(exceptionString.toString())) {
            result.setResult(false);
            result.setResultMsg(exceptionString.toString());
            return result;
        }
        String sql = StringUtils.EMPTY;
        if (allJe) {
            if (pagingInfo != null) {
                sql = "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`," +
                        "`Date`," +
                        "DebitSum," +
                        "CreditSum," +
                        "ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "TB.`CurrentPeriodBo`," +
                        "TB.`Group`," +
                        "TB.`VID`," +
                        "TB.`Date`," +
                        "TB.DebitSum," +
                        "TB.CreditSum," +
                        "TB.ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "c.`CurrentPeriodBo`," +
                        "c.`Group`," +
                        "c.`VID`," +
                        "c.`Date`," +
                        "sum(c.`Debit`) AS DebitSum," +
                        "sum(c.`Credit`) AS CreditSum," +
                        "count(1) AS ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`" +
                        "FROM" +
                        "Voucher " + sqlWhere +
                        "GROUP BY" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`" +
                        ") abc" +
                        "LEFT JOIN Voucher c ON abc.`CurrentPeriodBo` = c.`CurrentPeriodBo`" +
                        "AND abc.`Group` = c.`Group`" +
                        "AND abc.`VID` = c.`VID`" +
                        "GROUP BY" +
                        "c.`CurrentPeriodBo`," +
                        "c.`Group`," +
                        "c.`VID`," +
                        "c.`Date`" +
                        ") TB" +
                        ") tmp" +
                        "ORDER BY" +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`VID`" +
                        "LIMIT " + pagingInfo.getPageSize() * (pagingInfo.getPageIndex() - 1) + "," + pagingInfo.getPageSize();
            } else {
                sql = "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`," +
                        "`Date`," +
                        "DebitSum," +
                        "CreditSum," +
                        "ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "TB.`CurrentPeriodBo`," +
                        "TB.`Group`," +
                        "TB.`VID`," +
                        "TB.`Date`," +
                        "TB.DebitSum," +
                        "TB.CreditSum," +
                        "TB.ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "c.`CurrentPeriodBo`," +
                        "c.`Group`," +
                        "c.`VID`," +
                        "c.`Date`," +
                        "sum(c.`Debit`) AS DebitSum," +
                        "sum(c.`Credit`) AS CreditSum," +
                        "count(1) AS ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`" +
                        "FROM" +
                        "Voucher " + sqlWhere +
                        "GROUP BY" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`" +
                        ") abc" +
                        "LEFT JOIN Voucher c ON abc.`CurrentPeriodBo` = c.`CurrentPeriodBo`" +
                        "AND abc.`Group` = c.`Group`" +
                        "AND abc.`VID` = c.`VID`" +
                        "GROUP BY" +
                        "c.`CurrentPeriodBo`," +
                        "c.`Group`," +
                        "c.`VID`," +
                        "c.`Date`" +
                        ") TB" +
                        ") tmp" +
                        "ORDER BY" +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`VID`";
            }
        } else {
            if (pagingInfo != null) {
                sql = "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`," +
                        "`Date`," +
                        "DebitSum," +
                        "CreditSum," +
                        "ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "TB.`CurrentPeriodBo`," +
                        "TB.`Group`," +
                        "TB.`VID`," +
                        "TB.`Date`," +
                        "TB.DebitSum," +
                        "TB.CreditSum," +
                        "TB.ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`," +
                        "`Date`," +
                        "sum(`Debit`) AS DebitSum," +
                        "sum(`Credit`) AS CreditSum," +
                        "count(1) AS ItemCount" +
                        "FROM" +
                        "Voucher " + sqlWhere +
                        "GROUP BY" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`," +
                        "`Date`" +
                        ") TB" +
                        ") abc" +
                        "ORDER BY" +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`VID`" +
                        "LIMIT " + pagingInfo.getPageSize() * (pagingInfo.getPageIndex() - 1) + "," + pagingInfo.getPageSize();
            } else {
                sql = "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`," +
                        "`Date`," +
                        "DebitSum," +
                        "CreditSum," +
                        "ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "TB.`CurrentPeriodBo`," +
                        "TB.`Group`," +
                        "TB.`VID`," +
                        "TB.`Date`," +
                        "TB.DebitSum," +
                        "TB.CreditSum," +
                        "TB.ItemCount" +
                        "FROM" +
                        "(" +
                        "SELECT" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`," +
                        "`Date`," +
                        "sum(`Debit`) AS DebitSum," +
                        "sum(`Credit`) AS CreditSum," +
                        "count(1) AS ItemCount" +
                        "FROM" +
                        "Voucher " + sqlWhere +
                        "GROUP BY" +
                        "`CurrentPeriodBo`," +
                        "`Group`," +
                        "`VID`," +
                        "`Date`" +
                        ") TB" +
                        ") abc" +
                        "ORDER BY" +
                        "`CurrentPeriodBo`," +
                        "`Date`," +
                        "`Group`," +
                        "`VID`";
            }
        }
        List<Map<String, Object>> queryResult = jdbcTemplate.queryForList(sql);
        List<VoucherMainDto> voucherMainDtos = new ArrayList<>();

        queryResult.forEach(a -> {
            VoucherMainDto voucherMainDto = new VoucherMainDto();
            voucherMainDto.setPeriod(Integer.parseInt(a.get("CurrentPeriodBo").toString()));
            voucherMainDto.setGroup(a.get("Group").toString());
            voucherMainDto.setvID(a.get("VID").toString());
            voucherMainDto.setDate(DateTime.parse(a.get("Date").toString()).toDate());
            voucherMainDto.setDebitSum(new BigDecimal(Double.valueOf(a.get("DebitSum").toString())));
            voucherMainDto.setCreditSum(new BigDecimal(Double.valueOf(a.get("CreditSum").toString())));
            voucherMainDto.setGroupShow(a.get("Group").toString() + "-" + a.get("VID").toString());
            voucherMainDto.setItemCount(Integer.valueOf(a.get("ItemCount").toString()));
            voucherMainDtos.add(voucherMainDto);
        });

        Function<VoucherMainDto, List<Object>> compositeKey = voucherRecord -> Arrays.asList(voucherRecord.getPeriod(), voucherRecord.getvID(), voucherRecord.getGroup());
        Map<Object, List<VoucherMainDto>> resultGroupMap = voucherMainDtos.stream().collect(Collectors.groupingBy(compositeKey, Collectors.toList()));
        List<VoucherMainDto> resultGroup = new ArrayList<>();
        resultGroupMap.forEach((k, v) -> resultGroup.add(v.stream().findFirst().get()));
        if (voucherMainDtos.size() != resultGroup.size()) {
            List<VoucherMainDto> expectedList = new ArrayList<>(voucherMainDtos);
            expectedList.removeAll(resultGroup);
            for (VoucherMainDto temp : expectedList) {
                VoucherMainDto alerMto = resultGroup.stream()
                        .filter(t -> t.getPeriod() == temp.getPeriod() && t.getvID().equals(temp.getvID()) && t.getGroup().equals(temp.getGroup())).findFirst().get();
                alerMto.setDebitSum(alerMto.getDebitSum().add(temp.getDebitSum()));
                alerMto.setCreditSum(alerMto.getCreditSum().add(temp.getCreditSum()));
                alerMto.setItemCount(alerMto.getItemCount() + temp.getItemCount());
            }
        }
        QueryVoucherMainDto dto = new QueryVoucherMainDto();
        dto.setVoucherMainList(resultGroup);
        dto.setItemIDCount(resultGroup.stream().mapToInt(a -> a.getItemCount()).sum());
        dto.setCreditSum(new BigDecimal(resultGroup.stream().mapToDouble(a -> a.getCreditSum().doubleValue()).sum()));
        dto.setDebitSum(new BigDecimal(resultGroup.stream().mapToDouble(a -> a.getDebitSum().doubleValue()).sum()));
        dto.setvIDCount(resultGroup.size());

        resultDto.setResult(true);
        resultDto.setData(dto);
        return resultDto;
    }

    private String getSqlWhereCondition(List<QueryConditionDto> listQueryCondition, VoucherSearchEnum mainRelation, boolean allJe, StringBuffer exceptionString) throws Exception {
        //what does this statement used for?
        exceptionString = new StringBuffer(StringUtils.EMPTY);

        if (mainRelation != VoucherSearchEnum.S_AndAll && mainRelation != VoucherSearchEnum.S_OrAll) {
            throw new Exception("条件关系定义错误");
        }
        List<VoucherTmplDto> tmpls = new ArrayList<>();
        for (QueryConditionDto queryConditionDto : listQueryCondition) {
            switch (queryConditionDto.getConditionName()) {
                case VoucherSelect.S_CorpAcctCoding:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "`AcctCode`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_CorpAcctName:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "EnterpriseAccount.`Name`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_VoucherType:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "`Group`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_VoucherVID:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "`VID`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_VoucherSummary:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "`Summary`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_Period:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "`CurrentPeriodBo`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_Debit:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "`Debit`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_Credit:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "`Credit`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_Date:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "`Date`"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_CustomerCode:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "Voucher.CustomerCode"
                            , mainRelation
                            , null));
                    break;
                case VoucherSelect.S_CustomerName:
                    tmpls.add(createQueryScriptByTmplList(Arrays.asList(queryConditionDto.getSearchValue().split(" "))
                            , VoucherSearchEnum.values()[queryConditionDto.getSearchKeyWord()]
                            , "Voucher.CustomerName"
                            , mainRelation
                            , null));
                    break;
                default:
                    throw new Exception("未处理的条件类型：" + queryConditionDto.getConditionName());
            }
        }

        tmpls.forEach(p -> {
            if (StringUtils.isBlank(p.getSql())) {
                tmpls.remove(p);
            }
        });

        return getSqlWhereCondition(tmpls);
    }

    private VoucherTmplDto createQueryScriptByTmplList(List<String> strList, VoucherSearchEnum relation, String columnName, VoucherSearchEnum logic, String strValue) throws Exception {
        VoucherTmplDto temp = new VoucherTmplDto();
        temp.setLogic(logic);
        StringBuffer sb = new StringBuffer();
        sb.append("(");

        int count = strList.size();
        switch (relation) {
            case S_EqualAnyKey:
                for (int i = 0; i < count; i++) {
                    sb.append(columnName + "=`" + strList.get(i) + "`" + (i < (count - 1) ? " or " : ")"));
                }
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                break;
            case S_AnyHeaderKey:
                for (int i = 0; i < count; i++) {
                    sb.append(columnName + "=`" + strList.get(i) + "`" + (i < (count - 1) ? " or " : ")"));
                }
                temp.setSort(VoucherLevelEnum.SecondSort.getCode());
                break;
            case S_AllKey:
                for (int i = 0; i < count; i++) {
                    sb.append(columnName + "like'" + strList.get(i) + "'" + (i < (count - 1) ? " and " : ")"));
                }
                temp.setSort(VoucherLevelEnum.SecondSort.getCode());
                break;
            case S_ContainAnyKey:
                for (int i = 0; i < count; i++) {
                    sb.append(columnName + "like'" + strList.get(i) + "'" + (i < (count - 1) ? " or " : ")"));
                }
                temp.setSort(VoucherLevelEnum.SecondSort.getCode());
                break;
            case S_OrAll:
                break;
            case S_AndAll:
                break;
            case S_Equal:
                sb.append(columnName + " ='" + strValue + "'");
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                break;
            case S_Greater:
                sb.append(columnName + " >'" + strValue + "'");
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                break;
            case S_GreaterEqual:
                sb.append(columnName + " >='" + strValue + "'");
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                break;
            //小于
            case S_Less:
                sb.append(columnName + "< '" + strValue + "')");
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                break;
            //小于等于
            case S_LessEqual:
                sb.append(columnName + "<= '" + strValue + "')");
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                break;
            case S_Manual:
                sb.append(columnName + "=1");
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                break;
            case S_Industry:
                sb.append(columnName + " is null");
                temp.setSort(VoucherLevelEnum.ThirdSort.getCode());
                break;
            case S_FromTo:
                if (strList.size() != 2) {
                    throw new Exception("数据格式错误");
                }
                if (!"null".equals(strList.get(0).toLowerCase()) && !"null".equals(strList.get(1).toLowerCase())) {
                    if ("`Debit`".equals(columnName) || "`Credit`".equals(columnName)) {
                        sb.append(columnName + ">=" + Double.valueOf(strList.get(0)) + " And " + columnName + " <=" + Double.valueOf(strList.get(1)) + ")");
                    } else {
                        sb.append(columnName + ">='" + strList.get(0) + "' And " + columnName + " <= '" + strList.get(1) + "')");
                    }
                } else if ("null".equals(strList.get(0).toLowerCase()) && "null".equals(strList.get(1).toLowerCase())) {
                    break;
                } else if (!"null".equals(strList.get(0).toLowerCase()) && "null".equals(strList.get(1).toLowerCase())) {
                    if ("`Debit`".equals(columnName) || "`Credit`".equals(columnName)) {
                        sb.append(columnName + ">=" + Double.valueOf(strList.get(0)) + ")");
                    } else {
                        sb.append(columnName + ">='" + strList.get(0) + "')");
                    }
                } else if ("null".equals(strList.get(0).toLowerCase()) && !"null".equals(strList.get(1).toLowerCase())) {
                    if ("`Debit`".equals(columnName) || "`Credit`".equals(columnName)) {
                        sb.append(columnName + " <=" + Double.valueOf(strList.get(1)) + ")");
                    } else {
                        sb.append(columnName + " <= '" + strList.get(1) + "')");
                    }
                }
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                break;
            default:
        }
        temp.setSql(sb.toString());
        String acctStrs = StringUtils.EMPTY;
        if ("EnterpriseAccount.`Name`".equals(columnName)) {
            acctStrs = "select AcctCode from EnterpriseAccount where " + temp.getSql();

            List<Map<String, Object>> result = jdbcTemplate.queryForList(acctStrs);

            int num = result.size();
            if (num > 0) {
                sb = new StringBuffer();
                sb.append("(");
                for (int idx = 0; idx < num; idx++) {
                    Map<String, Object> item = result.get(idx);

                    sb.append("AcctCode='" + item.get("AcctCode") + "'" + (idx < (num - 1) ? " or " : ")"));
                }
                temp.setSort(VoucherLevelEnum.FirstSort.getCode());
                temp.setSql(sb.toString());
            } else {
                temp.setSql("(AcctCode is null)");
                temp.setSort(VoucherLevelEnum.ThirdSort.getCode());
            }
        }
        return temp;
    }

    private String getSqlWhereCondition(List<VoucherTmplDto> tmplList) throws Exception {
        List<VoucherTmplDto> smallTemp = tmplList.stream().sorted(Comparator.comparing(VoucherTmplDto::getSort).reversed()).collect(Collectors.toList());
        int count = 1;
        int tCount = smallTemp.size();
        StringBuilder sqlWhere = new StringBuilder();
        if (tCount > 0) {
            sqlWhere.append(" where");
        }
        for (VoucherTmplDto t : smallTemp) {
            String logic = StringUtils.EMPTY;
            if (t.getLogic().equals(VoucherSearchEnum.S_OrAll)) {
                logic = " OR";
            } else if (t.getLogic().equals(VoucherSearchEnum.S_AndAll)) {
                logic = " AND";
            } else {
                throw new Exception("未处理的条件关系：" + t.getLogic().toString());
            }

            sqlWhere.append(" ").append(t.getSql()).append(count < tCount ? logic + " " : " ");
            count++;
        }
        return sqlWhere.toString();
    }
}