package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.alibaba.fastjson.JSON;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.KeyValueConfigResultType;
import pwc.taxtech.atms.dto.vatdto.BSPLFormulaDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.FormulaDataSourceDto;
import pwc.taxtech.atms.vat.entity.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


//**
/// 名称：发生净值
/// 功能：计算指定科目发生额（根据科目自带方向而定）
/// 备注：科目在贷方 则用贷方发生额-借方发生额
///       科目在借方 则用借方发生额-贷方发生额
///       本公式取VAT试算平衡表 总账（经过科目重分类和审计调整之后的）中累计发生额相关数据。
/// web admin中需要新增科目(科目代码待确定)，6001.04 税率为17%， 6001.05税率为13%，6001.06税率为11%，6001.07税率为6%
/// </summary>
/// <param name="accountType">科目类型——两种取值结果“0”标准科目，“1”企业科目；</param>
/// <param name="accountCode">科目代码；</param>
/// <param name="year">会计年份——“0”为当前年份，其他年份则为具体年份；</param>
/// <param name="period">会计期间——“0”为当前月份，1-12为自然月；"99"当年累计</param>
/// <param name="keyParam">摘要（未实现）</param>

public class FSJZ extends FunctionBase implements FreeRefFunction {

    public FSJZ(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        Date creatime = new Date();
        int accountType, year, period;
        String accountCode, keyParam;

        if (args.length < 5) {
            return null;
        }

        accountType = this.getIntParam(args[0], ec);
        accountCode = this.getStringParam(args[1], ec);
        year = this.getIntParam(args[2], ec);
        period = this.getIntParam(args[3], ec);
        keyParam = this.getStringParam(args[4], ec);

        String formulaExpression = "FSJZ(" + accountType + ",\"" + accountCode + "\"," + year + "," + period + ",\"" + keyParam + "\")";
        logger.debug(formulaExpression);

        if (year == 0) {
            year = formulaContext.getYear();
        }

        int[] yearOffset = {};
        period = FormulaHelper.getPeriod(period, yearOffset, formulaContext);
        year = year + yearOffset[0];

        if (accountType == 0) {
            //标准账套
            VatStandardAccount vatStandardAccount = SpringContextUtil.vatStandardAccountMapper.selectOneByAccountCode(accountCode);
            if (vatStandardAccount == null) {
                return null;
            }

            BalanceStdManual balanceStdManual = null;
            if (year != formulaContext.getYear()) {
                String dbName = SpringContextUtil.projectDao.getDbNameWithYearAndOrgID(formulaContext.getOrganizationID(), year);

                if (period == 99) {
                    balanceStdManual = SpringContextUtil.balanceStdManualMapper.selectOneWithAccountCodeAndDBName(accountCode, dbName, 12);
                } else {
                    balanceStdManual = SpringContextUtil.balanceStdManualMapper.selectOneWithAccountCodeAndDBName(accountCode, dbName, period);
                }
            } else {
                if (period == 99) {
                    balanceStdManual = SpringContextUtil.balanceStdManualMapper.selectOneWithAccountCode(accountCode, 12);
                } else {
                    balanceStdManual = SpringContextUtil.balanceStdManualMapper.selectOneWithAccountCode(accountCode, period);
                }
            }

            BigDecimal val = new BigDecimal(0);
            List<FormulaDataSourceDto> formulaDataSourceDtoList = new ArrayList<>();
            if (balanceStdManual != null) {
                List<VatEnterpriseAccount> vatEnterpriseAccountList = SpringContextUtil
                        .vatEnterpriseAccountMapper
                        .getListWithAccountCode(accountCode);
                if (period == 99) {
                    val = new BigDecimal(((balanceStdManual.getYearDebitNet() != null ? balanceStdManual.getYearDebitNet() : 0).doubleValue()
                            - (balanceStdManual.getYearCreditNet() != null ? balanceStdManual.getYearCreditNet() : 0).doubleValue())
                            * vatStandardAccount.getDirection());
                    for (VatEnterpriseAccount vatEnterpriseAccount : vatEnterpriseAccountList) {
                        BigDecimal amount = SpringContextUtil
                                .balanceMapper
                                .getSumOfBalance(vatEnterpriseAccount.getAcctCode(), vatStandardAccount.getDirection(), period);
                        BSPLFormulaDataSourceDto bsplFormulaDataSourceDto = new BSPLFormulaDataSourceDto();
                        bsplFormulaDataSourceDto.setAccountCode(vatEnterpriseAccount.getAcctCode());
                        bsplFormulaDataSourceDto.setAccountName(vatEnterpriseAccount.getName());
                        bsplFormulaDataSourceDto.setName("TrialBalanceDataSource");
                        bsplFormulaDataSourceDto.setAmount(amount);
                        formulaDataSourceDtoList.add(bsplFormulaDataSourceDto);
                    }
                } else {
                    val = new BigDecimal(((balanceStdManual.getDebitNet() != null ? balanceStdManual.getDebitNet() : 0).doubleValue()
                            - (balanceStdManual.getCreditNet() != null ? balanceStdManual.getCreditNet() : 0).doubleValue())
                            * vatStandardAccount.getDirection());
                    for (VatEnterpriseAccount vatEnterpriseAccount : vatEnterpriseAccountList) {
                        BigDecimal amount = SpringContextUtil
                                .balanceMapper
                                .getFirstOfBalance(vatEnterpriseAccount.getAcctCode(), vatStandardAccount.getDirection(), period);
                        BSPLFormulaDataSourceDto bsplFormulaDataSourceDto = new BSPLFormulaDataSourceDto();
                        bsplFormulaDataSourceDto.setAccountCode(vatEnterpriseAccount.getAcctCode());
                        bsplFormulaDataSourceDto.setAccountName(vatEnterpriseAccount.getName());
                        bsplFormulaDataSourceDto.setName("TrialBalanceDataSource");
                        bsplFormulaDataSourceDto.setAmount(amount);
                        formulaDataSourceDtoList.add(bsplFormulaDataSourceDto);
                    }
                    val = FormulaHelper.roundValue(val, KeyValueConfigResultType.Accounting, null, formulaContext) != null
                            ? FormulaHelper.roundValue(val, KeyValueConfigResultType.Accounting, null, formulaContext) : new BigDecimal(0);

                }
            }
            //todo: sava data
            //save datasource
            DataSource dataSource = new DataSource();
            Long dataSourceID = SpringContextUtil.distributedIDService.nextId();
            dataSource.setId(dataSourceID);
            dataSource.setType(1);
            dataSource.setName("ReportDataSource");
            dataSource.setAmount(val);
            dataSource.setDescription("");
            dataSource.setCreateBy("Admin");
            dataSource.setUpdateBy("Admin");
            dataSource.setCreateTime(creatime);
            dataSource.setUpdateTime(creatime);
            dataSource.setRowIndex(ec.getRowIndex());
            dataSource.setColumnIndex(ec.getColumnIndex());
            dataSource.setRowName("");
            dataSource.setColumnName("");
            SpringContextUtil.dataSourceMapper.insertSelective(dataSource);

            for (FormulaDataSourceDto aFormulaDataSourceDtoList : formulaDataSourceDtoList) {
                BSPLFormulaDataSourceDto bsplFormulaDataSourceDto = ((BSPLFormulaDataSourceDto) aFormulaDataSourceDtoList);
                DataSourceDetail dataSourceDetail = new DataSourceDetail();
                dataSourceDetail.setId(SpringContextUtil.distributedIDService.nextId());
                dataSourceDetail.setDataSourceId(dataSourceID);
                dataSourceDetail.setDataSourceType(FormulaDataSourceDetailType.BSPLFormulaDataSourceDto.getCode());
                dataSourceDetail.setItemValue(JSON.toJSONString(bsplFormulaDataSourceDto));
                SpringContextUtil.dataSourceDetailMapper.insertSelective(dataSourceDetail);
            }

            //save formulablock
            Long cellTemplateID = getCellTemplateID(period, ec);

            PeriodFormulaBlock periodFormulaBlock = new PeriodFormulaBlock();
            periodFormulaBlock.setId(SpringContextUtil.distributedIDService.nextId());
            periodFormulaBlock.setPeriod(period);
            periodFormulaBlock.setReportId(0L);
            periodFormulaBlock.setCellTemplateId(cellTemplateID);
            periodFormulaBlock.setFormulaExpression(formulaExpression);
            periodFormulaBlock.setData(val.toString());
            periodFormulaBlock.setDataSourceId(dataSourceID);
            periodFormulaBlock.setCreateBy("Admin");
            periodFormulaBlock.setCreateTime(creatime);
            periodFormulaBlock.setUpdateBy("Admin");
            periodFormulaBlock.setUpdateTime(creatime);
            SpringContextUtil.periodFormulaBlockMapper.insertSelective(periodFormulaBlock);

            return new NumberEval(val.doubleValue());
        } else {
            //企业账套
            List<VatEnterpriseAccount> vatEnterpriseAccountList = SpringContextUtil
                    .vatEnterpriseAccountDao
                    .getListByAccountCode(accountCode);

            if (vatEnterpriseAccountList == null) {
                return null;
            }

            Balance balance = null;
            String dbName = SpringContextUtil.projectDao.getDbNameWithYearAndOrgID(formulaContext.getOrganizationID(), year);
            if (period == 99) {
                balance = SpringContextUtil.balanceMapper.getFirstWithDbNameAndAccountCodeAndPeriod(dbName, accountCode, 12);
            } else {
                balance = SpringContextUtil.balanceMapper.getFirstWithDbNameAndAccountCodeAndPeriod(dbName, accountCode, period);
            }

            BigDecimal val = new BigDecimal(0);
            List<FormulaDataSourceDto> dsList = new ArrayList<>();
            if (balance != null) {
                if (period == 99) {
                    val = new BigDecimal(((balance.getYearDebitNet() != null ? balance.getYearDebitNet() : 0).doubleValue()
                            - (balance.getYearCreditNet() != null ? balance.getYearCreditNet() : 0).doubleValue())
                            * vatEnterpriseAccountList.get(0).getDirection());
                    BigDecimal amount = SpringContextUtil
                            .balanceMapper
                            .getSumOfBalance(accountCode, vatEnterpriseAccountList.get(0).getDirection(), period);

                    BSPLFormulaDataSourceDto bsplFormulaDataSourceDto = new BSPLFormulaDataSourceDto();
                    bsplFormulaDataSourceDto.setAccountCode(vatEnterpriseAccountList.get(0).getAcctCode());
                    bsplFormulaDataSourceDto.setAccountName(vatEnterpriseAccountList.get(0).getName());
                    bsplFormulaDataSourceDto.setName("TrialBalanceDataSource");
                    bsplFormulaDataSourceDto.setAmount(amount);
                    dsList.add(bsplFormulaDataSourceDto);
                } else {
                    val = new BigDecimal(((balance.getDebitNet() != null ? balance.getDebitNet() : 0).doubleValue()
                            - (balance.getCreditNet() != null ? balance.getCreditNet() : 0).doubleValue())
                            * vatEnterpriseAccountList.get(0).getDirection());
                    BigDecimal amount = SpringContextUtil
                            .balanceMapper
                            .getFirstOfBalance(accountCode, vatEnterpriseAccountList.get(0).getDirection(), period);

                    BSPLFormulaDataSourceDto bsplFormulaDataSourceDto = new BSPLFormulaDataSourceDto();
                    bsplFormulaDataSourceDto.setAccountCode(vatEnterpriseAccountList.get(0).getAcctCode());
                    bsplFormulaDataSourceDto.setAccountName(vatEnterpriseAccountList.get(0).getName());
                    bsplFormulaDataSourceDto.setName("TrialBalanceDataSource");
                    bsplFormulaDataSourceDto.setAmount(amount);
                    dsList.add(bsplFormulaDataSourceDto);
                }

                val = FormulaHelper.roundValue(val, KeyValueConfigResultType.Accounting, null, formulaContext) != null
                        ? FormulaHelper.roundValue(val, KeyValueConfigResultType.Accounting, null, formulaContext)
                        : new BigDecimal(0);
            }
            //todo: sava data
            //save datasource
            DataSource dataSource = new DataSource();
            Long dataSourceID = SpringContextUtil.distributedIDService.nextId();
            dataSource.setId(dataSourceID);
            dataSource.setType(1);
            dataSource.setName("ReportDataSource");
            dataSource.setAmount(val);
            dataSource.setDescription("");
            dataSource.setCreateBy("Admin");
            dataSource.setUpdateBy("Admin");
            dataSource.setCreateTime(creatime);
            dataSource.setUpdateTime(creatime);
            dataSource.setRowIndex(ec.getRowIndex());
            dataSource.setColumnIndex(ec.getColumnIndex());
            dataSource.setRowName("");
            dataSource.setColumnName("");
            SpringContextUtil.dataSourceMapper.insertSelective(dataSource);

            for (FormulaDataSourceDto aFormulaDataSourceDtoList : dsList) {
                BSPLFormulaDataSourceDto bsplFormulaDataSourceDto = ((BSPLFormulaDataSourceDto) aFormulaDataSourceDtoList);
                DataSourceDetail dataSourceDetail = new DataSourceDetail();
                dataSourceDetail.setId(SpringContextUtil.distributedIDService.nextId());
                dataSourceDetail.setDataSourceId(dataSourceID);
                dataSourceDetail.setDataSourceType(FormulaDataSourceDetailType.BSPLFormulaDataSourceDto.getCode());
                dataSourceDetail.setItemValue(JSON.toJSONString(bsplFormulaDataSourceDto));
                SpringContextUtil.dataSourceDetailMapper.insertSelective(dataSourceDetail);
            }

            //save formulablock
            Long cellTemplateID = getCellTemplateID(period, ec);

            PeriodFormulaBlock periodFormulaBlock = new PeriodFormulaBlock();
            periodFormulaBlock.setId(SpringContextUtil.distributedIDService.nextId());
            periodFormulaBlock.setPeriod(period);
            periodFormulaBlock.setReportId(0L);//todo:update reportID when report data generated
            periodFormulaBlock.setCellTemplateId(cellTemplateID);
            periodFormulaBlock.setFormulaExpression(formulaExpression);
            periodFormulaBlock.setData(val.toString());
            periodFormulaBlock.setDataSourceId(dataSourceID);
            periodFormulaBlock.setCreateBy("Admin");
            periodFormulaBlock.setCreateTime(creatime);
            periodFormulaBlock.setUpdateBy("Admin");
            periodFormulaBlock.setUpdateTime(creatime);
            SpringContextUtil.periodFormulaBlockMapper.insertSelective(periodFormulaBlock);
            return new NumberEval(val.doubleValue());
        }
    }
}
