package pwc.taxtech.atms.controller;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;

import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.RoleCategoryDto;
import pwc.taxtech.atms.dto.navtree.NavTreeDto;
import pwc.taxtech.atms.dto.role.RoleDisplayDto;
import pwc.taxtech.atms.dto.role.RoleDto;
import pwc.taxtech.atms.dto.role.RoleDtoTree;
import pwc.taxtech.atms.dto.user.DimensionRole;
import pwc.taxtech.atms.dto.user.DimensionUser;
import pwc.taxtech.atms.dto.user.UserRoleDimensionValueDto;
import pwc.taxtech.atms.dto.user.UserRoleDto;
import pwc.taxtech.atms.dto.user.UserRoleInfo;
import pwc.taxtech.atms.dto.user.UserRoleUpdateDto;
import pwc.taxtech.atms.entitiy.RoleCategory;
import pwc.taxtech.atms.service.RoleService;

@RunWith(MockitoJUnitRunner.class)
public class RoleControllerTest {

    @Mock
    private RoleService roleService;

    @InjectMocks
    private RoleController roleController = new RoleController();

    @Test
    public void getRoleTreeListNotNull() {
        List<NavTreeDto> navTreeDtos = new ArrayList<>();
        when(roleService.getRoleTreeList()).thenReturn(navTreeDtos);
        assertThat(roleController.getRoleTreeList()).isNotNull();
    }

    @Test
    public void commonCopyPropertiesTest() {
        RoleCategory roleCategory = new RoleCategory();
        roleCategory.setIsActive(false);
        RoleCategoryDto roleCategoryDto = new RoleCategoryDto();
        CommonUtils.copyProperties(roleCategory, roleCategoryDto);
        Short short1 = 0;
        Assert.assertEquals(roleCategoryDto.getIsActive(), short1);
        roleCategory.setIsActive(true);
        CommonUtils.copyProperties(roleCategory, roleCategoryDto);
        Short short2 = 1;
        Assert.assertEquals(roleCategoryDto.getIsActive(), short2);
    }

    @Test
    public void testFindFirst() {
        NavTreeDto navTreeDto = new NavTreeDto();
        navTreeDto.setLabel("test");
        List<NavTreeDto> navTreeDtos = new ArrayList<>();
        navTreeDtos.add(navTreeDto);
        Boolean findNav = navTreeDtos.stream().filter(oneNav -> oneNav.getLabel().equals("good")).findFirst()
                .isPresent();
        Boolean findNav2 = navTreeDtos.stream().anyMatch(oneNav -> oneNav.getLabel().equals("test"));
        Assert.assertFalse(findNav);
        Assert.assertTrue(findNav2);
        List<NavTreeDto> list = navTreeDtos.stream().filter(oneNav -> oneNav.getLabel().equals("good"))
                .collect(Collectors.toList());
        Assert.assertTrue(list.isEmpty());
    }

    @Test
    public void getAllOwnUserRoleListNotNull() {
        List<UserRoleInfo> userRoleInfos = new ArrayList<>();
        when(roleService.getAllOwnUserRoleList()).thenReturn(userRoleInfos);
        assertThat(roleController.getAllOwnUserRoleList()).isNotNull();
    }

    @Test
    public void getAllUserRoleListNotNull() {
        List<UserRoleInfo> userRoleInfos = new ArrayList<>();
        when(roleService.getAllUserRoleList()).thenReturn(userRoleInfos);
        assertThat(roleController.getAllUserRoleList()).isNotNull();
    }

    @Test
    public void getAllRoleListByUserIDTest() {
        Map<String, String> aMap = new HashMap<>();
        aMap.put("123", "456");
        Map<String, String> bMap = new HashMap<>();
        bMap.put("123", "456");
        List<Map<String, String>> list = new ArrayList<>();
        list.add(aMap);
        list.add(bMap);
        list = list.stream().distinct().collect(Collectors.toList());
        Assert.assertEquals(1, list.size());

        List<UserRoleDto> userRoleDtos = new ArrayList<>();
        when(roleService.getAllRoleListByUserID("userID", "serviceTypeID")).thenReturn(userRoleDtos);
        assertThat(roleController.getAllRoleListByUserID("userID", "serviceTypeID")).isNotNull();
    }

    @Test
    public void getRoleListTest() {
        List<RoleDtoTree> roleDtoTrees = new ArrayList<>();
        when(roleService.getRoleList()).thenReturn(roleDtoTrees);
        assertThat(roleController.getRoleList()).isNotNull();
    }

    @Test
    public void getUsersByRoleIDTest() {
        List<UserRoleDto> userList = new ArrayList<>();
        when(roleService.getUsersByRoleID("roleId")).thenReturn(userList);
        assertThat(roleController.getUsersByRoleID("roleId")).isNotNull();
    }

    @Test
    public void getExtraUsersByRoleIDTest() {
        List<UserRoleDto> userList = new ArrayList<>();
        when(roleService.getExtraUsersByRoleID("roleId")).thenReturn(userList);
        assertThat(roleController.getExtraUsersByRoleID("roleId")).isNotNull();
    }

    @Test
    public void removeUserRoleTest() {
        assertThat(roleController.removeUserRole("userID", Arrays.asList("roleId"), "2")).isTrue();
    }

    @Test
    public void getRoleListByRoleTypeTest() {
        List<RoleDto> roleList = new ArrayList<>();
        when(roleService.getRoleListByRoleType("2")).thenReturn(roleList);
        assertThat(roleController.getRoleListByRoleType("2")).isNotNull();
    }

    @Test
    public void addRoleCategoryTest() {
        assertThat(roleController.addRoleCategory("name", "id")).isTrue();
    }

    @Test
    public void updateRoleCategoryTest() {
        assertThat(roleController.updateRoleCategory("name", "id")).isTrue();
    }

    @Test
    public void deleteRoleCategoryTest() {
        when(roleService.deleteRoleCategory("id")).thenReturn(new OperationResultDto());
        assertThat(roleController.deleteRoleCategory("id")).isNotNull();
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Test
    public void addRoleTestWhenRoleNameDuplicate() {
        RoleDisplayDto roleDto = new RoleDisplayDto();
        Map result = new HashMap();
        result.put("data", -1);
        when(roleService.addRole(roleDto)).thenReturn("-1");
        assertThat(roleController.addRole(roleDto)).isEqualTo(result);
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Test
    public void addRoleTest() {
        RoleDisplayDto roleDto = new RoleDisplayDto();
        Map result = new HashMap();
        result.put("data", new ArrayList<RoleDtoTree>());
        when(roleService.addRole(roleDto)).thenReturn("roleId");
        assertThat(roleController.addRole(roleDto)).isEqualTo(result);
    }

    @Test
    public void validateRoleNameUniqueTest() {
        when(roleService.validateRoleNameUnique("roleName", "oldRoleName")).thenReturn(true);
        assertThat(roleController.validateRoleNameUnique("roleName", "oldRoleName")).isTrue();

    }

    @Test
    public void checkExistReferenceTest() {
        when(roleService.checkUserRole("roleID")).thenReturn(true);
        assertThat(roleController.checkExistReference("roleID")).isTrue();
    }

    @Test
    public void deleteRoleTest() {
        RoleDto roleDto = new RoleDto();
        when(roleService.getRoleList()).thenReturn(new ArrayList<RoleDtoTree>());
        assertThat(roleController.deleteRole(roleDto)).isNotNull();
    }

    @Test
    public void getUserRoleListTest() {
        UserRoleDimensionValueDto userRoleDimensionValueDto = new UserRoleDimensionValueDto();
        when(roleService.getUserRoleList(userRoleDimensionValueDto)).thenReturn(new ArrayList<>());
        assertThat(roleController.getUserRoleList(userRoleDimensionValueDto.getOrganizationID(),
                userRoleDimensionValueDto.getDimensionID(), userRoleDimensionValueDto.getDimensionValueID()))
                        .isNotNull();
    }

    @Test
    public void getAllRolePermissionTest() {
        when(roleService.getAllRolePermission("2")).thenReturn(new ArrayList<>());
        assertThat(roleController.getAllRolePermission("2")).isNotNull();
    }

    @Test
    public void updateUserRoleTest() {
        assertThat(roleController.updateUserRole(new UserRoleUpdateDto())).isTrue();
    }

    @Test
    public void getDimensionRoleUserListTest() {
        List<DimensionRole> dimensionRoles = new ArrayList<>();
        when(roleService.getDimensionRoleUserList("1", "2")).thenReturn(dimensionRoles);
        assertThat(roleController.getDimensionRoleUserList("1", "2")).isNotNull();
    }

    @Test
    public void getDimensionUserRoleListTest() {
        List<DimensionUser> dimensionUsers = new ArrayList<>();
        when(roleService.getDimensionUserRoleList("1", "2")).thenReturn(dimensionUsers);
        assertThat(roleController.getDimensionUserRoleList("1", "2")).isNotNull();
    }

    @Test
    public void getActiveUserRoleListByAreaIDTest() {
        List<UserRoleInfo> userRoleInfos = new ArrayList<>();
        when(roleService.getActiveUserRoleListByAreaID("1")).thenReturn(userRoleInfos);
        assertThat(roleController.getActiveUserRoleListByAreaID("1")).isNotNull();
    }
}
