package pwc.taxtech.atms.service.impl;

import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.common.CommonConstants;
import pwc.taxtech.atms.common.message.AreaMessage;
import pwc.taxtech.atms.common.message.AreaRegionMessage;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.arearegion.AreaRegionDto;
import pwc.taxtech.atms.dto.arearegion.AreaRegionInfo;
import pwc.taxtech.atms.dto.arearegion.CityInfo;
import pwc.taxtech.atms.entitiy.Area;
import pwc.taxtech.atms.entitiy.AreaExample;
import pwc.taxtech.atms.entitiy.AreaRegion;
import pwc.taxtech.atms.entitiy.AreaRegionExample;
import pwc.taxtech.atms.entitiy.OperationLogBasicData;
import pwc.taxtech.atms.entitiy.OperationLogBasicDataExample;
import pwc.taxtech.atms.entitiy.Region;
import pwc.taxtech.atms.security.JwtAuthenticationProvider;
import pwc.taxtech.atms.security.JwtAuthenticationToken;
import pwc.taxtech.atms.service.AreaRegionService;

public class AreaRegionServiceIT extends CommonIT {

    @Autowired
    private AreaRegionService areaRegionService;
    @Autowired
    private JwtAuthenticationProvider jwtAuthenticationProvider;

    @Before
    public void setUp() {
        cleanUpAllTables();
    }

    @Test
    public void findAreaRegionsByArea() {

        // area data
        Area area = new Area();
        area.setID("area");
        area.setName("areaName");
        area.setIsActive(true);
        areaMapper.insert(area);

        // region data
        for (int i = 0; i < 4; i++) {
            Region region = new Region();
            region.setID("region" + (i + 1));
            region.setName("RegionName" + (i + 1));
            region.setShortName("ShortName" + (i + 1));
            region.setMergerName("RegionMergerName" + (i + 1));
            region.setLevelType(1);
            region.setPinYin("RegionPinYin" + (i + 1));
            region.setLongitude(1.1f);
            region.setLatitude(2.2f);
            region.setIsActive(true);

            regionMapper.insert(region);
        }

        // areaRegion data
        for (int i = 0; i < 4; i++) {
            AreaRegion areaRegion = new AreaRegion();
            areaRegion.setID("areaRegion" + (i + 1));
            areaRegion.setAreaID("area");
            areaRegion.setRegionID("region" + (i + 1));

            areaRegionMapper.insert(areaRegion);
        }

        List<AreaRegion> areaRegionList = areaRegionService.findAreaRegionsByArea("area");
        Assert.assertEquals(4, areaRegionList.size());
        AreaRegion areaRegion = areaRegionList.get(0);
        Assert.assertEquals("area", areaRegion.getArea().getID());
        Assert.assertEquals("areaName", areaRegion.getArea().getName());
        Assert.assertEquals("region1", areaRegion.getRegion().getID());
        Assert.assertEquals("RegionMergerName1", areaRegion.getRegion().getMergerName());
    }

    @Test
    public void addTest() {
        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);

        AreaRegionInfo areaRegionInfo = new AreaRegionInfo();
        areaRegionInfo.setName("TestingArea");
        areaRegionInfo.setParentId(null);

        CityInfo beijingCity = new CityInfo();
        beijingCity.setID("110100");
        beijingCity.setParentId("110000");
        beijingCity.setName("北京市");
        CityInfo tianjingCity = new CityInfo();
        tianjingCity.setID("120100");
        tianjingCity.setParentId("120000");
        tianjingCity.setName("天津市");
        CityInfo shijiazhuangCity = new CityInfo();
        shijiazhuangCity.setID("130100");
        shijiazhuangCity.setParentId("130000");
        shijiazhuangCity.setName("石家庄市");
        CityInfo tangshanCity = new CityInfo();
        tangshanCity.setID("130200");
        tangshanCity.setParentId("130000");
        tangshanCity.setName("唐山市");
        areaRegionInfo.setCityList(Arrays.asList(beijingCity, tianjingCity, shijiazhuangCity, tangshanCity));

        // check result
        OperationResultDto operationResultDto = areaRegionService.add(areaRegionInfo);
        Assert.assertEquals(true, operationResultDto.getResult());

        // check area
        AreaExample areaExample = new AreaExample();
        areaExample.createCriteria().andNameEqualTo("TestingArea");
        List<Area> areas = areaMapper.selectByExample(areaExample);
        Assert.assertEquals("TestingArea", areas.get(0).getName());
        Assert.assertEquals(CommonConstants.ACTIVE_STATUS, areas.get(0).getIsActive());
        Assert.assertNull(areas.get(0).getParentID());
        String expectedAreaId = areas.get(0).getID();

        // check areaRegion
        AreaRegionExample areaRegionExample = new AreaRegionExample();
        areaRegionExample.createCriteria().andAreaIDEqualTo(expectedAreaId);
        areaRegionExample.setOrderByClause("RegionID ASC");
        List<AreaRegion> areaRegionList = areaRegionMapper.selectByExample(areaRegionExample);
        Assert.assertEquals(7, areaRegionList.size());
        Assert.assertEquals("110000", areaRegionList.get(0).getRegionID());
        Assert.assertEquals("120100", areaRegionList.get(3).getRegionID());
        Assert.assertEquals("130000", areaRegionList.get(4).getRegionID());
    }

    @Test
    public void addTestWithDuplicateName() {

        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);

        AreaRegionInfo areaRegionInfo = new AreaRegionInfo();
        areaRegionInfo.setName("TestingArea");
        areaRegionInfo.setParentId(null);
        areaRegionService.add(areaRegionInfo);
        // check result
        OperationResultDto operationResultDto = areaRegionService.add(areaRegionInfo);
        Assert.assertEquals(false, operationResultDto.getResult());
        Assert.assertEquals(AreaRegionMessage.AreaRegionRepeat, operationResultDto.getResultMsg());
    }

    @Test
    public void updateTestNameChangeOnly() {

        // data preparation
        execSqlFile("sql/area_data_update_test.sql");
        execSqlFile("sql/areaRegion_data_update_test.sql");
        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);

        AreaRegionInfo areaRegionInfo = new AreaRegionInfo();
        areaRegionInfo.setID("bc08cf92-c75c-4fa3-ac66-fe10be346ceb");
        areaRegionInfo.setName("TOP22222");
        areaRegionInfo.setParentId("7c360a44-7a17-4ca0-9c2a-a88cde7670d5");

        OperationResultDto operationResultDto = areaRegionService.update(areaRegionInfo);

        // check result
        Assert.assertEquals(true, operationResultDto.getResult());

        // check area
        Area area = areaMapper.selectByPrimaryKey("bc08cf92-c75c-4fa3-ac66-fe10be346ceb");
        Assert.assertEquals("TOP22222", area.getName());
        Assert.assertEquals("7c360a44-7a17-4ca0-9c2a-a88cde7670d5", area.getParentID());

        // check operation log
        OperationLogBasicDataExample example = new OperationLogBasicDataExample();
        example.setOrderByClause("CreateTime ASC");
        List<OperationLogBasicData> operationLogs = operationLogBasicDataMapper.selectByExample(example);
        Assert.assertEquals(1, operationLogs.size());
        Assert.assertEquals("Name", operationLogs.get(0).getOperationContent());
        Assert.assertEquals("TOP22222", operationLogs.get(0).getOperationObject());
        Assert.assertEquals("TOP22222", operationLogs.get(0).getUpdateState());
        Assert.assertEquals("TOP11111", operationLogs.get(0).getOriginalState());
    }

    @Test
    public void updateTestNameAndRegion() {

        // data preparation
        execSqlFile("sql/area_data_update_test.sql");
        execSqlFile("sql/region_data.sql");
        execSqlFile("sql/areaRegion_data_update_test.sql");
        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);

        AreaRegionInfo areaRegionInfo = new AreaRegionInfo();
        areaRegionInfo.setID("bc08cf92-c75c-4fa3-ac66-fe10be346ceb");
        areaRegionInfo.setName("TOP22222");
        areaRegionInfo.setParentId("7c360a44-7a17-4ca0-9c2a-a88cde7670d5");
        CityInfo beijingCity = new CityInfo();
        beijingCity.setID("110100");
        beijingCity.setParentId("110000");
        beijingCity.setName("北京市");
        CityInfo tianjingCity = new CityInfo();
        tianjingCity.setID("120100");
        tianjingCity.setParentId("120000");
        tianjingCity.setName("天津市");
        CityInfo shijiazhuangCity = new CityInfo();
        shijiazhuangCity.setID("130100");
        shijiazhuangCity.setParentId("130000");
        shijiazhuangCity.setName("石家庄市");
        CityInfo tangshanCity = new CityInfo();
        tangshanCity.setID("130200");
        tangshanCity.setParentId("130000");
        tangshanCity.setName("唐山市");
        areaRegionInfo.setCityList(Arrays.asList(beijingCity, tianjingCity, shijiazhuangCity, tangshanCity));
        OperationResultDto operationResultDto = areaRegionService.update(areaRegionInfo);

        // check result
        Assert.assertEquals(true, operationResultDto.getResult());

        // check area
        Area area = areaMapper.selectByPrimaryKey("bc08cf92-c75c-4fa3-ac66-fe10be346ceb");
        Assert.assertEquals("TOP22222", area.getName());
        Assert.assertEquals("7c360a44-7a17-4ca0-9c2a-a88cde7670d5", area.getParentID());

        // check area region
        AreaRegionExample areaRegionExample = new AreaRegionExample();
        areaRegionExample.createCriteria().andAreaIDEqualTo("bc08cf92-c75c-4fa3-ac66-fe10be346ceb");
        areaRegionExample.setOrderByClause("RegionID ASC");
        List<AreaRegion> areaRegionList = areaRegionMapper.selectByExample(areaRegionExample);
        Assert.assertEquals(7, areaRegionList.size());
        Assert.assertEquals("110000", areaRegionList.get(0).getRegionID());
        Assert.assertEquals("120100", areaRegionList.get(3).getRegionID());
        Assert.assertEquals("130000", areaRegionList.get(4).getRegionID());

        // check operation log
        OperationLogBasicDataExample example = new OperationLogBasicDataExample();
        example.setOrderByClause("CreateTime ASC");
        List<OperationLogBasicData> operationLogs = operationLogBasicDataMapper.selectByExample(example);
        Assert.assertEquals(2, operationLogs.size());
        Assert.assertEquals("Name", operationLogs.get(0).getOperationContent());
        Assert.assertEquals("TOP22222", operationLogs.get(0).getOperationObject());
        Assert.assertEquals("TOP22222", operationLogs.get(0).getUpdateState());
        Assert.assertEquals("TOP11111", operationLogs.get(0).getOriginalState());
        Assert.assertEquals(AreaMessage.AreaRegion, operationLogs.get(1).getOperationContent());
        Assert.assertEquals("TOP22222", operationLogs.get(1).getOperationObject());
        Assert.assertEquals("北京市,天津市,韶关市,深圳市", operationLogs.get(1).getOriginalState());
        Assert.assertEquals("北京市,天津市,石家庄市,唐山市", operationLogs.get(1).getUpdateState());

    }

    @Test
    public void getCitiesTestWithRegion() {
        execSqlFile("sql/fullAreaRegion_data_update_test.sql");

        List<AreaRegionDto> areaRegionDtos = areaRegionService.getCities("110000");
        Assert.assertEquals("北京市", areaRegionDtos.get(0).getRegionName());

        areaRegionDtos = areaRegionService.getCities("130000");
        Assert.assertEquals(11, areaRegionDtos.size());

        areaRegionDtos = areaRegionService.getCities("640000");
        Assert.assertEquals("吴忠市", areaRegionDtos.get(2).getRegionName());

    }

    @Test
    public void getProvincesTestWithRegion() {
        execSqlFile("sql/fullAreaRegion_data_update_test.sql");

        List<AreaRegionDto> areaRegionDtos = areaRegionService.getProvinces();

        // 应能找到省级地名 130000 河北省
        Assert.assertEquals(true, areaRegionDtos.stream().anyMatch(s -> "130000".equals(s.getRegionID())));

        // 应无法找到地级市名 110100 海淀区
        Assert.assertEquals(false, areaRegionDtos.stream().anyMatch(s -> "110100".equals(s.getRegionID())));
    }
}
