package pwc.taxtech.atms.service.impl;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.nutz.lang.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.OperationAction;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.dao.BusinessUnitMapper;
import pwc.taxtech.atms.dao.OperationLogBasicDataMapper;
import pwc.taxtech.atms.dto.BusinessUnitDto;
import pwc.taxtech.atms.dto.BusinessUnitInputDto;
import pwc.taxtech.atms.dto.IdModel;
import pwc.taxtech.atms.entitiy.BusinessUnit;
import pwc.taxtech.atms.entitiy.OperationLogBasicData;
import pwc.taxtech.atms.entitiy.OperationLogBasicDataExample;
import pwc.taxtech.atms.security.JwtAuthenticationProvider;
import pwc.taxtech.atms.security.JwtAuthenticationToken;
import pwc.taxtech.atms.service.BusinessUnitService;

public class BusinessUnitServiceIT extends CommonIT {
    private static Logger logger = LoggerFactory.getLogger(BusinessUnitServiceIT.class);

    private static final String UUID1 = CommonUtils.getUUID();
    private static final String UUID2 = CommonUtils.getUUID();
    private static final String UUID3 = CommonUtils.getUUID();

    @Autowired
    private BusinessUnitService businessUnitService;

    @Autowired
    private BusinessUnitMapper businessUnitMapper;

    @Autowired
    private JwtAuthenticationProvider jwtAuthenticationProvider;

    @Autowired
    private OperationLogBasicDataMapper operationLogBasicDataMapper;

    @Autowired
    private AuthUserHelper authUserHelper;

    private BusinessUnit businessUnit1;

    private BusinessUnit businessUnit2;

    private BusinessUnitDto businessUnitDto1;

    private BusinessUnitDto businessUnitDto2;

    private IdModel idModel1;

    @Before
    public void init() {
        cleanUpAllTables();
        // for get and delete test
        businessUnit1 = new BusinessUnit();
        businessUnit1.setID(UUID1);
        businessUnit1.setIsActive(true);
        businessUnit1.setName("businessUnit1");
        businessUnit1.setCreateTime(new Date());
        businessUnit1.setUpdateTime(new Date());
        businessUnitMapper.insert(businessUnit1);

        idModel1 = new IdModel();
        idModel1.setID(UUID1);

        // for get and update test
        businessUnit2 = new BusinessUnit();
        businessUnit2.setID(UUID2);
        businessUnit2.setIsActive(true);
        businessUnit2.setName("businessUnit2");
        businessUnit2.setCreateTime(new Date());
        businessUnit2.setUpdateTime(new Date());
        businessUnitMapper.insert(businessUnit2);

        businessUnitDto1 = new BusinessUnitDto();
        businessUnitDto1.setID(UUID2);
        businessUnitDto1.setIsActive(false);
        businessUnitDto1.setName("businessUnitDto1");

        // for add test
        businessUnitDto2 = new BusinessUnitDto();
        businessUnitDto2.setID(UUID3);
        businessUnitDto2.setIsActive(true);
        businessUnitDto2.setName("businessUnitDto2");

        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);
    }

    @Test
    public void getBusinessUnitsTest() {
        logger.debug("Start to test getBusinessUnits");
        List<BusinessUnitDto> businessUnitDtoList = businessUnitService.getBusinessUnits();
        Assert.assertNotNull(businessUnitDtoList);
        Assert.assertTrue(!businessUnitDtoList.isEmpty());
        for (BusinessUnitDto businessUnitDto : businessUnitDtoList) {
            if (UUID1.equals(businessUnitDto.getID())) {
                compareBusinessUnits(businessUnit1, businessUnitDto);
            }
            if (UUID2.equals(businessUnitDto.getID())) {
                compareBusinessUnits(businessUnit2, businessUnitDto);
            }
        }
    }

    @Test
    public void deleteBusinessUnitTest() {
        logger.debug("Start to test deleteBusinessUnit");
        Integer result = businessUnitService.deleteBusinessUnit(idModel1);
        Assert.assertEquals("1", Integer.toString(result));
        BusinessUnit businessUnitAfterDelete = businessUnitMapper.selectByPrimaryKey(UUID1);
        Assert.assertNull(businessUnitAfterDelete);

        logger.debug("Start to check basic data operation log for delete");
        OperationLogBasicData operationLogBasicData = findOperationLogBasicData(businessUnit1.getName(),
                OperationAction.Delete.toString()).get(0);
        compareOperationLogBasicData(businessUnit1.getName(), OperationAction.Delete.toString(), operationLogBasicData);
    }

    @Test
    public void updateBusinessUnitsTest() {
        logger.debug("Start to test updateBusinessUnits");
        findByNameAndIsActive(businessUnit2.getName(), businessUnit2.getIsActive());
        Assert.assertFalse(businessUnit2.getName().equals(businessUnitDto1.getName()));
        Assert.assertFalse(businessUnit2.getIsActive().equals(businessUnitDto1.getIsActive()));
        List<BusinessUnitDto> businessUnitDtos = new ArrayList<>();
        businessUnitDtos.add(businessUnitDto1);
        Boolean result = businessUnitService.updateBusinessUnits(businessUnitDtos);
        Assert.assertTrue(result);
        BusinessUnit updatedBusinessUnit = findByNameAndIsActive(businessUnitDto1.getName(),
                businessUnitDto1.getIsActive());
        compareBusinessUnits(updatedBusinessUnit, businessUnitDto1);

        logger.debug("Start to check basic data operation log for update");
        List<OperationLogBasicData> operationLogBasicDatas = findOperationLogBasicData(businessUnit2.getName(),
                OperationAction.Update.toString());
        Assert.assertTrue(operationLogBasicDatas.size() > 1);
        // sort operation content: isActive and name
        operationLogBasicDatas.sort(new Comparator<OperationLogBasicData>() {
            @Override
            public int compare(OperationLogBasicData o1, OperationLogBasicData o2) {
                return o1.getOperationContent().compareTo(o2.getOperationContent());
            }
        });

        // isActive
        OperationLogBasicData logForFieldIsActive = operationLogBasicDatas.get(0);
        compareOperationLogForUpdate("ISACTIVE", Strings.upperFirst(businessUnit2.getIsActive().toString()),
                Strings.upperFirst(businessUnitDto1.getIsActive().toString()), businessUnit2.getName(),
                OperationAction.Update.toString(), logForFieldIsActive);

        // name
        OperationLogBasicData logForFieldName = operationLogBasicDatas.get(1);
        compareOperationLogForUpdate("NAME", businessUnit2.getName(), businessUnitDto1.getName(),
                businessUnit2.getName(), OperationAction.Update.toString(), logForFieldName);
    }

    @Test
    public void addBusinessUnitTest() {
        logger.debug("Start to test addBusinessUnit");
        List<BusinessUnitInputDto> businessUnitDtos = new ArrayList<>();
        BusinessUnitInputDto item = new BusinessUnitInputDto();
        CommonUtils.copyProperties(businessUnitDto2, item);
        businessUnitDtos.add(item);
        businessUnitService.addBusinessUnits(businessUnitDtos);
        BusinessUnit businessUnit = findByNameAndIsActive(businessUnitDto2.getName(), businessUnitDto2.getIsActive());
        Assert.assertEquals(businessUnit.getIsActive(), businessUnitDto2.getIsActive());
        Assert.assertEquals(businessUnit.getName(), businessUnitDto2.getName());

        logger.debug("Start to check basic data operation log for add");
        OperationLogBasicData operationLogBasicData = findOperationLogBasicData(businessUnitDto2.getName(),
                OperationAction.New.toString()).get(0);
        compareOperationLogBasicData(businessUnitDto2.getName(), OperationAction.New.toString(), operationLogBasicData);
    }

    private void compareBusinessUnits(BusinessUnit businessUnit, BusinessUnitDto businessUnitDto) {
        Assert.assertEquals(businessUnit.getID(), businessUnitDto.getID());
        Assert.assertEquals(businessUnit.getIsActive(), businessUnitDto.getIsActive());
        Assert.assertEquals(businessUnit.getName(), businessUnitDto.getName());
    }

    private List<OperationLogBasicData> findOperationLogBasicData(String name, String action) {
        OperationLogBasicDataExample example = new OperationLogBasicDataExample();
        pwc.taxtech.atms.entitiy.OperationLogBasicDataExample.Criteria criteria = example.createCriteria();
        if (!OperationAction.Delete.toString().equals(action)) {
            criteria.andOperationObjectEqualTo(name);
        }
        criteria.andOperationUserEqualTo(authUserHelper.getCurrentAuditor());
        criteria.andModuleNameEqualTo(OperationModule.BasicDataBusinessUnit.toString());
        criteria.andOperationActionEqualTo(action);
        List<OperationLogBasicData> operationLogBasicDataList = operationLogBasicDataMapper.selectByExample(example);
        Assert.assertNotNull(operationLogBasicDataList);
        Assert.assertTrue(!operationLogBasicDataList.isEmpty());
        return operationLogBasicDataList;
    }

    private void compareOperationLogForUpdate(String content, String originState, String updateState, String name,
            String action, OperationLogBasicData logForFieldIsActive) {
        Assert.assertEquals(content, logForFieldIsActive.getOperationContent().toUpperCase());
        Assert.assertEquals(originState, logForFieldIsActive.getOriginalState());
        Assert.assertEquals(updateState, logForFieldIsActive.getUpdateState());
        compareOperationLogBasicData(name, action, logForFieldIsActive);
    }

    private void compareOperationLogBasicData(String name, String action, OperationLogBasicData operationLogBasicData) {
        if (!OperationAction.Delete.toString().equals(action)) {
            Assert.assertEquals(name, operationLogBasicData.getOperationObject());
        }
        Assert.assertEquals(authUserHelper.getCurrentAuditor(), operationLogBasicData.getOperationUser());
        Assert.assertEquals(OperationModule.BasicDataBusinessUnit.toString(), operationLogBasicData.getModuleName());
        Assert.assertEquals(action, operationLogBasicData.getOperationAction());
    }

    private BusinessUnit findByNameAndIsActive(String name, Boolean isActive) {
        List<BusinessUnit> actualBusinessUnits = businessUnitService.findByNameAndIsActive(name, isActive);
        Assert.assertNotNull(actualBusinessUnits);
        Assert.assertTrue(!actualBusinessUnits.isEmpty());
        BusinessUnit businessUnit = actualBusinessUnits.get(0);
        return businessUnit;
    }


}
