package pwc.taxtech.atms.service.impl;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.CommonTestUtils;
import pwc.taxtech.atms.dto.dimension.DimensionDto;
import pwc.taxtech.atms.service.DemoService;

public class DemoServiceIT extends CommonIT {

    @Autowired
    private DemoService demoService;

    @Before
    public void setUp() {
        dimensionMapper.deleteByExample(null);
        dimensionValueMapper.deleteByExample(null);
        dimensionValueOrgMapper.deleteByExample(null);
        statisticAttributeMapper.deleteByExample(null);
        statisticAttributeDimensionMapper.deleteByExample(null);
        operationLogOrganizationMapper.deleteByExample(null);
        cleanUpOperationLogBasicDataTable();
    }

    @Test
    public void testOk() {
        CommonTestUtils.initTestAuth();
        List<DimensionDto> modelList = new ArrayList<>();
        {
            DimensionDto model = new DimensionDto();
            model.setName("单元测试name1");
            model.setIsActive(true);
            modelList.add(model);
        }
        {
            DimensionDto model = new DimensionDto();
            model.setName("单元测试name2");
            model.setIsActive(true);
            modelList.add(model);
        }
        assertThat(demoService.addDimensionWithoutTranction(modelList).getResult()).isTrue();

        assertThat(dimensionMapper.countByExample(null)).isEqualTo(2);

    }

    @Test
    public void testFailed() {
        CommonTestUtils.initTestAuth();
        List<DimensionDto> modelList = new ArrayList<>();
        {
            DimensionDto model = new DimensionDto();
            model.setName("单元测试name1");
            model.setIsActive(true);
            modelList.add(model);
        }
        {
            DimensionDto model = new DimensionDto();
            // 把name设置为null，数据库会报错
            model.setName(null);
            model.setIsActive(true);
            modelList.add(model);
        }
        assertThat(demoService.addDimensionWithoutTranction(modelList).getResult()).isFalse();

        // 没有回滚，有一条记录
        assertThat(dimensionMapper.countByExample(null)).isEqualTo(1);

    }

    @Test
    public void addDimensionWithTranction() {
        CommonTestUtils.initTestAuth();
        List<DimensionDto> modelList = new ArrayList<>();
        {
            DimensionDto model = new DimensionDto();
            model.setName("单元测试name1");
            model.setIsActive(true);
            modelList.add(model);
        }
        {
            DimensionDto model = new DimensionDto();
            // 把name设置为null，数据库会报错
            model.setName(null);
            model.setIsActive(true);
            modelList.add(model);
        }
        assertThat(demoService.addDimensionWithTranction(modelList).getResult()).isFalse();

        // 回滚了，有0条记录
        assertThat(dimensionMapper.countByExample(null)).isEqualTo(0);

    }

    @Test
    public void addDimensionWithTranctionType2() {
        CommonTestUtils.initTestAuth();
        List<DimensionDto> modelList = new ArrayList<>();
        {
            DimensionDto model = new DimensionDto();
            model.setName("单元测试name1");
            model.setIsActive(true);
            modelList.add(model);
        }
        {
            DimensionDto model = new DimensionDto();
            // 把name设置为null，数据库会报错
            model.setName(null);
            model.setIsActive(true);
            modelList.add(model);
        }

        assertThatThrownBy(() -> {
            demoService.addDimensionWithTranctionType2(modelList);
        }).isInstanceOf(DataIntegrityViolationException.class);

        // 回滚了，有0条记录
        assertThat(dimensionMapper.countByExample(null)).isEqualTo(0);

    }

    // @Test
    // public void failedTest() {
    // throw new RuntimeException();
    // }
}
