package pwc.taxtech.atms.service.impl;

import static org.assertj.core.api.Assertions.assertThat;

import java.util.Date;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.CommonTestUtils;
import pwc.taxtech.atms.constant.DimensionConstant;
import pwc.taxtech.atms.dto.DimensionValueOrgDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.dimension.DimensinTypeOrgDto;
import pwc.taxtech.atms.dto.dimension.DimensionDto;
import pwc.taxtech.atms.dto.dimension.DimensionTypeEnum;
import pwc.taxtech.atms.dto.dimension.DimensionValueDto;
import pwc.taxtech.atms.dto.navtree.DevTreeDto;
import pwc.taxtech.atms.entitiy.Dimension;
import pwc.taxtech.atms.entitiy.DimensionValue;
import pwc.taxtech.atms.entitiy.OperationLogBasicData;
import pwc.taxtech.atms.entitiy.OperationLogOrganization;
import pwc.taxtech.atms.entitiy.StatisticAttribute;
import pwc.taxtech.atms.entitiy.StatisticAttributeDimension;
import pwc.taxtech.atms.service.DimensionService;

public class DimensionServiceIT extends CommonIT {
    @Autowired
    private DimensionService dimensionService;

    @Before
    public void setUp() {
        dimensionMapper.deleteByExample(null);
        dimensionValueMapper.deleteByExample(null);
        dimensionValueOrgMapper.deleteByExample(null);
        statisticAttributeMapper.deleteByExample(null);
        statisticAttributeDimensionMapper.deleteByExample(null);
        operationLogOrganizationMapper.deleteByExample(null);
        cleanUpOperationLogBasicDataTable();
    }

    @Test
    public void getDimensionStatics() {
        execSqlFile("sql/dimension/dimension_data.sql");
        execSqlFile("sql/dimension/dimensionValue_data.sql");
        List<DimensionDto> resultList = dimensionService.getDimensionStatics();
        assertThat(resultList).isNotEmpty();
        assertThat(resultList).extracting("name").contains("机构", "事业部", "覆盖区域");
        assertThat(resultList).extracting("attributeID").contains(DimensionConstant.OrganizationStructureID,
                DimensionConstant.BusinessUnitID, DimensionConstant.AreaID);
        // logger.debug("print resultList:{}", JSON.toJSONString(resultList, true));

    }

    @Test
    public void getAllDimensionList() {
        execSqlFile("sql/dimension/dimension_data.sql");
        execSqlFile("sql/dimension/dimensionValue_data.sql");
        List<DimensionDto> resultList = dimensionService.getAllDimensionList();
        assertThat(resultList).isNotEmpty();

    }

    @Test(expected = RuntimeException.class)
    public void updateDimensionThenThrow() {
        dimensionService.updateDimension(null);

    }

    @SuppressWarnings("rawtypes")
    @Test
    public void updateDimensionOk() {
        CommonTestUtils.initTestAuth();
        execSqlFile("sql/dimension/dimension_data.sql");
        execSqlFile("sql/dimension/dimensionValue_data.sql");

        final String id = "6df98a1b-7330-4cbb-90ec-d6290267dcbc";
        DimensionDto input = new DimensionDto();
        input.setID(id);
        input.setName("事业1_updated"); // 原来是“事业1”
        input.setIsActive(true);
        input.setIsMandatory(true);
        OperationResultDto result = dimensionService.updateDimension(input);
        assertThat(result).isNotNull().matches(x -> x.getResult());
        Dimension recordUpdated = dimensionMapper.selectByPrimaryKey(id);
        assertThat(recordUpdated).isNotNull().matches(x -> x.getName().equals("事业1_updated"));

    }

    @Test(expected = RuntimeException.class)
    public void getDimensionValueListFailed() {
        dimensionService.getDimensionValueList(null);
    }

    @Test
    public void getDimensionValueList() {
        String did = "0ee9c871-fd1d-4498-a1b6-acef002d3bed";
        execSqlFile("sql/dimension/dimensionValue_data.sql");
        List<DimensionValueDto> resultList = dimensionService.getDimensionValueList(did);
        assertThat(resultList).isNotEmpty().extracting("name").contains("tttt6");
    }

    @Test(expected = RuntimeException.class)
    public void getDimensionByIdFailed() {
        dimensionService.getDimensionById(null);
    }

    @Test
    public void getDimensionByIdOk() {
        execSqlFile("sql/dimension/dimension_data.sql");
        String did = DimensionConstant.OrgSubChildrenID;
        DimensinTypeOrgDto result = dimensionService.getDimensionById(did);
        assertThat(result).isNotNull().matches(x -> x.getName().equals("机构"))
                .matches(x -> x.getDimensionType().equals(DimensionTypeEnum.OrgSubChildren.value()));
        // logger.debug("print resultList:{}", JSON.toJSONString(result, true));
    }

    @Test
    public void findByIsActiveTrue() {
        execSqlFile("sql/dimension/dimension_data.sql");
        List<Dimension> result = dimensionService.findByIsActiveTrue();
        assertThat(result).isNotEmpty();
    }

    @Test
    public void getAllDimensionOrgList() {
        execSqlFile("sql/dimension/dimension_data.sql");
        execSqlFile("sql/dimension/dimensionValue_data.sql");
        execSqlFile("sql/dimension/dimensionValueOrg_data.sql");
        List<DimensionValueOrgDto> result = dimensionService.getAllDimensionOrgList();
        assertThat(result).isNotEmpty();
        assertThat(result).extracting("dimensionName").contains("aaa", "frankTest");
        // logger.debug("print result:{}", JSON.toJSONString(result, true));
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
    @Test
    public void getDevDimensionTreeList() {
        execSqlFile("sql/dimension/dimension_data.sql");
        execSqlFile("sql/dimension/dimensionValue_data.sql");
        List<DevTreeDto> result = dimensionService.getDevDimensionTreeList();
        assertThat(result).isNotEmpty();
        assertThat(result).extracting("text").contains("事业部", "产品线");
        List<DevTreeDto> items = result.stream().filter(x -> x.getText().equals("产品线")).findFirst().get().getItems();
        assertThat(items).isNotEmpty().extracting("text").contains("产品线1");
        // assertThat(result).filteredOn(x->{x.getText().equals("产品线")}).

        // logger.debug("print result:{}", JSON.toJSONString(result, true));
    }

    @Test
    public void addDimension() {
        CommonTestUtils.initTestAuth();

        insertBaseDataForStatisticAttribute();

        DimensionDto model = new DimensionDto();
        model.setName("单元测试name1");
        model.setIsActive(true);
        assertThat(dimensionService.addDimension(model).getResult()).isTrue();

        List<Dimension> dlist = dimensionMapper.selectByExample(null);
        assertThat(dlist).isNotEmpty().extracting("name").contains("单元测试name1");
        final String targetDimensionId = dlist.stream().filter(x -> x.getName().equals("单元测试name1")).findFirst().get()
                .getID();

        List<StatisticAttributeDimension> list = statisticAttributeDimensionMapper.selectByExample(null);
        assertThat(list).isNotEmpty().extracting("statisticsAttributeID").contains(DimensionConstant.OrgSubChildrenID,
                DimensionConstant.UserID, DimensionConstant.AreaID);

        List<OperationLogBasicData> logs = operationLogBasicDataMapper.selectByExample(null);
        assertThat(logs).isNotEmpty().extracting("operationObject").contains("单元测试name1-机构", "单元测试name1-用户数",
                "单元测试name1-覆盖区域", "单元测试name1");

        DimensionValueDto dvdto = new DimensionValueDto();
        dvdto.setIsActive(true);
        dvdto.setName("单元测试valueName1");
        dvdto.setDimensionID(targetDimensionId);
        assertThat(dimensionService.addDimensionValue(dvdto).getResult()).isTrue();

        List<DimensionValue> dvlist = dimensionValueMapper.selectByExample(null);
        assertThat(dvlist).isNotEmpty().extracting("name").contains("单元测试valueName1");
        DimensionValue entity = dvlist.stream().filter(x -> "单元测试valueName1".equals(x.getName())).findFirst()
                .orElse(null);

        List<OperationLogOrganization> logs2 = operationLogOrganizationMapper.selectByExample(null);
        assertThat(logs2).isNotEmpty().extracting("operationObject").contains("单元测试name1-单元测试valueName1");

        List<OperationLogBasicData> logs3 = operationLogBasicDataMapper.selectByExample(null);
        assertThat(logs3).isNotEmpty().extracting("operationObject").contains("单元测试name1-单元测试valueName1");

        DimensionValueDto updateModel = new DimensionValueDto();
        updateModel.setIsActive(true);
        updateModel.setName("单元测试valueName1_updated");
        updateModel.setDimensionID(entity.getDimensionID());
        updateModel.setID(entity.getID());
        assertThat(dimensionService.updateDimensionValue(updateModel).getResult()).isTrue();

        List<OperationLogBasicData> logs4 = operationLogBasicDataMapper.selectByExample(null);
        assertThat(logs4).isNotEmpty().extracting("updateState").contains("单元测试valueName1_updated");
    }

    private void insertBaseDataForStatisticAttribute() {
        // 准备三条StatisticAttribute数据
        // select * from StatisticAttribute where id in
        // ('c61a5bd6-a996-4952-9869-d053995237e7','c61a5bd6-a996-4952-9869-d053996237e8','c61a5bd6-a996-4952-9869-d053995237e6')
        {
            StatisticAttribute record = newRecord();
            record.setID(DimensionConstant.OrgSubChildrenID);
            record.setName("机构");
            statisticAttributeMapper.insert(record);
        }
        {
            StatisticAttribute record = newRecord();
            record.setID(DimensionConstant.AreaID);
            record.setName("覆盖区域");
            statisticAttributeMapper.insert(record);
        }
        {
            StatisticAttribute record = newRecord();
            record.setID(DimensionConstant.UserID);
            record.setName("用户数");
            statisticAttributeMapper.insert(record);
        }
    }

    private StatisticAttribute newRecord() {
        StatisticAttribute record = new StatisticAttribute();
        record.setIsActive(true);
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        return record;
    }

}
