package pwc.taxtech.atms.service.impl;


import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.CommonTestUtils;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.message.EnterpriseAccountMessage;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.ValidateInfoDto;
import pwc.taxtech.atms.dto.epaccount.AccountMappingDto;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountAndValidateInfo;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountDto;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountSetDto;
import pwc.taxtech.atms.entitiy.AccountMapping;
import pwc.taxtech.atms.entitiy.EnterpriseAccount;
import pwc.taxtech.atms.entitiy.EnterpriseAccountExample;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSet;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSetOrg;
import pwc.taxtech.atms.entitiy.OperationLogEnterPrise;
import pwc.taxtech.atms.service.EnterpriseAccountService;

public class EnterpriseAccountServiceIT extends CommonIT {

	@Autowired private EnterpriseAccountService enterpriseAccountService;
	
	@Before
	public void setUp() {
		cleanUpAllTables();
		CommonTestUtils.initTestAuth();
	}
	
	@Test
    public void getListByEnterpriseAccountSetIDTest() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_data.sql");
        
        EnterpriseAccountAndValidateInfo epAccountAndValidateInfo = enterpriseAccountService.getListByEnterpriseAccountSetID("37175dae-0b7f-4dc7-ac58-d8b4009ef0be");
        
        //check dto
        Assert.assertEquals(97, epAccountAndValidateInfo.getEnterpriseAccountList().size());
        Assert.assertEquals(0, epAccountAndValidateInfo.getValidateInfoList().size());
        
        List<EnterpriseAccountDto> epAccountDtoList = epAccountAndValidateInfo.getEnterpriseAccountList();
        Assert.assertEquals("1001", epAccountDtoList.get(0).getCode());
        Assert.assertEquals("100101", epAccountDtoList.get(1).getCode());
        Assert.assertEquals("1001", epAccountDtoList.get(1).getParentCode());
        Assert.assertEquals("1002", epAccountDtoList.get(6).getCode());
        Assert.assertEquals("100201", epAccountDtoList.get(7).getCode());
        Assert.assertEquals("1002", epAccountDtoList.get(7).getParentCode());
        Assert.assertEquals("10020101", epAccountDtoList.get(8).getCode());
        Assert.assertEquals("10020149", epAccountDtoList.get(40).getCode());
        Assert.assertEquals("100202", epAccountDtoList.get(41).getCode());
        Assert.assertEquals("10020201", epAccountDtoList.get(42).getCode());
        Assert.assertEquals("100203", epAccountDtoList.get(59).getCode());
        Assert.assertEquals("1012", epAccountDtoList.get(84).getCode());
        Assert.assertEquals("101212", epAccountDtoList.get(96).getCode());
    }
	
	@Test
	public void getListByEnterpriseAccountSetIDTestWithoutEpAccounts() {
	    //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        
        EnterpriseAccountAndValidateInfo epAccountAndValidateInfo = enterpriseAccountService.getListByEnterpriseAccountSetID("10be679b-75e3-4cab-85d1-28f53930cba8");
        
        //check dto
        Assert.assertEquals(0, epAccountAndValidateInfo.getEnterpriseAccountList().size());
        Assert.assertEquals(0, epAccountAndValidateInfo.getValidateInfoList().size());
	}
	
	@Test
	public void getListByEnterpriseAccountSetIDTestWithStandardAccounts() {
	    //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_data.sql");
        
        EnterpriseAccountAndValidateInfo epAccountAndValidateInfo = enterpriseAccountService.getListByEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        
        //check dto
        Assert.assertEquals(6, epAccountAndValidateInfo.getEnterpriseAccountList().size());
        Assert.assertEquals(0, epAccountAndValidateInfo.getValidateInfoList().size());
        
        EnterpriseAccountDto topAccount = epAccountAndValidateInfo.getEnterpriseAccountList().get(0);
        Assert.assertEquals("1001", topAccount.getCode());
        Assert.assertEquals("库存现金", topAccount.getName());
        Assert.assertNull(topAccount.getParentCode());
        Assert.assertTrue(1 == topAccount.getAcctLevel());
        Assert.assertEquals("库存现金", topAccount.getStdName());
        EnterpriseAccountDto rmbCode = epAccountAndValidateInfo.getEnterpriseAccountList().get(1);
        Assert.assertEquals("100101", rmbCode.getCode());
        Assert.assertEquals("人民币", rmbCode.getName());
        Assert.assertEquals("1001", rmbCode.getParentCode());
        Assert.assertTrue(2 == rmbCode.getAcctLevel());
	}
	
	@Test
    public void getListByEnterpriseAccountSetIDTestWithValidateErrors() {
	    
	    //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_data.sql");
        
        EnterpriseAccountAndValidateInfo epAccountAndValidateInfo = enterpriseAccountService.getListByEnterpriseAccountSetID("e4ffadee-b545-4781-b504-fe686333d4d2");
        
        //check dto
        Assert.assertEquals(97, epAccountAndValidateInfo.getEnterpriseAccountList().size());
        Assert.assertEquals(6, epAccountAndValidateInfo.getValidateInfoList().size());
        
        List<ValidateInfoDto> validateInfoList = epAccountAndValidateInfo.getValidateInfoList();
        
        Assert.assertEquals(EnterpriseAccountMessage.EnterpriceAccountRepeat, validateInfoList.get(0).getType());
        Assert.assertEquals(Arrays.asList("100103", "10020140"), validateInfoList.get(0).getInValidateCodeList());
        Assert.assertEquals(EnterpriseAccountMessage.EnterpriseAccountNameEmpty, validateInfoList.get(1).getType());
        Assert.assertEquals(Arrays.asList("10020149","10020237", "10020504"), validateInfoList.get(1).getInValidateCodeList());
        Assert.assertEquals(EnterpriseAccountMessage.DirectionFormatError, validateInfoList.get(2).getType());
        Assert.assertEquals(Arrays.asList("10020109", "10020145", "10020420"), validateInfoList.get(2).getInValidateCodeList());
        Assert.assertEquals(EnterpriseAccountMessage.AcctPropNullError, validateInfoList.get(3).getType());
        Assert.assertEquals(Arrays.asList("10020147", "10020501", "101211"), validateInfoList.get(3).getInValidateCodeList());
        Assert.assertEquals(EnterpriseAccountMessage.AcctPropFormatError, validateInfoList.get(4).getType());
        Assert.assertEquals(Arrays.asList("100102", "10020102", "101203"), validateInfoList.get(4).getInValidateCodeList());
        Assert.assertEquals(EnterpriseAccountMessage.NoParentCode, validateInfoList.get(5).getType());
        Assert.assertEquals(Arrays.asList("101312"), validateInfoList.get(5).getInValidateCodeList());
        
	}
	
	@Test
	public void getEnterpriseAccountTest() {
	    
	    //data preparation
	    execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_data.sql");
        
        EnterpriseAccountDto enterpriseAccountDto1 = enterpriseAccountService.getEnterpriseAccount("aa771106-1e08-48a7-a6ef-69188e2bc7bc");
        Assert.assertEquals("1001", enterpriseAccountDto1.getCode());
        Assert.assertEquals("库存现金", enterpriseAccountDto1.getName());
        Assert.assertEquals("库存现金", enterpriseAccountDto1.getFullName());
        Assert.assertEquals("1", enterpriseAccountDto1.getAcctProp().toString());
        Assert.assertEquals("1", enterpriseAccountDto1.getAcctLevel().toString());
        Assert.assertEquals("1", enterpriseAccountDto1.getDirection().toString());
        Assert.assertEquals(false, enterpriseAccountDto1.getIsLeaf());
        Assert.assertEquals(true, enterpriseAccountDto1.getIsActive());
        Assert.assertEquals("库存现金", enterpriseAccountDto1.getStdName());
        
        EnterpriseAccountDto enterpriseAccountDto2 = enterpriseAccountService.getEnterpriseAccount("14ced4bb-f3ef-464b-9800-d6c7de46a554");
        Assert.assertEquals("1001", enterpriseAccountDto2.getParentCode());
        Assert.assertEquals("库存现金", enterpriseAccountDto2.getParentName());
        Assert.assertEquals("库存现金", enterpriseAccountDto2.getParentFullName());
	}
	
	/*@Test
	public void accountMappingGetMappingResultTest() {
	    String accountSetID = "6ee4441d-0155-4a0f-b2fe-116912db8776";
	    String industryID = "0";
	    String orgID = "5d6a7685-4dd0-46b2-83af-6db1380f8fc6";
        String parentCode = "1001";
	    List<EnterpriseAccount> epAccounts = customAccountMapper.getMappingResult(accountSetID, industryID, orgID, parentCode);
	    System.out.println(epAccounts);
	}*/
	
	@Test
	public void addEnterpriseAccountTestWithDuplicateEpAccount() {
	    
	    //enterpriseAccountSetId="181b735c-d75c-4e64-8e43-a2a12cd72788"
	    //organizationId=2b05abc8-9429-436d-91c6-178e50d25147
	    //data preparation
	    execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_add_test_data.sql");
        
        //dto preparation
        EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
        enterpriseAccountDto.setCode("1001");
        enterpriseAccountDto.setName("现金");
        enterpriseAccountDto.setFullName("现金");
        enterpriseAccountDto.setEnterpriseAccountSetID("181b735c-d75c-4e64-8e43-a2a12cd72788");
        enterpriseAccountDto.setDirection(1);
        enterpriseAccountDto.setAcctLevel(1);
        enterpriseAccountDto.setAcctProp(1);
        enterpriseAccountDto.setParentCode(null);
        enterpriseAccountDto.setParentFullName("");
        
        OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.addEnterpriseAccount(enterpriseAccountDto);
        
        //check result
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountMessage.EnterpriceAccountRepeat, result.getResultMsg());
        Assert.assertNotNull(result.getData());
	}
	
	@Test
    public void addEnterpriseAccountTestWithNoOrganizationLink() {
	    //enterpriseAccountSetId="181b735c-d75c-4e64-8e43-a2a12cd72788"
        //organizationId=2b05abc8-9429-436d-91c6-178e50d25147
        //data preparation
        execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_add_test_data.sql");
        
        //dto preparation
        EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
        enterpriseAccountDto.setCode("1001.01");
        enterpriseAccountDto.setName("人民币");
        enterpriseAccountDto.setFullName("现金-人民币");
        enterpriseAccountDto.setEnterpriseAccountSetID("181b735c-d75c-4e64-8e43-a2a12cd72788");
        enterpriseAccountDto.setDirection(1);
        enterpriseAccountDto.setAcctLevel(2);
        enterpriseAccountDto.setAcctProp(1);
        enterpriseAccountDto.setRuleType(2);
        enterpriseAccountDto.setIsActive(true);
        enterpriseAccountDto.setParentCode("1001");
        enterpriseAccountDto.setParentFullName("现金");
        
        OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.addEnterpriseAccount(enterpriseAccountDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        
        //check enterpriseAccount
        EnterpriseAccount epAccount1001 = enterpriseAccountService.getEnterpriseAccount("1001", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        Assert.assertEquals(false, epAccount1001.getIsLeaf());
        EnterpriseAccount epAccount100101 = enterpriseAccountService.getEnterpriseAccount("1001.01", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        Assert.assertEquals("人民币", epAccount100101.getName());
        Assert.assertEquals("1001", epAccount100101.getParentCode());
        Assert.assertEquals("现金-人民币", epAccount100101.getFullName());
        Assert.assertEquals(new Integer(1), epAccount100101.getAcctProp());
        Assert.assertEquals(new Integer(2), epAccount100101.getAcctLevel());
        Assert.assertEquals(new Integer(1), epAccount100101.getDirection());
        Assert.assertEquals(true, epAccount100101.getIsLeaf());
        
        //check operation log
        Assert.assertEquals(1, operationLogEnterpriseMapper.countByExample(null));
        
	}
	
	@Test
    public void addEnterpriseAccountTestWithOrganizationLink() {
	    //enterpriseAccountSetId="181b735c-d75c-4e64-8e43-a2a12cd72788"
        //organizationId=2b05abc8-9429-436d-91c6-178e50d25147
        //data preparation
        execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_add_test_data.sql");
        Calendar cal = Calendar.getInstance();
        EnterpriseAccountSetOrg enterpriseAccountSetOrg = new EnterpriseAccountSetOrg();
        enterpriseAccountSetOrg.setOrganizationID("2b05abc8-9429-436d-91c6-178e50d25147");
        //setorg1
        enterpriseAccountSetOrg.setID(CommonUtils.getUUID());
        enterpriseAccountSetOrg.setEnterpriseAccountSetID("181b735c-d75c-4e64-8e43-a2a12cd72788");
        cal.set(2018, Calendar.MARCH, 1);
        enterpriseAccountSetOrg.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.AUGUST, 31);
        enterpriseAccountSetOrg.setExpiredDate(cal.getTime());
        enterpriseAccountSetOrgMapper.insert(enterpriseAccountSetOrg);
        
        //dto preparation
        EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
        enterpriseAccountDto.setCode("1001.01");
        enterpriseAccountDto.setName("人民币");
        enterpriseAccountDto.setFullName("现金-人民币");
        enterpriseAccountDto.setEnterpriseAccountSetID("181b735c-d75c-4e64-8e43-a2a12cd72788");
        enterpriseAccountDto.setDirection(1);
        enterpriseAccountDto.setAcctLevel(2);
        enterpriseAccountDto.setAcctProp(1);
        enterpriseAccountDto.setRuleType(2);
        enterpriseAccountDto.setIsActive(true);
        enterpriseAccountDto.setParentCode("1001");
        enterpriseAccountDto.setParentFullName("现金");
        
        OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.addEnterpriseAccount(enterpriseAccountDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        //check enterpriseAccount
        EnterpriseAccount epAccount1001 = enterpriseAccountService.getEnterpriseAccount("1001", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        Assert.assertEquals(false, epAccount1001.getIsLeaf());
        EnterpriseAccount epAccount100101 = enterpriseAccountService.getEnterpriseAccount("1001.01", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        Assert.assertEquals("人民币", epAccount100101.getName());
        Assert.assertEquals("1001", epAccount100101.getParentCode());
        Assert.assertEquals("现金-人民币", epAccount100101.getFullName());
        Assert.assertEquals(new Integer(1), epAccount100101.getAcctProp());
        Assert.assertEquals(new Integer(2), epAccount100101.getAcctLevel());
        Assert.assertEquals(new Integer(1), epAccount100101.getDirection());
        Assert.assertEquals(true, epAccount100101.getIsLeaf());
        //check accountmapping
        List<AccountMapping> accountMappings = accountMappingMapper.selectByExample(null);
        Assert.assertEquals(1, accountMappings.size());
        AccountMapping accountMapping = accountMappings.get(0);
        Assert.assertEquals("1001.01", accountMapping.getEnterpriseAccountCode());
        Assert.assertEquals("00", accountMapping.getStandardAccountCode());
        Assert.assertEquals("181b735c-d75c-4e64-8e43-a2a12cd72788", accountMapping.getEnterpriseAccountSetID());
        Assert.assertEquals("2b05abc8-9429-436d-91c6-178e50d25147", accountMapping.getOrganizationID());
        Assert.assertEquals("23", accountMapping.getIndustryID());
        
        //check operation log
        Assert.assertEquals(1, operationLogEnterpriseMapper.countByExample(null));
        
	}
	
	@Test
    public void updateEnterpriseAccountTest () {
	    
	    //enterpriseAccountSetId="181b735c-d75c-4e64-8e43-a2a12cd72788"
        //organizationId=2b05abc8-9429-436d-91c6-178e50d25147
        //data preparation
        execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_update_test_data.sql");
        
        /*
         * before update
         * 1001         NULL        0
         * 1001.01      1001        0 
         * 1001.02      1001        1
         * 1001.01.01   1001.01     1
         * update 1001.01.01 -> 1001.02.01      1001.02
         * after update
         * 1001         NULL        0
         * 1001.01      1001        1
         * 1001.02      1001        0
         * 1001.02.01   1001.02     1
         */
        
        //dto preparation
        EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
        enterpriseAccountDto.setID("c861b9d1-d7f2-4ca0-b95c-5534de66b0c5");
        enterpriseAccountDto.setCode("1001.02.01");
        enterpriseAccountDto.setName("美元备用金");
        enterpriseAccountDto.setFullName("现金-美元-美元备用金");
        enterpriseAccountDto.setEnterpriseAccountSetID("181b735c-d75c-4e64-8e43-a2a12cd72788");
        enterpriseAccountDto.setDirection(-1);
        enterpriseAccountDto.setAcctLevel(3);
        enterpriseAccountDto.setAcctProp(3);
        enterpriseAccountDto.setRuleType(2);
        enterpriseAccountDto.setIsActive(true);
        enterpriseAccountDto.setParentCode("1001.02");
        enterpriseAccountDto.setParentFullName("现金-美元");
        
        OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.updateEnterpriseAccount(enterpriseAccountDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        //check enterpriseAccount
        EnterpriseAccount epAccount100101 = enterpriseAccountService.getEnterpriseAccount("1001.01", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        EnterpriseAccount epAccount100102 = enterpriseAccountService.getEnterpriseAccount("1001.02", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        EnterpriseAccount epAccount10010201 = enterpriseAccountService.getEnterpriseAccount("1001.02.01", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        
        Assert.assertEquals(true, epAccount100101.getIsLeaf());
        Assert.assertEquals(false, epAccount100102.getIsLeaf());
        Assert.assertEquals(true, epAccount10010201.getIsLeaf());
        Assert.assertEquals("美元备用金", epAccount10010201.getName());
        Assert.assertEquals("1001.02", epAccount10010201.getParentCode());
        Assert.assertEquals("现金-美元-美元备用金", epAccount10010201.getFullName());
        Assert.assertEquals(new Integer(3), epAccount10010201.getAcctProp());
        Assert.assertEquals(new Integer(3), epAccount10010201.getAcctLevel());
        Assert.assertEquals(new Integer(-1), epAccount10010201.getDirection());
        
        //check operation log
        Assert.assertEquals(1, operationLogEnterpriseMapper.countByExample(null));
	}
	
	@Test
    public void updateEnterpriseAccountTestWithInactiveAccountSet () {
	    /*
	     * enterpriseAccountSetId="181b735c-d75c-4e64-8e43-a2a12cd72788"
	     * organizationId=2b05abc8-9429-436d-91c6-178e50d25147
	     */
        //data preparation
        execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_update_test_data.sql");
        //set account inactive
        EnterpriseAccountSet enterpriseAccountSet = enterpriseAccountSetMapper.selectByPrimaryKey("181b735c-d75c-4e64-8e43-a2a12cd72788");
        enterpriseAccountSet.setIsActive(false);
        enterpriseAccountSetMapper.updateByPrimaryKey(enterpriseAccountSet);
        /*
         * before update
         * 1001         NULL        0
         * 1001.01      1001        0 
         * 1001.02      1001        1
         * 1001.01.01   1001.01     1
         * update 1001.01.01 -> 1001.02.01      1001.02
         * after update
         * 1001         NULL        0
         * 1001.01      1001        1
         * 1001.02      1001        0
         * 1001.02.01   1001.02     1
         */
        
        //dto preparation
        EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
        enterpriseAccountDto.setID("c861b9d1-d7f2-4ca0-b95c-5534de66b0c5");
        enterpriseAccountDto.setCode("1001.02.01");
        enterpriseAccountDto.setName("美元备用金");
        enterpriseAccountDto.setFullName("现金-美元-美元备用金");
        enterpriseAccountDto.setEnterpriseAccountSetID("181b735c-d75c-4e64-8e43-a2a12cd72788");
        enterpriseAccountDto.setDirection(-1);
        enterpriseAccountDto.setAcctLevel(3);
        enterpriseAccountDto.setAcctProp(3);
        enterpriseAccountDto.setRuleType(2);
        enterpriseAccountDto.setIsActive(true);
        enterpriseAccountDto.setParentCode("1001.02");
        enterpriseAccountDto.setParentFullName("现金-美元");
        
        OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.updateEnterpriseAccount(enterpriseAccountDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        //check enterpriseAccount
        EnterpriseAccount epAccount100101 = enterpriseAccountService.getEnterpriseAccount("1001.01", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        EnterpriseAccount epAccount100102 = enterpriseAccountService.getEnterpriseAccount("1001.02", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        EnterpriseAccount epAccount10010201 = enterpriseAccountService.getEnterpriseAccount("1001.02.01", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        
        Assert.assertEquals(true, epAccount100101.getIsLeaf());
        Assert.assertEquals(false, epAccount100102.getIsLeaf());
        Assert.assertEquals(true, epAccount10010201.getIsLeaf());
        Assert.assertEquals("美元备用金", epAccount10010201.getName());
        Assert.assertEquals("1001.02", epAccount10010201.getParentCode());
        Assert.assertEquals("现金-美元-美元备用金", epAccount10010201.getFullName());
        Assert.assertEquals(new Integer(3), epAccount10010201.getAcctProp());
        Assert.assertEquals(new Integer(3), epAccount10010201.getAcctLevel());
        Assert.assertEquals(new Integer(-1), epAccount10010201.getDirection());
        
        //check enterprise account set
        EnterpriseAccountSet updatedEnterpriseAccountSet = enterpriseAccountSetMapper.selectByPrimaryKey("181b735c-d75c-4e64-8e43-a2a12cd72788");
        Assert.assertEquals(true, updatedEnterpriseAccountSet.getIsActive());
        
        //check operation log
        Assert.assertEquals(1, operationLogEnterpriseMapper.countByExample(null));
	}
	
	@Test
    public void updateEnterpriseAccountTestWithValidationError () {
	    /*
         * enterpriseAccountSetId="181b735c-d75c-4e64-8e43-a2a12cd72788"
         * organizationId=2b05abc8-9429-436d-91c6-178e50d25147
         */
        //data preparation
        execSqlFile("sql/standardAccount_data.sql");
        execSqlFile("sql/enterpriseAccount_update_test_data.sql");
        //set account inactive
        EnterpriseAccountSet enterpriseAccountSet = enterpriseAccountSetMapper.selectByPrimaryKey("181b735c-d75c-4e64-8e43-a2a12cd72788");
        enterpriseAccountSet.setIsActive(false);
        enterpriseAccountSetMapper.updateByPrimaryKey(enterpriseAccountSet);
        /*
         * before update
         * 1001         NULL        0
         * 1001.01      1001        0 
         * 1001.02      1001        1
         * 1001.01.01   1001.01     1
         * update 1001.01 -> 1001.03      1001
         * after update
         * 1001         NULL        0
         * 1001.03      1001        1
         * 1001.02      1001        1
         * 1001.01.01   1001.01     1
         */
        
        //dto preparation
        EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
        enterpriseAccountDto.setID("d3ce1d49-94bc-42b8-8add-b0894e3e8835");
        enterpriseAccountDto.setCode("1001.03");
        enterpriseAccountDto.setName("欧元");
        enterpriseAccountDto.setFullName("现金-欧元");
        enterpriseAccountDto.setEnterpriseAccountSetID("181b735c-d75c-4e64-8e43-a2a12cd72788");
        enterpriseAccountDto.setDirection(-1);
        enterpriseAccountDto.setAcctLevel(2);
        enterpriseAccountDto.setAcctProp(3);
        enterpriseAccountDto.setRuleType(2);
        enterpriseAccountDto.setIsActive(true);
        enterpriseAccountDto.setParentCode("1001");
        enterpriseAccountDto.setParentFullName("现金");
        
        OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.updateEnterpriseAccount(enterpriseAccountDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        //check enterpriseAccount
        EnterpriseAccount epAccount100103 = enterpriseAccountService.getEnterpriseAccount("1001.03", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        EnterpriseAccount epAccount100102 = enterpriseAccountService.getEnterpriseAccount("1001.02", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        EnterpriseAccount epAccount10010101 = enterpriseAccountService.getEnterpriseAccount("1001.01.01", "181b735c-d75c-4e64-8e43-a2a12cd72788");
        
        //Assert.assertEquals(true, epAccount100103.getIsLeaf());
        Assert.assertEquals(true, epAccount100102.getIsLeaf());
        Assert.assertEquals(true, epAccount10010101.getIsLeaf());
        Assert.assertEquals("欧元", epAccount100103.getName());
        Assert.assertEquals("1001", epAccount100103.getParentCode());
        Assert.assertEquals("现金-欧元", epAccount100103.getFullName());
        Assert.assertEquals(new Integer(3), epAccount100103.getAcctProp());
        Assert.assertEquals(new Integer(2), epAccount100103.getAcctLevel());
        Assert.assertEquals(new Integer(-1), epAccount100103.getDirection());
        
        //check enterprise account set
        EnterpriseAccountSet updatedEnterpriseAccountSet = enterpriseAccountSetMapper.selectByPrimaryKey("181b735c-d75c-4e64-8e43-a2a12cd72788");
        Assert.assertEquals(true, updatedEnterpriseAccountSet.getIsActive());
        
        //check operation log
        Assert.assertEquals(1, operationLogEnterpriseMapper.countByExample(null));
	}
	
	@Test
	public void validateImportEnterpriseAccountTestImportFormatError() {
	    
	    OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.validateImportEnterpriseAccount("excel/customer-atms.xlsx");
	    Assert.assertEquals(false, result.getResult());
	    Assert.assertEquals(EnterpriseAccountMessage.EnterpriceAccountImportDataFormatError, result.getResultMsg());
	}
	
	@Test
    public void validateImportEnterpriseAccountTestDataError() {
        
        OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount-acctprop-format-error.xlsx");
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountMessage.AcctPropFormatError, result.getResultMsg());
        
        result = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount-code-empty-error.xlsx");
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountMessage.CodeEmpty, result.getResultMsg());
        
        result = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount-code-maxlength-error.xlsx");
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountMessage.CodeMaxLength, result.getResultMsg());
        
        result = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount-direction-format-error.xlsx");
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountMessage.DirectionFormatError, result.getResultMsg());
        
        result = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount-name-maxlength-error.xlsx");
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountMessage.NameMaxLength, result.getResultMsg());
    }
	
	@Test
    public void validateImportEnterpriseAccountTest() {
	    
	    OperationResultDto<List<EnterpriseAccountDto>> result = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount.xlsx");
        Assert.assertEquals(true, result.getResult());
        List<EnterpriseAccountDto> enterpriseAccountDtoList = result.getData();
        Assert.assertEquals(6, enterpriseAccountDtoList.size());
        EnterpriseAccountDto epAccount1001 = enterpriseAccountDtoList.get(0);
        Assert.assertEquals("1001", epAccount1001.getCode());
        Assert.assertEquals("库存现金", epAccount1001.getName());
        Assert.assertEquals(new Integer(-1), epAccount1001.getAcctProp());
        Assert.assertEquals(new Integer(1), epAccount1001.getDirection());
        EnterpriseAccountDto epAccount100102 = enterpriseAccountDtoList.get(2);
        Assert.assertEquals("100102", epAccount100102.getCode());
        Assert.assertEquals("", epAccount100102.getName());
        Assert.assertEquals(new Integer(2), epAccount100102.getAcctProp());
        Assert.assertEquals(new Integer(-1), epAccount100102.getDirection());
        
	}
	
	@Test
	public void addEnterpriseAccountSetAndImportDataTest() {
	    
	    /*
         * enterpriseAccountSetId="dd424793-46a4-4102-beaa-a7efe161d1ee"
         * enterpriseAccountSetCode="CSBZKM"
         * enterpriseAccountSetName="测试标准科目"
         */
	    //dto preparation
        List<EnterpriseAccountDto> importData 
            = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount_upload.xlsx").getData();
        
        OperationResultDto<?> result = enterpriseAccountService.addEnterpriseAccountSetAndImportData("测试标准科目", "CSBZKM", importData);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        
        //check enterpriseAccountSet
        EnterpriseAccountSet enterpriseAccountSet = enterpriseAccountSetMapper.selectByExample(null).get(0);
        String enterpriseAccountSetID = enterpriseAccountSet.getID();
        Assert.assertEquals("CSBZKM", enterpriseAccountSet.getCode());
        Assert.assertEquals("测试标准科目", enterpriseAccountSet.getName());
        Assert.assertEquals(true, enterpriseAccountSet.getIsActive());
        
        //check enterpriseAccount
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID);
        List<EnterpriseAccount> epAccountList = enterpriseAccountMapper.selectByExample(example);
        Assert.assertEquals(97, epAccountList.size());
        //1001
        example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID).andCodeEqualTo("1001");
        EnterpriseAccount epAccount1001 = enterpriseAccountMapper.selectByExample(example).get(0);
        Assert.assertEquals("1001", epAccount1001.getCode());
        Assert.assertEquals("库存现金", epAccount1001.getName());
        Assert.assertEquals("库存现金", epAccount1001.getFullName());
        Assert.assertEquals(false, epAccount1001.getIsLeaf());
        Assert.assertEquals(new Integer(1), epAccount1001.getAcctLevel());
        Assert.assertNull(epAccount1001.getParentCode());
        //100105
        example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID).andCodeEqualTo("100105");
        EnterpriseAccount epAccount100105 = enterpriseAccountMapper.selectByExample(example).get(0);
        Assert.assertEquals("100105", epAccount100105.getCode());
        Assert.assertEquals("新加坡元", epAccount100105.getName());
        Assert.assertEquals("库存现金-新加坡元", epAccount100105.getFullName());
        Assert.assertEquals(true, epAccount100105.getIsLeaf());
        Assert.assertEquals(new Integer(2), epAccount100105.getAcctLevel());
        Assert.assertEquals("1001", epAccount100105.getParentCode());
        //10020116
        example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID).andCodeEqualTo("10020116");
        EnterpriseAccount epAccount10020116 = enterpriseAccountMapper.selectByExample(example).get(0);
        Assert.assertEquals("10020116", epAccount10020116.getCode());
        Assert.assertEquals("浦发银行", epAccount10020116.getName());
        Assert.assertEquals("银行存款-RMB-浦发银行", epAccount10020116.getFullName());
        Assert.assertEquals(true, epAccount10020116.getIsLeaf());
        Assert.assertEquals(new Integer(3), epAccount10020116.getAcctLevel());
        Assert.assertEquals("100201", epAccount10020116.getParentCode());
        //100203
        example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID).andCodeEqualTo("100203");
        EnterpriseAccount epAccount100203 = enterpriseAccountMapper.selectByExample(example).get(0);
        Assert.assertEquals("100203", epAccount100203.getCode());
        Assert.assertEquals("VND", epAccount100203.getName());
        Assert.assertEquals("银行存款-VND", epAccount100203.getFullName());
        Assert.assertEquals(false, epAccount100203.getIsLeaf());
        Assert.assertEquals(new Integer(2), epAccount100203.getAcctLevel());
        Assert.assertEquals("1002", epAccount100203.getParentCode());
        
        //check operation log
        List<OperationLogEnterPrise> operationLogs = operationLogEnterpriseMapper.selectByExample(null);
        Assert.assertEquals(1, operationLogs.size());
	}
	
	@Test
    public void repeatImportEnterpriseAccountSetTestImportAppend() {
	    /*
         * enterpriseAccountSetId="dd424793-46a4-4102-beaa-a7efe161d1ee"
         * enterpriseAccountSetCode="CSBZKM"
         * enterpriseAccountSetName="测试标准科目"
         */
	    execSqlFile("sql/enterpriseAccountSet_data.sql");
	    EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
	    enterpriseAccountSetDto.setID("dd424793-46a4-4102-beaa-a7efe161d1ee");
	    enterpriseAccountSetDto.setName("测试标准科目");
	    enterpriseAccountSetDto.setIsImportAppend(false);
	    //dto preparation
	    List<EnterpriseAccountDto> importData 
            = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount_upload.xlsx").getData();
	    OperationResultDto<?> result = enterpriseAccountService.repeatImportEnterpriseAccountSet(enterpriseAccountSetDto, importData);
	    
	    //check result
        Assert.assertEquals(true, result.getResult());
        
        //check enterpriseAccount
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee");
        List<EnterpriseAccount> epAccountList = enterpriseAccountMapper.selectByExample(example);
        Assert.assertEquals(97, epAccountList.size());
        //1001
        example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee").andCodeEqualTo("1001");
        EnterpriseAccount epAccount1001 = enterpriseAccountMapper.selectByExample(example).get(0);
        Assert.assertEquals("1001", epAccount1001.getCode());
        Assert.assertEquals("库存现金", epAccount1001.getName());
        Assert.assertEquals("库存现金", epAccount1001.getFullName());
        Assert.assertEquals(false, epAccount1001.getIsLeaf());
        Assert.assertEquals(new Integer(1), epAccount1001.getAcctLevel());
        Assert.assertNull(epAccount1001.getParentCode());
        //100105
        example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee").andCodeEqualTo("100105");
        EnterpriseAccount epAccount100105 = enterpriseAccountMapper.selectByExample(example).get(0);
        Assert.assertEquals("100105", epAccount100105.getCode());
        Assert.assertEquals("新加坡元", epAccount100105.getName());
        Assert.assertEquals("库存现金-新加坡元", epAccount100105.getFullName());
        Assert.assertEquals(true, epAccount100105.getIsLeaf());
        Assert.assertEquals(new Integer(2), epAccount100105.getAcctLevel());
        Assert.assertEquals("1001", epAccount100105.getParentCode());
        //10020116
        example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee").andCodeEqualTo("10020116");
        EnterpriseAccount epAccount10020116 = enterpriseAccountMapper.selectByExample(example).get(0);
        Assert.assertEquals("10020116", epAccount10020116.getCode());
        Assert.assertEquals("浦发银行", epAccount10020116.getName());
        Assert.assertEquals("银行存款-RMB-浦发银行", epAccount10020116.getFullName());
        Assert.assertEquals(true, epAccount10020116.getIsLeaf());
        Assert.assertEquals(new Integer(3), epAccount10020116.getAcctLevel());
        Assert.assertEquals("100201", epAccount10020116.getParentCode());
        //100203
        example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee").andCodeEqualTo("100203");
        EnterpriseAccount epAccount100203 = enterpriseAccountMapper.selectByExample(example).get(0);
        Assert.assertEquals("100203", epAccount100203.getCode());
        Assert.assertEquals("VND", epAccount100203.getName());
        Assert.assertEquals("银行存款-VND", epAccount100203.getFullName());
        Assert.assertEquals(false, epAccount100203.getIsLeaf());
        Assert.assertEquals(new Integer(2), epAccount100203.getAcctLevel());
        Assert.assertEquals("1002", epAccount100203.getParentCode());
        
        //check operation log
        List<OperationLogEnterPrise> operationLogs = operationLogEnterpriseMapper.selectByExample(null);
        Assert.assertEquals(0, operationLogs.size());
	    
	}
	
	@Test
    public void repeatImportEnterpriseAccountSetTestNoImportAppend() {
        /*
         * enterpriseAccountSetId="dd424793-46a4-4102-beaa-a7efe161d1ee"
         * enterpriseAccountSetCode="CSBZKM"
         * enterpriseAccountSetName="测试标准科目"
         */
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        enterpriseAccountSetDto.setName("测试标准科目");
        enterpriseAccountSetDto.setIsImportAppend(false);
        //dto preparation
        List<EnterpriseAccountDto> importData 
            = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount_upload.xlsx").getData();
        OperationResultDto<?> result = enterpriseAccountService.repeatImportEnterpriseAccountSet(enterpriseAccountSetDto, importData);
        
        enterpriseAccountSetDto.setIsImportAppend(true);
        result = enterpriseAccountService.repeatImportEnterpriseAccountSet(enterpriseAccountSetDto, importData);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        
        //check enterpriseAccount
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee");
        List<EnterpriseAccount> epAccountList = enterpriseAccountMapper.selectByExample(example);
        Assert.assertEquals(97+97, epAccountList.size());
        
        //check operation log
        List<OperationLogEnterPrise> operationLogs = operationLogEnterpriseMapper.selectByExample(null);
        Assert.assertEquals(0, operationLogs.size());
        
    }
	
	@Test
	public void clearRepeatEnterpriseAccountListTest() {
	    
	    /*
         * enterpriseAccountSetId="dd424793-46a4-4102-beaa-a7efe161d1ee"
         * enterpriseAccountSetCode="CSBZKM"
         * enterpriseAccountSetName="测试标准科目"
         */
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        enterpriseAccountSetDto.setName("测试标准科目");
        enterpriseAccountSetDto.setIsImportAppend(false);
        
        //upload twice
        List<EnterpriseAccountDto> importData 
            = enterpriseAccountService.validateImportEnterpriseAccount("excel/epaccount_upload_for_repeatcode_test.xlsx").getData();
        OperationResultDto<?> result = enterpriseAccountService.repeatImportEnterpriseAccountSet(enterpriseAccountSetDto, importData);
        
        enterpriseAccountSetDto.setIsImportAppend(true);
        result = enterpriseAccountService.repeatImportEnterpriseAccountSet(enterpriseAccountSetDto, importData);
        
        //before clean repeat code
        //check result
        Assert.assertEquals(true, result.getResult());
        
        //check enterpriseAccount
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee");
        List<EnterpriseAccount> epAccountList = enterpriseAccountMapper.selectByExample(example);
        Assert.assertEquals(6+6, epAccountList.size());
        
        //dto preparation for clean
        EnterpriseAccountSetDto cleanDto = new EnterpriseAccountSetDto();
        cleanDto.setID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        cleanDto.setRepeatCodeList(Arrays.asList("1001", "100101", "100102", "100103", "100104", "100105"));
        
        //clean repeat account
        result = enterpriseAccountService.clearRepeatEnterpriseAccountList(cleanDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        //check enterpriseAccount
        epAccountList = enterpriseAccountMapper.selectByExample(example);
        Assert.assertEquals(6, epAccountList.size());
	}
	
	@Test
	public void getAccountMappingOrgTest() {
	    
	    //data preparation
	    execSqlFile("sql/get_accountmapping_org_data.sql");
	    
	    List<AccountMappingDto> accountMappingDtoList = enterpriseAccountService.getAccountMappingOrg("5b4b8f88-571d-44c6-8002-c0716017d41e");
	    
	    Assert.assertEquals(4, accountMappingDtoList.size());
	}
}

