package pwc.taxtech.atms.service.impl;

import static org.assertj.core.api.Assertions.assertThat;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import org.joda.time.DateTime;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.CommonTestUtils;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.OperationAction;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.message.EnterpriseAccountSetMessage;
import pwc.taxtech.atms.common.message.EnterpriseAccountSetOrgMessage;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountSetDto;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountSetOrgDto;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSet;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSetOrg;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSetOrgExample;
import pwc.taxtech.atms.entitiy.OperationLogEnterPrise;
import pwc.taxtech.atms.entitiy.OperationLogEnterPriseExample;
import pwc.taxtech.atms.service.EnterpriseAccountSetService;

public class EnterpriseAccountSetServiceIT  extends CommonIT {

    @Autowired private EnterpriseAccountSetService enterpriseAccountSetService;
    
    @Before
    public void setUp() {
        cleanUpAllTables();
    }
    
    @Test
    public void addEnterpriseAccountSetOrgTest() {
    
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_org_validation_data.sql");
        
        Calendar cal = Calendar.getInstance();
        EnterpriseAccountSetOrg enterpriseAccountSetOrg = new EnterpriseAccountSetOrg();
        enterpriseAccountSetOrg.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        //setorg1
        enterpriseAccountSetOrg.setID(CommonUtils.getUUID());
        enterpriseAccountSetOrg.setEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        cal.set(2018, Calendar.MARCH, 1);
        enterpriseAccountSetOrg.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.AUGUST, 1);
        enterpriseAccountSetOrg.setExpiredDate(cal.getTime());
        enterpriseAccountSetOrgMapper.insert(enterpriseAccountSetOrg);
        
        CommonTestUtils.initTestAuth();
        
        //dto preparation
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        enterpriseAccountSetDto.setName("测试标准科目");
        enterpriseAccountSetDto.setEnterpriseAccountSetOrgList(new ArrayList<>());
        //setorg2
        EnterpriseAccountSetOrgDto enterpriseAccountSetOrgDto = new EnterpriseAccountSetOrgDto();
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        enterpriseAccountSetOrgDto.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        enterpriseAccountSetOrgDto.setEffectiveDateStr("2019-04");
        enterpriseAccountSetOrgDto.setExpiredDateStr("2019-10");
        enterpriseAccountSetDto.getEnterpriseAccountSetOrgList().add(enterpriseAccountSetOrgDto);
        //setorg3
        enterpriseAccountSetOrgDto = new EnterpriseAccountSetOrgDto();
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        enterpriseAccountSetOrgDto.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        enterpriseAccountSetOrgDto.setEffectiveDateStr("2020-04");
        enterpriseAccountSetOrgDto.setExpiredDateStr(null);
        enterpriseAccountSetDto.getEnterpriseAccountSetOrgList().add(enterpriseAccountSetOrgDto);
        
        OperationResultDto<?> result = enterpriseAccountSetService.addEnterpriseAccountSetOrg(enterpriseAccountSetDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        //check data
        EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee");
        example.setOrderByClause("EffectiveDate ASC");
        List<EnterpriseAccountSetOrg> enterpriseAccountSetOrgList = enterpriseAccountSetOrgMapper.selectByExample(example);
        Assert.assertEquals(3, enterpriseAccountSetOrgList.size());
        EnterpriseAccountSetOrg setOrg2 = enterpriseAccountSetOrgList.get(1);
        Assert.assertEquals("dd424793-46a4-4102-beaa-a7efe161d1ee", setOrg2.getEnterpriseAccountSetID());
        Assert.assertEquals("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182", setOrg2.getOrganizationID());
        Assert.assertEquals("2019-04-01", new DateTime(setOrg2.getEffectiveDate()).toString("yyyy-MM-dd"));
        Assert.assertEquals("2019-10-31", new DateTime(setOrg2.getExpiredDate()).toString("yyyy-MM-dd"));
        EnterpriseAccountSetOrg setOrg3 = enterpriseAccountSetOrgList.get(2);
        Assert.assertEquals("dd424793-46a4-4102-beaa-a7efe161d1ee", setOrg3.getEnterpriseAccountSetID());
        Assert.assertEquals("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182", setOrg3.getOrganizationID());
        Assert.assertEquals("2020-04-01", new DateTime(setOrg3.getEffectiveDate()).toString("yyyy-MM-dd"));
        Assert.assertNull(setOrg3.getExpiredDate());
        //check operation log
        OperationLogEnterPriseExample operationLogEnterPriseExample = new OperationLogEnterPriseExample();
        operationLogEnterPriseExample.setOrderByClause("createtime asc");
        List<OperationLogEnterPrise> operationLogs = operationLogEnterpriseMapper.selectByExample(operationLogEnterPriseExample);
        Assert.assertEquals(2, operationLogs.size());
        OperationLogEnterPrise setOrgAddLog1 = operationLogs.get(0);
        Assert.assertEquals("通用机构", setOrgAddLog1.getOperationContent());
        Assert.assertEquals("测试标准科目", setOrgAddLog1.getOperationObject());
        Assert.assertEquals(OperationModule.BasicDataEnterpriceAccount.name(), setOrgAddLog1.getModuleName());
        Assert.assertEquals(OperationAction.AddRelevantOrgAction.name(), setOrgAddLog1.getOperationAction());
    }

    @Test
    public void deleteEnterpriseAccountSetOrgTest() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        execSqlFile("sql/enterpriseAccountSetOrg_data.sql");
        CommonTestUtils.initTestAuth();
        
        //dto preparation
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("2891e3dd-d673-49a2-845d-c759202056c6");
        EnterpriseAccountSetOrgDto enterpriseAccountSetOrgDto = new EnterpriseAccountSetOrgDto();
        enterpriseAccountSetOrgDto.setID("06430cbf-e177-4f65-9a2a-d650c36cb564");
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetID("2891e3dd-d673-49a2-845d-c759202056c6");
        enterpriseAccountSetOrgDto.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        enterpriseAccountSetOrgDto.setOrganizationName("通用机构");
        enterpriseAccountSetDto.setEnterpriseAccountSetOrgList(new ArrayList<>());
        enterpriseAccountSetDto.getEnterpriseAccountSetOrgList().add(enterpriseAccountSetOrgDto);
        
        OperationResultDto<?> result = enterpriseAccountSetService.deleteEnterpriseAccountSetOrg(enterpriseAccountSetDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        
        //check data
        EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("2891e3dd-d673-49a2-845d-c759202056c6");
        Assert.assertEquals(4, enterpriseAccountSetOrgMapper.countByExample(example));
        
        //check operation log
        OperationLogEnterPriseExample operationLogEnterPriseExample = new OperationLogEnterPriseExample();
        List<OperationLogEnterPrise> operationLogs = operationLogEnterpriseMapper.selectByExample(operationLogEnterPriseExample);
        Assert.assertEquals(1, operationLogs.size());
        OperationLogEnterPrise setOrgDeleteLog = operationLogs.get(0);
        Assert.assertEquals("通用机构", setOrgDeleteLog.getOperationContent());
        Assert.assertEquals("指标分析", setOrgDeleteLog.getOperationObject());
        Assert.assertEquals(OperationModule.BasicDataEnterpriceAccount.name(), setOrgDeleteLog.getModuleName());
        Assert.assertEquals(OperationAction.DeleteRelevantOrg.name(), setOrgDeleteLog.getOperationAction());
        
    }

    @Test
    public void enterpriseAccountSetOrgValidateTestWithEffectiveDateGreaterThanExpiredDate() {
        
        //data preparation
        //e4ffadee-b545-4781-b504-fe686333d4d2, VALERROR, 测试验证错误
        //execSqlFile("sql/enterpriseAccountSet_org_validation_data.sql");
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("e4ffadee-b545-4781-b504-fe686333d4d2");
        EnterpriseAccountSetOrgDto enterpriseAccountSetOrgDto = new EnterpriseAccountSetOrgDto();
        Calendar cal = Calendar.getInstance();
        cal.set(2018, Calendar.APRIL, 1);
        enterpriseAccountSetOrgDto.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.MARCH, 1);
        enterpriseAccountSetOrgDto.setExpiredDate(cal.getTime());
        enterpriseAccountSetDto.setEnterpriseAccountSetOrgList(new ArrayList<>());
        enterpriseAccountSetDto.getEnterpriseAccountSetOrgList().add(enterpriseAccountSetOrgDto);
        OperationResultDto<?> result = enterpriseAccountSetService.enterpriseAccountSetOrgValidate(enterpriseAccountSetDto);
        
        //check result
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountSetOrgMessage.EffectiveDateAreaProblem, result.getResultMsg());
    }

    @Test
    public void enterpriseAccountSetOrgValidateTestWithTimeOverlap() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_org_validation_data.sql");
        Calendar cal = Calendar.getInstance();
        EnterpriseAccountSetOrg enterpriseAccountSetOrg = new EnterpriseAccountSetOrg();
        enterpriseAccountSetOrg.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        //setorg1
        enterpriseAccountSetOrg.setID(CommonUtils.getUUID());
        enterpriseAccountSetOrg.setEnterpriseAccountSetID("e4ffadee-b545-4781-b504-fe686333d4d2");
        cal.set(2018, Calendar.MARCH, 1);
        enterpriseAccountSetOrg.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.AUGUST, 1);
        enterpriseAccountSetOrg.setExpiredDate(cal.getTime());
        enterpriseAccountSetOrgMapper.insert(enterpriseAccountSetOrg);
        //setorg2
        enterpriseAccountSetOrg.setID(CommonUtils.getUUID());
        enterpriseAccountSetOrg.setEnterpriseAccountSetID("37175dae-0b7f-4dc7-ac58-d8b4009ef0be");
        cal.set(2018, Calendar.SEPTEMBER, 1);
        enterpriseAccountSetOrg.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.DECEMBER, 1);
        enterpriseAccountSetOrg.setExpiredDate(cal.getTime());
        enterpriseAccountSetOrgMapper.insert(enterpriseAccountSetOrg);
        
        //dto preparation
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        EnterpriseAccountSetOrgDto enterpriseAccountSetOrgDto = new EnterpriseAccountSetOrgDto();
        //setorg3
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        enterpriseAccountSetOrgDto.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        cal.set(2018, Calendar.APRIL, 1);
        enterpriseAccountSetOrgDto.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.OCTOBER, 1);
        enterpriseAccountSetOrgDto.setExpiredDate(cal.getTime());
        enterpriseAccountSetDto.setEnterpriseAccountSetOrgList(new ArrayList<>());
        enterpriseAccountSetDto.getEnterpriseAccountSetOrgList().add(enterpriseAccountSetOrgDto);
        
        OperationResultDto<?> result = enterpriseAccountSetService.enterpriseAccountSetOrgValidate(enterpriseAccountSetDto);
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountSetOrgMessage.EnterpriseAccountSetOrgDateTimeOverlap, result.getResultMsg());
    }

    @Test
    public void enterpriseAccountSetOrgValidateTestWithTwicePerYear() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_org_validation_data.sql");
        Calendar cal = Calendar.getInstance();
        EnterpriseAccountSetOrg enterpriseAccountSetOrg = new EnterpriseAccountSetOrg();
        enterpriseAccountSetOrg.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        
        //setorg1
        enterpriseAccountSetOrg.setID(CommonUtils.getUUID());
        enterpriseAccountSetOrg.setEnterpriseAccountSetID("e4ffadee-b545-4781-b504-fe686333d4d2");
        cal.set(2018, Calendar.MARCH, 1);
        enterpriseAccountSetOrg.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.MAY, 1);
        enterpriseAccountSetOrg.setExpiredDate(cal.getTime());
        enterpriseAccountSetOrgMapper.insert(enterpriseAccountSetOrg);
        
        //dto preparation
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("e4ffadee-b545-4781-b504-fe686333d4d2");
        EnterpriseAccountSetOrgDto enterpriseAccountSetOrgDto = new EnterpriseAccountSetOrgDto();
        //setorg2
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetID("e4ffadee-b545-4781-b504-fe686333d4d2");
        enterpriseAccountSetOrgDto.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        cal.set(2018, Calendar.JULY, 1);
        enterpriseAccountSetOrgDto.setEffectiveDate(cal.getTime());
        cal.set(2019, Calendar.FEBRUARY, 1);
        enterpriseAccountSetOrgDto.setExpiredDate(cal.getTime());
        enterpriseAccountSetDto.setEnterpriseAccountSetOrgList(new ArrayList<>());
        enterpriseAccountSetDto.getEnterpriseAccountSetOrgList().add(enterpriseAccountSetOrgDto);
        
        OperationResultDto<?> result = enterpriseAccountSetService.enterpriseAccountSetOrgValidate(enterpriseAccountSetDto);
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountSetOrgMessage.EnterpriseAccountSetOrgOnlyOncePerYear, result.getResultMsg());
        
    }

    @Test
    public void getEnterpriseAccountSetListTest() {
    	
    	//data preparation
    	execSqlFile("sql/enterpriseAccountSet_data.sql");
    	List<EnterpriseAccountSetDto> enterpriseAccountSetList = enterpriseAccountSetService.getEnterpriseAccountSetList();
    
    	Assert.assertEquals(7, enterpriseAccountSetList.size());
    	EnterpriseAccountSetDto aacDemo = enterpriseAccountSetList.get(0);
    	Assert.assertEquals("885c2ddc-3164-4d57-bab8-12de3d9f83a0", aacDemo.getID());
    	Assert.assertEquals("518000", aacDemo.getCode());
    	Assert.assertEquals("AAC Demo", aacDemo.getName());
    	Assert.assertEquals(true, aacDemo.getIsActive());
    }

    @Test
    public void getEnterpriseAccountSetTestWithNoOrganization() {
        
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        execSqlFile("sql/enterpriseAccountSetOrg_data.sql");
        execSqlFile("sql/organization_data_for_enterpriseAccountSet_test.sql");
        
        EnterpriseAccountSetDto enterpriseAccountSetDto = enterpriseAccountSetService.getEnterpriseAccountSet("7bba71c9-97a9-4841-b9cb-8140347800ea");
    
        //check dto
        //Assert.assertEquals(null, enterpriseAccountSetDto.getEnterpriseAccountSetOrgList());
        Assert.assertTrue("Not empty", enterpriseAccountSetDto.getEnterpriseAccountSetOrgList().isEmpty());
        Assert.assertEquals("000010", enterpriseAccountSetDto.getCode());
        Assert.assertEquals("账套无机构", enterpriseAccountSetDto.getName());
    }

    @Test
    public void getEnterpriseAccountSetTestWithOrganizations() {
        
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        execSqlFile("sql/enterpriseAccountSetOrg_data.sql");
        execSqlFile("sql/organization_data_for_enterpriseAccountSet_test.sql");
        
        EnterpriseAccountSetDto enterpriseAccountSetDto = enterpriseAccountSetService.getEnterpriseAccountSet("2891e3dd-d673-49a2-845d-c759202056c6");
    
        //check dto
        Assert.assertEquals("6A274BB1-4360-4DD0-95C3-A785FFCCDCC2", enterpriseAccountSetDto.getCode());
        Assert.assertEquals("指标分析", enterpriseAccountSetDto.getName());
        
        //check organization
        Assert.assertEquals(5, enterpriseAccountSetDto.getEnterpriseAccountSetOrgList().size());
        List<EnterpriseAccountSetOrgDto> epAccountSetOrgDtoList = enterpriseAccountSetDto.getEnterpriseAccountSetOrgList()
                .stream().sorted(Comparator.comparing(epAccountSetOrgDto -> epAccountSetOrgDto.getOrganizationName()))
                .collect(Collectors.toList());
        
        Assert.assertEquals("06430cbf-e177-4f65-9a2a-d650c36cb564", epAccountSetOrgDtoList.get(4).getID());
        Assert.assertEquals("2891e3dd-d673-49a2-845d-c759202056c6", epAccountSetOrgDtoList.get(4).getEnterpriseAccountSetID());
        Assert.assertEquals("2017-01", epAccountSetOrgDtoList.get(4).getEffectiveDateStr());
        Assert.assertEquals("2017-06", epAccountSetOrgDtoList.get(4).getExpiredDateStr());
        Assert.assertEquals("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182", epAccountSetOrgDtoList.get(4).getOrganizationID());
        Assert.assertEquals("通用机构", epAccountSetOrgDtoList.get(4).getOrganizationName());
        
    }

    @Test
    public void updateEnterpriseAccountSetOrgTestWithoutUpdateOrganization() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_org_validation_data.sql");
        
        Calendar cal = Calendar.getInstance();
        EnterpriseAccountSetOrg enterpriseAccountSetOrg = new EnterpriseAccountSetOrg();
        enterpriseAccountSetOrg.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        //setorg1
        String enterpriseAccountSetOrgID = CommonUtils.getUUID();
        enterpriseAccountSetOrg.setID(enterpriseAccountSetOrgID);
        enterpriseAccountSetOrg.setEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        cal.set(2018, Calendar.MARCH, 1);
        enterpriseAccountSetOrg.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.AUGUST, 31);
        enterpriseAccountSetOrg.setExpiredDate(cal.getTime());
        enterpriseAccountSetOrgMapper.insert(enterpriseAccountSetOrg);
        
        CommonTestUtils.initTestAuth();
        
        //dto preparation
        EnterpriseAccountSetOrgDto enterpriseAccountSetOrgDto = new EnterpriseAccountSetOrgDto();
        enterpriseAccountSetOrgDto.setID(enterpriseAccountSetOrgID);
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetName("测试标准科目");
        enterpriseAccountSetOrgDto.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        enterpriseAccountSetOrgDto.setOrganizationName("通用机构");
        enterpriseAccountSetOrgDto.setEffectiveDateStr("2019-04");
        enterpriseAccountSetOrgDto.setExpiredDateStr("2019-10");
        
        OperationResultDto<?> result = enterpriseAccountSetService.updateEnterpriseAccountSetOrg(enterpriseAccountSetOrgDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        //check data
        EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee");
        example.setOrderByClause("EffectiveDate ASC");
        List<EnterpriseAccountSetOrg> enterpriseAccountSetOrgList = enterpriseAccountSetOrgMapper.selectByExample(example);
        Assert.assertEquals(1, enterpriseAccountSetOrgList.size());
        EnterpriseAccountSetOrg setOrg = enterpriseAccountSetOrgList.get(0);
        Assert.assertEquals("dd424793-46a4-4102-beaa-a7efe161d1ee", setOrg.getEnterpriseAccountSetID());
        Assert.assertEquals("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182", setOrg.getOrganizationID());
        Assert.assertEquals("2019-04-01", new DateTime(setOrg.getEffectiveDate()).toString("yyyy-MM-dd"));
        Assert.assertEquals("2019-10-31", new DateTime(setOrg.getExpiredDate()).toString("yyyy-MM-dd"));
        //check operation log
        OperationLogEnterPriseExample operationLogEnterPriseExample = new OperationLogEnterPriseExample();
        operationLogEnterPriseExample.setOrderByClause("createtime asc");
        List<OperationLogEnterPrise> operationLogs = operationLogEnterpriseMapper.selectByExample(operationLogEnterPriseExample);
        Assert.assertEquals(1, operationLogs.size());
        OperationLogEnterPrise setOrgUpdateLog = operationLogs.get(0);
        Assert.assertEquals("通用机构", setOrgUpdateLog.getOperationContent());
        Assert.assertEquals("测试标准科目", setOrgUpdateLog.getOperationObject());
        Assert.assertEquals("2018-03,2018-08", setOrgUpdateLog.getOriginalState());
        Assert.assertEquals("2019-04,2019-10", setOrgUpdateLog.getUpdateState());
        Assert.assertEquals(OperationModule.BasicDataEnterpriceAccount.name(), setOrgUpdateLog.getModuleName());
        Assert.assertEquals(OperationAction.UpdateRelevantOrg.name(), setOrgUpdateLog.getOperationAction());
    }

    @Test
    public void updateEnterpriseAccountSetOrgTestWithUpdateOrganization() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_org_validation_data.sql");
        
        Calendar cal = Calendar.getInstance();
        EnterpriseAccountSetOrg enterpriseAccountSetOrg = new EnterpriseAccountSetOrg();
        enterpriseAccountSetOrg.setOrganizationID("bd5cfb8c-bc0d-4baf-8fe9-4b55811bf182");
        //setorg1
        String enterpriseAccountSetOrgID = CommonUtils.getUUID();
        enterpriseAccountSetOrg.setID(enterpriseAccountSetOrgID);
        enterpriseAccountSetOrg.setEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        cal.set(2018, Calendar.MARCH, 1);
        enterpriseAccountSetOrg.setEffectiveDate(cal.getTime());
        cal.set(2018, Calendar.AUGUST, 31);
        enterpriseAccountSetOrg.setExpiredDate(cal.getTime());
        enterpriseAccountSetOrgMapper.insert(enterpriseAccountSetOrg);
        
        CommonTestUtils.initTestAuth();
        
        //dto preparation
        EnterpriseAccountSetOrgDto enterpriseAccountSetOrgDto = new EnterpriseAccountSetOrgDto();
        enterpriseAccountSetOrgDto.setID(enterpriseAccountSetOrgID);
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetID("dd424793-46a4-4102-beaa-a7efe161d1ee");
        enterpriseAccountSetOrgDto.setEnterpriseAccountSetName("测试标准科目");
        enterpriseAccountSetOrgDto.setOrganizationID("0db8caa7-2e15-4f83-89f3-221da79625d1");
        enterpriseAccountSetOrgDto.setOrganizationName("华天证券有限公司");
        enterpriseAccountSetOrgDto.setEffectiveDateStr("2019-04");
        enterpriseAccountSetOrgDto.setExpiredDateStr("2019-10");
        
        OperationResultDto<?> result = enterpriseAccountSetService.updateEnterpriseAccountSetOrg(enterpriseAccountSetOrgDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        //check data
        EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo("dd424793-46a4-4102-beaa-a7efe161d1ee");
        example.setOrderByClause("EffectiveDate ASC");
        List<EnterpriseAccountSetOrg> enterpriseAccountSetOrgList = enterpriseAccountSetOrgMapper.selectByExample(example);
        Assert.assertEquals(1, enterpriseAccountSetOrgList.size());
        EnterpriseAccountSetOrg setOrg = enterpriseAccountSetOrgList.get(0);
        Assert.assertEquals("dd424793-46a4-4102-beaa-a7efe161d1ee", setOrg.getEnterpriseAccountSetID());
        Assert.assertEquals("0db8caa7-2e15-4f83-89f3-221da79625d1", setOrg.getOrganizationID());
        Assert.assertEquals("2019-04-01", new DateTime(setOrg.getEffectiveDate()).toString("yyyy-MM-dd"));
        Assert.assertEquals("2019-10-31", new DateTime(setOrg.getExpiredDate()).toString("yyyy-MM-dd"));
        //check operation log
        OperationLogEnterPriseExample operationLogEnterPriseExample = new OperationLogEnterPriseExample();
        operationLogEnterPriseExample.setOrderByClause("createtime asc");
        List<OperationLogEnterPrise> operationLogs = operationLogEnterpriseMapper.selectByExample(operationLogEnterPriseExample);
        Assert.assertEquals(1, operationLogs.size());
        OperationLogEnterPrise setOrgUpdateLog = operationLogs.get(0);
        Assert.assertEquals("通用机构", setOrgUpdateLog.getOperationContent());
        Assert.assertEquals("测试标准科目", setOrgUpdateLog.getOperationObject());
        Assert.assertEquals("通用机构:2018-03,2018-08", setOrgUpdateLog.getOriginalState());
        Assert.assertEquals("华天证券有限公司:2019-04,2019-10", setOrgUpdateLog.getUpdateState());
        Assert.assertEquals(OperationModule.BasicDataEnterpriceAccount.name(), setOrgUpdateLog.getModuleName());
        Assert.assertEquals(OperationAction.UpdateRelevantOrg.name(), setOrgUpdateLog.getOperationAction());
    }

    @Test
    public void updateEnterpriseAccountSetTest() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        CommonTestUtils.initTestAuth();
        
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("885c2ddc-3164-4d57-bab8-12de3d9f83a0");
        enterpriseAccountSetDto.setName("NameUpdate");
        enterpriseAccountSetDto.setCode("CodeUpdate");
        OperationResultDto<?> result = enterpriseAccountSetService.updateEnterpriseAccountSet(enterpriseAccountSetDto);
        
        //check result
        Assert.assertEquals(true, result.getResult());
        
        //check data
        EnterpriseAccountSet updateEntity = enterpriseAccountSetMapper.selectByPrimaryKey("885c2ddc-3164-4d57-bab8-12de3d9f83a0");
        Assert.assertEquals("NameUpdate", updateEntity.getName());
        Assert.assertEquals("CodeUpdate", updateEntity.getCode());
        
        //check operation Log
        OperationLogEnterPriseExample operationLogEnterPriseExample = new OperationLogEnterPriseExample();
        operationLogEnterPriseExample.setOrderByClause("createtime asc");
        List<OperationLogEnterPrise> operationLogs = operationLogEnterpriseMapper.selectByExample(operationLogEnterPriseExample);
        Assert.assertEquals(2, operationLogs.size());
        assertThat(operationLogs).extracting("operationObject").contains("518000");
        assertThat(operationLogs).extracting("originalState").contains("518000", "AAC Demo");
        assertThat(operationLogs).extracting("updateState").contains("CodeUpdate", "NameUpdate");
        
        /*OperationLogEnterPrise codeChangeLog = operationLogs.get(0);
        Assert.assertEquals("518000", codeChangeLog.getOperationObject());
        Assert.assertEquals("518000", codeChangeLog.getOriginalState());
        Assert.assertEquals("CodeUpdate", codeChangeLog.getUpdateState());
        OperationLogEnterPrise nameChangeLog = operationLogs.get(1);
        Assert.assertEquals("518000", nameChangeLog.getOperationObject());
        Assert.assertEquals("AAC Demo", nameChangeLog.getOriginalState());
        Assert.assertEquals("NameUpdate", nameChangeLog.getUpdateState());*/
    }

    @Test
    public void updateEnterpriseAccountSetTestWithCodeDuplicate() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        CommonTestUtils.initTestAuth();
        
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("885c2ddc-3164-4d57-bab8-12de3d9f83a0");
        enterpriseAccountSetDto.setName("NameUpdate");
        enterpriseAccountSetDto.setCode("6A274BB1-4360-4DD0-95C3-A785FFCCDCC2");
        OperationResultDto<?> result = enterpriseAccountSetService.updateEnterpriseAccountSet(enterpriseAccountSetDto);
        
        //check result
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountSetMessage.EnterpriseAccountSetCodeRepeat, result.getResultMsg());
    }

    @Test
    public void updateEnterpriseAccountSetTestWithNameDuplicate() {
        //data preparation
        execSqlFile("sql/enterpriseAccountSet_data.sql");
        CommonTestUtils.initTestAuth();
        
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setID("885c2ddc-3164-4d57-bab8-12de3d9f83a0");
        enterpriseAccountSetDto.setName("指标分析");
        enterpriseAccountSetDto.setCode("CodeUpdate");
        OperationResultDto<?> result = enterpriseAccountSetService.updateEnterpriseAccountSet(enterpriseAccountSetDto);
        
        //check result
        Assert.assertEquals(false, result.getResult());
        Assert.assertEquals(EnterpriseAccountSetMessage.EnterpriseAccountSetNameRepeat, result.getResultMsg());
    }

}
