package pwc.taxtech.atms.service.impl;

import java.util.List;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.dto.MenuDisplayDto;
import pwc.taxtech.atms.dto.MenuDto;
import pwc.taxtech.atms.security.JwtAuthenticationProvider;
import pwc.taxtech.atms.security.JwtAuthenticationToken;
import pwc.taxtech.atms.service.MenuService;

public class MenuServiceIT extends CommonIT {

    private static Logger logger = LoggerFactory.getLogger(MenuServiceIT.class);

    @Autowired
    private MenuService menuService;

    @Autowired
    private JwtAuthenticationProvider jwtAuthenticationProvider;

    @Before
    public void init() {

        execSqlFile("sql/menu.sql");

        // get current user name
        Authentication request = new JwtAuthenticationToken("xxxx");
        Authentication authenticaiton = jwtAuthenticationProvider.authenticate(request);
        SecurityContextHolder.getContext().setAuthentication(authenticaiton);

    }

    @Test
    public void getMenusForDisplayTest() {
        logger.debug("Start to test getMenusForDisplay");
        Integer serviceId = 1;
        List<MenuDisplayDto> menuDisplayDtos = menuService.getMenusForDisplay(serviceId);
        Assert.assertNotNull(menuDisplayDtos);
        Assert.assertEquals(2, menuDisplayDtos.size());
        for (MenuDisplayDto menuDisplayDto : menuDisplayDtos) {
            assertSubDisplayMenus("apps", menuDisplayDto, 2);
            assertSubDisplayMenus("assignment", menuDisplayDto, 4);
            for (MenuDisplayDto subMenuDto : menuDisplayDto.getSubMenus()) {
                assertSubDisplayMenus("dns", subMenuDto, 4);
                assertSubDisplayMenus("local_atm", subMenuDto, 2);
            }
        }
    }

    @Test
    public void getMenusTest() {
        logger.debug("Start to test getMenus");
        Integer serviceId = 1;
        List<MenuDto> menuDtos = menuService.getMenus(serviceId);
        Assert.assertNotNull(menuDtos);
        Assert.assertEquals(2, menuDtos.size());
        for (MenuDto menuDto : menuDtos) {
            assertSubMenus("apps", menuDto, 2);
            assertSubMenus("assignment", menuDto, 5);
            for (MenuDto subMenuDto : menuDto.getSubMenus()) {
                assertSubMenus("dns", subMenuDto, 4);
                assertSubMenus("local_atm", subMenuDto, 2);
            }
        }
    }

    private void assertSubDisplayMenus(String iconClassName, MenuDisplayDto menuDisplayDto, int menuSize) {
        if (menuDisplayDto.getIconClassName().equals(iconClassName)) {
            Assert.assertNotNull(menuDisplayDto.getSubMenus());
            Assert.assertEquals(menuDisplayDto.getSubMenus().size(), menuSize);
        }
    }

    private void assertSubMenus(String iconClassName, MenuDto menuDto, int menuSize) {
        if (menuDto.getIconClassName().equals(iconClassName)) {
            Assert.assertNotNull(menuDto.getSubMenus());
            Assert.assertEquals(menuDto.getSubMenus().size(), menuSize);
        }
    }

    @After
    public void clean() {
        cleanUpMenuTable();
    }

}
