package pwc.taxtech.atms.service.impl;

import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import com.fasterxml.jackson.core.JsonProcessingException;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.dto.navtree.DevTreeDto;
import pwc.taxtech.atms.dto.navtree.RolePermissionDisplayDto;
import pwc.taxtech.atms.service.PermissionService;

public class PermissionServiceIT extends CommonIT {

    private static final Logger logger = LoggerFactory.getLogger(PermissionServiceIT.class);
    
    @Autowired private PermissionService permissionService;
    
    @Before
    public void setUp() {
        cleanUpAllTables();
    }
    
    /*@Test
    public void test() {
        rolePermissionMapper.selectByRoleListAndServiceTypeWithAssociation(
                Arrays.asList("1", "2"), "2");
    }*/
    
    @Test
    public void getIvhTreePermissionsByRoleIDTest() throws JsonProcessingException {
        
        /*
         * role: id=07CCEC5C-F059-4864-8009-D460689F1007
         */
        //data preparation
        execSqlFile("sql/permission_data.sql");
        execSqlFile("sql/user_role_permission_data.sql");
        
        RolePermissionDisplayDto rolePermissionDisplayDto = permissionService.getIvhTreePermissionsByRoleID("07CCEC5C-F059-4864-8009-D460689F1007", "2");
        logger.debug("print result json:{}", objectMapper.writeValueAsString(rolePermissionDisplayDto));

        //check dto
        Assert.assertEquals(2, rolePermissionDisplayDto.getRolePermissionList().size());
        Assert.assertEquals(1, rolePermissionDisplayDto.getPermissionDevTreeList().size());
    }
    
    @Test
    public void getIvhTreePermissionsByRoleIDListTest() throws JsonProcessingException {
        
        /*
         * 07CCEC5C-F059-4864-8009-D460689F1007,36FB1BA2-EBFA-4FDF-9B8F-0E04E9DBCB97
         */
        //data preparation
        execSqlFile("sql/permission_data.sql");
        execSqlFile("sql/user_role_permission_data.sql");
        
        List<DevTreeDto> tree = permissionService.getIvhTreePermissionsByRoleIDList(Arrays.asList("07CCEC5C-F059-4864-8009-D460689F1007" ,"36FB1BA2-EBFA-4FDF-9B8F-0E04E9DBCB97"), "2");
        
        logger.debug("print result json:{}", objectMapper.writeValueAsString(tree));
        Assert.assertEquals(1, tree.size());
    
    }
    
    @Test
    public void getAllPermissionsTest() throws JsonProcessingException {
        
        //data preparation
        execSqlFile("sql/permission_data.sql");
        
        List<DevTreeDto> tree = permissionService.getAllPermissions("2");
        logger.debug("print result json:{}", objectMapper.writeValueAsString(tree));
        
        Assert.assertEquals(1, tree.size());
    
    }
}
