package pwc.taxtech.atms.service.impl;

import static org.junit.Assert.assertEquals;

import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.dto.VMRegionDevTreeData;
import pwc.taxtech.atms.dto.navtree.DevTreeDto;
import pwc.taxtech.atms.dto.navtree.IvhTreeDto;
import pwc.taxtech.atms.entitiy.Region;
import pwc.taxtech.atms.service.RegionService;

public class RegionServiceIT extends CommonIT {

	@Autowired
	private RegionService regionService;

	@Before
	public void setUp() {
		cleanUpAllTables();
	}

	@Test
	public void getRegionAreaTreeTest() {

		// data preparatino
		execSqlFile("sql/area_data.sql");
		execSqlFile("sql/areaRegion_data.sql");
		execSqlFile("sql/region_data.sql");

		List<DevTreeDto<VMRegionDevTreeData>> tree = regionService.getRegionAreaTree();
		assertEquals(31, tree.size());
		DevTreeDto<VMRegionDevTreeData> firstArea = tree.get(0);
		assertEquals("7c360a44-7a17-4ca0-9c2a-a88cde7670d5", firstArea.getId());
		assertEquals("1111", firstArea.getText());
		List<DevTreeDto<VMRegionDevTreeData>> itemsOfFirstArea = firstArea.getItems();
		assertEquals(8, itemsOfFirstArea.size());
		DevTreeDto<VMRegionDevTreeData> secondSubAreaOfFirstArea = itemsOfFirstArea.get(1);
		assertEquals("65476f34-58b2-40b7-865f-8a12b8b0a5e6", secondSubAreaOfFirstArea.getId());
		assertEquals("top", secondSubAreaOfFirstArea.getText());
		DevTreeDto<VMRegionDevTreeData> firstRegionOfFirstArea = itemsOfFirstArea.get(6);
		assertEquals("110000", firstRegionOfFirstArea.getId());
		assertEquals("北京市", firstRegionOfFirstArea.getText());
		assertEquals(1, firstRegionOfFirstArea.getItems().size());
		assertEquals("110000", firstRegionOfFirstArea.getItems().get(0).getParentId());
	}

	@Test
	public void getProvinceAndCityTreeListTest() {

		// data preparatino
		execSqlFile("sql/region_data.sql");
		
		List<DevTreeDto<IvhTreeDto<Region>>> tree = regionService.getProvinceAndCityTreeList();
		assertEquals(35, tree.size());
		DevTreeDto<IvhTreeDto<Region>> hebeiRegion = tree.get(2);
		assertEquals(11, hebeiRegion.getData().getChildren().size());
		assertEquals(11, hebeiRegion.getItems().size());
	}
	
	@Test
	public void getAreRegionTreeByNodeIDTest() {
        // TODO
    }

}
