﻿systemConfigurationModule
    .controller('DeclarationFormConfigurationController', ['$scope', '$log', '$timeout', '$translate', '$rootScope', '$interval', '$compile', '$uibModal', '$q', 'SweetAlert', 'templateGroupService',
        'templateService', 'templateFormulaService', 'stdAccountService', 'spreadJsTipService', 'enums', 'KeyValueConfigService', 'modelConfigurationService', 'industryService',
        'formulaService', 'serviceTypeService', 'mentioUtil', 'region',
        function ($scope, $log, $timeout, $translate, $rootScope, $interval, $compile, $uibModal, $q, SweetAlert, templateGroupService, templateService, templateFormulaService,
                  stdAccountService, spreadJsTipService, enums, KeyValueConfigService, modelConfigurationService, industryService, formulaService, serviceTypeService, mentioUtil, region) {
            'use strict';

            var keyValueExp = /@\S*/g;
            var keyValueIllegalExp = /[^a-zA-z0-9\.]+/;
            var blankExp = /\s/g;
            var notDuplicateOperatorExp = /(,[\*\/]+)|([\+\-\*\/]+)/g;
            var missingExpressionExp = /([\+\-\*\/,]+$)|([\+\-\*\/]+[\),])/;
            var missingOperatorExp = /([^\+\-\*\/,\(]+@)|(@[^\+\-\*\/,\)]+)/;

            var _spread = null;

            // 本页面常量
            var thisConstant = {
                deleteClass: 'material-icons delete',
                editClass: 'material-icons edit',
                findAddClass: 'fa fa-search-plus organge-color',
            };


            var serviceTypeMapping = {
                VAT: $translate.instant('VAT'),
                RPT: $translate.instant('RPT'),
                CIT: $translate.instant('CIT'),
                DTA: $translate.instant('DTA'),
                CF: $translate.instant('CashFlow'),
                AssetsManage: $translate.instant('AssetsManage')
            };

            // Config if demo, if current service is only demo, will not validate the formula
            var demoMapping = {
                VAT: false,
                RPT: true,
                CIT: true,
                DTA: true,
                CF: true,
                AssetsManage: true
            };

            var filingTypeList = _.map(constant.payTaxTypeList, function (x) {
                return {
                    value: x.value,
                    name: $translate.instant(x.name)
                };
            });

            var citReportTypeList = _.map(constant.citReportTypeList, function (x) {
                return {
                    value: x.value,
                    name: $translate.instant(x.name)
                };
            });

            var cfReportTypeList = _.map(constant.cfReportTypeList, function (x) {
                return {
                    value: x.value,
                    name: $translate.instant(x.name)
                };
            });

            var hidePanel = function () {
                $('#editReportFormulaPop').hide('normal');
            };

            var togglePanel = function () {
                if ($('#editReportFormulaPop #editReportFormulaContent').css('display') == 'none') {
                    $('#editReportFormulaPop').height(610);
                } else {
                    $('#editReportFormulaPop').height(40);
                }
                $timeout(function () {
                    $('#editReportFormulaPop #editReportFormulaContent').toggle('normal');
                }, 50);
            };

            //显示纳税申报表的单元格信息详情
            var showConfigPage = function () {
                var h = -3 * $(window).height() / 4;
                var w = 1 * $(window).width() / 4
                $('#editReportFormulaPop').css({
                    'position': 'relative', 'z-index': '1', top: h + 'px', left: w + 'px'
                });
                $('#editReportFormulaPop').show('normal');
            };

            //加载'报表模板'源数据
            var loadTemplateGroup = function () {

                if ($scope.curIndustry && $scope.curIndustry.id) {

                    var payTaxType = null;
                    if ($scope.curServiceTypeId !== enums.serviceType.CIT && $scope.curServiceTypeId !== enums.serviceType.Others) {
                        if ($scope.payTaxType && $scope.payTaxType.value) {
                            payTaxType = $scope.payTaxType.value;
                        }
                        else {
                            return;
                        }
                    }

                    templateGroupService.getByIndustry($scope.curServiceTypeId, payTaxType, $scope.curIndustry.id)
                        .success(function (templateGroupData) {
                            templateGroupData = _.sortBy(templateGroupData, function (num) {
                                return -num.isSystemType;
                            });
                            $scope.templateGroups = templateGroupData;

                            var defaultValue = templateGroupData[0];
                            if ($scope.curTemplateGroup && $scope.curTemplateGroup.name) {
                                var find = _.find(templateGroupData, function (row) {
                                    return row.name === $scope.curTemplateGroup.name || row.id === $scope.curTemplateGroup.id;
                                });

                                if (find) {
                                    defaultValue = find;
                                    $scope.templateGroupId = find.id;
                                }
                            }

                            dxControl.renderTemplateGroupDropDown(templateGroupData, defaultValue);

                            if (templateGroupData && templateGroupData.length > 0) {
                                loadReportType();
                            } else {
                                $('#dx-select-template-groups').dxDropDownBox('instance').option('dataSource', []);
                                $('#dx-select-template').dxDropDownBox('instance').option('dataSource', []);
                            }
                        });

                }
            };

            //加载'报表类型'源数据
            var loadReportType = function () {
                var defaultReportTypeOpt = {
                    displayExpr: 'name',
                    valueExpr: "name",
                    itemTemplate: 'reportTypeItem',
                    onSelectionChanged: function (e) {
                        $scope.reportType = e.selectedItem;
                        listTemplates();
                    }
                };

                if ($scope.curTemplateGroup && $scope.curTemplateGroup.id) {
                    if ($scope.curServiceTypeId === enums.serviceType.CIT) {
                        $scope.detailReportTypeList = angular.copy(defaultReportTypeOpt);
                        $scope.detailReportTypeList.dataSource = new DevExpress.data.ArrayStore({
                            data: citReportTypeList
                        });
                        $scope.detailReportTypeList.value = citReportTypeList[0].name;
                        $('#dx-select-report-type').dxSelectBox('instance').option($scope.detailReportTypeList);
                        $scope.reportType = citReportTypeList[0];
                        listTemplates();
                    } else if ($scope.curServiceTypeId === enums.serviceType.CF) {
                        $scope.detailReportTypeList = angular.copy(defaultReportTypeOpt);
                        $scope.detailReportTypeList.dataSource = new DevExpress.data.ArrayStore({
                            data: cfReportTypeList
                        });
                        $scope.detailReportTypeList.value = cfReportTypeList[1].name;
                        $('#dx-select-report-type').dxSelectBox('instance').option($scope.detailReportTypeList);
                        $scope.reportType = cfReportTypeList[1];
                        listTemplates();
                    } else if ($scope.curServiceTypeId === enums.serviceType.Others) {
                        $scope.detailReportTypeList = angular.copy(defaultReportTypeOpt);
                        $scope.detailReportTypeList.dataSource = new DevExpress.data.ArrayStore({
                            data: citReportTypeList
                        });
                        $scope.detailReportTypeList.value = citReportTypeList[0].name;
                        $('#dx-select-report-type').dxSelectBox('instance').option($scope.detailReportTypeList);
                        $scope.reportType = citReportTypeList[0];
                        listTemplates();
                    } else {
                        $('#dx-select-report-type').dxSelectBox('instance').option('dataSource', []);
                        $scope.reportType = null;
                        listTemplates();
                    }
                }
            };

            var loadIndustry = function () {
                industryService.getAllAvailableIndustry().success(function (industryData) {
                    if (industryData && industryData.length > 0) {
                        if (region !== 'zh-CN') {
                            angular.forEach(industryData, function (x) {
                                switch (x.id) {
                                    case '0':
                                        x.name = $translate.instant('GeneralIndustry');
                                        break;
                                    case '1':
                                        x.name = $translate.instant('IndustryIndustry');
                                        break;
                                    case '10':
                                        x.name = $translate.instant('AssetManagement');
                                        break;
                                    case '13':
                                        x.name = $translate.instant('MobileManufacturingIndustry');
                                        break;
                                    case '14':
                                        x.name = $translate.instant('FinancialServiceIndustry');
                                        break;
                                    case '20':
                                        x.name = $translate.instant('ManufacturingIndustry');
                                        break;
                                    case '23':
                                        x.name = $translate.instant('EstateIndustry');
                                        break;
                                    default:
                                        x.name = x.englishName;
                                        break;
                                }
                            });
                        }

                        //配置行业下拉列表
                        $scope.dataSourceIndustryList = {
                            dataSource: new DevExpress.data.ArrayStore({
                                data: industryData
                            }),
                            displayExpr: 'name',
                            valueExpr: "id",
                            value: industryData[0].id,
                            itemTemplate: 'industryItem',
                            onItemClick: function (e) {
                                $scope.curIndustry = e.itemData;
                                loadTemplateGroup();
                            },
                            onInitialized: function (e) {
                                $scope.dataSourceIndustryListInstance = e.component;
                            }
                        }
                        $scope.curIndustry = industryData[0];
                        $('#dx-select-industry').dxSelectBox('instance').option('value', $scope.curIndustry.id);
                        loadTemplateGroup();
                    }
                });
            };

            var loadServiceType = function () {
                serviceTypeService.getServiceTypeList().success(function (data) {
                    if (data && data.length > 0) {
                        //配置服务类型列表
                        $scope.serviceTypeList = _.chain(data).sortBy(function (s) {
                            return Number(s.id);
                        }).map(function (s) {
                            var rtn = null;
                            if (serviceTypeMapping.hasOwnProperty(s.prefix)) {
                                rtn = {
                                    id: s.id,
                                    name: serviceTypeMapping[s.prefix]
                                };

                                if (demoMapping.hasOwnProperty(s.prefix)) {
                                    rtn.isDemo = demoMapping[s.prefix];
                                }
                            }

                            return rtn;
                        }).reject(function (s) {
                            return _.isEmpty(s);
                        }).value();

                        $scope.serviceTypeList.push({
                            id: '99',
                            isDemo: true,
                            name: $translate.instant('Others')
                        });

                        $scope.curServiceTypeId = $scope.serviceTypeList[0].id;
                        $scope.curServiceTypeName = $scope.serviceTypeList[0].name;
                        $scope.isDemo = $scope.serviceTypeList[0].isDemo;

                        $scope.payTaxType = filingTypeList[0];
                        $('#dx-select-pay-tax-type').dxSelectBox('instance').option('value', $scope.payTaxType.name);

                        loadIndustry();
                    }
                });
            };

            var getServiceTypeList = function () {
                var rtn = [];
                if (!_.isEmpty($scope.serviceTypeList)) {
                    rtn = _.reject($scope.serviceTypeList, function (s) {
                        return s.id === $scope.curServiceTypeId;
                    });
                }

                return rtn;
            };

            var toggleServiceType = function (s) {
                $scope.curServiceTypeId = s.id;
                $scope.curServiceTypeName = s.name;
                $scope.isDemo = s.isDemo;
                $scope.payTaxType = filingTypeList[0];
                $('#dx-select-pay-tax-type').dxSelectBox('instance').option('value', $scope.payTaxType.name);

                loadIndustry();
            };

            var hasError = function () {
                return $scope.editReportFormulaForm &&
                    ($scope.editReportFormulaForm.formula && !_.isEmpty($scope.editReportFormulaForm.formula.$error) ||
                        $scope.editReportFormulaForm.voucher && !_.isEmpty($scope.editReportFormulaForm.voucher.$error) ||
                        $scope.editReportFormulaForm.invoice && !_.isEmpty($scope.editReportFormulaForm.invoice.$error) ||
                        $scope.editReportFormulaForm.model && !_.isEmpty($scope.editReportFormulaForm.model.$error) ||
                        $scope.editReportFormulaForm.validation && !_.isEmpty($scope.editReportFormulaForm.validation.$error))
            };

            //加载'报表'源数据
            var listTemplates = function () {
                templateGroupService.getTemplateList($scope.curTemplateGroup.id, $scope.reportType && $scope.reportType.value).success(function (templateData) {

                    templateData = _.filter(templateData, function (row) {
                        return row.isActiveAssociation;
                    });

                    var defaultValue = templateData[0];
                    if ($scope.currentTemplate && ($scope.currentTemplate.name || $scope.currentTemplate.id)) {
                        var find = _.find(templateData, function (row) {
                            return row.name === $scope.currentTemplate.name || row.id === $scope.currentTemplate;
                        });

                        if (find) {
                            defaultValue = find;
                        }
                    }

                    $scope.templates = templateData; //当前组下的所有模板数据

                    dxControl.renderTemplateDropDown(templateData, defaultValue);
                    //配置报表下拉框
                    if (templateData && templateData.length > 0) {
                        $scope.ToggleSaveAs = showSavebutton();
                    }
                });
            };
            //模板不是默认模板才可以编辑和保存 
            var showSavebutton = function () {
                var isShow = false;
                if ($scope.curTemplateGroup && $scope.curTemplateGroup !== undefined) {
                    var isSystemType = $scope.curTemplateGroup.isSystemType;
                    if (!isSystemType) {
                        isShow = true;
                    }
                }

                return isShow;
            };

            var setCellFormula = function (templateID) {
                $log.debug('current templateID:' + templateID + ', getting template formula date...');

                //根据templateId获取公式配置数据
                templateFormulaService.getCellTemplateConfigList(templateID).success(function (getResult) {

                    if (getResult.result === false) {
                        SweetAlert.warning($translate.instant(getResult.resultMsg));
                        return;
                    }

                    if (!getResult.data) {
                        getResult.data = [];
                    }

                    $scope.templateFormulaList = getResult.data.filter(function (item) {
                        return !item.isReadOnly;
                    });

                    refreshSpreadUI();
                });

            };

            // 刷新选中的过滤凭证科目代码
            var refreshVoucherFilter = function () {
                if (!$scope.editModel.accountCodes) {
                    $scope.editModel.accountCodes = [];
                }

                if (!_.isEmpty($scope.editModel.accountCodes)) {
                    var selectedData = _.filter($scope.accountDataSource, function (data) {
                        return $scope.editModel.accountCodes.indexOf(data.code) > -1;
                    });

                    $scope.selectedAccount = selectedData;
                } else {
                    $scope.selectedAccount = [];
                }

                $scope.refreshGrids();
            };

            // 刷新选中的关联模型
            var refreshModelFilter = function () {
                if (!$scope.editModel.modelIDs) {
                    $scope.editModel.modelIDs = [];
                }

                if (!_.isEmpty($scope.editModel.modelIDs)) {
                    var selectedData = _.filter($scope.modelDataSource, function (data) {
                        return $scope.editModel.modelIDs.indexOf(data.id) > -1;
                    });

                    $scope.selectedModel = selectedData;
                } else {
                    $scope.selectedModel = [];
                }

                $scope.refreshGrids();
            };

            // 获取发票设置
            var getInvoiceFilter = function () {
                var items = [{
                    type: 'PleaseSelect',
                    value: 0,
                    categoryOptions: [],
                    rateOptions: [],
                    amountTypeOptions: [
                        {name: 'PleaseSelect', value: 0},
                        {name: 'TotalAmount', value: 1},
                        {name: 'TaxAmount', value: 2},
                        {name: 'CopyNumber', value: 3}
                    ]
                },
                    {
                        type: 'Income',
                        value: 1,
                        categoryOptions: _.chain(enums.invoiceType).pairs().map(function (it) {
                            return {key: $translate.instant(it[0]), value: it[1], ticked: false};
                        }).value(),
                        rateOptions: [
                            {key: '17%', value: '17%', ticked: false},
                            {key: '13%', value: '13%', ticked: false},
                            {key: '11%', value: '11%', ticked: false},
                            {key: '6%', value: '6%', ticked: false},
                            {key: '5%', value: '5%', ticked: false},
                            {key: '4%', value: '4%', ticked: false},
                            {key: '3%', value: '3%', ticked: false},
                            {key: '1.5%', value: '1.5%', ticked: false}
                        ],
                        amountTypeOptions: [
                            {name: 'PleaseSelect', value: 0},
                            {name: 'TotalAmount', value: 1},
                            {name: 'TaxAmount', value: 2},
                            {name: 'CopyNumber', value: 3}
                        ]
                    },
                    {
                        type: 'Output',
                        value: 2,
                        categoryOptions: _.chain(enums.outputInvoiceType).pairs().map(function (it) {
                            return {key: $translate.instant(it[0]), value: it[1], ticked: false};
                        }).value(),
                        rateOptions: [
                            {key: '17%', value: '17%', ticked: false},
                            {key: '13%', value: '13%', ticked: false},
                            {key: '11%', value: '11%', ticked: false},
                            {key: '6%', value: '6%', ticked: false},
                            {key: '5%', value: '5%', ticked: false},
                            {key: '4%', value: '4%', ticked: false},
                            {key: '3%', value: '3%', ticked: false},
                            {key: '1.5%', value: '1.5%', ticked: false}
                        ],
                        amountTypeOptions: [
                            {name: 'PleaseSelect', value: 0},
                            {name: 'TotalAmount', value: 1},
                            {name: 'TaxAmount', value: 2},
                            {name: 'CopyNumber', value: 3}
                        ]
                    },
                    {
                        type: 'CustomsAudit',
                        value: 3,
                        categoryOptions: [],
                        rateOptions: [],
                        amountTypeOptions: [
                            {name: 'PleaseSelect', value: 0},
                            {name: 'TotalAmount', value: 1},
                            {name: 'TaxAmount', value: 2},
                            {name: 'CopyNumber', value: 3}
                        ]
                    }
                ];

                return items;
            };

            // 选中某一发票类型级联更新税率与金额类型列表
            var selectInvoiceOption = function (editModel, invoiceType, isInit) {
                var selectedInvoice = _.where($scope.invoiceOptions, {value: invoiceType});
                var op = selectedInvoice.length === 0 ? $scope.invoiceOptions[0] : selectedInvoice[0];

                // 如果获取到的editModel.invoiceCategory为[ 99 ],代表发票类型全选；
                // 否则只勾选editModel.invoiceCategory数组中有的税率
                var categoryOptions = _.map(op.categoryOptions, function (o) {
                    var rtn = angular.copy(o);
                    if (isInit && editModel.invoiceCategory &&
                        (_.contains(editModel.invoiceCategory, Number(constant.selectAllValue)) || _.contains(editModel.invoiceCategory, rtn.value))) {
                        rtn.ticked = true;
                    } else {
                        rtn.ticked = false;
                    }

                    return rtn;
                });

                // 如果获取到的editModel.taxRate为[ '99' ],代表税率全选；
                // 否则只勾选editModel.taxRate数组中有的税率
                var rateOptions = _.map(op.rateOptions, function (o) {
                    var rtn = angular.copy(o);
                    if (isInit && editModel.taxRate &&
                        (_.contains(editModel.taxRate, constant.selectAllValue) || _.contains(editModel.taxRate, rtn.value))) {
                        rtn.ticked = true;
                    } else {
                        rtn.ticked = false;
                    }

                    return rtn;
                });

                var choseAmoutType = isInit && editModel.invoiceAmountType && editModel.invoiceAmountType > 0 ?
                    _.where(op.amountTypeOptions, {value: editModel.invoiceAmountType})[0] : op.amountTypeOptions[0];

                editModel.chosenInvoiceOption = {
                    type: op.type,
                    value: op.value,
                    categoryOptions: categoryOptions,
                    rateOptions: rateOptions,
                    invoiceAmountType: choseAmoutType
                };
            };

            var resetFormHeight = function () {
                var sumHeight = $(document).height();
                var headerHeight = $('.menu-header').height();
                var topHeight = $('.templates-top').height();

                var bodyHeigh = $('.templates-body').css('height', (sumHeight - headerHeight - topHeight - 50) + 'px');
                $('#report-view').css('height', (sumHeight - headerHeight - topHeight - 50) + 'px');
            };

            var loadSheet = function (template) {
                $scope.currentTemplate = template;

                $('#report-view').html('');

                templateService.renderSpreadExcelSimple('report-view', template.id).then(function (spread) {
                    $(window).on('resize', function () {
                        resetFormHeight();
                        $timeout(function () {
                            if (spread && spread.refresh) {
                                spread.refresh();
                            }
                        }, 500);
                    });

                    _spread = spread;
                    var activeSheet = _spread.getSheet(1);
                    //设置整个sheet不可编辑
                    activeSheet.isProtected = true;
                    activeSheet.options.protectionOption = {
                        allowEditObjects: false
                    };

                    $scope.activeSheet = activeSheet;

                    setCellFormula(template.id);

                    //给单元格增加双击事件
                    activeSheet.bind(GC.Spread.Sheets.Events.CellDoubleClick, function (sender, args) {
                        var sheet = args.sheet;
                        var cell = sheet.getCell(args.row, args.col);
                        showEditReportFormulaPop(template, args.row, args.col, true);
                    });

                    //给单元格增加单击事件
                    activeSheet.bind(GC.Spread.Sheets.Events.CellClick, function (sender, args) {
                        var sheet = args.sheet;
                        var cell = sheet.getCell(args.row, args.col);
                        showEditReportFormulaPop(template, args.row, args.col, false);
                    });
                });
            };

            var processSpreadContextMenu = function (e) {
                // move the context menu to the position of the mouse point
                var sheet = _spread.getActiveSheet(),
                    target = getHitTest(e.pageX, e.pageY, sheet),
                    hitTestType = target.hitTestType,
                    row = target.row,
                    col = target.col,
                    selections = sheet.getSelections();

                var isHideContextMenu = false;
                if ($scope.currentTemplate.isSystemType) {
                    isHideContextMenu = true;
                }
                else if (hitTestType === GC.Spread.Sheets.SheetArea.colHeader) {
                    if (getCellInSelections(selections, row, col) === null) {
                        sheet.setSelection(-1, col, sheet.getRowCount(), 1);
                    }
                    if (row !== undefined && col !== undefined) {
                        $(".context-header").show();
                        $(".context-cell").hide();
                    }
                } else if (hitTestType === GC.Spread.Sheets.SheetArea.rowHeader) {
                    if (getCellInSelections(selections, row, col) === null) {
                        sheet.setSelection(row, -1, 1, sheet.getColumnCount());
                    }
                    if (row !== undefined && col !== undefined) {
                        $(".context-header").show();
                        $(".context-cell").hide();
                    }
                } else if (hitTestType === GC.Spread.Sheets.SheetArea.viewport) {
                    if (getCellInSelections(selections, row, col) === null) {
                        sheet.clearSelection();
                        sheet.setActiveCell(row, col);
                    }
                    if (row !== undefined && col !== undefined) {
                        $(".context-header").hide();
                        $(".context-cell").hide();
                    } else {
                        isHideContextMenu = true;
                    }
                } else if (hitTestType === GC.Spread.Sheets.SheetArea.corner) {
                    sheet.setSelection(-1, -1, sheet.getRowCount(), sheet.getColumnCount());
                    if (row !== undefined && col !== undefined) {
                        $(".context-header").hide();
                        $(".context-cell").show();
                    }
                }

                var $contextMenu = $("#spreadContextMenu");
                $contextMenu.data("sheetArea", hitTestType);
                if (isHideContextMenu) {
                    hideSpreadContextMenu();
                } else {
                    $contextMenu.css({left: e.pageX, top: e.pageY});
                    $contextMenu.show();

                    $(document).on("click.contextmenu", function () {
                        if ($(event.target).parents("#spreadContextMenu").length === 0) {
                            hideSpreadContextMenu();
                        }
                    });
                }
            };

            var processContextMenuClicked = function () {
                var action = $(this).data("action");
                var sheet = _spread.getActiveSheet();
                var sheetArea = $("#spreadContextMenu").data("sheetArea");

                hideSpreadContextMenu();

                switch (action) {
                    case "applyRowColName":
                        setRowColName(sheet);
                        break;
                    default:
                        break;
                }
            };

            var hideSpreadContextMenu = function () {
                $("#spreadContextMenu").hide();
                $(document).off("click.contextmenu");
            };

            var getCellInSelections = function (selections, row, col) {
                var count = selections.length, range;
                for (var i = 0; i < count; i++) {
                    range = selections[i];
                    if (range.contains(row, col)) {
                        return range;
                    }
                }
                return null;
            };

            var getHitTest = function (pageX, pageY, sheet) {
                var offset = $("#report-view").offset(),
                    x = pageX - offset.left,
                    y = pageY - offset.top;
                return sheet.hitTest(x, y);
            };

            var setRowColName = function (sheet) {
                var candidateNames = getSelectedCells();
                candidateNames.forEach(function (c) {
                    var cell = sheet.getCell(c.rowIndex, c.columnIndex);
                    if (cell) {
                        var cellValue = cell.value();
                        if (!_.isEmpty(cellValue)) {
                            cellValue = cellValue.replaceToEnd('\r', '').replaceToEnd('\n', '').trim();
                        }

                        c.value = cellValue;
                    }
                });
                candidateNames = _.reject(candidateNames, function (x) {
                    return _.isEmpty(x.value);
                });
                templateService.setRowColName($scope.currentTemplate.id, candidateNames).then(function (data) {
                    if (data) {
                        SweetAlert.success($translate.instant('CellRowColNameSetSuccess'));
                        //var template = _.findWhere($scope.templates, { id: $scope.currentTemplate.id })
                        loadSheet($scope.currentTemplate);
                    }
                });
            };

            var getSelectedCells = function () {
                var activeSheet = _spread.getActiveSheet();
                var selection = activeSheet.getSelections();
                var spans = activeSheet.getSpans();
                var exceptedCells = [];
                for (var k = 0; k < spans.length; k++) {
                    for (var i = 0; i < spans[k].rowCount; i++) {
                        for (var j = 0; j < spans[k].colCount; j++) {
                            if (i > 0 || j > 0) {
                                exceptedCells.push({rowIndex: spans[k].row + i, columnIndex: spans[k].col + j});
                            }
                        }
                    }
                }

                var validPosition = [];
                for (var k = 0; k < selection.length; k++) {
                    var n = selection[k];
                    var startRow = n.row >= 0 ? n.row : 0;
                    var startCol = n.col >= 0 ? n.col : 0;

                    if (n.rowCount >= 1 && n.colCount >= 1) {
                        for (var i = 0; i < n.rowCount; i++) {
                            for (var j = 0; j < n.colCount; j++) {
                                var row = startRow + i;
                                var col = startCol + j;
                                if (_.every(exceptedCells, function (c) {
                                        return c.rowIndex !== row || c.columnIndex !== col;
                                    })) {
                                    validPosition.push({rowIndex: row, columnIndex: col});
                                }
                            }
                        }
                    }
                }

                return validPosition;
            };

            var confirmWarningWindow = function (title, text) {
                var deferred = $q.defer();
                SweetAlert.swal({
                        title: title,
                        text: text,
                        type: "warning",
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        allowOutsideClick: false,
                        confirmButtonText: $translate.instant('Confirm'),
                        cancelButtonText: $translate.instant('Cancel'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                    function (isConfirm) {
                        deferred.resolve(isConfirm);
                    });

                return deferred.promise;
            };

            var showEditReportFormulaPop = function (template, row, col, isDoubleClick) {

                if ($scope.templateFormulaList && $scope.templateFormulaList.length > 0) {

                    var formulaModel = _.find($scope.templateFormulaList, function (num) {
                        return num.templateID === template.id && num.rowIndex === row && num.columnIndex === col
                    });
                    if (formulaModel) {
                        $scope.editModel = angular.copy(formulaModel);
                        $scope.editModel.cell = PWC.numToExcelChar(row, col);

                        $scope.editModel.isSystemType = template.isSystemType;
                        $scope.selectedAccount = [];
                        $scope.selectedModel = [];

                        changeTab($scope.editModel);

                        refreshVoucherFilter();
                        refreshModelFilter();

                        // Select default invoice option if no related info in editModel
                        selectInvoiceOption($scope.editModel, $scope.editModel.invoiceType, true);

                        // invoiceOption: Used to save invoice tax rate and amount type options temporarily
                        $scope.invoiceOption = _.findWhere($scope.invoiceOptions, {type: $scope.editModel.chosenInvoiceOption.type});

                        $scope.mentionApi.triggerValidator($scope.editModel.formula, $scope.editReportFormulaForm.formula);
                        $scope.mentionApiForValidation.triggerValidator($scope.editModel.validation, $scope.editReportFormulaForm.validation);
                        asyncFormulaValidate();
                        asyncFormulaValidateForValidation();
                        if (isDoubleClick) {
                            var cell = $scope.activeSheet.getCell(row, col);
                            if (cell.value() === '——') {
                                confirmWarningWindow($translate.instant('ReportCellUpdateWarning'), '').then(function (data) {
                                    if (data) {
                                        showConfigPage();
                                    }
                                });
                            } else {
                                showConfigPage();
                            }
                        }
                        //$('#editReportFormulaPop').show('normal');
                        $(".editReportFormulaPop-modal-content").css('top', '0px');
                        $(".editReportFormulaPop-modal-content").css('left', '0px');
                        $(".editReportFormulaPop-modal-content").draggable({
                            cursor: "cursor",
                            scroll: true,
                            cancel: 'input, label, .btn, .form-control, .close, div.label, .has-error'
                        });
                    }
                }
            };

            var changeTab = function (editModel) {
                if (_.isEmpty(editModel)) {
                    // Do nothing
                }
                else if (editModel.hasFormula) {
                    $scope.selectedTab = 0;
                }
                else if (editModel.hasInvoice || editModel.hasVoucher || editModel.hasKeyIn) {
                    $scope.selectedTab = 1;
                }
                else if (editModel.hasModel) {
                    $scope.selectedTab = 2;
                }
                else if (editModel.hasValidation) {
                    $scope.selectedTab = 3;
                }
                else { // all options are not configured
                    $scope.selectedTab = 0;
                }
            };

            var initAccountSelector = function () {
                $scope.selectedAccount = [];

                $scope.voucherGridOptions = {
                    bindingOptions: {
                        dataSource: {
                            deep: false,
                            dataPath: 'selectedAccount'
                        }
                    },
                    columns: [
                        {caption: $translate.instant('SubjectCodeCol'), dataField: 'code'},
                        {caption: $translate.instant('SubjectNameCol'), dataField: 'name'},
                        {caption: $translate.instant('SubjectDirectionCol'), dataField: 'directionName'},
                        {caption: $translate.instant('SubjectTypeCol'), dataField: 'AcctPropName'},
                        {
                            alignment: 'center',
                            width: '40px',
                            cellTemplate: function (container, options) {
                                var compileInlineBtn = $compile('<a class="dx-link dx-link-delete"\
ng-show="hasEditPermission && ToggleSaveAs && editModel.hasVoucher"/>');
                                compileInlineBtn($scope)
                                    .html('<i class="fa fa-trash" aria-hidden="true"></i>')
                                    .on('dxclick', function () {
                                        options.component.removeRow(options.rowIndex);
                                    })
                                    .appendTo(container);
                            }
                        }

                    ],
                    loadPanel: {
                        enabled: false
                    },
                    paging: {
                        enabled: false
                    },
                    scrolling: {
                        mode: "virtual"
                    },
                    sorting: {
                        mode: "none"
                    },
                    height: '100%',
                    showBorders: true,
                    showRowLines: true,
                    showColumnLines: true,
                    onRowRemoving: function (e) {
                        var idx = $scope.selectedAccount.indexOf(e.data);
                        if (idx >= 0) {
                            $scope.selectedAccount.splice(idx, 1);
                        }
                    }
                };
            };

            var initModelSelector = function () {
                $scope.selectedModel = [];

                $scope.modelGridOptions = {
                    bindingOptions: {
                        dataSource: {
                            deep: false,
                            dataPath: 'selectedModel'
                        }
                    },
                    columns: [
                        {caption: $translate.instant('ModelCode'), dataField: 'code', visible: false},
                        {caption: $translate.instant('ModelName'), dataField: 'name'},
                        {caption: $translate.instant('ModelDescription'), dataField: 'description'},
                        {caption: $translate.instant('ModelFeature'), dataField: 'featureName'},
                        {caption: $translate.instant('ModelType'), dataField: 'modelTypeName'},
                        {caption: $translate.instant('ModelOrganization'), dataField: 'organizationName'},
                        {
                            alignment: 'center',
                            width: '40px',
                            cellTemplate: function (container, options) {
                                var compileInlineBtn = $compile('<a class="dx-link dx-link-delete"\
ng-show="hasEditPermission && ToggleSaveAs && editModel.hasModel"/>');
                                compileInlineBtn($scope)
                                    .html('<i class="fa fa-trash" aria-hidden="true"></i>')
                                    .on('dxclick', function () {
                                        options.component.removeRow(options.rowIndex);
                                    })
                                    .appendTo(container);
                            }
                        }
                    ],
                    loadPanel: {
                        enabled: false
                    },
                    paging: {
                        enabled: false
                    },
                    scrolling: {
                        mode: "virtual"
                    },
                    sorting: {
                        mode: "none"
                    },
                    height: '100%',
                    showBorders: true,
                    showRowLines: true,
                    showColumnLines: true,
                    onRowRemoving: function (e) {
                        var idx = $scope.selectedModel.indexOf(e.data);
                        if (idx >= 0) {
                            $scope.selectedModel.splice(idx, 1);
                        }
                    }
                };
            };

            var showSelectAccountPop = function () {
                $scope.isShowAccounts = true;
            };

            var showSelectModelPop = function () {
                $scope.isShowModels = true;
            };

            /*********************** Formula Editor ***************************************/
            var showSelectFormulaPop = function () {
                $scope.operateType = constant.Operation.Add;
            };

            var showSelectKeyValuePop = function (keyCode) {
                if (!_.isEmpty(keyCode)) {
                    $scope.defaultKeyCode = keyCode;
                    $scope.isUpdate = true;
                }
                else {
                    $scope.defaultKeyCode = null;
                    $scope.isUpdate = false;
                }

                $scope.isShowKeyValue = true;
            };

            var editFormulaParamOrKeyValue = function (e) {
                if ($scope.hasEditPermission && $scope.ToggleSaveAs && $scope.editModel.hasFormula) {
                    $scope.isUpdate = true;
                    if (e && e.tagInfo) {
                        if (e.tagType === '0') { // key value selector
                            $scope.focusTag = e.tagInfo;
                            showSelectKeyValuePop(e.name);
                        }
                        else if (e.tagType === '1') { // formula parameter editor
                            $scope.focusTag = e.tagInfo;
                            $scope.editFormula = e.name;
                            $scope.editParams = e.formulaParams;
                            $scope.isShowParam = true;
                        }
                    }
                }
            };

            var selectParam = function (formula) {
                $scope.isUpdate = false;
                $scope.editFormula = formula.code;
                $scope.editParams = ')';
                $scope.isShowParam = true;
            };

            var cancelHideTranslator = function () {
                $scope.cancelHide = true;
            };

            var timerHideTranslator = function () {
                $scope.cancelHide = false;
                $timeout(function () {
                    if (!$scope.cancelHide && $scope.firedType === '1') {
                        $scope.firedName = '';
                        $scope.firedParams = '';
                    }
                }, 500);
            };

            // 公式验证
            var mentionMenuValidator = function (ngModelValue, ngModel) {
                ngModel.$setValidity('requiredValidator', true);
                ngModel.$setValidity('keyValueNotEmptyValidator', true);
                ngModel.$setValidity('keyValueSeparatorValidator', true);
                ngModel.$setValidity('operatorDuplicateValidator', true);
                ngModel.$setValidity('missingExpressionValidator', true);
                ngModel.$setValidity('missingOperatorValidator', true);

                if (!$scope.hasEditPermission || !$scope.ToggleSaveAs || !$scope.editModel.hasFormula) {
                    return true;
                }

                if (PWC.isNullOrEmpty(ngModelValue)) {
                    ngModel.$setValidity('requiredValidator', false);
                    return false;
                }

                var keyValueMatches = ngModelValue.match(keyValueExp);
                if (!_.isEmpty(keyValueMatches)) {
                    for (var i = 0; i < keyValueMatches.length; i++) {
                        var m = keyValueMatches[i];
                        m = m.substring(1); // Remove prefix @

                        if (m.length === 0) { // The key value is empty
                            ngModel.$setValidity('keyValueNotEmptyValidator', false);
                            return false;
                        }

                        // Check if suffix blank exists for each key value
                        if (keyValueIllegalExp.test(m)) {
                            ngModel.$setValidity('keyValueSeparatorValidator', false);
                            return false;
                        }
                    }
                }

                // Check if operators(+-*/) missed between key codes
                // replacedVal is like: 123+@+@-@
                var replacedVal = ngModelValue.replace(keyValueExp, '@').replace(blankExp, '');
                if (missingOperatorExp.test(replacedVal)) {
                    ngModel.$setValidity('missingOperatorValidator', false);
                    return false;
                }

                // Find the operators and make sure they are not duplicated
                var operatorMatches = ngModelValue.match(notDuplicateOperatorExp);
                if (_.any(operatorMatches, function (m) {
                        return m.length > 1;
                    })) {
                    ngModel.$setValidity('operatorDuplicateValidator', false);
                    return false;
                }

                // Check if the expression ends with operators(+-*/,)
                if (missingExpressionExp.test(ngModelValue)) {
                    ngModel.$setValidity('missingExpressionValidator', false);
                    return false;
                }

                return true;
            };

            var asyncFormulaValidate = function () {
                if (!$scope.isDemo && $scope.mentionApi.isValid()) {
                    var validateFormula = $scope.editModel.formula;
                    if (!_.isEmpty(validateFormula)) {
                        validateFormula = validateFormula.replace(keyValueExp, '0').trim();
                    }

                    templateFormulaService.validate(validateFormula).success(function (validation) {
                        if (!validation.result) {
                            $scope.editReportFormulaForm.formula.$setValidity('formulaFormatValidator', false);
                        } else {
                            $scope.editReportFormulaForm.formula.$setValidity('formulaFormatValidator', true);
                        }
                    });
                }
            };

            // 设置Key Value选择器
            $scope.keyValueTitle = $translate.instant('AddKeyValue');
            $scope.keyValueSelectorOptions = {
                top: 120
            };

            $scope.keyValueSelectorApi = {
                onSelectorConfirm: function () {
                    var selection = this.getSelection();
                    if (!_.isEmpty(selection)) {
                        if ($scope.isUpdate) {
                            var node = $scope.focusTag;
                            $(node).attr('data-code', '@' + selection.keyCode);
                            $(node).attr('title', selection.description);
                            $(node).text('@' + selection.name);
                        } else {
                            $scope.editModel.formula = $.trim($scope.editModel.formula) + ' @' + selection.keyCode + ' ';
                        }
                    }
                }
            };

            // 设置公式参数选择器
            $scope.paramSelectorApi = {
                onSelectorConfirm: function (e) {
                    if ($scope.isUpdate) {
                        var childNodes = $scope.focusTag.childNodes;
                        if (childNodes.length > 2) {
                            for (var i = childNodes.length; i > 1; i--) {
                                childNodes[i].remove();
                            }
                        }

                        childNodes[1].nodeValue = e.param;
                    } else {
                        if (!_.isEmpty(e.formula)) {
                            $scope.editModel.formula = $.trim($scope.editModel.formula) + e.formula + '(' + e.param;
                        }
                    }
                }
            };

            /*********************** End Formula Editor ***********************************/

            /*********************** Formula Editor for Validation ************************/
            var showSelectFormulaPopForValidation = function () {
                $scope.operateTypeForValidation = constant.Operation.Add;
            };

            var showSelectKeyValuePopForValidation = function (keyCode) {
                if (!_.isEmpty(keyCode)) {
                    $scope.defaultKeyCodeForValidation = keyCode;
                    $scope.isUpdateForValidation = true;
                }
                else {
                    $scope.defaultKeyCodeForValidation = null;
                    $scope.isUpdateForValidation = false;
                }

                $scope.isShowKeyValueForValidation = true;
            };

            var editFormulaParamOrKeyValueForValidation = function (e) {
                if ($scope.hasEditPermission && $scope.ToggleSaveAs && $scope.editModel.hasValidation) {
                    $scope.isUpdateForValidation = true;
                    if (e && e.tagInfo) {
                        if (e.tagType === '0') { // key value selector
                            $scope.focusTagForValidation = e.tagInfo;
                            showSelectKeyValuePopForValidation(e.name);
                        }
                        else if (e.tagType === '1') { // formula parameter editor
                            $scope.focusTagForValidation = e.tagInfo;
                            $scope.editFormulaForValidation = e.name;
                            $scope.editParamsForValidation = e.formulaParams;
                            $scope.isShowParamForValidation = true;
                        }
                    }
                }
            };

            var selectParamForValidation = function (formula) {
                $scope.isUpdateForValidation = false;
                $scope.editFormulaForValidation = formula.code;
                $scope.editParamsForValidation = ')';
                $scope.isShowParamForValidation = true;
            };

            var cancelHideTranslatorForValidation = function () {
                $scope.cancelHideForValidation = true;
            };

            var timerHideTranslatorForValidation = function () {
                $scope.cancelHideForValidation = false;
                $timeout(function () {
                    if (!$scope.cancelHideForValidation && $scope.firedValidationType === '1') {
                        $scope.firedValidationName = '';
                        $scope.firedValidationParams = '';
                    }
                }, 500);
            };

            // 公式验证
            var mentionMenuValidatorForValidation = function (ngModelValue, ngModel) {
                ngModel.$setValidity('requiredValidator', true);
                ngModel.$setValidity('keyValueNotEmptyValidator', true);
                ngModel.$setValidity('keyValueSeparatorValidator', true);
                ngModel.$setValidity('operatorDuplicateValidator', true);
                ngModel.$setValidity('missingExpressionValidator', true);
                ngModel.$setValidity('missingOperatorValidator', true);

                if (!$scope.hasEditPermission || !$scope.ToggleSaveAs || !$scope.editModel.hasValidation) {
                    return true;
                }

                if (PWC.isNullOrEmpty(ngModelValue)) {
                    ngModel.$setValidity('requiredValidator', false);
                    return false;
                }

                var keyValueMatches = ngModelValue.match(keyValueExp);
                if (!_.isEmpty(keyValueMatches)) {
                    for (var i = 0; i < keyValueMatches.length; i++) {
                        var m = keyValueMatches[i];
                        m = m.substring(1); // Remove prefix @

                        if (m.length === 0) { // The key value is empty
                            ngModel.$setValidity('keyValueNotEmptyValidator', false);
                            return false;
                        }

                        // Check if suffix blank exists for each key value
                        if (keyValueIllegalExp.test(m)) {
                            ngModel.$setValidity('keyValueSeparatorValidator', false);
                            return false;
                        }
                    }
                }

                // Check if operators(+-*/) missed between key codes
                // replacedVal is like: 123+@+@-@
                var replacedVal = ngModelValue.replace(keyValueExp, '@').replace(blankExp, '');
                if (missingOperatorExp.test(replacedVal)) {
                    ngModel.$setValidity('missingOperatorValidator', false);
                    return false;
                }

                // Find the operators and make sure they are not duplicated
                var operatorMatches = ngModelValue.match(notDuplicateOperatorExp);
                if (_.any(operatorMatches, function (m) {
                        return m.length > 1;
                    })) {
                    ngModel.$setValidity('operatorDuplicateValidator', false);
                    return false;
                }

                // Check if the expression ends with operators(+-*/,)
                if (missingExpressionExp.test(ngModelValue)) {
                    ngModel.$setValidity('missingExpressionValidator', false);
                    return false;
                }

                return true;
            };

            var asyncFormulaValidateForValidation = function () {
                if ($scope.mentionApiForValidation.isValid()) {
                    var validateFormula = $scope.editModel.validation;
                    if (!_.isEmpty(validateFormula)) {
                        validateFormula = validateFormula.replace(keyValueExp, '0').trim();
                    }

                    templateFormulaService.validate(validateFormula).success(function (validation) {
                        if (!validation.result) {
                            $scope.editReportFormulaForm.validation.$setValidity('formulaFormatValidator', false);
                        } else {
                            $scope.editReportFormulaForm.validation.$setValidity('formulaFormatValidator', true);
                        }
                    });
                }
            };

            // 设置Key Value选择器
            $scope.keyValueTitleForValidation = $translate.instant('AddKeyValue');
            $scope.keyValueSelectorOptionsForValidation = {
                top: 120
            };

            $scope.keyValueSelectorApiForValidation = {
                onSelectorConfirm: function () {
                    var selection = this.getSelection();
                    if (!_.isEmpty(selection)) {
                        if ($scope.isUpdateForValidation) {
                            var node = $scope.focusTagForValidation;
                            $(node).attr('data-code', '@' + selection.keyCode);
                            $(node).attr('title', selection.description);
                            $(node).text('@' + selection.name);
                        } else {
                            $scope.editModel.validation = $.trim($scope.editModel.validation) + ' @' + selection.keyCode + ' ';
                        }
                    }
                }
            };

            // 设置公式参数选择器
            $scope.paramSelectorApiForValidation = {
                onSelectorConfirm: function (e) {
                    if ($scope.isUpdateForValidation) {
                        var childNodes = $scope.focusTagForValidation.childNodes;
                        if (childNodes.length > 2) {
                            for (var i = childNodes.length; i > 1; i--) {
                                childNodes[i].remove();
                            }
                        }

                        childNodes[1].nodeValue = e.param;
                    } else {
                        if (!_.isEmpty(e.formula)) {
                            $scope.editModel.validation = $.trim($scope.editModel.validation) + e.formula + '(' + e.param;
                        }
                    }
                },
            };

            /*********************** End Formula Editor ***********************************/

                //模板另存为
            var saveTemplateGroup = function () {
                    var groupName = $scope.groupName;
                    if (groupName === '' || groupName === undefined) {
                        SweetAlert.warning($translate.instant('InputTemplateGroupNameWarning'));
                        return;
                    }

                    $scope.newTemplateGroup.Name = groupName;
                    $scope.newTemplateGroup.ServiceTypeID = $scope.curServiceTypeId;
                    $scope.newTemplateGroup.CopyFrom = $scope.curTemplateGroup.id;
                    $scope.newTemplateGroup.groupType = $scope.curTemplateGroup.groupType;
                    $scope.newTemplateGroup.payTaxType = $scope.curTemplateGroup.payTaxType;
                    $scope.newTemplateGroup.industryIDs = $scope.curTemplateGroup.industryIDs;
                    $scope.newTemplateGroup.ChangedFormulas = $scope.formulasArrayTemp;

                    templateGroupService.addTemplateGroup($scope.newTemplateGroup).success(function (or) {
                        if (or && or.result) {
                            SweetAlert.success($translate.instant('TemplateGroupSaveAsSuccess'));
                            loadTemplateGroup();
                            $('#templateName').val('');
                            $('.templates-save-form').css('display', 'none');
                        } else {
                            SweetAlert.warning($translate.instant(or.resultMsg));
                        }
                    });
                };


            var loadTemplateMenu = function () {
                templateGroupService.getTemplateList($scope.curTemplateGroup).success(function (templateData) {
                    $scope.templates = templateData;
                });
            };
            //保存模板
            var saveTemplate = function () {
                templateFormulaService.saveTemplateFormulaList($scope.formulasArrayTemp).success(function (or) {
                    if (or && or.result) {
                        SweetAlert.success($translate.instant('TemplateSaveSuccess'));
                    } else {
                        SweetAlert.warning($translate.instant(or.resultMsg));
                    }
                });
            };


            var saveFormulaToTemp = function () {
                //find exist
                /*
                var a = _.findWhere($scope.formulasArrayTemp, { id: $scope.editModel.cellTemplateID });

                if (a && a != undefined) {
                    //delete old
                    $scope.formulasArrayTemp.forEach(function (f, index) {
                        if (f.cellTemplateID === $scope.editModel.cellTemplateID) {
                            $scope.formulasArrayTemp.splice(index, 1);
                        }
                    });
                    //add new
                    $scope.formulasArrayTemp.push($scope.editModel);
                }
                else {
                    $scope.formulasArrayTemp.push($scope.editModel);
                }
                */

                if (!$scope.editModel.hasFormula) {
                    $scope.editModel.formula = null;
                }

                if ($scope.editModel.hasInvoice) {
                    // 如果发票类型全选，则将editModel.invoiceCategory设置为[ 99 ]；
                    // 否则将选中的发票类型存入editModel.invoiceCategory数组，如[1, 2, 3]
                    if ($scope.editModel.chosenInvoiceOption.categoryOptions.length > 0 &&
                        _.every($scope.editModel.chosenInvoiceOption.categoryOptions, {ticked: true})) {
                        $scope.editModel.invoiceCategory = [Number(constant.selectAllValue)];
                    } else {
                        $scope.editModel.invoiceCategory = _.chain($scope.editModel.chosenInvoiceOption.categoryOptions)
                            .where({ticked: true}).pluck('value').value();
                    }

                    // 如果税率全选，则将editModel.taxRate设置为[ '99' ]；
                    // 否则将选中的税率存入editModel.taxRate数组，如['17%', '11%', '6%']
                    if ($scope.editModel.chosenInvoiceOption.rateOptions.length > 0 &&
                        _.every($scope.editModel.chosenInvoiceOption.rateOptions, {ticked: true})) {
                        $scope.editModel.taxRate = [constant.selectAllValue];
                    } else {
                        $scope.editModel.taxRate = _.chain($scope.editModel.chosenInvoiceOption.rateOptions)
                            .where({ticked: true}).pluck('value').value();
                    }

                    $scope.editModel.invoiceType = $scope.editModel.chosenInvoiceOption.value;
                    $scope.editModel.invoiceAmountType = $scope.editModel.chosenInvoiceOption.invoiceAmountType.value;

                } else {
                    $scope.editModel.invoiceCategory = [];
                    $scope.editModel.taxRate = [];
                    $scope.editModel.invoiceType = null;
                    $scope.editModel.invoiceAmountType = null;
                }

                if (!$scope.editModel.hasVoucher) {
                    $scope.editModel.accountCodes = [];
                    $scope.editModel.voucherKeyword = '';
                }

                if (!$scope.editModel.hasModel) {
                    $scope.editModel.modelIDs = [];
                }

                if (!$scope.editModel.hasValidation) {
                    $scope.editModel.validation = null;
                    $scope.editModel.validationDescription = null;
                }

                templateFormulaService.saveOrUpdateCellTemplateConfig($scope.editModel).success(function (data) {
                    if (data && data.result) {
                        SweetAlert.success($translate.instant('CellTemplateSaveSuccess'));

                        for (var i = 0; i < $scope.templateFormulaList.length; i++) {
                            if ($scope.templateFormulaList[i].cellTemplateID === $scope.editModel.cellTemplateID) {
                                $scope.editModel.formulaDataSource = data.resultMsg;
                                $scope.templateFormulaList[i] = $scope.editModel;
                            }
                        }
                    } else {
                        SweetAlert.warning($translate.instant(data.resultMsg));
                    }

                    //$('#editReportFormulaPop').modal('hide');
                    //$('#editReportFormulaPop').hide('normal');
                    hidePanel();
                    refreshSpreadUI();
                });
            };

            $scope.showOperateLogPop = function () {
                $scope.isShowLog = true;
                // $('#showOperatePop').modal('show');
            };

            var refreshSpreadUI = function () {

                spreadJsTipService.initialize($scope.activeSheet);
                if ($scope.templateFormulaList && $scope.templateFormulaList.length > 0) {
                    $scope.templateFormulaList.forEach(function (row, index) {
                        var cell = $scope.activeSheet.getCell(row.rowIndex, row.columnIndex);
                        if (row.formulaDataSource) {
                            cell.value(row.formulaDataSource);
                            cell.hAlign(0);
                        } else {
                            if (cell.value() !== '——') {
                                cell.value(null);
                            } else {
                                //is ——
                                if (row.hasFormula || row.hasInvoice || row.hasVoucher || row.hasKeyIn) {
                                    cell.value(null);
                                }
                            }
                        }
                        spreadJsTipService.setCellTip(cell, row);

                        if ($scope.cellTemplateId && $scope.cellTemplateId === row.cellTemplateID) {
                            $scope.activeSheet.setActiveCell(row.rowIndex, row.columnIndex);
                            releaseParameter();
                        }


                        //spreadJsTipService.setCellTipString(cell, "测试数据源");
                    });
                }

                //加载修改了但还未保存下来的数据到excel中            
                if ($scope.formulasArrayTemp && $scope.formulasArrayTemp.length > 0) {
                    $scope.formulasArrayTemp.forEach(function (row) {
                        var cell = $scope.activeSheet.getCell(row.rowIndex, row.columnIndex);
                        if (row.formulaDataSource) {
                            cell.value(row.formulaDataSource);
                            cell.hAlign(0);
                        } else {
                            cell.value(null);
                        }
                        spreadJsTipService.setCellTip(cell, row);
                    });
                }

                spreadJsTipService.paintSheet($scope.activeSheet);
            };

            var getFormulaList = function () {
                $q.all([
                    formulaService.getAll(),
                    formulaService.getAllParam(),
                    formulaService.getAllParamMapping()
                ]).then(function (result) {
                    if (!_.isEmpty(result[0]) && !_.isEmpty(result[0].data) && !_.isEmpty(result[0].data.data)) {
                        var formulaList = _.map(result[0].data.data, function (config) {
                            return {
                                id: config.id,
                                code: config.formulaName,
                                name: config.chineseName,
                                description: config.description,
                                englishName: config.englishName,
                                requiredParamNum: config.requiredParamNum
                            }
                        });

                        if (!_.isEmpty(result[1]) && !_.isEmpty(result[1].data) && !_.isEmpty(result[1].data.data) &&
                            !_.isEmpty(result[2]) && !_.isEmpty(result[2].data) && !_.isEmpty(result[2].data.data)) {
                            var groupedParamList = _.groupBy(result[2].data.data, 'formulaID');
                            var paramList = result[1].data.data;
                            _.each(groupedParamList, function (val, key) {
                                var formula = _.findWhere(formulaList, {id: key});
                                if (formula) {
                                    formula.params = _.chain(val)
                                        .map(function (p) {
                                            return {id: p.formulaParamID, index: p.paramIndex};
                                        })
                                        .sortBy(function (p) {
                                            return p.index;
                                        }).value();
                                    _.each(formula.params, function (p, idx) {
                                        var param = _.findWhere(paramList, {id: p.id});
                                        _.extend(p, param);
                                    });
                                }
                            });
                        }

                        $scope.formulaList = formulaList;
                    }
                });
            };

            var getModelList = function () {
                modelConfigurationService.getModelListByIndustry($scope.curServiceTypeId, $scope.curIndustry.id)
                    .success(function (data) {
                        var rtn = [];
                        var modelTypes = _.invert(enums.modelType);
                        _.each(data, function (m) {
                            if (m.type.toString() === modelTypes.SystemCommonModel) {
                                if (!_.some(rtn, {id: m.id})) {
                                    m.orgID = null;
                                    m.organizationName = $translate.instant('All');
                                    m.featureName = $translate.instant(enums.modelFeature[m.feature]);
                                    rtn.push(m);
                                }
                            } else {
                                var existsModel = _.findWhere(rtn, {id: m.id});
                                if (!existsModel) {
                                    m.featureName = $translate.instant(enums.modelFeature[m.feature]);
                                    rtn.push(m);
                                } else {
                                    existsModel.orgID = existsModel.orgID + ',' + m.orgID;
                                    existsModel.organizationName = existsModel.organizationName + ',' + m.organizationName;
                                }
                            }
                        });

                        $scope.modelDataSource = rtn;
                    });
            };

            var checkVoucherValid = function () {
                if (!$scope.editModel.hasVoucher || !PWC.isNullOrEmpty($scope.editModel.voucherKeyword)
                    || !_.isEmpty($scope.editModel.accountCodes)) {
                    if ($scope.editReportFormulaForm && $scope.editReportFormulaForm.voucher
                        && $scope.editReportFormulaForm.voucher.$error.voucherNotSelectValidator) {
                        $scope.editReportFormulaForm.voucher.$setValidity('voucherNotSelectValidator', true);
                    }
                } else if ($scope.editReportFormulaForm && $scope.editReportFormulaForm.voucher
                    && !$scope.editReportFormulaForm.voucher.$error.voucherNotSelectValidator) {
                    $scope.editReportFormulaForm.voucher.$setValidity('voucherNotSelectValidator', false);
                }
            };

            var checkPermission = function () {
                $scope.$root.checkUserPermission(constant.adminPermission.systemConfiguration.declarationFormConfiguration.editCode).success(function (data) {
                    $scope.hasEditPermission = data;
                });
            };

            $scope.$watch('curIndustry.id', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.accountDataSource = [];
                    $scope.modelDataSource = [];
                } else {
                    stdAccountService.getStdAccountByIndustryID(newValue)
                        .success(function (data) {
                            _.each(data, function (account) {
                                account.directionName = $translate.instant(enums.standardAccountDirection[account.direction]);
                                account.AcctPropName = $translate.instant(enums.standardAccountAcctProp[account.acctProp]);
                            });

                            $scope.accountDataSource = data;
                        });
                    getModelList();
                    getFormulaList();
                }
            });

            $scope.$watch('curServiceTypeId', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.modelDataSource = [];
                } else {
                    getModelList();
                }
            });

            // If changed to no formula config, empty the input.
            $scope.$watch('editModel.hasFormula', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.editModel.formula = '';
                    if ($scope.editReportFormulaForm && $scope.editReportFormulaForm.formula
                        && $scope.editReportFormulaForm.formula.$error.requiredValidator) {
                        $scope.editReportFormulaForm.formula.$setValidity('requiredValidator', true);
                    }
                } else if (PWC.isNullOrEmpty($scope.editModel.formula) &&
                    !$scope.editReportFormulaForm.formula.$error.requiredValidator) {
                    $scope.editReportFormulaForm.formula.$setValidity('requiredValidator', false);
                }
            });

            $scope.$watch('editModel.hasValidation', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.editModel.validation = '';
                    if ($scope.editReportFormulaForm && $scope.editReportFormulaForm.validation
                        && $scope.editReportFormulaForm.validation.$error.requiredValidator) {
                        $scope.editReportFormulaForm.validation.$setValidity('requiredValidator', true);
                    }
                } else if (PWC.isNullOrEmpty($scope.editModel.validation)
                    && !$scope.editReportFormulaForm.validation.$error.requiredValidator) {
                    $scope.editReportFormulaForm.validation.$setValidity('requiredValidator', false);
                }
            });

            // If changed to no voucher config, empty the datasource.
            $scope.$watch('editModel.hasVoucher', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.editModel.voucherKeyword = '';
                    $scope.selectedAccount = [];
                    $scope.editModel.accountCodes = [];
                }

                checkVoucherValid();
            });

            // If invoice option is not selected yet, alert user.
            $scope.$watch('editModel.chosenInvoiceOption', function (newValue, oldValue) {
                if ($scope.editModel.hasInvoice &&
                    (_.isEmpty(newValue) ||
                        newValue.value <= 0 ||
                        newValue.invoiceAmountType.value <= 0 ||
                        newValue.value !== 3 &&
                        (_.every(newValue.categoryOptions, {ticked: false}) ||
                            _.every(newValue.rateOptions, {ticked: false})))) {
                    $scope.editReportFormulaForm.invoice.$setValidity('invoiceNotSelectValidator', false);
                } else {
                    $scope.editReportFormulaForm.invoice.$setValidity('invoiceNotSelectValidator', true);
                }
            }, true);

            $scope.$watch('editModel.hasInvoice', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.editReportFormulaForm.invoice.$setValidity('invoiceNotSelectValidator', true);
                } else if (_.isEmpty($scope.editModel.chosenInvoiceOption) ||
                    $scope.editModel.chosenInvoiceOption.value <= 0 ||
                    $scope.editModel.chosenInvoiceOption.invoiceAmountType.value <= 0 ||
                    $scope.editModel.chosenInvoiceOption.value !== 3 &&
                    (_.every($scope.editModel.chosenInvoiceOption.rateOptions, {ticked: false}) ||
                        _.every($scope.editModel.chosenInvoiceOption.categoryOptions, {ticked: false}))) {
                    $scope.editReportFormulaForm.invoice.$setValidity('invoiceNotSelectValidator', false);
                }
            });

            // If changed to no model config, empty the datasource.
            $scope.$watch('editModel.hasModel', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.selectedModel = [];
                    $scope.editModel.modelIDs = [];
                    $scope.editReportFormulaForm.model.$setValidity('modelNotSelectValidator', true);
                } else if ($scope.editModel.modelIDs.length === 0) {
                    $scope.editReportFormulaForm.model.$setValidity('modelNotSelectValidator', false);
                }
            });

            $scope.$watch('voucherFilterType', function (newValue, oldValue) {
                $scope.editModel.voucherKeyword = '';
                $scope.selectedAccount = [];
                $scope.editModel.accountCodes = [];

                if (newValue === 0) {
                    $scope.refreshGrids();
                }

                checkVoucherValid();
            });

            $scope.$watch('editModel.voucherKeyword', function (newValue, oldValue) {
                checkVoucherValid();
            });

            $scope.$watch('selectedAccount.length', function (newValue, oldValue) {
                if (newValue !== oldValue) {
                    $scope.editModel.accountCodes = _.pluck($scope.selectedAccount, 'code');
                }

                checkVoucherValid();
            });

            $scope.$watch('selectedModel.length', function (newValue, oldValue) {
                if (newValue !== 0 || !$scope.editModel.hasModel) {
                    $scope.editReportFormulaForm.model.$setValidity('modelNotSelectValidator', true);
                } else {
                    $scope.editReportFormulaForm.model.$setValidity('modelNotSelectValidator', false);
                }

                if (newValue !== oldValue) {
                    $scope.editModel.modelIDs = _.pluck($scope.selectedModel, 'id');
                }
            });

            var releaseParameter = function () {
                $scope.templateGroupId = null;
                $scope.templateId = null;
                $scope.cellTemplateId = null;
            };

            // 选中报表
            var selectTemplate = function (model) {
                $scope.currentTemplate = model;

                listTemplates();
            };

            $scope.selectTemplate = selectTemplate;

            // 改变模板的的时候，重新选择
            var selectTemplateGroup = function (model) {
                $scope.curTemplateGroup = null;
                $scope.currentTemplate = null;
                if (model) {
                    $scope.curTemplateGroup = model;
                }

                loadTemplateGroup();
            };

            $scope.selectTemplateGroup = selectTemplateGroup;

            // dx 控件
            var dxControl = {
                groupDropDownId: 'dx-select-template-groups',
                groupDropDownContentId: 'dx-template-group-content',
                templateDropDownId: 'dx-select-template',
                templateDropDownContentId: 'dx-template-content',

                // dx drop down 模板绘制
                renderTemplateGroupDropDown: function (data, selectValue) {
                    $scope.curTemplateGroup = selectValue;

                    if (!selectValue) {
                        selectValue = {};
                        selectValue.name = '';
                    }


                    var bindAddEvent = function () {
                        $(".add-template-group").off("click").on("click", function () {
                            // to create order

                            $scope.$apply(function () {
                                $scope.templateGroupModel = {};
                                $scope.templateGroupModel.serviceTypeID = $scope.curServiceTypeId;
                                $scope.templateGroupModel.industryIDs = $scope.curIndustry.id;

                                if ($scope.curServiceTypeId === enums.serviceType.CIT || $scope.curServiceTypeId === enums.serviceType.Others) {
                                    $scope.templateGroupModel.reportType = $scope.reportType.value;
                                    $scope.operateTemplateGroupType = constant.TemplateGroupOperation.AddGroupFirst;
                                }
                                else {
                                    $scope.templateGroupModel.payTaxType = $scope.payTaxType.value;

                                    // Mock data
                                    $scope.curServiceTypeId = enums.serviceType.VAT;

                                    $scope.operateTemplateGroupType = constant.TemplateGroupOperation.AddDirectly;
                                }
                            });


                            $('#' + dxControl.groupDropDownId).dxDropDownBox('close');
                        });

                    };

                    // 鼠标移开，关闭drop down
                    var bindMouseLeave = function () {

                        $("#" + dxControl.groupDropDownContentId).off("mouseleave").on("mouseleave", function () {
                            // to create order
                            $('#' + dxControl.groupDropDownId).dxDropDownBox('close');
                        });
                    };

                    // 报表模板组下拉列表
                    var listTemplateGroupView = function (e, data, value) {

                        var height = null;

                        if (data && data.length >= 8) {
                            height = '300px';
                        }

                        var list = $('<div class="list-view" id="templateGroupListView">').dxList({
                            dataSource: data,
                            height: height,
                            selectionMode: "single",
                            pageLoadMode: 'scrollBottom',
                            itemTemplate: function (itemData, itemIndex, itemElement) {
                                var div = '<span class="left-part" title="' + itemData.name + '"><span>' + itemData.name + '</span></span>';
                                if (!itemData.isSystemType) {
                                    div += '<span class="right-operation"><i class="material-icons edit">create</i>';
                                    div += ' <i class="material-icons delete">delete</i>';
                                }
                                div += '</span>';

                                itemElement.append(div);
                            },
                            onItemClick: function (args) {

                                $scope.curTemplateGroup = args.itemData;
                                // 单击即使选中
                                if (args.jQueryEvent.target.className === thisConstant.deleteClass) {
                                    $scope.$apply(function () {
                                        $scope.templateGroupModel = args.itemData;
                                        $scope.operateTemplateGroupType = constant.Operation.Delete;
                                    });

                                } else if (args.jQueryEvent.target.className === thisConstant.editClass) {

                                    $scope.$apply(function () {
                                        $scope.templateGroupModel = args.itemData;
                                        $scope.operateTemplateGroupType = constant.Operation.Edit;
                                    });
                                } else {
                                    loadReportType();
                                }

                                $('#' + dxControl.groupDropDownId).dxDropDownBox('close');
                                e.component.option('value', $scope.curTemplateGroup.name);
                            }

                        });

                        list.dxList('instance');
                        return list;
                    };

                    $('#' + dxControl.groupDropDownId).dxDropDownBox({
                        value: selectValue.name,
                        valueExpr: "id",
                        displayExpr: "name",
                        showClearButton: false,
                        onContentReady: function (args) {
                            // var value = args.component.option("value");
                            // syncTreeViewSelection(thisData.orgTreeView, thisData.selectOrgList, false);
                        },
                        contentTemplate: function (e) {
                            // thisData.DropDownEvent = e;
                            var value = e.component.option("value");
                            var $treeView = listTemplateGroupView(e, data, value);

                            var addTemplateGroup = '<div class="add-template-group"><span class="left-part">\
                                <span class="organge-color">\
                                    <i class="material-icons add">add_circle_outline</i>' + $translate.instant('AddTemplate') + '</span>\
                            </span></div>';

                            var div = "<div id='" + dxControl.groupDropDownContentId + "'>";
                            var wrapper = $(div).append($treeView).append(addTemplateGroup);

                            $timeout(function () {
                                bindAddEvent();
                                bindMouseLeave();
                            }, 50);
                            return wrapper;
                        }
                    });
                },

                // 报表下拉框
                renderTemplateDropDown: function (data, selectValue) {
                    if (!data) {
                        data = [];
                    }


                    if (selectValue) {
                        loadSheet(selectValue);
                    } else {
                        $('#report-view').html('');
                        selectValue = {};
                        selectValue.name = '';
                    }

                    var bindAddEvent = function () {
                        $(".add-template-btn").off("click").on("click", function () {
                            // to create order

                            var currentTemplateCode = _.pluck($scope.templates, 'code');

                            $scope.$apply(function () {

                                var serviceTypeId = '2';
                                if ($scope.curServiceTypeId === enums.serviceType.CIT || $scope.curServiceTypeId === enums.serviceType.VAT
                                    || $scope.curServiceTypeId === enums.serviceType.Others) {
                                    serviceTypeId = $scope.curServiceTypeId;
                                }

                                $scope.templateModel = {
                                    templateGroupID: $scope.curTemplateGroup.id,
                                    serviceTypeID: serviceTypeId,
                                    payTaxType: serviceTypeId === enums.serviceType.CIT || serviceTypeId === enums.serviceType.Others ? '' : $scope.payTaxType.value,
                                    reportType: serviceTypeId === enums.serviceType.CIT || serviceTypeId === enums.serviceType.Others ? $scope.reportType.value : '',
                                    industryIDs: $scope.curIndustry.id,
                                    currentTemplateCodeList: currentTemplateCode
                                };

                                $scope.operateTemplateType = constant.Operation.Add;
                            });

                            $('#' + dxControl.templateDropDownId).dxDropDownBox('close');
                        });
                    };

                    // 鼠标移开，关闭drop down
                    var bindMouseLeave = function () {

                        $("#" + dxControl.templateDropDownContentId).off("mouseleave").on("mouseleave", function () {
                            // to create order
                            $('#' + dxControl.templateDropDownId).dxDropDownBox('close');
                        });
                    };

                    var getDisplayName = function (value) {

                        if (!value) {
                            return '';
                        }
                        var displayName = value.name;
                        if (value.code) {
                            displayName = value.code + '_' + value.name;
                        }

                        return displayName;
                    };

                    // 报表模板下拉列表
                    var listTemplateView = function (e, data, value) {

                        var height = null;

                        if (data && data.length >= 8) {
                            height = '300px';
                        }

                        var list = $('<div class="list-view" id="templateListView">').dxList({
                            dataSource: data,
                            height: height,
                            selectionMode: "single",
                            pageLoadMode: 'scrollBottom',
                            noDataText: '无数据',
                            itemTemplate: function (itemData, itemIndex, itemElement) {

                                var disPlay = getDisplayName(itemData);

                                var div = '<span class="left-part" title="' + disPlay + '"><span>' + disPlay + '</span></span><span class="right-operation">';

                                if (!itemData.isSystemType) {
                                    div += '<i class="material-icons edit">create</i>';
                                }

                                div += ' <i class="material-icons delete">delete</i>';

                                div += '</span>';

                                itemElement.append(div);
                            },
                            onItemClick: function (args) {

                                $scope.currentTemplate = args.itemData;
                                // 单击即使选中
                                if (args.jQueryEvent.target.className === thisConstant.deleteClass) {

                                    $scope.$apply(function () {
                                        $scope.operateTemplateType = constant.Operation.Delete;
                                        $scope.templateModel = args.itemData;
                                    });

                                } else if (args.jQueryEvent.target.className === thisConstant.editClass) {

                                    $scope.$apply(function () {
                                        $scope.operateTemplateType = constant.Operation.Edit;
                                        $scope.templateModel = args.itemData;
                                    });

                                } else {
                                    loadSheet(args.itemData);
                                }

                                var disPlay = getDisplayName(args.itemData);

                                $('#' + dxControl.templateDropDownId).dxDropDownBox('close');
                                e.component.option('value', disPlay);
                            }

                        });

                        list.dxList('instance');
                        return list;
                    };

                    var selectName = getDisplayName(selectValue);

                    $('#' + dxControl.templateDropDownId).dxDropDownBox({
                        value: selectName,
                        showClearButton: false,
                        onContentReady: function (args) {
                        },
                        contentTemplate: function (e) {
                            // thisData.DropDownEvent = e;
                            var value = e.component.option("value");
                            var $treeView = listTemplateView(e, data, value);

                            var addTemplateGroup = '<div class="add-template-btn"><span class="left-part">\
                                <span class="organge-color">\
                                    <i class="material-icons add">add_circle_outline</i>' + $translate.instant('AddReport') + '</span>\
                            </span></div>';


                            var div = "<div id='" + dxControl.templateDropDownContentId + "'>";

                            var wrapper = $(div).append($treeView).append(addTemplateGroup);

                            $timeout(function () {
                                bindAddEvent();
                                bindMouseLeave();
                            }, 50);
                            return wrapper;
                        }
                    });
                }

            };


            (function initialize() {
                $log.debug('DeclarationFormConfigurationController.ctor()...');

                $log.debug('...cellTemplateId:' + $scope.templateGroupId);

                resetFormHeight();
                $scope.selectedTab = 0;
                $scope.voucherFilterType = 0;

                $scope.selectFormula = null;
                $scope.cancelHide = false;
                $scope.firedName = '';
                $scope.firedType = '0';
                $scope.firedParams = '';
                $scope.editFormula = '';
                $scope.editParams = '';
                $scope.mentionApi = {};

                $scope.selectFormulaForValidation = null;
                $scope.cancelHideForValidation = false;
                $scope.firedValidationName = '';
                $scope.firedValidationType = '0';
                $scope.firedValidationParams = '';
                $scope.editFormulaForValidation = '';
                $scope.editParamsForValidation = '';
                $scope.mentionApiForValidation = {};

                $scope.curTemplateGroup = {};
                $scope.newTemplateGroup = {};
                $scope.templateGroups = [];
                $scope.templates = [];
                $scope.currentTemplate = {};
                $scope.editModel = {};
                $scope.formulasArrayTemp = [];
                $scope.curIndustry = {};
                $scope.payTaxType = filingTypeList[0];
                $scope.formulaList = [];

                $scope.dataSourcePayTaxTypeListInstance = {};

                $scope.dataSourceIndustryListInstance = {};
                //配置报表类型下拉框
                $scope.dataSourcePayTaxTypeList = {
                    dataSource: new DevExpress.data.ArrayStore({
                        data: filingTypeList
                    }),
                    displayExpr: 'name',
                    valueExpr: "name",
                    value: filingTypeList[0].name,
                    itemTemplate: 'filingTypeItem',
                    onInitialized: function (e) {
                        $scope.dataSourcePayTaxTypeListInstance = e.component;
                    },
                    onSelectionChanged: function (e) {
                        $scope.payTaxType = e.selectedItem;
                        loadTemplateGroup();
                    }
                };

                checkPermission();
                loadServiceType();

                KeyValueConfigService.getAll().success(function (data) {
                    $scope.mentionList = _.map(data, function (config) {
                        return {
                            code: '@' + config.keyCode,
                            name: config.name,
                            description: config.description
                        }
                    });
                });

                var tabs = [
                    {id: 0, text: $translate.instant('AutoCalculateTab')},
                    {id: 1, text: $translate.instant('ManualInputTab')},
                    {id: 2, text: $translate.instant('ModelTab')},
                    {id: 3, text: $translate.instant('ValidationTab')}
                ];
                $scope.tabOptions = {
                    dataSource: tabs,
                    bindingOptions: {
                        selectedIndex: 'selectedTab'
                    },
                    onItemClick: function () {
                        $scope.refreshGrids();
                    }
                };

                initAccountSelector();
                initModelSelector();

                $scope.invoiceOptions = getInvoiceFilter();
                //配置发票类型下拉选项
                $scope.dataSourceInvoice = {
                    bindingOptions: {
                        disabled: '!hasEditPermission || !ToggleSaveAs || !editModel.hasInvoice'
                    },
                    dataSource: $scope.invoiceOptions,
                    displayExpr: function (item) {
                        if (item) {
                            return $translate.instant(item.type);
                        }
                        return "";
                    },
                    onItemClick: function (e) {
                        $scope.invoiceOption = e.itemData;
                        selectInvoiceOption($scope.editModel, $scope.invoiceOption.value);
                    }
                };

                //配置发票产生类型下拉选项
                $scope.dataSourceInvoiceAmount = {
                    bindingOptions: {
                        dataSource: 'invoiceOption.amountTypeOptions',
                        disabled: '!hasEditPermission || !ToggleSaveAs || !editModel.hasInvoice'
                    },
                    displayExpr: function (item) {
                        if (item) {
                            return $translate.instant(item.name);
                        }
                        return "";
                    },
                    onItemClick: function (e) {
                        $scope.editModel.chosenInvoiceOption.invoiceAmountType = e.itemData;
                    }
                };

                // 设置科目代码选择器
                $scope.accountTitle = $translate.instant('AddVoucher');
                $scope.accountSelectorOptions = {
                    height: 500,
                    top: 140,
                    width: 730,
                    selectNodesRecursive: true,
                };
                $scope.accountSelectorApi = {
                    onSelectorConfirm: refreshVoucherFilter
                };

                // 设置模型选择器
                $scope.modelTitle = $translate.instant('AddLinkModel');
                $scope.modelSelectorOptions = {
                    height: 500,
                    top: 140,
                    width: 1030
                };
                $scope.modelSelectorApi = {
                    onSelectorConfirm: refreshModelFilter
                };

                // 为发票税率多选下拉菜单进行国际化
                $scope.langSetting = {
                    selectAll: $translate.instant('SelectAll'),
                    selectNone: $translate.instant('SelectNone'),
                    reset: $translate.instant('Reset'),
                    nothingSelected: $translate.instant('PleaseSelect'),
                    search: $translate.instant('Search')
                };

                $scope.$on(enums.adminEvent.layoutChanged, function () {
                    return $timeout(function () {
                        if (_spread && _spread.refresh) {
                            _spread.refresh();
                        }
                    }, 50);
                });

                $scope.loadSheet = loadSheet;

                $scope.saveTemplateGroup = saveTemplateGroup;

                $scope.loadTemplateMenu = loadTemplateMenu;

                $scope.listTemplates = listTemplates;

                $scope.saveFormulaToTemp = saveFormulaToTemp;

                $scope.saveTemplate = saveTemplate;

                $scope.selectInvoiceOption = selectInvoiceOption;

                $scope.showSelectAccountPop = showSelectAccountPop;
                $scope.showSelectModelPop = showSelectModelPop;
                $scope.getServiceTypeList = getServiceTypeList;
                $scope.toggleServiceType = toggleServiceType;
                $scope.hasError = hasError;
                $scope.hidePanel = hidePanel;
                $scope.togglePanel = togglePanel;

                $scope.showSelectFormulaPop = showSelectFormulaPop;
                $scope.showSelectKeyValuePop = showSelectKeyValuePop;
                $scope.editFormulaParamOrKeyValue = editFormulaParamOrKeyValue;
                $scope.selectParam = selectParam;
                $scope.mentionMenuValidator = mentionMenuValidator;
                $scope.asyncFormulaValidate = asyncFormulaValidate;
                $scope.cancelHideTranslator = cancelHideTranslator;
                $scope.timerHideTranslator = timerHideTranslator;

                $scope.showSelectFormulaPopForValidation = showSelectFormulaPopForValidation;
                $scope.showSelectKeyValuePopForValidation = showSelectKeyValuePopForValidation;
                $scope.editFormulaParamOrKeyValueForValidation = editFormulaParamOrKeyValueForValidation;
                $scope.selectParamForValidation = selectParamForValidation;
                $scope.mentionMenuValidatorForValidation = mentionMenuValidatorForValidation;
                $scope.asyncFormulaValidateForValidation = asyncFormulaValidateForValidation;
                $scope.cancelHideTranslatorForValidation = cancelHideTranslatorForValidation;
                $scope.timerHideTranslatorForValidation = timerHideTranslatorForValidation;

                $('.templates-body').off("click").on("click", function () {
                    $('#' + dxControl.groupDropDownId).dxDropDownBox('close');
                    $('#' + dxControl.templateDropDownId).dxDropDownBox('close');
                    $("#dx-select-industry").dxSelectBox('close');
                    $('#dx-select-pay-tax-type').dxSelectBox('close');
                });

                $('#report-view').unbind("contextmenu").bind("contextmenu", processSpreadContextMenu);
                $("#spreadContextMenu a").click(processContextMenuClicked);
                $(document).on("contextmenu", function () {
                    event.preventDefault();
                    return false;
                });
            })();
        }
    ]);