﻿systemConfigurationModule
.controller('EnterpriseSubjectCorrespondingController', ['$scope', '$log', 'uiGridConstants', 'SweetAlert', '$translate', 'stdAccountService', 'enterpriceAccountService', '$interval',
    function ($scope, $log, uiGridConstants, SweetAlert, $translate, stdAccountService, enterpriceAccountService, $interval) {
        'use strict';

        var defaultSelectStandardAccount = {
            id: 0,
            name: '',
            fullName: '',
            code: '',
            parentCode: '',
            acctLevel: 0,
            selectValue: ' '
        };

        var isActiveMap = { false: 'Disable', true: 'Enable' };
        var isActivedMap = { false: 'Disabled', true: 'Enabled' };

        var InitStdSubject = function () {
            stdAccountService.getStdAccountHierarchy().success(function (stdAccountsCategories) {
                $scope.stdAccountsCategories = stdAccountsCategories;
            });
        };

        var InitEnterpriseSubject = function () {
            enterpriceAccountService.getEnterpriceAccountList().success(function (data) {
                data.forEach(function (row) {
                    row.directionStr = _.find($scope.directionList, function (num) {
                        return num.id === row.direction
                    }).name;
                    row.subjectTypeStr = _.find($scope.acctPropList, function (num) {
                        return num.id === row.acctProp
                    }).name;
                });

                $scope.gridOptionsSubjectList.data = data;
            });
        };


        var mapStdAccount = function (code, name) {
            var accountMap = {};
            accountMap.enterpriseAccountCodes = [];
            accountMap.standardAccountCode = code;
            var selectedRows = $scope.gridApi.selection.getSelectedRows();
            if (selectedRows.length === 0) {
                SweetAlert.warning($translate.instant('SelectOneAccountToMap'));
                return;
            }
            selectedRows.forEach(function (v) {
                accountMap.enterpriseAccountCodes.push(v['code']);
            });
            enterpriceAccountService.mapAccount(accountMap).success(function (or) {
                if (or.result === true) {
                    $scope.gridApi.selection.clearSelectedRows();
                    SweetAlert.success($translate.instant('AccountMapSuccess'));
                    selectedRows.forEach(function (v) {
                        v['stdCode'] = code;
                        v['stdName'] = name;
                    });
                }
                else {
                    SweetAlert.warning($translate.instant(or.resultMsg).formatObj({ code: or.data.standardAccountCode }));
                }
            });
        };

        $scope.autoMap = function () {

            enterpriceAccountService.autoMap().success(function () {
                // mapping success
                InitEnterpriseSubject();
            });
        };

        $scope.showLinkModal = function () {
            $scope.isShowLinkModal = true;
            $scope.isShowLinkReport = false;
        };

        $scope.showLinkReport = function () {
            $scope.isShowLinkReport = true;
            $scope.isShowLinkModal = false;

        };

        var loadTemplateFormula = function () {
            // 先清空上次记录
            $scope.reportGridOptions.data = [];
            if ($scope.selectStandardAccount && $scope.selectStandardAccount.code) {
                templateFormulaService.getByStandardCode($scope.selectStandardAccount.code).success(function (data) {
                    $scope.reportGridOptions.data = data;
                });
            }
        };

        var showDetail = function (entity) {
            $scope.dbSelectedModel = entity;
            // 选中
            enterpriceAccountService.get($scope.dbSelectedModel.id).success(function (data) {
                $scope.editModel = data;
                if (data.stdCode && data.stdName) {
                    $scope.editModel.standardAccount = data.stdCode + '-' + data.stdName;
                }
                else {
                    $scope.editModel.standardAccount = $translate.instant('NotMap');
                }

                $scope.isAdd = false;

                $scope.editModel.title = $translate.instant('EnterpriceAccount') + '-' + '(' + data.code + ' ' + data.name + ')';

                $scope.selectStandardAccount = _.find($scope.standardAccountList, function (num) { return num.code === data.stdCode });

                if (!$scope.selectStandardAccount || !data.stdCode || data.stdCode === '') {
                    $scope.selectStandardAccount = defaultSelectStandardAccount;
                }

                $scope.selectDirection = _.find($scope.directionList, function (num) { return num.id === data.direction });
                $scope.selectAcctProp = _.find($scope.acctPropList, function (num) { return num.id === data.acctProp });

                $scope.isShowLinkModal = false;
                $scope.isShowLinkReport = true;
                $scope.iseditDetail = false;

                loadTemplateFormula();

                $('#enterpriseAccountDetailPopup').modal('show');
            });
        };

        $scope.reportGridOptions = {
            rowHeight: 28,
            selectionRowHeaderWidth: 28,
            enableFullRowSelection: false,
            enableRowSelection: false,
            enableSorting: false,
            enableFiltering: false,
            enableColumnMenus: false,
            enableRowHeaderSelection: false,
            enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
            columnDefs: [
            {
                field: 'templateName', name: $translate.instant('TemplateName'), width: '20%', headerCellClass: '',

                cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.templateName}}<span></div>'
            },
              {
                  field: 'serviceTypeName', name: $translate.instant('ServiceTypeName'), width: '20%', headerCellClass: '',

                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.serviceTypeName}}<span></div>'
              },

              {
                  field: 'cell', name: $translate.instant('Cell'), width: '10%', headerCellClass: '',

                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.cell}}</span></div>'
              },
              {
                  field: 'rowName', name: $translate.instant('RowName'), width: '15%', headerCellClass: '',

                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.rowName}}</span></div>'
              },
              {
                  field: 'columnName', name: $translate.instant('ColumnName'), width: '15%', headerCellClass: '',

                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.columnName}}</span></div>'
              },
               {
                   field: 'formula', name: $translate.instant('Formula'), width: '20%', headerCellClass: '',

                   cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.formula}}</span></div>'
               }
            ]
        };

        (function initializa() {
            $log.debug('EnterpriseSubjectCorrespondingController.ctor()...');

            $scope.stdAccountsCategories = [];

            $scope.stdSubjectList = [];

            $scope.directionList = [
            { id: 1, name: $translate.instant('StandardAccountDebit') },
            { id: -1, name: $translate.instant('StandardAccountCredit') },
            ];

            $scope.acctPropList = [
            { id: 1, name: $translate.instant('StandardAccountAcctPropAsset') },
            { id: 2, name: $translate.instant('StandardAccountAcctPropDebt') },
            { id: 3, name: $translate.instant('StandardAccountAcctPropCommon') },
            { id: 4, name: $translate.instant('StandardAccountAcctPropInterest') },
            { id: 5, name: $translate.instant('StandardAccountAcctPropCost') },
            { id: 6, name: $translate.instant('StandardAccountAcctPropProfitAndLoss') },
            ];


            $scope.gridOptionsSubjectList = {
                rowHeight: 55,
                selectionRowHeaderWidth: 55,
                enableSorting: false,
                enableColumnMenus: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                enableRowSelection: true,
                enableRowHeaderSelection: false,
                columnDefs: [
                  {
                      field: 'code', name: $translate.instant('EnterpriseSubjectCodeCol'), width: '20%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.code}}<span></div>'
                  },
                  {
                      field: 'name', name: $translate.instant('EnterpriseSubjectNameCol'), width: '25%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.name}}</span></div>'
                  },
                  {
                      field: 'directionStr', name: $translate.instant('SubjectDirectionCol'), width: '15%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.directionStr}}</span></div>'
                  },
                  {
                      field: 'subjectTypeStr', name: $translate.instant('SubjectTypeCol'), width: '15%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.subjectTypeStr}}</span></div>'
                  },
                  {
                      field: 'stdCode', name: $translate.instant('StandardSubjectCol'), width: '25%', headerCellClass: 'text-align-center',
                      cellTemplate: '<div class="text-align-center" ng-dblClick="grid.appScope.showDetail(row.entity)"><span><i class="material-icons map-status-flag" ng-if="row.entity.stdCode == null">flag</i>{{row.entity.stdCode === null ? "未对应" : (row.entity.stdCode + "-" + row.entity.stdName)}}</span></div>'
                  }
                ],
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    // call resize every 500 ms for 5 s after modal finishes opening - usually only necessary on a bootstrap modal
                    $interval(function () {
                        $scope.gridApi.core.handleWindowResize();
                    }, 500, 60 * 60 * 8);
                }
            };


            InitStdSubject();
            InitEnterpriseSubject();

            $scope.mapStdAccount = mapStdAccount;
            $scope.showDetail = showDetail;

        })();
    }
]);