﻿commonModule.controller('importAuditAdjustController', ['$scope', '$log', '$timeout', '$q', '$translate', 'loginContext'
    , 'apiInterceptor', 'Upload', 'dataImportService', 'SweetAlert', 'vatImportService', 'citSessionService'
    , 'uiGridConstants', 'enums', 'vatCommonService', '$uibModal',
function ($scope, $log, $timeout, $q, $translate, loginContext
    , apiInterceptor, Upload, dataImportService, SweetAlert, vatImportService, citSessionService
    , uiGridConstants, enums, vatCommonService, $uibModal) {
    'use strict';

    var uploadUrl = apiInterceptor.vatWebApiHostUrl + '/FileUpload/NewFile';
    var successCount = 0;
    var resumable = true;
    var projectID = citSessionService.project.id;
    var period = $scope.periodId;
    var projectDbName = citSessionService.project.dbName;
    $scope.chunkSize = 100000;
    var validationErrorType = {
        PeriodParseErrorType: $translate.instant('PeriodParseErrorType'),
        AdjustDateParseErrorType: $translate.instant('AdjustDateParseErrorType'),
        DebitParseErrorType: $translate.instant('DebitParseErrorType'),
        CreditParseErrorType: $translate.instant('CreditParseErrorType'),
        EmptyFieldErrorType: $translate.instant('EmptyFieldErrorType'),
        OverLengthFieldErrorType: $translate.instant('OverLengthFieldErrorType')
    };

    var serverErrorType = {
        EmptyFieldErrorType: 'EmptyField',
        OverLengthFieldErrorType: 'OverLengthField'
    };

    //初始化ack-pagination
    $scope.pagingOptions = {
        pageIndex: 1, //当前页码
        totalItems: 0, //总数据
        totalPages: 10, //总页数
        maxSize: 5, //分页数字的限制。
        pageSize: 5,//constant.page.pageSizeArrary[3], //每页多少条数据,100条
        pageSizeString: constant.page.pageSizeArrary[3].toString(),
        firstPage: $translate.instant('PagingFirstPage'),
        previousPage: $translate.instant('PagingPreviousPage'),
        nextPage: $translate.instant('PagingNextPage'),
        lastPage: $translate.instant('PagingLastPage'),
    };

    //初始化查询参数
    $scope.queryParams = {
        pageInfo: {
            totalCount: 0,
            pageIndex: 0,
            pageSize: 0,
            totalPage: 0,
        },
        periodId: period
    };

    //初始化汇总参数
    $scope.totalData = {
        totalNumber: 0,
        debit: 0,
        credit: 0
    };


    var initColumns = function () {

        $scope.allColumns = [
            $translate.instant('PleaseSelectColumn'),
            $translate.instant('period'),
            $translate.instant('adjustDate'),
            $translate.instant('JournalKMDM'),
            $translate.instant('JournalZY'),
            $translate.instant('JournalJFJE'),
            $translate.instant('JournalDFJE')
        ];

        $scope.necessoryColumns = [
            $translate.instant('JournalKMDM'),
            $translate.instant('JournalJFJE'),
            $translate.instant('JournalDFJE')
        ];
    };

    var initGrid = function () {

        $scope.gridOptions = {
            rowHeight: constant.UIGrid.rowHeight,
            selectionRowHeaderWidth: constant.UIGrid.rowHeight,
            enableFullRowSelection: false,
            enableRowSelection: false,
            enableSorting: false,
            enableFiltering: false,
            enableColumnMenus: false,
            enableRowHeaderSelection: false,
            enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
            columnDefs: [
            {
                field: 'index', name: $translate.instant('ImportErrorPopUpNoCol'), width: '10%', headerCellClass: '', enableFiltering: false,

                cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.index}}<span></div>'
            }
            ],
            onRegisterApi: function (gridApi) {
                $scope.gridApiTotal = gridApi;
            }
        };

        $scope.gridOptions.noData = false;
    };

    var setGridCols = function (data) {

        var colCnt = 3;
        var showPeriod = _.filter(data, function (i) {
            return !PWC.isNullOrEmpty(i.monthID);
        }).length > 0;
        if (showPeriod) colCnt++;

        var showDate = _.filter(data, function (i) {
            return i.date != i.createTime;
        }).length > 0;
        if (showDate) colCnt++;

        var showSummary = _.filter(data, function (i) {
            return !PWC.isNullOrEmpty(i.summary);
        }).length > 0;
        if (showSummary) colCnt++;

        var colWidth = PWC.round(90 / colCnt, 0);
        var lastColWidth = 90 - (colCnt - 1) * colWidth;

        $scope.gridOptions.columnDefs = [
        {
            name: $translate.instant('ImportErrorPopUpNoCol'), width: '10%', headerCellClass: '',
            cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.index}}<span></div>'
        }];

        if (showPeriod) {
            $scope.gridOptions.columnDefs.push({
                name: $translate.instant('period'), width: colWidth.toString() + '%', headerCellClass: 'right',
                cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.period}}</span></div>'
            });
        }

        if (showDate) {
            $scope.gridOptions.columnDefs.push({
                name: $translate.instant('adjustDate'), width: colWidth.toString() + '%', headerCellClass: '',
                cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.date | date:"yyyy-MM-dd"}}</span></div>'
            });
        }

        $scope.gridOptions.columnDefs.push({
            name: $translate.instant('JournalKMDM'), width: colWidth.toString() + '%', headerCellClass: '',
            cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.acctCode}}</span></div>'
        });

        if (showSummary) {
            $scope.gridOptions.columnDefs.push({
                name: $translate.instant('JournalZY'), width: colWidth.toString() + '%', headerCellClass: '',
                cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.summary}}</span></div>'
            });
        }

        $scope.gridOptions.columnDefs.push(
         {
             name: $translate.instant('JournalJFJE'), width: colWidth.toString() + '%', headerCellClass: 'right', format: { type: 'fixedPoint', precision: 2 },
             cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.debit}}</span></div>'
         },
         {
             name: $translate.instant('JournalDFJE'), width: lastColWidth.toString() + '%', headerCellClass: 'right', format: { type: 'fixedPoint', precision: 2 },
             cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.credit}}</span></div>'
         });
    };

    $scope.$on(enums.vatEvent.layoutChanged, function () {
        $scope.gridApiTotal.core.handleWindowResize();
    });

    //parentIndex：column index
    //index：selecteditem index
    var mappingColumn = function (parentIndex, index, colName) {
        if (colName) {

            var oldColumn = $scope.selectedColumnMap[parentIndex];
            var newColumn = $scope.allColumns[index];

            if (oldColumn === newColumn) {
                return;
            }
            else {
                $scope.selectedColumnMap[parentIndex] = colName;

                if (colName !== $translate.instant('PleaseSelectColumn')) {
                    $scope.allColumns.splice(index, 1);

                    if (oldColumn && oldColumn !== $translate.instant('PleaseSelectColumn')) {
                        $scope.allColumns.push(oldColumn);
                    }
                }
                else {
                    if (oldColumn) {
                        $scope.allColumns.push(oldColumn);
                    }
                }
            }
        }
        $log.debug($scope.selectedColumnMap);
    };

    var UploadFile = function (file) {
        if (file) {
            if (!file.name.endsWith('.xls') && !file.name.endsWith('.xlsx')) {
                SweetAlert.warning($translate.instant('ImportFileInvalidType'));
                return;
            };
            $scope.initIncomeInvoiceList = [];
            successCount = 0;
            if (!file.$error) {
                var tempFileName = PWC.newGuid() + '.dat';
                Upload.upload({
                    url: uploadUrl,
                    data: {
                        filename: file.name,
                        tempFileName: tempFileName,
                        file: file
                    },
                    resumeChunkSize: resumable ? $scope.chunkSize : null,
                    headers: {
                        'Access-Control-Allow-Origin': '*',
                        Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken
                    },
                    __RequestVerificationToken: $scope.token,
                    withCredentials: true
                }).then(function (resp) {
                    successCount++;
                    $scope.fileName = file.name;
                    if (successCount === 1) {
                        $scope.tempFileName = resp.data;
                        changeSheet(0);
                    }
                });
            }
        }
    }

    var resetMappingRow = function () {
        initColumns();
        $scope.selectedColumnMap = [];
    };

    var changeSheet = function (index) {
        $scope.selectedSheet.sheetIndex = index;
        resetMappingRow();
        if (_.isString($scope.tempFileName)) {
            //$scope.excelSource = { sheetNameList: [], dataList: [] };
            vatImportService.getFileContent($scope.tempFileName, $scope.selectedSheet.sheetIndex, $scope.selectedSheet.topRowNumber).success(function (data) {
                if (data && data.result === false) {
                    SweetAlert.warning(data.resultMsg);
                    return;
                }
                $scope.excelSource = data;
                $scope.firstDataRow = data.dataList[0];
                $scope.startRowList = _.range(data.lastRowIndex);
                $scope.selectedSheet.sheetName = $scope.excelSource.sheetNameList[$scope.selectedSheet.sheetIndex];
                citSessionService.dataChanged = true;
            }).error(function () {
                SweetAlert.error($translate.instant('PleaseContactAdministrator'));
            });
        }
    };

    var validateClientSide = function (dataList) {

        var periodIntCheckMsg = '';
        var periodCnt = 0;
        var dateCheckMsg = '';
        var dateCnt = 0;
        var creditNumberCheckMsg = '';
        var creditCnt = 0;
        var debitNumberCheckMsg = '';
        var debitCnt = 0;

        if (parseInt($scope.StartRowNum) > $scope.excelSource.dataList.length) {
            SweetAlert.warning($translate.instant('StartRowNumberCheckMsg'));
            return;
        }

        for (var i = $scope.StartRowNum - 1; i < $scope.excelSource.dataList.length; i++) {
            var period1 = null;
            if ($.inArray($translate.instant('period'), $scope.selectedColumnMap) >= 0) {
                period1 = $scope.excelSource.dataList[i][$.inArray($translate.instant('period'), $scope.selectedColumnMap)];
            }
            var summary = null;
            if ($.inArray($translate.instant('JournalZY'), $scope.selectedColumnMap) >= 0) {
                summary = $scope.excelSource.dataList[i][$.inArray($translate.instant('JournalZY'), $scope.selectedColumnMap)];
            }
            var adjustDate = null;
            if ($.inArray($translate.instant('adjustDate'), $scope.selectedColumnMap) >= 0) {
                adjustDate = $scope.excelSource.dataList[i][$.inArray($translate.instant('adjustDate'), $scope.selectedColumnMap)];
            }
            var credit = $scope.excelSource.dataList[i][$.inArray($translate.instant('JournalDFJE'), $scope.selectedColumnMap)];
            var debit = $scope.excelSource.dataList[i][$.inArray($translate.instant('JournalJFJE'), $scope.selectedColumnMap)];
            var code = $scope.excelSource.dataList[i][$.inArray($translate.instant('JournalKMDM'), $scope.selectedColumnMap)];

            var curRowInfo = '(' + $translate.instant('RowIndex').formatObj({ rowIndex: i + 1 }) + ')' +
                            $translate.instant('period') + ':' + period1 + ',' +
                            $translate.instant('adjustDate') + ':' + adjustDate + ',' +
                            $translate.instant('JournalKMDM') + ':' + code + ',' +
                            $translate.instant('JournalZY') + ':' + summary + ',' +
                            $translate.instant('JournalJFJE') + ':' + debit + ',' +
                            $translate.instant('JournalDFJE') + '：' + credit + ';<br />';

            $scope.hasOtherPeriod = false;
            if ($.inArray($translate.instant('period'), $scope.selectedColumnMap) >= 0) {
                if (isNaN(parseInt(period1))) {
                    periodIntCheckMsg += curRowInfo;
                    periodCnt++;
                }
                else {
                    if (period1 != period) {
                        //CIT全年时不做此判断
                        if (period !== constant.WholeYearPeriod) {
                            $scope.hasOtherPeriod = true;
                            continue;
                        }                       
                    }
                }
            }

            if ($.inArray($translate.instant('adjustDate'), $scope.selectedColumnMap) >= 0) {
                if (!PWC.checkDate(adjustDate)) {
                    dateCheckMsg += curRowInfo;
                    dateCnt++;
                }
            }

            if (isNaN(parseFloat(credit))) {
                creditNumberCheckMsg += curRowInfo;
                creditCnt++;
            }

            if (isNaN(parseFloat(debit))) {
                debitNumberCheckMsg += curRowInfo;
                debitCnt++;
            }

            if (periodCnt == 0 && creditCnt == 0 && debitCnt == 0 && dateCnt == 0) {
                var item = {
                    'AcctCode': code,
                    'Period': period !== constant.WholeYearPeriod ? period : period1 ,
                    'Debit': debit,
                    'Credit': credit,
                    'Summary': summary,
                    'MonthID': period1,
                    'CreateTime': null,
                    'Date': adjustDate
                };
                if ($.inArray($translate.instant('adjustDate'), $scope.selectedColumnMap) >= 0) {
                    item.CreateTime = new Date();
                };
                dataList.push(item);
            }
        }

        $scope.errorMsgMap = [];

        if (periodCnt > 0) {
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.PeriodParseErrorType,
                    'errorCount': periodCnt,
                    'errorContent': $translate.instant('PeriodParseErrorMsg').formatObj({ rows: periodIntCheckMsg })
                });
        };

        if (dateCnt > 0) {
            $scope.errorMsgMap.push(
                           {
                               'errorType': validationErrorType.AdjustDateParseErrorType,
                               'errorCount': periodCnt,
                               'errorContent': $translate.instant('AdjustDateParseErrorMsg').formatObj({ rows: dateCheckMsg })
                           });
        }

        if (debitCnt > 0) {
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.DebitParseErrorType,
                    'errorCount': debitCnt,
                    'errorContent': $translate.instant('DebitParseErrorMsg').formatObj({ rows: debitNumberCheckMsg })
                });
        };

        if (creditCnt > 0) {
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.CreditParseErrorType,
                    'errorCount': creditCnt,
                    'errorContent': $translate.instant('CreditParseErrorMsg').formatObj({ rows: creditNumberCheckMsg })
                });
        };

        if ($scope.errorMsgMap.length > 0) {
            $('#errorListModal').modal('show');
            return false;
        }
        return true;
    };

    var alerts = function () {
        var deferred = $q.defer();
        $scope.isContinue = true;
        if ($.inArray($translate.instant('period'), $scope.selectedColumnMap) >= 0) {
            if ($scope.hasOtherPeriod) {
                swal({
                    title: "warning!",
                    text: $translate.instant('PeriodDifferenceTip'),
                    type: "warning",
                    showCancelButton: true,
                    confirmButtonColor: "#DD6B55",
                    confirmButtonText: $translate.instant('Yes'),
                    cancelButtonText: $translate.instant('No'),
                    closeOnConfirm: true,
                    closeOnCancel: true
                },
                function (isConfirm) {
                    $scope.isContinue = isConfirm;
                    deferred.resolve();
                });
            }
            else {
                $scope.isContinue = true;
                deferred.resolve();
            }
        }
        else {
            swal({
                title: "warning!",
                text: $translate.instant('NoPeriod'),
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                confirmButtonText: $translate.instant('Yes'),
                cancelButtonText: $translate.instant('No'),
                closeOnConfirm: true,
                closeOnCancel: true
            },
            function (isConfirm) {
                $scope.isContinue = isConfirm;
                deferred.resolve();
            });
        };

        return deferred.promise;
    };

    function doSave2Server(saveType) {
        // saveType 1：override，2：append
        if ($scope.excelSource.sheetNameList.length == 0) {
            SweetAlert.warning($translate.instant('PleaseSelectFileFirst'));
            return;
        }
        var notMappedColumns = '';
        if ($scope.selectedColumnMap.length >= 0) {
            var count1 = 0;
            $scope.necessoryColumns.forEach(function (m) {
                if (m !== $translate.instant('PleaseSelectColumn')) {
                    if ($.inArray(m, $scope.selectedColumnMap) < 0) {
                        notMappedColumns += m + ',';
                        count1++;
                    }
                }
            });
            if (notMappedColumns) {
                SweetAlert.warning($translate.instant('ColumnsMapErrorMsg').formatObj({ columns: notMappedColumns.substring(0, notMappedColumns.length - 1) }));
                return;
            }
        }
        var dataList = [];
        if (!validateClientSide(dataList)) return;
        if (dataList.length == 0) {
            SweetAlert.warning($translate.instant('noCurMonthRecordErrorMsg'));
            return;
        };

        alerts().then(function () {
            if ($scope.isContinue) {
                $scope.errorMsgMap = [];
                if ($scope.errorMsgMap.length > 0) {
                    $('#errorListModal').modal('show');
                    return;
                }
                else {
                    vatImportService.ImportAuditAdjust(dataList, period, saveType).success(function (data) {
                        if (data) {
                            if (data.result) {
                                vatImportService.getImportType(projectID, period).success(function (data1) {
                                    var type = data1;

                                    if (type == enums.vatDataImportStatus.ErpImported) {
                                        dataImportService.manageData(period).success(function (data2) {
                                            vatCommonService.importSetProjectStatus(projectDbName, period, constant.DictionaryDictKey.WFImportAuditAdjust
                                                                                    , enums.FinishStatusEnum.Finished);
                                            vatCommonService.setImportSubStatus(enums.VatImportSubStatus.isAdjustImport, true);
                                            //$log.debug("import auditAdjust save2Server: " + citSessionService.project.importSubStatus.isAdjustImport);
                                            SweetAlert.success($translate.instant('ImportSuccess'));
                                            resetMappingRow();
                                            getVouchers();
                                        }).error(function () {
                                            $log.debug('ErpImported-Error()...');
                                            SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                                        });
                                    }
                                    else if (type == enums.vatDataImportStatus.TbImported) {
                                        vatImportService.refreshTrialBalance(period).success(function (data3) {
                                            if (data3 === true) {
                                                vatCommonService.importSetProjectStatus(projectDbName, period, constant.DictionaryDictKey.WFImportAuditAdjust
                                                                                       , enums.FinishStatusEnum.Finished);
                                                vatCommonService.setImportSubStatus(enums.VatImportSubStatus.isAdjustImport, true);
                                                //$log.debug("import auditAdjust save2Server: " + citSessionService.project.importSubStatus.isAdjustImport);
                                                SweetAlert.success($translate.instant('ImportSuccess'));
                                                resetMappingRow();
                                                getVouchers();
                                            }
                                            else {
                                                $log.debug('TbImported-Server-Error()...');
                                            }
                                        }).error(function () {
                                            $log.debug('TbImported-Error()...');
                                            SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                                        });
                                    }
                                    else {
                                        vatCommonService.importSetProjectStatus(projectDbName, period, constant.DictionaryDictKey.WFImportAuditAdjust
                                                                                 , enums.FinishStatusEnum.Finished);
                                        vatCommonService.setImportSubStatus(enums.VatImportSubStatus.isAdjustImport, true);
                                        //$log.debug("import auditAdjust save2Server: " + citSessionService.project.importSubStatus.isAdjustImport);
                                        SweetAlert.success($translate.instant('ImportSuccess'));
                                        resetMappingRow();
                                        getVouchers();
                                    }
                                    $scope.fileName = '';
                                }).error(function () {
                                    $log.debug('getImportType-Error()...');
                                    SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                                });
                            }
                            else {
                                processValidateMsg(data.data);
                            }
                        }
                    }).error(function () {
                        $log.debug('ImportAuditAdjust-Error()...');
                        SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                    });
                }
            }
        });
    }

    var save2Server = function (saveType) {
        if (citSessionService.project.projectStatusList[period] >= constant.ProjectStatusEnum.Generated) {
            swal({
                title: "warning!",
                text: $translate.instant('IsConfirmToReImport').formatObj({ status: vatCommonService.getProjectStautsEnumDesc(citSessionService.project.projectStatusList[period]) }),
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                confirmButtonText: $translate.instant('Yes'),
                cancelButtonText: $translate.instant('No'),
                closeOnConfirm: true,
                closeOnCancel: true
            },
            function (isConfirm) {
                if (isConfirm) {
                    doSave2Server(saveType);
                }
                else {
                    swal.close();
                }
            });
        }
        else {
            doSave2Server(saveType);
        }
    };

    var buildDataStr = function (p) {
        var empty = '<' + $translate.instant('empty') + '>';
        return '(' + $translate.instant('RowIndex').formatObj({ rowIndex: p.rowIndex }) + ')' +
            $translate.instant('period') + ':' + (PWC.isNullOrEmpty(p.tranCode) ? empty : p.tranCode) + ',' +
            $translate.instant('JournalKMDM') + ':' + (PWC.isNullOrEmpty(p.period) ? empty : p.period) + ',' +
            $translate.instant('JournalZY') + ':' + (PWC.isNullOrEmpty(p.vid) ? empty : p.vid) + ',' +
            $translate.instant('JournalJFJE') + ':' + (PWC.isNullOrEmpty(p.group) ? empty : p.group) + ',' +
            $translate.instant('JournalDFJE') + ':' + (PWC.isNullOrEmpty(p.itemID) ? empty : p.itemID) + ';<br />';
        return '';
    };

    var processValidateMsg = function (errorList) {
        if (errorList && errorList.length > 0) {
            errorList.forEach(function (v) {
                if (v.resultMsg === serverErrorType.EmptyFieldErrorType) {
                    var vString = '';
                    v.data.forEach(function (p) {
                        vString += buildDataStr(p);
                    });
                    $scope.errorMsgMap.push(
                        {
                            'errorType': validationErrorType.EmptyFieldErrorType,
                            'errorCount': v.data.length,
                            'errorContent': $translate.instant('EmptyFieldErrorMsg').formatObj({ rows: vString })
                        });
                }
                else if (v.resultMsg === serverErrorType.OverLengthFieldErrorType) {
                    var vString = '';
                    v.data.forEach(function (p) {
                        vString += buildDataStr(p);
                    });
                    $scope.errorMsgMap.push(
                        {
                            'errorType': validationErrorType.OverLengthFieldErrorType,
                            'errorCount': v.data.length,
                            'errorContent': $translate.instant('OverLengthErrorMsg').formatObj({ rows: vString })
                        });
                }

            });
        };
        if ($scope.errorMsgMap.length > 0) {
            $('#errorListModal').modal('show');
        }
    };
    
    var clear = function () {
        if (period === constant.WholeYearPeriod) {
            showPeriodDrapDown();
        }
        else {
            $scope.selectedPeriod = period;
            deleteAdjustAuditByPeriod();
        }
          
    };
  
    //显示期间选择弹出框
    var showPeriodDrapDown = function () {
        $scope.modalInstance = $uibModal.open({
            animation: false,
            ariaLabelledBy: 'modal-title',
            ariaDescribedBy: 'modal-body',
            templateUrl: 'audit-model-period-dropdown.html',
            scope: $scope,
            windowClass: "audit-model-period-dropdow-popup"
        });

        $scope.closePeriodSelectModal = function () {
            $scope.modalInstance.dismiss('cancel');
        };

        $scope.selectBox = {
            simple: {
                dataSource: $scope.auditPeriods,
                displayExpr: "Period",
                valueExpr: "ID",
                searchEnabled: false,
                placeholder: $translate.instant('PleaseSelectPeriod'),
                noDataText: $translate.instant('NoDataToDispaly'),
                maxLength: 2,
                onSelectionChanged: function (e) {     
                    $scope.selectedPeriod = e.selectedItem.Period;
                }
            }
        };

        vatImportService.getAdjustAuditPeriods().success(function (re) {
            if (re.result) {
                $scope.auditPeriods = [];
                var id = 1;
                $.each(re.data, function (index, item) {
                    var periodDto = {
                        ID: -1,
                        Period: -1
                    };
                    periodDto.ID = id++;
                    periodDto.Period = item;
                    $scope.auditPeriods.push(periodDto);
                });

                $("#auditPeriodSelectBox").dxSelectBox("instance").option("dataSource", $scope.auditPeriods);
            }
        }).error(function () {
            SweetAlert.error($translate.instant('PleaseContactAdministrator'));
        });
    }

    var deleteAdjustAuditByPeriod = function () {
        if (_.isNull($scope.selectedPeriod)) {
            SweetAlert.warning($translate.instant('PleaseSelectPeriod'));
            return;
        }

        var alertMsg = period === constant.WholeYearPeriod
            ? $translate.instant('AdjustAuditClearWarning') : $translate.instant('AdjustAuditClearTips');
      swal({
          title: "warning!",
          text: alertMsg,
          type: "warning",
          showCancelButton: true,
          confirmButtonColor: "#DD6B55",
          confirmButtonText: $translate.instant('Yes'),
          cancelButtonText: $translate.instant('No'),
          closeOnConfirm: true,
          closeOnCancel: true
      }, function (isConfirm) {
          if (isConfirm) {
              var dataList = [];
              var saveType = 1;//overide
              var deleteDto = {
                  serviceTypeId: $scope.serviceTypeId,
                  periodIds: [$scope.selectedPeriod]
              };

              vatImportService.deleteAdjustAudit(deleteDto).success(function (data) {
                  if (data.result) {
                      $scope.selectedPeriod = null;
                      if (period === constant.WholeYearPeriod) {
                          $scope.closePeriodSelectModal();
                      }

                      vatImportService.getImportType(projectID, period).success(function (data1) {
                          var type = data1;
                          if (type === enums.vatDataImportStatus.ErpImported) {
                              dataImportService.manageData(period).success(function (data2) {
                                  vatCommonService.clearDataSetProjectStatus(projectDbName, period, constant.DictionaryDictKey.WFImportAuditAdjust
                                                                            , enums.FinishStatusEnum.NotFinished);
                                  vatCommonService.setImportSubStatus(enums.VatImportSubStatus.isAdjustImport, false);
                                  $log.debug("clear auditAdjust clear: " + citSessionService.project.importSubStatus.isAdjustImport);
                                  SweetAlert.success($translate.instant('ClearSuccess'));                                   
                                  resetMappingRow();
                                  getVouchers();
                              }).error(function () {
                                  $log.debug('ErpImported-Error()...');
                                  SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                              });
                          }
                          else if (type === enums.vatDataImportStatus.TbImported) {
                              vatImportService.refreshTrialBalance(period).success(function (data3) {
                                  if (data3 === true) {
                                      vatCommonService.clearDataSetProjectStatus(projectDbName, period, constant.DictionaryDictKey.WFImportAuditAdjust
                                                                                , enums.FinishStatusEnum.NotFinished);
                                      vatCommonService.setImportSubStatus(enums.VatImportSubStatus.isAdjustImport, false);
                                      //$log.debug("import auditAdjust clear: " + citSessionService.project.importSubStatus.isAdjustImport);
                                      SweetAlert.success($translate.instant('ClearSuccess'));                                       
                                      resetMappingRow();
                                      getVouchers();
                                  }
                                  else {
                                      $log.debug('TbImported-Server-Error()...');
                                  }
                              }).error(function () {
                                  $log.debug('TbImported-Error()...');
                                  SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                              });
                          }
                          else {
                              vatCommonService.clearDataSetProjectStatus(projectDbName, period);
                              vatCommonService.setImportSubStatus(enums.VatImportSubStatus.isAdjustImport, false);
                              //$log.debug("import auditAdjust clear: " + citSessionService.project.importSubStatus.isAdjustImport);
                              SweetAlert.success($translate.instant('ClearSuccess'));
                              resetMappingRow();
                              getVouchers();
                          }

                      }).error(function () {
                          $log.debug('getImportType-Error()...');
                          SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                      });
                  }
              });
          }
      })     
    };

    var getVouchers = function () {
        $scope.queryParams.periodId = period;
        $scope.queryParams.pageInfo = {
            totalCount: $scope.pagingOptions.totalItems,
            pageIndex: $scope.pagingOptions.pageIndex,
            pageSize: $scope.pagingOptions.pageSize,
            totalPage: 0,
        };
        vatImportService.queryAuditAdjust($scope.queryParams).success(function (re) {
            if (re) {
                $scope.excelSource = { sheetNameList: [], dataList: [] };
                initPagingControl(re.pageInfo.totalCount);
                $scope.mapppingList = re.list;
                var seqNo = 1;
                $scope.mapppingList.forEach(function (i) {
                    i.index = seqNo++;
                    i.debit = PWC.round(i.debit, 2);
                    i.credit = PWC.round(i.credit, 2);
                });
                $scope.totalData.totalNumber = re.pageInfo.totalCount;
                $scope.totalData.debit = PWC.round(re.calculateData.debitSum, 2);
                $scope.totalData.credit = PWC.round(re.calculateData.creditSum, 2);
                setGridCols($scope.mapppingList);
                $scope.gridOptions.data = $scope.mapppingList;
                $scope.gridOptions.noData = $scope.mapppingList.length === 0;
                $scope.firstDataRow = null;
                citSessionService.dataChanged = false;
            }
            
        }).error(function () {
            SweetAlert.error($translate.instant('PleaseContactAdministrator'));
        });
    };

    var initPagingControl = function (totalItemsCount) {
        $scope.pagingOptions.totalItems = totalItemsCount;
    }

    $scope.$watch('selectedFileName', function (newValue, oldValue) {
        if (newValue && newValue !== oldValue) {
            UploadFile($scope.selectedFileName);
            resetMappingRow();
        }
    });

    $scope.pagingService = {
        refreshInvoiceDataGrid: function () {
            $log.debug("refreshInvoiceDataGrid");
            getVouchers();
        },
    };

    (function initialize() {
        $scope.StartRowNum = 2;
        $scope.mapppingList = [];
        $scope.mappingRow = null;
        $scope.firstDataRow = null;
        $scope.selectedPeriod = null;
        $scope.selectedColumnMap = [];
        $scope.selectedFileName = '';
        $scope.excelSource = { sheetNameList: [], dataList: [] };
        $scope.selectedSheet = { sheetName: '', sheetIndex: 0, topRowNumber: 0 };
        $scope.deleteAdjustAuditByPeriod = deleteAdjustAuditByPeriod;


        initColumns();
        initGrid();
        getVouchers();
        $scope.refresh = getVouchers;
        $scope.clear = clear;

        $scope.changeSheet = changeSheet;
        $scope.mappingColumn = mappingColumn;
        $scope.save2Server = save2Server;
    })();

   
}
]);