﻿invoiceModule.controller('importShipmentListController', ['$scope', '$log', '$document', '$uibModal', 'messagebox',
    '$translate', '$q', 'apiInterceptor', '$interval', 'region', '$timeout', '$location', 'InvoiceManageService',
    'vatOutputInvoiceManageService', 'userService', '$state', 'SweetAlert', 'Upload',
    function($scope, $log, $document, $uibModal, messagebox, $translate, $q, apiInterceptor, $interval, region,
        $timeout, $location, InvoiceManageService, vatOutputInvoiceManageService, userService, $state, SweetAlert, Upload) {
        'use strict';

        var thisConstant = {
            Company: $translate.instant('Company'),
            PgiWeek: $translate.instant('PgiWeek'),
            ConfirmationNumber: $translate.instant('ConfirmationNumber'),
            VIN: $translate.instant('VIN'),
            CarType: $translate.instant('CarType'),
            IsMarketingCar: $translate.instant('IsMarketingCar'),
            TaxableSubtotal: $translate.instant('TaxableSubtotal'),
            VehiclePriceafterDiscounts: $translate.instant('VehiclePriceafterDiscounts'),
            VAT: $translate.instant('VAT'),
            Revenue: $translate.instant('Revenue'),
            Discounts: $translate.instant('Discounts'),
            Subsidy: $translate.instant('Subsidy'),
            Cost: $translate.instant('Cost'),
            GDRevenue: $translate.instant('GDRevenue'),
            GDVAT: $translate.instant('GDVAT'),
            GDCost: $translate.instant('GDCost'),
            Inventory: $translate.instant('Inventory'),
            OrginalbeforeIntercompany: $translate.instant('OrginalbeforeIntercompany'),
            Notes: $translate.instant('Notes')
        };


        var mainModule = {

            main: function() {
                $scope.importDataModal = mainModule.importDataModal;
                $scope.chunkSize = 10 * 1024 * 1024;
                mainModule.initControls();

                $scope.toggleTriggerSyncHistory = mainModule.toggleTriggerSyncHistory;
                $scope.closeTriggerSyncHistory = mainModule.closeTriggerSyncHistory;

                mainModule.loadInvoiceData();

            },
            toggleTriggerSyncHistory: function(event) {

                vatOutputInvoiceManageService.getOutputInvoiceImportShipmentSyncHistory().success(function(data) {

                    if (data && data.length > 0) {
                        _.each(data, function(item) {
                            item.createTime = (new Date(item.createTime)).dateTimeToString('yyyyMMdd hh:mm');
                        });
                    }

                    $scope.syncHistoryList = data;

                    var mouseX = event.clientX;
                    var mouseY = event.clientY;
                    if (mouseX && mouseY) {
                        $('#shipmenthistory-pop-container').css({ 'top': mouseY - 32 + 'px', 'left': mouseX - 180 + 'px' }).css("display", 'block');
                    }

                });

            },

            closeTriggerSyncHistory: function() {
                $('#shipmenthistory-pop-container').hide(500);
            },

            // 初始化一些控件配置参数
            initControls: function() {

                $scope.searchEntity = {};
                var isShowClearButton = true;
                $scope.pagingOptions = {
                    pageIndex: 1, //当前页码
                    totalItems: 0, //总数据
                    pageSize: constant.page.pageSizeArrary[1], //每页多少条数据 
                };

                $scope.searchEntityOptions = {
                    //1. entity name
                    txtVINOptions: {
                        bindingOptions: {
                            value: 'searchEntity.vin',
                        },
                        // placeholder: $scope.pleaseSelect,
                        showClearButton: isShowClearButton,
                    }

                };


                $scope.loadInvoiceData = mainModule.loadInvoiceData;

                mainModule.initInvoiceGrid();
            },

            // 导入数据模态框
            importDataModal: function() {

                var editModel = {};

                $scope.editModel = editModel;

                var parent = angular.element($document[0].querySelector('#output-invoice-container'));
                var modalInstance = $uibModal.open({
                    animation: true,
                    // ariaLabelledBy: 'modal-title',
                    // ariaDescribedBy: 'modal-body',
                    templateUrl: 'importshipmentModal.html',
                    // windowClass: 'import-shipment-list-wrapper',
                    // appendTo: parent,
                    scope: $scope,
                    resolve: {
                        editModel: editModel
                    }
                });

                modalInstance.result.then(function(data) {
                    $scope.operateType = null;
                    $scope.isUpdate = true;
                }, function() {
                    $scope.operateType = null;
                    $scope.isUpdate = false;
                    $log.info('Modal dismissed at: ' + new Date());
                });

                // 确定
                $scope.overlayUpload = function() {
                    mainModule.importData(modalInstance);
                };

                // 取消
                $scope.cancel = function() {
                    modalInstance.dismiss('cancel');
                };
            },

            // 覆盖导入
            overlayUpload: function() {
                mainModule.importData();
            },

            // 导入
            importData: function(modalInstance) {
                if (!$scope.editModel.file || !$scope.editModel.file.name) {

                    SweetAlert.info($translate.instant('SelectCustomerFileRequired'));
                    return;
                }

                var deferred = $q.defer();

                var tempFileName = PWC.newGuid() + '.dat';
                var token = $('input[name="__RequestVerificationToken"]').val();

                var uploadUrl = apiInterceptor.webApiHostUrl + '/outputInvoiceImportShipment/importShipment';

                $('#busy-indicator-container').show();
                Upload.upload({
                    url: uploadUrl,
                    data: {
                        cancel: false,
                        filename: $scope.editModel.file.name,
                        tempFileName: tempFileName
                    },
                    file: $scope.editModel.file,
                    resumeChunkSize: $scope.chunkSize,
                    headers: {
                        'Access-Control-Allow-Origin': '*',
                        Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken,
                        __RequestVerificationToken: token,
                        withCredentials: true
                    },
                    __RequestVerificationToken: token,
                    withCredentials: true
                }).then(function(resp) {
                    var ret = resp.data;
                    $('#busy-indicator-container').hide();
                    deferred.resolve();
                    if (ret.result) {
                        // 全部导入成功
                        SweetAlert.success($translate.instant('ImportSuccess'));
                        modalInstance.close(ret);
                        mainModule.loadInvoiceData();
                    } else {

                        if (ret.resultMsg && ret.resultMsg.length > 0) {
                            SweetAlert.warning($translate.instant(ret.resultMsg));
                        } else {
                            SweetAlert.warning($translate.instant("ImportFailed"));
                        }
                    }

                }, function(resp) {
                    deferred.resolve();

                    if (resp.statusText === 'HttpRequestValidationException') {
                        SweetAlert.warning($translate.instant('HttpRequestValidationException'));
                    } else {
                        SweetAlert.warning('SaveFail');
                    }

                    console.log('Error status: ' + resp.status);
                }, function(evt) {
                    deferred.resolve();
                    var progressPercentage = parseInt(100.0 * evt.loaded / evt.total);
                    $log.debug('progress: ' + progressPercentage + '% ' + evt.config.data.filename);
                });
            },

            // 初始化发票表
            initInvoiceGrid: function() {

                $scope.invoiceGridDataSource = [];
                $scope.invoiceEditGridOptions = {
                    bindingOptions: {
                        dataSource: 'invoiceGridDataSource'
                    },
                    // keyExpr: "ID",
                    allowColumnResizing: true,
                    columnAutoWidth: true,
                    showRowLines: true,
                    showColumnLines: true,
                    // rowAlternationEnabled: true,
                    showBorders: true,
                    // height: '400px',
                    noDataText: $translate.instant('NoDataText'),
                    selectAllText: $translate.instant('SelectAll'),
                    allowColumnReordering: true,
                    height: 400,
                    paging:{
                        enabled:false
                    },
                    onContentReady: function(e) {
                        $scope.invoiceGridInstance = e.component;
                    },
                    onSelectionChanged: function(data) {
                        mainModule.SelectedInvoiceList = data.selectedRowsData;
                    },
                    columns: [{
                            caption: "",
                            columns: [{
                                caption: "",
                                columns: [{ dataField: 'pgIweek', caption: thisConstant.PgiWeek }]
                            }]
                        },

                        {
                            caption: 'Step 3- From BD to Final Customer',
                            alignment: 'center',
                            columns: [{
                                    caption: "",
                                    columns: [
                                        { dataField: 'company', caption: thisConstant.Company },
                                        { dataField: 'companyVloolup', caption: thisConstant.Company }
                                    ]
                                },
                                {
                                    caption: "Account Code",
                                    columns: [
                                        { dataField: 'confirmationNumber', caption: thisConstant.ConfirmationNumber }
                                    ]
                                },
                                {
                                    caption: "",
                                    columns: [
                                        { dataField: 'vin', caption: thisConstant.VIN },
                                        { dataField: 'carType', caption: thisConstant.CarType },
                                        { dataField: 'isMarketingCar', caption: thisConstant.IsMarketingCar },
                                        { dataField: 'taxableSubtotal', caption: thisConstant.TaxableSubtotal, format: { type: 'fixedPoint', precision: 2 } }
                                    ]
                                },
                                {
                                    caption: "120000",
                                    columns: [
                                        { dataField: 'vehiclePriceafterDiscounts', caption: thisConstant.VehiclePriceafterDiscounts, format: { type: 'fixedPoint', precision: 2 } }
                                    ]
                                },
                                {
                                    caption: "220143",
                                    columns: [
                                        { dataField: 'vat', caption: thisConstant.VAT, format: { type: 'fixedPoint', precision: 2 } }
                                    ]
                                },
                                {
                                    caption: "400005",
                                    columns: [
                                        { dataField: 'revenue', caption: thisConstant.Revenue, format: { type: 'fixedPoint', precision: 2 } }
                                    ]
                                },
                                {
                                    caption: "400005",
                                    columns: [
                                        { dataField: 'discounts', caption: thisConstant.Discounts, format: { type: 'fixedPoint', precision: 2 } }
                                    ]
                                },
                                {
                                    caption: "472000 US/ 640990 CN",
                                    columns: [
                                        { dataField: 'subsidy', caption: thisConstant.Subsidy }
                                    ]
                                },
                                {
                                    caption: "502000",
                                    columns: [
                                        { dataField: 'cost', caption: thisConstant.Cost, format: { type: 'fixedPoint', precision: 2 } }
                                    ]
                                }
                            ]
                        },
                        {
                            caption: "Step 2- From GD to BD Intercompany",
                            columns: [{
                                    caption: "For GD 2060 Fapiao Issue",
                                    columns: [
                                        { dataField: 'gdRevenue', caption: thisConstant.GDRevenue, format: { type: 'fixedPoint', precision: 2 } },
                                        { dataField: 'gdvat', caption: thisConstant.GDVAT, format: { type: 'fixedPoint', precision: 2 } }
                                    ]
                                },
                                {
                                    caption: "",
                                    columns: [{ dataField: 'gdCost', caption: thisConstant.GDCost, format: { type: 'fixedPoint', precision: 2 } }]
                                }

                            ]

                        },
                        {
                            caption: "Current Invontory",
                            columns: [{
                                    caption: "130450",
                                    columns: [{ dataField: 'inventory', caption: thisConstant.Inventory, format: { type: 'fixedPoint', precision: 2 } }]
                                },
                                {
                                    caption: "Warehouse in WD",
                                    columns: [{ dataField: 'orginalbeforeIntercompany', caption: thisConstant.OrginalbeforeIntercompany }]
                                }
                            ]
                        },
                        {
                            caption: "",
                            columns: [{
                                caption: "",
                                columns: [{ dataField: 'notes', caption: thisConstant.Notes }]
                            }]
                        }
                    ]

                    // columns

                };
            },

            // 获取编辑发票数据
            loadInvoiceData: function() {

                var model = {
                    queryParam: $scope.searchEntity,
                    pagingParam: $scope.pagingOptions
                };


                var numberList = ['discounts', 'gdRevenue', 'gdvat', 'gdCost','subsidy'];
                var parseViewModel = function(r) {

                    numberList.forEach(function(column) {
                        if (r[column]) {

                            var value = r[column];
                            try {
                                if (value.indexOf('(') > -1 && value.indexOf(')') > -1) {
                                    value = value.replace(/[^\d.]/g, '');
                                    value = '(' + parseFloat(value).formatAmount() + ')';
                                } else if (value === '-') {

                                }  else if (value === '#N/A'){
                                    
                                }
                                else {
                                    value = value.replace(/[^\d.]/g, '');
                                    value = parseFloat(value).formatAmount();
                                }

                                r[column] = value;
                            } catch (e) {
                                $log.debug(e +'data:'  + value);
                            }
                        }
                    });
                    return r;
                };


                vatOutputInvoiceManageService.getImportShipmentList(model).success(function(data) {

                    if (data.list && data.list.length > 0) {
                        data.list.forEach(function(r) {
                            r = parseViewModel(r);
                        });
                    }
                    $scope.invoiceGridDataSource = data.list;
                    $scope.pagingOptions.totalItems = data.pageInfo.totalCount;

                    $timeout(function() {
                        $scope.setInvoiceGridHeight();
                    }, 50);
                });
            },

            // 转换时间格式
            parseData: function(result) {
                if (result && result.length > 0) {
                    result.forEach(function(row) {

                        if (row.lastModifiedBy) {
                            var d = window.PWC.parseFromLocalTimeString(row.lastModifiedBy);
                            row.lastModifiedBy = d.formatDateTime('yyyy-MM-dd');
                        }

                        var driverLicenseOrgCode = '';
                        if (row.driverLicense) {
                            driverLicenseOrgCode += row.driverLicense;
                        }

                        if (row.organizationCode) {
                            driverLicenseOrgCode += '/' + row.organizationCode;
                        }

                        row.driverLicenseOrgCode = driverLicenseOrgCode;

                        if (row.operateStatus === 0) {
                            row.status = "normal";
                        } else if (row.operateStatus === 1) {
                            row.status = "issued";
                        }
                    });
                }

                return result;
            },

            // 获取用户所属机构
            loadOwnerOrg: function() {
                userService.getUserOwnerOrganization().success(function(data) {
                    mainModule.OwnerOrgModel = data;
                });
            },
        };


        (function initialize() {
            $log.debug('importShipmentListController.ctor()...');
            mainModule.main();

        })();
    }
]);