//
// app module for online DPMS
//
var app = angular.module('app', ['ui.tree', 'ui.bootstrap', 'ui.bootstrap.tpls', 'ui.event', 'pascalprecht.translate', 'ngMessages', 'ui.utils',
    'ngRoute', 'ngCookies', 'ngSanitize', 'ct.ui.router.extras', 'chieffancypants.loadingBar', 'ngDraggable', 'ngFileUpload', 'LocalStorageModule', 'exceptionless',
    'app.config', 'app.common', 'app.webservices', 'app.framework', 'app.vat', 'app.invoice', 'app.cit', "app.dataImport", 'app.assetsManage', 'app.taxDocumentManage',
    'app.taxTaskManage', 'pasvaz.bindonce', 'app.vatDashboard', 'app.vatDongfeng', 'vs-repeat', 'ivh.treeview', 'angular-cache', 'app.cache', 'angularBootstrapNavTree',
    'ngAnimate', 'cgNotify', 'dx', 'ngNumeraljs', 'app.noPermissionPage'])
    //run blocks are executed after the injector is created and are the first
    //methods that are executed in any Angular app.
    .run(['$log', '$http', 'CacheFactory', 'userService', 'loginContext', '$rootScope', 'vatSessionService', '$q', function ($log, $http, CacheFactory, userService, loginContext, $rootScope, vatSessionService, $q) {
        $log.debug('app.run()...');

        $rootScope.vatPermission = constant.vatPermission;

        // 控制用户机构维度上的权限
        $rootScope.checkUserOrganizationPermissionList = function (permissionCodeList) {
            var deferred = $q.defer();
            var promise = deferred.promise;
            var model = {};

            var orgID = vatSessionService.project.organizationID;

            userService.getUserPermissionNew(loginContext.userName, function (data) {
                permissionCodeList.forEach(function (permissionCode) {
                    var ret = window.PWC.isHaveOrganizationPermission(orgID, permissionCode, data);
                    model[permissionCode] = ret;
                });

                deferred.resolve(model);
            });

            var successFunction = function (fn) {
                promise.then(function (model) {
                    fn(model);
                });
                return this;
            }

            return {
                success: successFunction,
            };
        };


        $rootScope.checkUserOrganizationPermissionList([]);

        $rootScope.checkUserPermissionList = function (permissionCodeList) {
            var deferred = $q.defer();
            var promise = deferred.promise;
            var model = {};
            // 这种方式 重新登录，并发去后台 取数据 会出现问题
            //userService.getUserPermission(loginContext.userName).success(function (data) {
            //    permissionCodeList.forEach(function (permissionCode) {
            //        var ret = window.PWC.isHavePermission(permissionCode, data);
            //        model[permissionCode] = ret;
            //    });

            //    deferred.resolve(model);
            //});

            userService.getUserPermissionNew(loginContext.userName, function (data) {
                permissionCodeList.forEach(function (permissionCode) {
                    var ret = window.PWC.isHavePermission(permissionCode, data);
                    model[permissionCode] = ret;
                });

                deferred.resolve(model);
            });

            var successFunction = function (fn) {
                promise.then(function (model) {
                    fn(model);
                });
                return this;
            }

            return {
                success: successFunction,
            };
        };
        $http.defaults.headers.common['X-XSRF-Token'] =
            angular.element('input[name="__RequestVerificationToken"]').attr('value');


    }])
    // We always place constant at the beginning of all configuration blocks.
    .constant('application', {
        // the current logged on user
        currentUser: {},
        // convert date to display time based on current user
        toDisplayTimeString: function (dateTime) {
            if (!_.isDate(dateTime)) {
                throw new TypeError('"dateTime" should be "Date" type!');
            }
            return dateTime.toString('h:mmtt').toLowerCase();
        },
        // define angular events for $broadcast/$emit and $on
        events: {
            beforeUnload: 'event:beforeUnload',
            navigateToTab: 'event:navigateToTab',
            showNotificationBar: 'event:showNotificationBar'
        }
    })
    // only providers and constants should be injected in config block
    .config(['$logProvider', '$translateProvider', '$translatePartialLoaderProvider', 'region', '$compileProvider',
        function ($logProvider, $translateProvider, $translatePartialLoaderProvider, region, $compileProvider) {
            'use strict';

            // to disable various debug runtime information in the compiler to DOM elements.
            $compileProvider.debugInfoEnabled(false);
            // enable output $log.debug by default
            $logProvider.debugEnabled(true);

            // angular-translate configuration
            var configurateTranslation = function () {

                $translateProvider.useLoader('$translatePartialLoader', {
                    // the translation table are organized by language and module under folder 'app/i18n/'
                    urlTemplate: '/app-resources/i18n/{lang}/{part}.json'
                });

                $translateProvider
                    .preferredLanguage(region)
                    .fallbackLanguage(region);

                //https://github.com/Contactis/translations-manager/issues/7
                //https://angular-translate.github.io/docs/#/guide/19_security
                $translateProvider.useSanitizeValueStrategy('escape');
            };

            configurateTranslation();
        }
    ])
    // IE10 fires input event when a placeholder is defined so that form element is in dirty instead of pristine state 
    // refer to: https://github.com/angular/angular.js/issues/2614
    .config(['$provide', function ($provide) {
        $provide.decorator('$sniffer', ['$delegate', function ($sniffer) {
            var msieVersion = parseInt((/msie (\d+)/.exec(angular.lowercase(navigator.userAgent)) || [])[1], 10);
            var hasEvent = $sniffer.hasEvent;
            $sniffer.hasEvent = function (event) {
                if (event === 'input' && msieVersion === 10) {
                    return false;
                }
                hasEvent.call(this, event);
            };
            return $sniffer;
        }]);
    }])
    .config(['$ExceptionlessClient', 'exceptionlessServerURL', function ($ExceptionlessClient, exceptionlessServerURL) {
        if (!PWC.isNullOrEmpty(exceptionlessServerURL)) {
            $ExceptionlessClient.config.apiKey = 'HK0XK49LbufV6E4q8HLW7CGncwSBJvdBrJQwUnzw';
            $ExceptionlessClient.config.serverUrl = exceptionlessServerURL;
            $ExceptionlessClient.config.setUserIdentity('0', 'Anonymous');
            $ExceptionlessClient.config.useSessions();
            $ExceptionlessClient.config.defaultTags.push('JavaScript', 'Angular');
        }
    }])
    .config(['CacheFactoryProvider', function (CacheFactoryProvider) {
        var options = {
            storageMode: 'localStorage', // This cache will use `localStorage`.
            storagePrefix: 'atms.'
        };
        if (!window.localStorage) {
            options.storageImpl = localStoragePolyfill;
        }
        angular.extend(CacheFactoryProvider.defaults, options);
    }])
    .config(['ivhTreeviewOptionsProvider', function (ivhTreeviewOptionsProvider) {
        ivhTreeviewOptionsProvider.set({
            defaultSelectedState: false,
            validate: true,
            // Twisties can be images, custom html, or plain text
            twistieCollapsedTpl: '<i class="fa fa-plus" aria-hidden="true"></i>',
            twistieExpandedTpl: '<i class="fa fa-minus" aria-hidden="true"></i>',
            twistieLeafTpl: '<span style="color:white">&#9679;</span>'
        });
    }])
    // refer to: https://github.com/oitozero/ngSweetAlert
    .factory('SweetAlert', ['$rootScope', '$log', '$translate', function ($rootScope, $log, $translate) {
        $log.debug('SweetAlert.run()...');
        var swal = window.swal;
        //public methods
        var self = {

            swal: function (arg1, arg2, arg3) {
                $rootScope.$evalAsync(function () {
                    if (typeof (arg2) === 'function') {
                        swal(arg1, function (isConfirm) {
                            $rootScope.$evalAsync(function () {
                                arg2(isConfirm);
                            });
                        }, arg3);
                    } else {
                        swal(arg1, arg2, arg3);
                    }
                });
            },
            success: function (title, message) {
                $rootScope.$evalAsync(function () {
                    //swal(title, message, 'success');

                    swal({
                        title: title,
                        text: message,
                        type: "success",
                        confirmButtonText: $translate.instant('Confirm'),
                    });
                });
            },
            error: function (title, message) {
                $rootScope.$evalAsync(function () {
                    //swal(title, message, 'error');
                    swal({
                        title: title,
                        text: message,
                        type: "error",
                        confirmButtonText: $translate.instant('Confirm'),
                    });

                });
            },
            warning: function (title, message) {
                $rootScope.$evalAsync(function () {
                    //swal(title, message, 'warning');
                    swal({
                        title: title,
                        text: message,
                        type: "warning",
                        confirmButtonText: $translate.instant('Confirm'),
                    });
                });
            },
            info: function (title, message) {
                $rootScope.$evalAsync(function () {
                    //swal(title, message, 'info');
                    swal({
                        title: title,
                        text: message,
                        type: "info",
                        confirmButtonText: $translate.instant('Confirm'),
                    });
                });
            }
        };

        return self;
    }])
    //turn off angular-loading-bar spinner
    //refer to https://github.com/chieffancypants/angular-loading-bar
    .config(['cfpLoadingBarProvider', function (cfpLoadingBarProvider) {
        cfpLoadingBarProvider.includeSpinner = false;
    }])
    //initialize localStorage
    //refer to https://github.com/grevory/angular-local-storage#get-started
    .config(['localStorageServiceProvider', function (localStorageServiceProvider) {
        localStorageServiceProvider.setPrefix('pwcdashboard').setStorageType('sessionStorage');
    }])
    // Provide the localization function for application and support async load translation table by parts on demand.
    // Note: When trying to adding new translation resource into .json file, please check if the same KEY is existing 
    // in .json files under i18n folder. Because if two parts have the same property, the property value will be overwrited 
    // by the loaded last part.
    // for example,
    // We load app.json first and then load patient.json. "app.json" file contains a property {"Text" : "Test"} 
    // and "patient.json" file contains property {"Text" : "Overwrite Test"} the "Text" on view will be translated to 
    // be "Overwrite Test".
    .factory('appTranslation', ['$log', '$translatePartialLoader', '$translate',
        function ($log, $translatePartialLoader, $translate) {
            'use strict';
            $log.debug('appTranslation.ctor()...');

            var translation = {
                // part names for modules
                // AppPart is for the translation of application level, not for a module for a business logic.
                appPart: 'app',
                vat: 'vat',
                cit: 'cit',
                noPermissionPage: 'noPermissionPage',
                /// <summary>
                /// async load translation tables into application for specified part names that required for the view.
                /// </summary>
                /// <param name="partNames">part names of array type</param>
                load: function (partNames) {
                    if (!angular.isArray(partNames)) {
                        throw new TypeError('"partNames" should be an array!');
                    }

                    partNames.forEach(function (name) {
                        $translatePartialLoader.addPart(name);
                    });

                    $translate.refresh();
                },
                loadAll: function () {
                    _.map(_.values(translation), function (part) {
                        if (_.isString(part)) {
                            $translatePartialLoader.addPart(part);
                        }
                    });
                    $translate.refresh();
                }
            };
            return translation;
        }
    ])
    .controller('AppController', ['$scope', '$log', '$location', '$translate', '$translatePartialLoader',
        '$window', 'appRoute', 'application', 'appTranslation', '$timeout', '$uibModal', 'loginContext', '$ExceptionlessClient', 'signalRSvc',
        'exceptionlessServerURL', function ($scope, $log, $location, $translate, $translatePartialLoader, $window, appRoute,
            application, appTranslation, $timeout, $uibModal, loginContext, $ExceptionlessClient, signalRSvc, exceptionlessServerURL) {
            'use strict';
            $log.debug('AppController.ctor()...');

            $scope.localName = loginContext.localName;

            if (!PWC.isNullOrEmpty(exceptionlessServerURL)) {
                $ExceptionlessClient.config.setUserIdentity(loginContext.userId, loginContext.localName);
                $ExceptionlessClient.config.useSessions();
            }

            $scope.$on('$locationChangeSuccess', function () {
                $scope.actualLocation = $location.path();
            });

            $scope.$watch(function () { return $location.path() }, function (newLocation) {
                if ($scope.actualLocation === newLocation) {
                    // back or forward
                    $log.debug('Go back or go forward to view browser history. Url: ' + newLocation);
                }
            });

            $scope.$on('$stateChangeSuccess',
                function () {
                    $log.debug('$stateChangeSuccess: ');
                });


            // publish unbeforeunload event to child scopes
            $scope.onbeforeunload = function () {
                $scope.$broadcast(application.events.beforeUnload);
            };

            $scope.main = function () {
                $location.path('/main');
                $location.url('main');
            };

            $scope.createDemo = function () {
                $location.path('/accountDemo');
                $location.url($location.path());
            };

            appTranslation.loadAll();

            signalRSvc.initialize();
        }
    ]);
(function () {
    'use strict';

    angular.module('app.config', [])
        // region of current application
        .constant('region', 'zh-CN')
        //.constant('region', 'en-US')
        // version of current application
        .constant('version', '1.0.0.0')
        // Exceptionless Server URL
        .constant('exceptionlessServerURL', '')
        .run([
            'region', function (region) {
                $.when(
                    $.getJSON("/Scripts/cldr/main/zh/numbers.json"),
                    $.getJSON("/Scripts/cldr/main/zh/ca-gregorian.json"),
                    $.getJSON("/Scripts/cldr/supplemental/likelySubtags.json")
                ).then(function () {
                    return [].slice.apply(arguments, [0]).map(function (result) {
                        return result[0];
                    });
                }).then(Globalize.load).then(function () {
                     Globalize.locale("zh-CN");
                   // Globalize.locale("en-US");
                    DevExpress.localization.locale(region);
                    //DevExpress.localization.locale(navigator.language || navigator.browserLanguage);
                });
            }
        ]);
}());
//Common Bind Module Method
var bindModule = function (thisModule, controllerProvider, compileProvider, filterProvider, provide) {

    thisModule.controller = controllerProvider.register;
    thisModule.directive = compileProvider.directive;
    thisModule.filter = filterProvider.register;
    thisModule.factory = provide.factory;
    thisModule.service = provide.service;

    return thisModule;
};

// register common module for shared functionalities for all other feature modules
var commonModule = angular.module('app.common', ['pascalprecht.translate', 'ngAnimate', 'ui.grid', 'ui.grid.selection', 'ui.grid.selection', 'ui.grid.treeView', 'ui.grid.resizeColumns', 'ui.grid.grouping', 'ui.grid.exporter', 'app.config', 'ui.grid.edit'])
	.run(['$log', function ($log) {
	    $log.debug('app.common.run()...');
	}])
	// define the script files for lazy loading and method to create depencency map for resovle config in route
	.provider('scriptDependency', ['version', function (version) {
	    'use strict';
	    // Here must add version parameter ?v=***, 
	    // to solve the problem that the bundle is not refreshed when its content changed.

	    this.commdataimport = [{
	        url: '/bundles/commdataimport.js?v=' + version,
	        type: 'text/javascript'
	    }, {
	        url: '/bundles/commdataimport.css?v=' + version,
	        type: 'text/css'
	    }];

	    this.vat = [{
	        url: '/bundles/vat.js?v=' + version,
	        type: 'text/javascript'
	    }, {
	        url: '/bundles/vat.css?v=' + version,
	        type: 'text/css'
	    }, {
	        url: '/bundles/vat.less?v=' + version,
	        type: 'text/css'
	    }];

	    this.cit = [{
	        url: '/bundles/cit.js?v=' + version,
	        type: 'text/javascript'
	    }, {
	        url: '/bundles/cit.css?v=' + version,
	        type: 'text/css'
	    }, {
	        url: '/bundles/cit.less?v=' + version,
	        type: 'text/css'
	    }];

        this.dataImport = [
            {
                url: '/bundles/globalDataImport.js?v=' + version,
                type: 'text/javascript'
            },
            {
                url: '/bundles/globalDataImport.css?v=' + version,
                type: 'text/css'
            },
            {
                url: '/bundles/globalDataImport.less?v=' + version,
                type: 'text/css'
            }
        ];

	    this.noPermissionPage = [{
	        url: '/bundles/noPermissionPage.js?v=' + version,
	        type: 'text/javascript'
	    }, {
	        url: '/bundles/noPermissionPage.less?v=' + version,
	        type: 'text/css'
	    }];

	    this.assetsManage = [{
	        url: '/bundles/assetsManage.js?v=' + version,
	        type: 'text/javascript'
	    }, {
	        url: '/bundles/assetsManage.css?v=' + version,
	        type: 'text/css'
	    }, {
	        url: '/bundles/assetsManage.less?v=' + version,
	        type: 'text/css'
	    }];

        this.taxDocumentManage = [{
            url: '/bundles/taxDocumentManage.js?v=' + version,
            type: 'text/javascript'
        }, {
            url: '/bundles/taxDocumentManage.css?v=' + version,
            type: 'text/css'
        }, {
            url: '/bundles/taxDocumentManage.less?v=' + version,
            type: 'text/css'
        }];


        this.taxTaskManage = [{
            url: '/bundles/taxTaskManage.js?v=' + version,
            type: 'text/javascript'
        }, {
            url: '/bundles/taxTaskManage.less?v=' + version,
            type: 'text/css'
        }];


	    this.summary = [
		//    {
		//    url: '/bundles/summary.js?v=' + version,
		//    type: 'text/javascript'
		//},
		{
		    url: '/bundles/summary.css?v=' + version,
		    type: 'text/css'
		}];

	    this.createDependenciesMap = function (dependencies) {
	        if (!angular.isArray(dependencies)) {
	            throw new TypeError('"scriptUrls" should be an array type!');
	        }

	        var dependenciesMap = {
	            dependency: ['$q', '$rootScope',
					function ($q, $rootScope) {
					    var deferred = $q.defer();
					    PWC.Loader.load(dependencies, function (hasNewJsLoaded) {
					        // If has new JavaScript resource loaded, should call $apply, otherwise, 
					        // should not call it.
					        if (hasNewJsLoaded) {
					            $rootScope.$apply(function () {
					                deferred.resolve();
					            });
					        } else {
					            deferred.resolve();
					        }
					    });
					    return deferred.promise;
					}
	            ]
	        };

	        return dependenciesMap;
	    };

	    this.$get = function () {
	        return {
	            createDependenciesMap: createDependenciesMap,
	            infrastructure: infrastructure,
	            vat: vat,
	            cit: cit,
                dataImport: dataImport,
	            assetsManage: assetsManage,
                taxDocumentManage: taxDocumentManage,
                noPermissionPageModule: noPermissionPageModule,
                taxTaskManage: taxTaskManage
	            //subjectCorrespondingInfrastructure: subjectCorrespondingInfrastructure
	            //subjectCorresponding: subjectCorresponding,
	            //configurationInfrastructure: configurationInfrastructure,
	            //declarationFormConfiguration: declarationFormConfiguration
	        };
	    };
	}]);

// register services module
var webservices = angular.module('app.webservices', ['app.common'])
	.run(['$log', function ($log) {
	    $log.debug('app.webservices.run()...');
	}]);

// register framework module for application framework
var frameworkModule = angular.module('app.framework', ['app.webservices', 'app.common', 'app.vatDashboard', 'ui.dashboard'])
	.run(['$log', function ($log) {
	    $log.debug('app.framework.run()...');
	}])
	.config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', 'scriptDependencyProvider', '$stateProvider',
		function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, scriptDependencyProvider, $stateProvider) {
		    'use strict';
		    // this is required to add controller/directive/filter/service after angular bootstrap
		    bindModule(frameworkModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

		    $stateProvider.state({
		        name: 'overview',
		        url: '/overview',
		        views: {
		            '@': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
							function ($scope, $stateParams, appTranslation) {
							    appTranslation.load([appTranslation.appPart]);
							}],
		                template: '<app-overview servicetypeid="2"></app-overview>',
		            }
		        },
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'notification',
		        url: '/notification',
		        views: {
		            '@': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
							function ($scope, $stateParams, appTranslation) {
							    appTranslation.load([appTranslation.appPart]);
							}],
		                template: '<app-push-notification></app-push-notification>',
		            }
		        },
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'overviewVat',
		        url: '/overview/vat',
		        views: {
		            '@': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
							function ($scope, $stateParams, appTranslation) {
							    appTranslation.load([appTranslation.appPart]);
							}],
		                template: '<app-overview servicetypeid="2"></app-overview>',
		            }
		        },
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'overviewAssetsManage',
		        url: '/overview/assetsManage',
		        views: {
		            '@': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
							function ($scope, $stateParams, appTranslation) {
							    appTranslation.load([appTranslation.appPart]);
							}],
		                template: '<app-overview servicetypeid="12"></app-overview>',
		            }
		        },
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'overviewCit',
		        url: '/overview/cit',
		        views: {
		            '@': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
							function ($scope, $stateParams, appTranslation) {
							    appTranslation.load([appTranslation.appPart]);
							}],
		                template: '<app-overview servicetypeid="6"></app-overview>',
		            }
		        },
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
                name: 'taxDocumentManage',
                url: '/documentManage/taxDocumentManage/:id',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<document-manage></document-manage>',
                    }
                },
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
		        name: 'summary',
		        url: '/summary',
		        views: {
		            '@': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
							function ($scope, $stateParams, appTranslation) {
							    appTranslation.load([appTranslation.appPart]);
							}],
		                template: '<app-summary></app-summary>',
		            }
		        },
                resolve: scriptDependencyProvider.createDependenciesMap(_.flatten([scriptDependencyProvider.summary, scriptDependencyProvider.vat])),
		        deepStateRedirect: true,
		        sticky: true
		    });
		}
	]);

// register cache module for application framework
var cacheModule = angular.module('app.cache', ['app.common'])
	.run(['$log', 'cacheService', function ($log, cacheService) {
	    $log.debug('app.cache.run()...');

	}]);

var vatModule = angular.module('app.vat', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngAnimate', 'ngSanitize', 'ui.bootstrap', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning',])
	.run(['$log', function ($log) {
	    $log.debug('app.vat.run()...');
	}])
	.config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
		function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
		    'use strict';
		    // this is required to add controller/directive/filter/service after angular bootstrap
		    bindModule(vatModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

		    uiSelectConfig.theme = 'select2';

		    $stateProvider.state({
		        name: 'vat',
		        url: '/vat',
		        cache: false,
		        views: {
		            '@': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
							function ($scope, $stateParams, appTranslation) {
							    appTranslation.load([appTranslation.vat]);
							}],
		                template: '<vat-layout></vat-layout>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData',
		        url: '/importData',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-import-layout></vat-import-layout>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });


		    $stateProvider.state({
		        name: 'vat.importData.balanceSheet',
		        url: '/balanceSheet',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-import-trial-balance></vat-import-trial-balance>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData.journalEntry',
		        url: '/journalEntry',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-import-journal-entry></vat-import-journal-entry>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData.erpData',
		        url: '/erpData',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-import-erp></vat-import-erp>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData.outputInvoice',
		        url: '/outputInvoice',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-import-output-invoice></vat-import-output-invoice>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData.inputInvoice',
		        url: '/inputInvoice',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-import-income-invoice></vat-import-income-invoice>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData.customInvoice',
		        url: '/customInvoice',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-import-custom-invoice></vat-import-custom-invoice>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData.voucherMapping',
		        url: '/voucherMapping',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-voucher-mapping></vat-voucher-mapping>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData.invoiceMapping',
		        url: '/invoiceMapping',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-invoice-mapping></vat-invoice-mapping>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.importData.auditAdjust',
		        url: '/auditAdjust',
		        views: {
		            '@vat.importData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-audit-adjust></vat-audit-adjust>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.previewData',
		        url: '/previewData',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
													function ($scope, $stateParams, appTranslation) {
													    appTranslation.load([appTranslation.vat]);
													}],
		                template: '<vat-import-layout></vat-import-layout>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.previewData.balanceSheet',
		        url: '/balanceSheet',
		        views: {
		            '@vat.previewData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-preview-balance-sheet></vat-preview-balance-sheet>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.previewData.accountVoucher',
		        url: '/accountVoucher',
		        views: {
		            '@vat.previewData': {
		                controller: [
							'$scope', '$stateParams', 'appTranslation',
							function ($scope, $stateParams, appTranslation) {
							    appTranslation.load([appTranslation.vat]);
							}
		                ],
		                template: '<vat-preview-account-voucher></vat-preview-account-voucher>'
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.previewData.outputInvoice',
		        url: '/outputInvoice',
		        views: {
		            '@vat.previewData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-preview-output-invoice></vat-preview-output-invoice>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.previewData.inputInvoice',
		        url: '/inputInvoice',
		        views: {
		            '@vat.previewData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-preview-input-invoice></vat-preview-input-invoice>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.previewData.customInvoice',
		        url: '/customInvoice',
		        views: {
		            '@vat.previewData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-preview-custom-invoice-sheet></vat-preview-custom-invoice-sheet>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.reductionData',
		        url: '/reductionData',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-import-layout></vat-import-layout>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.reductionData.accountMapping',
		        url: '/accountMapping',
		        views: {
		            '@vat.reductionData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-account-mapping></vat-account-mapping>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.reductionData.goodsMapping',
		        url: '/goodsMapping',
		        views: {
		            '@vat.reductionData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-goods-mapping></vat-goods-mapping>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.reductionData.caculateData',
		        url: '/caculateData',
		        views: {
		            '@vat.reductionData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-caculate-data></vat-caculate-data>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.reductionData.inputInvoice',
		        url: '/inputInvoice',
		        views: {
		            '@vat.reductionData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<p>inputInvoice</p>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.reductionData.unbilledInvoice',
		        url: '/unbilledInvoice',
		        views: {
		            '@vat.reductionData': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-unbilled-invoice></vat-unbilled-invoice>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });


		    $stateProvider.state({
		        name: 'vat.generateReport',
		        url: '/generateReport',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    $scope.isFromTopMenu = true;
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-report-layout></vat-report-layout>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: false,
		        sticky: false
		    });

		    $stateProvider.state({
		        name: 'vat.generateReport.reportView',
		        url: '/reportView?{id:string}&{templateid:string}&{name:string}&{templatecode:string}',
		        views: {
		            '@vat.generateReport': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    $scope.reportId = $stateParams.id;
										    $scope.templateId = $stateParams.templateid;
										    $scope.templateName = $stateParams.name;
										    $scope.templateCode = $stateParams.templatecode;
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-report-view report-id="reportId" template-id="templateId" template-name="templateName" template-code="templateCode" service-type="\'2\'"></vat-report-view>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: false,
		        sticky: false
		    });


		    /**********a beautiful separated line for analyzeReport*********/
		    $stateProvider.state({
		        name: 'vat.analyzeLayout',
		        url: '/analyzeLayout',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-analysis-menu></vat-analysis-menu>',

		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.analyzeLayout.analyzeReport',
		        url: '/analyzeReport',
		        views: {
		            '@vat.analyzeLayout': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-model-analysis></vat-model-analysis>',

		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.analyzeLayout.vatTaxDifference',
		        url: '/vatTaxDifference',
		        views: {
		            '@vat.analyzeLayout': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-tax-difference></vat-tax-difference>',

		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });
		    $stateProvider.state({
		        name: 'vat.analyzeReport',
		        url: '/analyzeReport',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-model-analysis></vat-model-analysis>',

		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.analyzeReport.dataSummary',
		        url: '/dataSummary',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-data-summary></vat-data-summary>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.analyzeReport.vatAnalysis',
		        url: '/vatAnalysis',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-vat-analysis></vat-vat-analysis>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.analyzeReport.financeTargetAnalysis',
		        url: '/financeTargetAnalysis',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-finance-target-analysis></vat-finance-target-analysis>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    $stateProvider.state({
		        name: 'vat.analyzeReport.vatModelAnalysis',
		        url: '/vatModelAnalysis',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<vat-model-analysis></vat-model-analysis>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });

		    /**********the end of a beautiful separated line for analyzeReport*********/

		    $stateProvider.state({
		        name: 'vat.devGrid',
		        url: '/devGrid',
		        views: {
		            '@vat': {
		                controller: ['$scope', '$stateParams', 'appTranslation',
										function ($scope, $stateParams, appTranslation) {
										    appTranslation.load([appTranslation.vat]);
										}],
		                template: '<dev-grid></dev-grid>',
		            }
		        },
		        resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
		        deepStateRedirect: true,
		        sticky: true
		    });
		}]);
//Common Bind Module Method
var bindModule = function (thisModule, controllerProvider, compileProvider, filterProvider, provide) {

    thisModule.controller = controllerProvider.register;
    thisModule.directive = compileProvider.directive;
    thisModule.filter = filterProvider.register;
    thisModule.factory = provide.factory;
    thisModule.service = provide.service;

    return thisModule;
};

var invoiceModule = angular.module('app.invoice', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngSanitize', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning'])
    .run(['$log', function ($log) {
        $log.debug('app.invoice.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(invoiceModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            uiSelectConfig.theme = 'select2';

            //发票管理框架 
            $stateProvider.state({
                name: 'invoiceManagement',
                url: '/invoiceManagement',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation', '$state',
                                        function ($scope, $stateParams, appTranslation, $state) {
                                            appTranslation.load([appTranslation.vat]);
                                            $scope.state = $state;
                                        }],
                        template: '<invoice-manage-infrastructure state="state"></invoice-manage-infrastructure>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            //发票管理主要界面
            $stateProvider.state('invoiceManagement.main', {
                url: "/main",
                sticky: true,
                dsr: true,
                views: {
                    'main-view@invoiceManagement': {
                        template: '<invoice-manage-main></invoice-manage-main>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理上传index
            $stateProvider.state('invoiceManagement.uploadIndex', {
                url: "/main/uploadIndex",
                sticky: true,
                dsr: true,
                views: {
                    'uploadIndex-view@invoiceManagement': {

                        template: '<invoice-manage-main-upload-index></invoice-manage-main-upload-index>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });
            //发票管理上传paper invoice
            $stateProvider.state('invoiceManagement.uploadPaperInvoice', {
                url: "/main/uploadPaperInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'uploadPaperInvoice-view@invoiceManagement': {

                        template: '<invoice-manage-main-upload-paper-invoice></invoice-manage-main-upload-paper-invoice>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理上传electonic invoice
            $stateProvider.state('invoiceManagement.uploadElecInvoice', {
                url: "/main/uploadElecInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'uploadElecInvoice-view@invoiceManagement': {

                        template: '<invoice-manage-main-upload-electronic-invoice></invoice-manage-main-upload-electronic-invoice>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理上传scan invoice
            $stateProvider.state('invoiceManagement.uploadScanInvoice', {
                url: "/main/uploadScanInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'uploadScanInvoice-view@invoiceManagement': {

                        template: '<invoice-manage-main-upload-scan-invoice></invoice-manage-main-upload-scan-invoice>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //发票管理未收票认证
            $stateProvider.state('invoiceManagement.noInvoiceVerify', {
                url: "/noInvoiceVerify",
                sticky: true,
                dsr: true,
                views: {
                    'noInvoiceVerify-view@invoiceManagement': {
                        template: '<invoice-manage-verify></invoice-manage-verify>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //开票管理
            $stateProvider.state('invoiceManagement.billingManage',
            {
                url: "/billingManage",
                sticky: true,
                dsr: true,
                views: {
                    'billingManage-view@invoiceManagement': {
                        template: '<iframe id="dfiframe" src="/app/df/code/pages/PreViewFapiao.html" scrolling="auto" height="100%" style="border:none" width="100%"></iframe>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });



            //发票查询
            $stateProvider.state('invoiceManagement.invoiceQuery',
            {
                url: "/invoiceQuery",
                sticky: true,
                dsr: true,
                views: {
                    'invoiceQuery-view@invoiceManagement': {
                        template: '<iframe id="dfiframe" src="/app/df/code/pages/FapiaoInqury.html" scrolling="auto" height="100%" style="border:none" width="100%"></iframe>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //发票管理待退票管理
            $stateProvider.state('invoiceManagement.pendingRefundInvoice', {
                url: "/pendingRefundInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'pendingRefundInvoice-view@invoiceManagement': {
                        template: '<invoice-manage-refund></invoice-manage-refund>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理关联管理
            $stateProvider.state('invoiceManagement.mapInvoice', {
                url: "/mapInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'mapInvoice-view@invoiceManagement': {
                        template: '<invoice-manage-map></invoice-manage-map>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //发票管理框架 
            $stateProvider.state({
                name: 'outputInvoiceManage',
                url: '/outputInvoiceManage',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation', '$state',
                                        function ($scope, $stateParams, appTranslation, $state) {
                                            appTranslation.load([appTranslation.vat]);
                                            $scope.state = $state;
                                        }],
                        template: '<output-invoice-infrastructure state="state"></output-invoice-infrastructure>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            //发票管理主要界面
            $stateProvider.state('outputInvoiceManage.imported', {
                url: "/imported",
                sticky: false,
                dsr: true,
                views: {
                    'outputInvoiceImported-view@outputInvoiceManage': {
                        template: '<output-invoice-imported></output-invoice-imported>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理编辑界面
            $stateProvider.state('outputInvoiceManage.outputInvoiceEdit', {
                url: "/outputInvoiceEdit",
                sticky: false,
                dsr: true,
                views: {
                    'outputInvoiceEdit-view@outputInvoiceManage': {
                        template: '<output-invoice-edit></output-invoice-edit>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理已打印BD-USER发票
            $stateProvider.state('outputInvoiceManage.issuedInvoiceBdView', {
                url: "/issuedInvoiceBdView",
                sticky: false,
                dsr: true,
                views: {
                    'issuedInvoiceBd-view@outputInvoiceManage': {
                        template: '<issued-invoice-bd-view></issued-invoice-bd-view>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.bdViewByVin', {
                url: "/bdViewByVin",
                sticky: false,
                dsr: true,
                views: {
                    'bdViewByVin-view@outputInvoiceManage': {
                        template: '<bd-view-by-vin></bd-view-by-vin>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //GD已开具发票特殊处理
            $stateProvider.state('outputInvoiceManage.issuedInvoiceSpecial', {
                url: "/issuedInvoiceSpecial",
                sticky: false,
                dsr: true,
                views: {
                    'issuedInvoiceSpecial-view@outputInvoiceManage': {
                        template: '<special-vat-invoice></special-vat-invoice>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });
            //GD已开具发票特殊处理
            $stateProvider.state('outputInvoiceManage.GDViewByVIN', {
                url: "/GDViewByVIN",
                sticky: false,
                dsr: true,
                views: {
                    'GDViewByVIN-view@outputInvoiceManage': {
                        template: '<gd-view-by-vin></gd-view-by-vin>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.outputReconciliationOverview', {
                url: "/outputReconciliationOverview",
                sticky: false,
                dsr: true,
                views: {
                    'output-reconciliation-overview@outputInvoiceManage': {
                        template: '<output-reconciliation-overview></output-reconciliation-overview>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.reconciliationRuleConfig', {
                url: "/reconciliationRuleConfig",
                sticky: false,
                dsr: true,
                views: {
                    'reconciliation-rule-config@outputInvoiceManage': {
                        template: '<reconciliation-rule-config></reconciliation-rule-config>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.outputReconciliationConfiguration', {
                url: "/outputReconciliationConfiguration",
                sticky: false,
                dsr: true,
                views: {
                    'output-reconciliation-configuration@outputInvoiceManage': {
                        template: '<output-reconciliation-configuration></output-reconciliation-configuration>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.taxControlDisk', {
                url: "/taxControlDisk",
                sticky: false,
                dsr: true,
                views: {
                    'tax-control-disk@outputInvoiceManage': {
                        template: '<tax-control-disk></tax-control-disk>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            ////GD已开具发票详情
            //$stateProvider.state('outputInvoiceManage.issuedInvoiceSpecialDetail', {
            //    url: "/issuedInvoiceSpecialDetail/{gdid}/{parentPage}",
            //    sticky: true,
            //    dsr: true,
            //    //params: {
            //    //    GDID: null,
            //    //    parentPage:null
            //    //},              
            //    views: {
            //        'issuedInvoiceSpecialDetail-view@outputInvoiceManage': {
            //            controller: ['$scope', '$stateParams', 'appTranslation',
            //                          function ($scope, $stateParams, appTranslation) {
            //                              console.log($stateParams.gdid);
            //                              $scope.gdInvoiceID = $stateParams.gdid;
            //                              $scope.parentPage = $stateParams.parentPage;
            //                          }],
            //            template: '<invoice-detail-vehicle></invoice-detail-vehicle>'

            //        }
            //    },
            //    resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            //});
            //发票管理已打印GD-BD发票
            $stateProvider.state('outputInvoiceManage.issuedInvoiceGdView', {
                url: "/issuedInvoiceGdView",
                sticky: false,
                dsr: true,
                views: {
                    'issuedInvoiceGd-view@outputInvoiceManage': {
                        template: '<issued-invoice-gd-view></issued-invoice-gd-view>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });



            //发票管理编辑界面
            $stateProvider.state('outputInvoiceManage.gdInvoicePrint', {
                url: "/gdInvoicePrint",
                sticky: false,
                params: { "invoiceList": null },
                dsr: true,
                views: {
                    'gdInvoicePrint-view@outputInvoiceManage': {
                        template: '<gd-invoice-print></gd-invoice-print>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //发票管理编辑界面
            $stateProvider.state('outputInvoiceManage.importShipmentList', {
                url: "/importShipmentList",
                sticky: false,
                dsr: true,
                views: {
                    'importShipment-view@outputInvoiceManage': {
                        template: '<import-shipment-list></import-shipment-list>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

        }]);

var citModule = angular.module('app.cit', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngAnimate', 'ngSanitize', 'ui.bootstrap', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning'])
    .run(['$log', function ($log) {
        $log.debug('app.cit.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(citModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            uiSelectConfig.theme = 'select2';

            $urlRouterProvider.when('/cit/previewData', '/cit/previewData/balanceSheet');

            $urlRouterProvider.when('/cit/reductionData', 'cit/reductionData/caculateData');

            $urlRouterProvider.when('/cit/analyzeReport', 'cit/analyzeReport/citModelAnalysis');

            $stateProvider.state({
                name: 'cit',
                url: '/cit',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.cit]);
                            }],
                        template: '<cit-layout></cit-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData',
                url: '/importData',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-layout></cit-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });


            $stateProvider.state({
                name: 'cit.importData.balanceSheet',
                url: '/balanceSheet',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-trial-balance></cit-import-trial-balance>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.assetList',
                url: '/assetList',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-asset></cit-import-asset>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.documentList',
                url: '/documentList?{docId:string}&{docTemplateId:string}&{docName:string}&{treeTypeCode:string}&{selectedReportCode:string} \
                      &{hasDocument}&{docNum}',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.docId = $stateParams.docId;
                                            $scope.docTemplateId = $stateParams.docTemplateId;
                                            $scope.docName = $stateParams.docName;
                                            $scope.treeTypeCode = $stateParams.treeTypeCode;
                                            $scope.selectedReportCode = $stateParams.selectedReportCode;
                                            $scope.hasDocument = $stateParams.hasDocument;
                                            $scope.docNum = $stateParams.docNum;
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-document-list doc-id="docId" doc-template-id="docTemplateId" doc-name="docName" \
                                   tree-type-code="treeTypeCode" selected-report-code="selectedReportCode" has-document="hasDocument" doc-num="docNum"> \
                                   </cit-import-document-list>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'cit.importData.documentList.reportView',
                url: '/reportView?{id:string}&{templateid:string}&{name:string}',
                views: {
                    '@cit.importData.documentList': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.reportId = $stateParams.id;
                                            $scope.templateId = $stateParams.templateid;
                                            $scope.templateName = $stateParams.name;
                                            $scope.templateCode = $stateParams.templatecode;
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<vat-report-view report-id="reportId" template-id="templateId" template-name="templateName" template-code="templateCode" service-type="\'6\'" is-document-list="true"></vat-report-view>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'cit.importData.journalEntry',
                url: '/journalEntry',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-journal-entry></cit-import-journal-entry>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.erpData',
                url: '/erpData',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-erp></cit-import-erp>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.outputInvoice',
                url: '/outputInvoice',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-output-invoice></cit-import-output-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.inputInvoice',
                url: '/inputInvoice',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-income-invoice></cit-import-income-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.customInvoice',
                url: '/customInvoice',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-custom-invoice></cit-import-custom-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.voucherMapping',
                url: '/voucherMapping',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-voucher-mapping></cit-voucher-mapping>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.invoiceMapping',
                url: '/invoiceMapping',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-invoice-mapping></cit-invoice-mapping>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.auditAdjust',
                url: '/auditAdjust',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-audit-adjust></cit-audit-adjust>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData',
                url: '/previewData',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                                    function ($scope, $stateParams, appTranslation) {
                                                        appTranslation.load([appTranslation.cit]);
                                                    }],
                        template: '<cit-import-layout></cit-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.balanceSheet',
                url: '/balanceSheet',
                views: {
                    '@cit.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-preview-balance-sheet></cit-preview-balance-sheet>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.accountVoucher',
                url: '/accountVoucher',
                views: {
                    '@cit.previewData': {
                        controller: [
                            '$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.cit]);
                            }
                        ],
                        template: '<cit-preview-account-voucher></cit-preview-account-voucher>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.outputInvoice',
                url: '/outputInvoice',
                views: {
                    '@cit.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-preview-output-invoice></cit-preview-output-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.inputInvoice',
                url: '/inputInvoice',
                views: {
                    '@cit.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-preview-input-invoice></cit-preview-input-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.customInvoice',
                url: '/customInvoice',
                views: {
                    '@cit.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-preview-custom-invoice-sheet></cit-preview-custom-invoice-sheet>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData',
                url: '/reductionData',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-layout></cit-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.accountMapping',
                url: '/accountMapping',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-account-mapping></cit-account-mapping>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.goodsMapping',
                url: '/goodsMapping',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-goods-mapping></cit-goods-mapping>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.caculateData',
                url: '/caculateData',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-caculate-data></cit-caculate-data>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.inputInvoice',
                url: '/inputInvoice',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<p>inputInvoice</p>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.unbilledInvoice',
                url: '/unbilledInvoice',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-unbilled-invoice></cit-unbilled-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });


            $stateProvider.state({
                name: 'cit.generateReport',
                url: '/generateReport',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-report-layout></cit-report-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'cit.generateReport.reportView',
                url: '/reportView?{id:string}&{templateid:string}&{name:string}&{templatecode:string}',
                views: {
                    '@cit.generateReport': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.reportId = $stateParams.id;
                                            $scope.templateId = $stateParams.templateid;
                                            $scope.templateName = $stateParams.name;
                                            $scope.templateCode = $stateParams.templatecode;
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<vat-report-view report-id="reportId" template-id="templateId" template-name="templateName" template-code="templateCode" \
service-type="\'6\'" is-read-only="true"></vat-report-view>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'cit.analyzeReport',
                url: '/analyzeReport',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-layout></cit-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.analyzeReport.citModelAnalysis',
                url: '/citModelAnalysis',
                views: {
                    '@cit.analyzeReport': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-model-analysis></cit-model-analysis>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.analyzeReport.citIndexAnalysis',
                url: '/citIndexAnalysis',
                views: {
                    '@cit.analyzeReport': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<iframe id="dfiframe" src="/app/df/code/pages/citdashboard.html" scrolling="auto" height="100%" style="border:none" width="100%"></iframe>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.analyzeReport.dataSummary',
                url: '/dataSummary',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-data-summary></cit-data-summary>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            /**********the end of a beautiful separated line for analyzeReport*********/

            $stateProvider.state({
                name: 'cit.devGrid',
                url: '/devGrid',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<dev-grid></dev-grid>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });


        }]);
//Common Bind Module Method
var bindModule = function (thisModule, controllerProvider, compileProvider, filterProvider, provide) {

    thisModule.controller = controllerProvider.register;
    thisModule.directive = compileProvider.directive;
    thisModule.filter = filterProvider.register;
    thisModule.factory = provide.factory;
    thisModule.service = provide.service;

    return thisModule;
};

var assetsManageModule = angular.module('app.assetsManage', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngSanitize', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning'])
    .run(['$log', function ($log) {
        $log.debug('app.assetsManage.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(assetsManageModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            uiSelectConfig.theme = 'select2';

            $urlRouterProvider.when('/assetsManage', '/assetsManage/importData/productItem');

            $urlRouterProvider.when('/assetsManage/importData', '/assetsManage/importData/productItem');

            $urlRouterProvider.when('/assetsManage/previewData', '/assetsManage/previewData/productItem');

            $urlRouterProvider.when('/assetsManage/productMix', '/assetsManage/productMix/productItem');

            $stateProvider.state({
                name: 'assetsManage',
                url: '/assetsManage',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.cit]);
                            }],
                        template: '<assets-manage-layout></assets-manage-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'assetsManage.importData',
                url: '/importData',
                views: {
                    '@assetsManage': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<assets-manage-import-layout></assets-manage-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'assetsManage.importData.productItem',
                url: '/productItem',
                views: {
                    '@assetsManage.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<product-item-import></product-item-import>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'assetsManage.previewData',
                url: '/previewData',
                views: {
                    '@assetsManage': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                                    function ($scope, $stateParams, appTranslation) {
                                                        appTranslation.load([appTranslation.cit]);
                                                    }],
                        template: '<assets-manage-import-layout></assets-manage-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'assetsManage.previewData.productItem',
                url: '/productItem',
                views: {
                    '@assetsManage.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<product-item-preview></product-item-preview>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'assetsManage.generateReport',
                url: '/generateReport',
                views: {
                    '@assetsManage': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<assets-manage-report-layout></assets-manage-report-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'assetsManage.productMix',
                url: '/productMix',
                views: {
                    '@assetsManage': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                                    function ($scope, $stateParams, appTranslation) {
                                                        appTranslation.load([appTranslation.cit]);
                                                    }],
                        template: '<assets-manage-import-layout></assets-manage-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'assetsManage.productMix.productItem',
                url: '/productMix',
                views: {
                    '@assetsManage.productMix': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<product-item-mix></product-item-mix>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'assetsManage.generateReport.reportView',
                url: '/reportView?{id:string}&{templateid:string}&{name:string}',
                views: {
                    '@assetsManage.generateReport': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.reportId = $stateParams.id;
                                            $scope.templateId = $stateParams.templateid;
                                            $scope.templateName = $stateParams.name;
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<assets-manage-report-view report-id="reportId" template-id="templateId" template-name="templateName"></assets-manage-report-view>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'assetsManage.generateReport.reportViewCit',
                url: '/reportViewCit?{id:string}&{templateid:string}&{name:string}',
                views: {
                    '@assetsManage.generateReport': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.reportId = $stateParams.id;
                                            $scope.templateId = $stateParams.templateid;
                                            $scope.templateName = $stateParams.name;
                                            appTranslation.load([appTranslation.cit]);
                                        }]
                        //template: '<cit-report-view2 report-id="reportId" template-id="templateId" template-name="templateName"></cit-report-view2>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.assetsManage),
                deepStateRedirect: false,
                sticky: false
            });
        }]);

var vatDashboardModule = angular.module('app.vatDashboard', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngSanitize', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning'])
    .run(['$log', function ($log) {
        $log.debug('app.vatDashboard.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(vatDashboardModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            $stateProvider.state({
                name: 'summaryDashboard',
                url: '/summaryDashboard',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<summary-dashboard></summary-dashboard>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'summaryDashboardConfig',
                url: '/summaryDashboardConfig/{id}',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<summary-dashboard-config></summary-dashboard-config>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'reportAnalysisDashboardConfig',
                url: '/reportAnalysisDashboardConfig/{id}',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<report-analysis-dashboard-config></report-analysis-dashboard-config>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'organizationDashboard',
                url: '/organizationDashboard/{id}',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<organization-dashboard></organization-dashboard>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'analyzeOrganizeDashboard',
                url: '/analyzeOrganizeDashboard/{id}',
                views: {
                    '@vat.analyzeLayout': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<organization-dashboard></organization-dashboard>',

                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'analyzeOrganizeDashboardConfig',
                url: '/analyzeOrganizeDashboardConfig/{id}',
                views: {
                    '@vat.analyzeLayout': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<report-analysis-dashboard-config></report-analysis-dashboard-config>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

        }]);

var vatDongfengModule = angular.module('app.vatDongfeng', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngSanitize', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning'])
    .run(['$log', function ($log) {
        $log.debug('app.vatDongfeng.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(vatDongfengModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            $stateProvider.state({
                name: 'IncomeTax',
                url: '/IncomeTax',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<dongfeng-nav-bar type="1"></dongfeng-nav-bar>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'CashFlow',
                url: '/CashFlow',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<dongfeng-nav-bar type="2"></dongfeng-nav-bar>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'DataSummary',
                url: '/DataSummary',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<dongfeng-nav-bar type="3"></dongfeng-nav-bar>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'GlobalSearch',
                url: '/GlobalSearch',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<dongfeng-nav-bar type="4"></dongfeng-nav-bar>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'RiskPanel',
                url: '/RiskPanel',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat]);
                                        }],
                        template: '<dongfeng-nav-bar type="5"></dongfeng-nav-bar>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });
        }]);

var noPermissionPageModule = angular.module('app.noPermissionPage', [])
    .run(['$log', function ($log) {
        $log.debug('app.noPermissionPage.run()...');
    }])
    .config(['$controllerProvider', '$compileProvider', '$filterProvider', '$provide', 'scriptDependencyProvider', '$stateProvider', '$urlRouterProvider',
        function ($controllerProvider, $compileProvider, $filterProvider, $provide, scriptDependencyProvider, $stateProvider, $urlRouterProvider) {
            'use strict';


            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(noPermissionPageModule, $controllerProvider, $compileProvider, $filterProvider, $provide);


            $stateProvider.state({
                name: 'noPermissionPage',
                url: '/noPermissionPage',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                       function ($scope, $stateParams, appTranslation) {
                           appTranslation.load([appTranslation.noPermissionPage]);
                       }],
                        template: '<no-permission-page></no-permission-page>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.noPermissionPage),
                deepStateRedirect: true,
                sticky: true
            });
        }]);

var taxDocumentManageModule = angular.module('app.taxDocumentManage', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns'
    , 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns'
    , 'mc.resizer', 'ui.select', 'ngSanitize', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning'])
    .run(['$log', function ($log) {
        $log.debug('app.taxDocumentManage.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(taxDocumentManageModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            //uiSelectConfig.theme = 'select2';

            //$urlRouterProvider.when('/taxDocumentManage', '/taxDocumentManage/document-manage/documentManage');


            //$stateProvider.state({
            //    name: 'taxDocumentManage',
            //    url: '/taxDocumentManage',
            //    cache: false,
            //    views: {
            //        '@': {
            //            controller: ['$scope', '$stateParams', 'appTranslation',
            //                function ($scope, $stateParams, appTranslation) {
            //                    appTranslation.load([appTranslation.cit]);
            //                }],
            //            template: '<document-manage></document-manage>',
            //        }
            //    },
            //    resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.taxDocumentManage), //???
            //    deepStateRedirect: true,
            //    sticky: true
            //});

        }]);


var taxTaskManageModule = angular.module('app.taxTaskManage', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns'
    , 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns'
    , 'mc.resizer', 'ui.select', 'ngSanitize', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning'])
    .run(['$log', function ($log) {
        $log.debug('app.taxTaskManage.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(taxTaskManageModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            $stateProvider.state({
                name: 'taxTaskManage',
                url: '/taxTaskManage?{code:string}',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                console.log("taxTaskManageModule $stateParams:" + $stateParams.code);
                                $scope.code = $stateParams.code;
                                console.log("taxTaskManageModule $scope.code:" + $scope.code);
                                appTranslation.load([appTranslation.cit]);
                            }],
                        template: '<tax-task-manage code="code"></tax-task-manage>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.taxTaskManage),
                deepStateRedirect: false,
                sticky: false
            });
        }]);

var dataImportModule = angular.module('app.dataImport', [])
    .run(['$log', function ($log) {
        $log.debug('app.dataImport.run()...');
    }])
    .config(['$controllerProvider', '$compileProvider', '$filterProvider', '$provide', 'scriptDependencyProvider', '$stateProvider', '$urlRouterProvider',
        function ($controllerProvider, $compileProvider, $filterProvider, $provide, scriptDependencyProvider, $stateProvider, $urlRouterProvider) {
            'use strict';

            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(dataImportModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            $stateProvider.state({
                name: 'GlobalDataImportLayout',
                url: '/globalDataImportLayout',
                template: '<div global-data-import-layout></div>',
                controller: ['$scope', '$stateParams', 'appTranslation', function ($scope, $stateParams, appTranslation) {
                    appTranslation.load([appTranslation.appPart]);
                }],
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImport),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'GlobalDataImportView',
                url: '/globalDataImportView',
                params: {
                    "organization": null,
                    "dataType": null,
                    "periodYear": null,
                    "periodMonth": null,
                    "project": null,
                    "isBatchImport": null
                },
                template: '<div global-data-import-view="stateParams"></div>',
                controller: ['$scope', '$stateParams', 'appTranslation', function ($scope, $stateParams, appTranslation) {
                    appTranslation.load([appTranslation.appPart]);
                    $scope.stateParams = $stateParams;
                }],
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImport),
                deepStateRedirect: true,
                sticky: true
            });
        }]);